<?php
namespace Teknisa\FIN\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\FIN\Services\ParamgeralGeralrecService;

class ParamgeralGeralrec extends \Teknisa\FIN\Controllers\TeknisaCrud {
    protected $dataSourceName = "paramgeral_geralrec";
    protected $connection;
    protected $crudService;
    protected $entityManager;

    public function __construct($crudService, $parameterBag, $entityManager, $paramGeralService) {
    	parent::__construct($crudService, $parameterBag);
    	$this->crudService = $crudService;
    	$this->connection = $entityManager->getConnection();
    	$this->entityManager = $entityManager;
    	$this->paramGeralService = $paramGeralService;
	}

    public function save(DTO\Request $request, DTO\Response $response) {
		$dataSet = $request->getDataSet();
		$rows = $dataSet->getRows();
		
		$paramGeral = $rows[0];
		$geralRec = $rows[1];

		$routePath = $request->getRoutePath();
		$method = $request->getMethod();
		$userId = $request->getUserId();

		$this->connection->beginTransaction();
		try{
			$newRequestPG = $this->buildRequest('paramgeral', $paramGeral, $routePath, $method, $userId);
			$rowToSave = $this->crudService->getRowsToSave($newRequestPG, 'paramgeral');
            $this->crudService->save('paramgeral', $rowToSave);
            $this->entityManager->flush();

            $newRequestGR = $this->buildRequest('geral_rec', $geralRec, $routePath, $method, $userId);
            $rowToSave = $this->crudService->getRowsToSave($newRequestGR, 'geral_rec');
            $this->crudService->save('geral_rec', $rowToSave);
            $this->entityManager->flush();
            
            $this->paramGeralService->atualizaGerParamGeral($paramGeral, $geralRec);
            
            $this->connection->commit();

		} catch (\Exception $e){
			$this->connection->rollback();
			$response->setError(new Error($e->getMessage(), $e->getCode()));
		}
 	}

	public function buildRequest($dataSourceName, $row, $method, $routePath, $userId) {
		$dataSet = new DataSet($dataSourceName, array($row));
        $request = new DTO\Request\DataSet($dataSet, $method, $routePath, $userId);
        return $request;
	}
}