<?php
namespace Teknisa\FIN\Factories;

use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Ger38200ImportacaoContasPagarFactory {

    protected $connection;
    protected $entityManager;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }
    
    /**
     * Busca Parametrização Geral
     * @return Paramgeral
     */
    public function buscaParamgeral() {
        $sql = "SELECT DSURLHCM, DSIDHCM, DSSECRETHCM, DSTOKENHCM FROM PARAMGERAL";
        
        $statement = $this->connection->prepare($sql);
        $statement->execute();
        
        return $statement->fetch();
    }
    
    
    /**
     * Atualiza Token da tabela PARAMGERAL
     * @
     */
    public function updateToken($dataUpdate){
        
        $sql = "UPDATE PARAMGERAL
                   SET DSTOKENHCM = :DSTOKENHCM
                   WHERE CDPARGERAL = :CDPARGERAL
                   AND NRORG = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($dataUpdate);
    }
    
    /**
     * Busca Filial de Origem
     * @
     */
     public function getFilialDeOrigem($cdempresa, $cdfilial, $nrorg){
        $params = array();
        $params['P_CDEMPRESA'] = $cdempresa;
        $params['P_CDFILIAL'] = $cdfilial;
        $params['P_NRORG'] = $nrorg;
        $sql = "SELECT CDFILIAL, NMFILIAL, CDINSCESTA, CDINSCMUNI, NRORG FROM(SELECT FI.CDFILIAL, FI.NMFILIAL, FI.CDEMPRESA, FI.CDINSCESTA, FI.CDINSCMUNI, NVL(PF.NRTAMNUMNFUTDFE,0) AS NRTAMNUMNFUTDFE, FI.CDCENTCUST, PF.NRORG AS NRORG FROM PARAMFILIAL PF, FILIAL FI WHERE(FI.CDEMPRESA = :P_CDEMPRESA) AND (FI.CDFILIAL = :P_CDFILIAL) AND (FI.IDCNTRRESC = 'N') AND (PF.CDFILIAL = FI.CDFILIAL) AND (PF.NRORG = FI.NRORG) UNION SELECT EF.CDFILIAL, FI.NMFILIAL, EF.CDEMPRESA, EF.CDINSCESTA, EF.CDINSCMUNI, NVL(PF.NRTAMNUMNFUTDFE,0) AS NRTAMNUMNFUTDFE, FI.CDCENTCUST, PF.NRORG AS NRORG FROM PARAMFILIAL PF, FILIAL FI, EMPRCNTRFIL EF WHERE (EF.CDEMPRESA = :P_CDEMPRESA) AND (FI.CDFILIAL = :P_CDFILIAL) AND (FI.IDCNTRRESC = 'N') AND (FI.CDFILIAL = EF.CDFILIAL) AND (PF.CDFILIAL = FI.CDFILIAL) AND (FI.NRORG = EF.NRORG)) A WHERE A.NRORG = :P_NRORG ORDER BY CDFILIAL";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
     /**
     * Busca dados do CNPJ/CPF pelo CDFORNECED
     * @
     */
    public function getCNPJOrCPFPagar($nrinsjurforn, $nrorg){
        $params = array();
        $params['P_NRINSJURFORN'] = $nrinsjurforn;
        $params['P_NRORG'] = $nrorg;
        $sql = "SELECT CDFORNECED, IDTPIJURFORN, NRINSJURFORN, NMRAZSOCFORN FROM FORNECEDOR WHERE NRINSJURFORN = :P_NRINSJURFORN AND NRORG = :P_NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
     /**
     * Busca dados do Tipo de Conta a Pagar
     * @
     */
   public function getTipoDeContaPagar($cdtpctpagar, $nrorg){
        $params = array();
        $params['P_CDTPCTPAGAR'] = $cdtpctpagar;
        $params['P_NRORG'] = $nrorg;
        $sql = "SELECT CDTPCTPAGAR, DSTPCTPAGAR, IDCTBLCTPAG, DSDEFACTPAG, CDCLASSFINA, IDUTLANCCPAG FROM TIPOCTPAGAR WHERE CDTPCTPAGAR = :P_CDTPCTPAGAR AND NRORG = :P_NRORG ORDER BY CDTPCTPAGAR";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
     /**
     * Busca dados do Tipo de Conta a Pagar cadastradoas
     * @
     */
   public function getNotaFiscalUtilizada($cdforneced, $cdserieentr,$nrnotafisc){
        $params = array();
        $params['CDFORNECED'] = $cdforneced;
        $params['CDSERIEENTR'] = $cdserieentr;
        $params['NRNOTAFISC'] = $nrnotafisc;
        $params['IDENTRSAIDOP'] = 'E';
        $sql = "SELECT CDFILIAL, NRLANCTONF,CDFORNECED,NRNOTAFISC FROM NOTAFISCAL WHERE  CDFORNECED = :CDFORNECED AND CDSERIEENTR =:CDSERIEENTR AND NRNOTAFISC =:NRNOTAFISC AND IDENTRSAIDOP =: IDENTRSAIDOP";  
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
     /**
     * Busca os centros de custo cadastrados
     * @
     */
    public function getCentroDeCusto($cdcentcust, $nrorg){
        $params = array();
        $params['P_CDCENTCUST'] = $cdcentcust;
        $params['P_NRORG'] = $nrorg;
        $sql = "SELECT CDCENTCUST FROM CENTCUST where CDCENTCUST = :P_CDCENTCUST AND NRORG = :P_NRORG ORDER BY CDCENTCUST"; 
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    /**
     * Busca as filiais cadastradas
     * @
     */
    public function getFilial($cdfilial,$cdempresa, $nrorg){
        $params = array();
        $params['P_CDFILIAL'] = $cdfilial;
        $params['P_CDEMPRESA'] = $cdempresa;
        $params['P_NOROG'] = $nrorg;
        $sql = "SELECT CDFILIAL FROM FILIAL WHERE CDFILIAL = :P_CDFILIAL AND CDEMPRESA = :P_CDEMPRESA AND NRORG = :P_NOROG"; 
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    /**
     * Busca o tamanho da nota fiscal para gera o número do tamanho correto
     * @
     */
    public function getTamanhoNotaFiscal($cdfilial,$cdempresa){
        $params = array();
        $params['P_CDFILIAL'] = $cdfilial;
        $params['P_CDEMPRESA'] = $cdempresa;
        $sql = "SELECT NVL(PF.NRTAMNUMNFUTDFE,0) AS NRTAMNUMNFUTDFE FROM PARAMFILIAL PF, FILIAL FI WHERE (FI.CDFILIAL = :P_CDFILIAL ) AND (FI.CDEMPRESA = :P_CDEMPRESA ) AND (PF.CDFILIAL = FI.CDFILIAL) AND (FI.IDCNTRRESC = 'N' )"; 
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    /**
     * Busca se a conta já foi inserida
     * @
     */
    public function getContaInserida($cdempresa,$dtatuavenpag, $dtorigvenpag, $cdtpctpagar, $cdfilibpag, $vrorigpag, $vratuapag, $nrinsjurnf){
        $params = array();
        $params['P_CDEMPRESA']    = $cdempresa;
        $params['P_DTATUAVENPAG'] = $dtatuavenpag;
        $params['P_DTORIGVENPAG'] = $dtorigvenpag;
        $params['P_CDTPCTPAGAR']  = $cdtpctpagar;
        $params['P_CDFILIBPAG']   = $cdfilibpag;
        $params['P_VRORIGPAG']    = $vrorigpag;
        $params['P_VRATUAPAG']    = $vratuapag;
        $params['P_NRINSJURNF']   = $nrinsjurnf;
        
        $sql = "SELECT 
                  C.CDFILIAL, 
                  C.NRLANCTONF, 
                  C.DSCONTAPAG 
                FROM 
                  CONTAPAGAR C, 
                  NOTAFISCAL N 
                WHERE 
                  C.CDEMPRESA = :P_CDEMPRESA 
                  AND C.DTATUAVENPAG = :P_DTATUAVENPAG 
                  AND C.DTORIGVENPAG = :P_DTORIGVENPAG 
                  AND C.CDTPCTPAGAR = :P_CDTPCTPAGAR 
                  AND C.CDFILIBPAG = :P_CDFILIBPAG 
                  AND C.VRORIGPAG = :P_VRORIGPAG 
                  AND C.VRATUAPAG = :P_VRATUAPAG 
                  AND C.CDFILIAL = N.CDFILIAL 
                  AND C.NRLANCTONF = N.NRLANCTONF 
                  AND N.NRINSJURNF = :P_NRINSJURNF"; 
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    
}