<?php

namespace Teknisa\FIN\Modules\ContaPagar\Factory;

use Teknisa\Model\Geral\Entities\Banco;
use Teknisa\Model\Financeiro\Entities\Retornodda;
use Teknisa\Model\Fiscal\Entities\Configurasmtp;
use Teknisa\Model\Fiscal\Entities\Emprbanc;
use Teknisa\Model\Geral\Entities\Empresa;
use Doctrine\ORM\EntityManager;

class Pag30000RetornoDDAFactory
{

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager = $entityManager;
    }

    public function getBanco($cdbanco, $nrorg){
        $parans = array();
        $parans['cdbanccnab'] = $cdbanco;
        $parans['nrorg'] = $nrorg;
        $banco = $this->entityManager->getRepository(Banco::class)->findOneBy($parans);
        return $banco;
    }

    public function deleteRetornoDDA($retorno, $row, $cdempresa){
        $parans = array(
            'CDFORNECED'  => $retorno->getCdforneced(),
            'DTVENBOLPAG' => $retorno->getDtvenbolpag(),
            'NRNOTAFISC'  => $retorno->getNrnotafisc(),
            'VRFINALBPAG' => $retorno->getVrfinalbpag(),
            'CDEMPRESA'   => $cdempresa,
            'CDBANCO'     => $row['CDBANCCNAB']
        );
        
        $sql = "DELETE FROM RETORNODDA
                    WHERE ((CDEMPRESA = :CDEMPRESA) OR (CDEMPRESA IS NULL))
                    AND CDFORNECED = :CDFORNECED
                    AND DTVENBOLPAG = :DTVENBOLPAG
                    AND ((LPAD (NRNOTAFISC, 20, '0') = LPAD (:NRNOTAFISC, 20, '0')) OR (NRNOTAFISC IS NULL))
                    AND VRFINALBPAG = :VRFINALBPAG
                    AND ((CDBANCO = :CDBANCO) OR (CDBANCO IS NULL))"; 
        $delete = $this->entityManager->getConnection()->prepare($sql);
        $delete->execute($parans);
    }
    
    public function getConfigurasmtp(){
        $parans = array();
        $configurasmtp = $this->entityManager->getRepository(Configurasmtp::class)->findOneBy($parans);
        return $configurasmtp;
    }

    public function getRetornoDDA(){
        $parans = array();
        $retornodda = $this->entityManager->getRepository(Retornodda::class)->findBy($parans);
        return $retornodda;
    }

    public function getEmprbanc($cdbanco, $cdempresa, $nrorg){
        $params = array();
        $params['nrorg'] = $nrorg;
        $params['cdbanco'] = $cdbanco;
        $params['cdempresa'] = $cdempresa;
        $emprbanc = $this->entityManager->getRepository(Emprbanc::class)->findOneBy($params);
        return $emprbanc;
    }

    public function getEmpresa($retorno, $nrorg){
        $params = array(
            'nrorg'        => $nrorg,
            'nrinsjurempr' => $retorno->getCnpjEmpresa()
        );
        
        $empresa = $this->entityManager->getRepository(Empresa::class)->findOneBy($params);
        return $empresa;
    }
    
    public function getEmpresaRaizCNPJ($nrinsjurempr){
        $params = array(
            'NRINSJUREMPR'  => $nrinsjurempr
        );
        
        $sql = "SELECT DISTINCT CDEMPRESA
                  FROM EMPRESA
                 WHERE NRINSJUREMPR = :NRINSJUREMPR
                 UNION
                SELECT DISTINCT CDEMPRESA
                  FROM EMPRESA
                 WHERE SUBSTR(NRINSJUREMPR,1,8) = SUBSTR(:NRINSJUREMPR,1,8)";
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        
        return $std->fetch();
    }
}