<?php

namespace Teknisa\FIN\Modules\ContaPagar\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\Environment;
use Teknisa\FIN\Modules\ContaPagar\Factory\InconsistenciasDDAFactory;

class InconsistenciasDDAService {

    const ABERTO    = 'A';
    const PLANEJADO = 'P';
    const BAIXADO   = 'B';
    const CANCELADO = 'C';

    private $existeInconsistencia;
    private $existeBoleto;

    public function __construct(
        EntityManager $entityManager,
        Environment $environment,
        InconsistenciasDDAFactory $InconsistenciasDDAFactory
    )
    {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->inconsistenciasDDAFactory = $InconsistenciasDDAFactory;
    }

    public function validaInconsistenciasBoletosDoArquivo($retornosDDA)
    {
        try {
            $nrorg = $this->environment->getNrOrgTrab();
            foreach ($retornosDDA as $retorno) {
                
                $nrnotafisc = str_pad(preg_replace("/[^0-9]/", "", $retorno->getNrnotafisc()), 20, 0, STR_PAD_LEFT);
                $retorno->setNrnotafisc($nrnotafisc);
                
                $empresa = $this->inconsistenciasDDAFactory->getEmpresa($retorno, $nrorg);
                $row = array(
                    'NRORG'       => $nrorg,
                    'CDEMPRESA'   => $empresa->getCdempresa(),
                    'DTVENBOLPAG' => $this->formataData($retorno->getDtvenbolpag())
                );
                $this->validaInconsistencias($retorno, $row);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaInconsistencias($retorno, $row)
    {
        try {
            $this->existeBoleto         = false;
            $this->existeInconsistencia = false;

            // Busca título com todos os dados do arquivo DDA - EM ABERTO
            $parans = array(
                'cnpjempresa'   => $retorno->getCnpjEmpresa(),
                'cdforneced'    => $retorno->getCdforneced(),
                'nrnotafisc'    => $retorno->getNrnotafisc(),
                'dtvenbolpag'   => $retorno->getDtvenbolpag(),
                'vrfinalbpag'   => $retorno->getVrfinalbpag(),
                'todosvr'       => 'N',
                'idstatusa'     => self::ABERTO,
                'idstatusb'     => self::ABERTO,
                'idstatusc'     => self::ABERTO,
                'tcdforneced'   => 'N',
                'tdtvenbolpag'  => 'N',
                'tnrnotafisc'   => 'N'
            );
            $boletPagar = $this->inconsistenciasDDAFactory->getBoletos($parans, $this->environment->getNrOrgTrab());
            // Verifica existencia do Título
            if ($boletPagar){
                $this->existeBoleto = true;
                
                $row['CDREDUCONT']   = $boletPagar[0]['CDREDUCONT'];
                $row['NRINSJURFILI'] = $boletPagar[0]['NRINSJURFILI'];
                
                // Verifica Quantidade de Titulos Relacionados
                if (count($boletPagar) == 1){
                    // Verifica Preenchimento Código de Barras
                    if (!$boletPagar[0]['CDBARRAPAG']){
                        // Gera Representação Númerica
                        $this->geraRepresentacaoNumerica($boletPagar, $retorno);
                        
                        // Gera Retorno DDA - OK
                        $row['STATUS'] = 'OK';
                        $retornoDda = $this->inconsistenciasDDAFactory->insereRetornoDDA($retorno, '', $row);
                        $this->entityManager->persist($retornoDda);
                    }else{
                        // Gera Inconsistência - Código de Barras Preenchido
                        $this->existeInconsistencia = true;
                        $this->codigoDeBarrasPreenchido($retorno, $row);
                    }
                }else{
                    // Gera Inconsistência - Títulos Relacionados
                    $this->existeInconsistencia = true;
                    $this->boletosRelacionados($retorno, $row);
                }
            }else{
                // Busca título com todos os dados do arquivo DDA - PLANEJADO/BAIXADO/CANCELADO
                $parans = array(
                    'cnpjempresa'   => $retorno->getCnpjEmpresa(),
                    'cdforneced'    => $retorno->getCdforneced(),
                    'nrnotafisc'    => $retorno->getNrnotafisc(),
                    'dtvenbolpag'   => $retorno->getDtvenbolpag(),
                    'vrfinalbpag'   => $retorno->getVrfinalbpag(),
                    'todosvr'       => 'N',
                    'idstatusa'     => self::PLANEJADO,
                    'idstatusb'     => self::BAIXADO,
                    'idstatusc'     => self::CANCELADO,
                    'tcdforneced'   => 'N',
                    'tdtvenbolpag'  => 'N',
                    'tnrnotafisc'   => 'N'
                );
                $boletPagar = $this->inconsistenciasDDAFactory->getBoletos($parans, $this->environment->getNrOrgTrab());
                
                // Verifica Divergencia Status
                if ($boletPagar){
                    $this->existeInconsistencia = true;
                    $this->existeBoleto = true;
                    
                    $row['CDREDUCONT']   = $boletPagar[0]['CDREDUCONT'];
                    $row['NRINSJURFILI'] = $boletPagar[0]['NRINSJURFILI'];
                    
                    $this->statusBoletoInvalido($retorno, $row);
                }else{
                    // Verifica Divergencia Fornecedor
                    if(!$this->existeInconsistencia)
                        $this->existeInconsistencia = $this->existeFornecedorDivergente($retorno, $row);
                    
                    // Verifica Divergencia Documento
                    if(!$this->existeInconsistencia)
                        $this->existeInconsistencia = $this->existeDocumentoDivergente($retorno, $row);
                    
                    // Verifica Divergencia Valor
                    if(!$this->existeInconsistencia)
                        $this->existeInconsistencia = $this->existeDivergenciaDeValor($retorno, $row);
                    
                    // Verifica Divergencia Data
                    if(!$this->existeInconsistencia)
                        $this->existeInconsistencia = $this->existeDivergenciaDeData($retorno, $row);
                }
            }
            
            if (!$this->existeBoleto && !$this->existeInconsistencia){
                // Busca título com todos os dados do arquivo DDA - EM ABERTO - SEM DOCUMENTO
                $parans = array(
                    'cnpjempresa'   => $retorno->getCnpjEmpresa(),
                    'cdforneced'    => $retorno->getCdforneced(),
                    'nrnotafisc'    => null,
                    'dtvenbolpag'   => $retorno->getDtvenbolpag(),
                    'vrfinalbpag'   => $retorno->getVrfinalbpag(),
                    'todosvr'       => 'N',
                    'idstatusa'     => self::ABERTO,
                    'idstatusb'     => self::ABERTO,
                    'idstatusc'     => self::ABERTO,
                    'tcdforneced'   => 'N',
                    'tdtvenbolpag'  => 'N',
                    'tnrnotafisc'   => 'T'
                );
                $boletPagar = $this->inconsistenciasDDAFactory->getBoletos($parans, $this->environment->getNrOrgTrab());
                
                if ($boletPagar){
                    $this->existeBoleto = true;
                    
                    $row['CDREDUCONT']   = $boletPagar[0]['CDREDUCONT'];
                    $row['NRINSJURFILI'] = $boletPagar[0]['NRINSJURFILI'];
                    
                    // Verifica Quantidade de Titulos Relacionados
                    if (count($boletPagar) == 1){
                        // Verifica Preenchimento Código de Barras
                        if (!$boletPagar[0]['CDBARRAPAG']){
                            // Gera Representação Númerica
                            $this->geraRepresentacaoNumerica($boletPagar, $retorno);
                            
                            // Gera Retorno DDA - OK
                            $row['STATUS'] = 'OK';
                            $retornoDda = $this->inconsistenciasDDAFactory->insereRetornoDDA($retorno, '', $row);
                            $this->entityManager->persist($retornoDda);
                        }else{
                            // Gera Inconsistência - Código de Barras Preenchido
                            $this->existeInconsistencia = true;
                            $this->codigoDeBarrasPreenchido($retorno, $row);
                        }
                    }else{
                        // Gera Inconsistência - Títulos Relacionados
                        $this->existeInconsistencia = true;
                        $this->boletosRelacionados($retorno, $row);
                    }
                }else{
                    // Busca título com todos os dados do arquivo DDA - PLANEJADO/BAIXADO/CANCELADO
                    $parans = array(
                        'cnpjempresa'   => $retorno->getCnpjEmpresa(),
                        'cdforneced'    => $retorno->getCdforneced(),
                        'nrnotafisc'    => null,
                        'dtvenbolpag'   => $retorno->getDtvenbolpag(),
                        'vrfinalbpag'   => $retorno->getVrfinalbpag(),
                        'todosvr'       => 'N',
                        'idstatusa'     => self::PLANEJADO,
                        'idstatusb'     => self::BAIXADO,
                        'idstatusc'     => self::CANCELADO,
                        'tcdforneced'   => 'N',
                        'tdtvenbolpag'  => 'N',
                        'tnrnotafisc'   => 'T'
                    );
                    $boletPagar = $this->inconsistenciasDDAFactory->getBoletos($parans, $this->environment->getNrOrgTrab());
                    
                    // Verifica Divergencia Status
                    if ($boletPagar){
                        $this->existeInconsistencia = true;
                        $this->existeBoleto = true;
                        
                        $row['CDREDUCONT']   = $boletPagar[0]['CDREDUCONT'];
                        $row['NRINSJURFILI'] = $boletPagar[0]['NRINSJURFILI'];
                        
                        $this->statusBoletoInvalido($retorno, $row);
                    }else{
                        $this->boletoInexistenteNoSistema($retorno, $row);
                    }
                }
            }
                
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function geraRepresentacaoNumerica($boleto, $retorno){
        $parans = array(
            'cdfilial'    => $boleto[0]['CDFILIAL'],
            'nrlanbolpag' => $boleto[0]['NRLANBOLPAG'],
            'nrorg'       => $this->environment->getNrOrgTrab()
        );
        $boletPagar = $this->inconsistenciasDDAFactory->getBoletPagar($parans);
        $linhaDigitavel = $this->geraLinhaDigitavelPorCodigoBarras($retorno->getCdbarrapag());
        $emprbanc = $this->inconsistenciasDDAFactory->getEmprbanc(
            $boletPagar->getCdempresa(),
            $retorno->getCodigoBanco(),
            $this->environment->getNrOrgTrab()
        );
        $boletPagar->setCdbarrapag($retorno->getCdbarrapag());
        $boletPagar->setDslinhbarpag($linhaDigitavel);
        $boletPagar->setIdcdbarradda('S');
        if ($emprbanc){
            $boletPagar->setCdtpbaixapag($emprbanc['CDTPBAIXAPAG']);
            $boletPagar->setCdreducont($emprbanc['CDREDUCONT']);
        }else{
            $boletPagar->setCdtpbaixapag(null);
            $boletPagar->setCdreducont(null);
        }
        $this->entityManager->persist($boletPagar);
    }

    public function boletosRelacionados($retorno, $row){
        $paramdda = $this->inconsistenciasDDAFactory->getParamdda(1, $this->environment->getNrOrgTrab());
        
        if($paramdda[0]['IDCONSISTE'] == 'S'){
            $row['STATUS'] = 'ERRO';
            $dsincons = "Existem mais de um título relacionado a Empresa, Data de Vencimento, Fornecedor e Valor.";
            $retornoDda = $this->inconsistenciasDDAFactory->insereRetornoDDA($retorno, $dsincons, $row);
            $this->entityManager->persist($retornoDda);
        }
    }

    public function codigoDeBarrasPreenchido($retorno, $row){
        $paramdda = $this->inconsistenciasDDAFactory->getParamdda(2, $this->environment->getNrOrgTrab());
        
        if($paramdda[0]['IDCONSISTE'] == 'S'){
            $row['STATUS'] = 'ERRO';
            $dsincons = "Título já possui código de barras preenchido.";
            $retornoDda = $this->inconsistenciasDDAFactory->insereRetornoDDA($retorno, $dsincons, $row);
            $this->entityManager->persist($retornoDda);
        }
    }

    public function statusBoletoInvalido($retorno, $row){
        $paramdda = $this->inconsistenciasDDAFactory->getParamdda(3, $this->environment->getNrOrgTrab());
        
        if($paramdda[0]['IDCONSISTE'] == 'S'){
            $row['STATUS'] = 'ERRO';
            $dsincons = "Título encontra-se baixado, planejado ou cancelado.";
            $retornoDda = $this->inconsistenciasDDAFactory->insereRetornoDDA($retorno, $dsincons, $row);
            $this->entityManager->persist($retornoDda);
        }
    }

    public function boletoInexistenteNoSistema($retorno, $row){
        $paramdda = $this->inconsistenciasDDAFactory->getParamdda(4, $this->environment->getNrOrgTrab());
        
        if($paramdda[0]['IDCONSISTE'] == 'S'){
            $row['CDREDUCONT']   = '';
            $row['NRINSJURFILI'] = '';
            $row['STATUS']       = 'ERRO';
            $dsincons = "Não foi encontrado nenhum título no sistema correspondente ao do arquivo DDA.";
            $retornoDda = $this->inconsistenciasDDAFactory->insereRetornoDDA($retorno, $dsincons, $row);
            $this->entityManager->persist($retornoDda);
        }
    }

    public function existeDivergenciaDeValor($retorno, $row){
        $this->existeInconsistencia = false;
        
        $paramdda = $this->inconsistenciasDDAFactory->getParamdda(5, $this->environment->getNrOrgTrab());
        
        if($paramdda[0]['IDCONSISTE'] == 'S'){
            $parans = array(
                'cnpjempresa'   => $retorno->getCnpjEmpresa(),
                'cdforneced'    => $retorno->getCdforneced(),
                'nrnotafisc'    => $retorno->getNrnotafisc(),
                'dtvenbolpag'   => $retorno->getDtvenbolpag(),
                'vrfinalbpag'   => 0,
                'todosvr'       => 'T',
                'idstatusa'     => 'T',
                'idstatusb'     => 'T',
                'idstatusc'     => 'T',
                'tcdforneced'   => 'N',
                'tdtvenbolpag'  => 'N',
                'tnrnotafisc'   => 'N'
            );
            $boletPagar = $this->inconsistenciasDDAFactory->getBoletos($parans, $this->environment->getNrOrgTrab());
            if ($boletPagar){
                $this->existeInconsistencia = true;
                $dsincons = "Valor inconsistente.";
                $row['CDREDUCONT']   = $boletPagar[0]['CDREDUCONT'];
                $row['NRINSJURFILI'] = $boletPagar[0]['NRINSJURFILI'];
                $row['STATUS']       = 'ERRO';
                $retornoDda = $this->inconsistenciasDDAFactory->insereRetornoDDA($retorno, $dsincons, $row);
                $this->entityManager->persist($retornoDda);
                
                $this->existeInconsistencia = true;
            }
        }
        
        return $this->existeInconsistencia;
    }

    public function existeDivergenciaDeData($retorno, $row){
        $this->existeInconsistencia = false;
        
        $paramdda = $this->inconsistenciasDDAFactory->getParamdda(6, $this->environment->getNrOrgTrab());
        
        if($paramdda[0]['IDCONSISTE'] == 'S'){
            $parans = array(
                'cnpjempresa'  => $retorno->getCnpjEmpresa(),
                'cdforneced'   => $retorno->getCdforneced(),
                'nrnotafisc'   => $retorno->getNrnotafisc(),
                'dtvenbolpag'  => $retorno->getDtvenbolpag(),
                'vrfinalbpag'  => $retorno->getVrfinalbpag(),
                'todosvr'      => 'N',
                'idstatusa'    => 'T',
                'idstatusb'    => 'T',
                'idstatusc'    => 'T',
                'tcdforneced'  => 'N',
                'tdtvenbolpag' => 'T',
                'tnrnotafisc'  => 'N'
            );
            $boletPagar = $this->inconsistenciasDDAFactory->getBoletos($parans, $this->environment->getNrOrgTrab());
            if ($boletPagar){
                $this->existeInconsistencia = true;
                $dsincons = "Data de vencimento inconsistente.";
                $row['CDREDUCONT']   = $boletPagar[0]['CDREDUCONT'];
                $row['NRINSJURFILI'] = $boletPagar[0]['NRINSJURFILI'];
                $row['STATUS']       = 'ERRO';
                $retornoDda = $this->inconsistenciasDDAFactory->insereRetornoDDA($retorno, $dsincons, $row);
                $this->entityManager->persist($retornoDda);
                
                $this->existeInconsistencia = true;
            }
        }
        
        return $this->existeInconsistencia;
    }

    public function existeFornecedorDivergente($retorno, $row){
        $this->existeInconsistencia = false;
        
        $paramdda = $this->inconsistenciasDDAFactory->getParamdda(7, $this->environment->getNrOrgTrab());
        
        if($paramdda[0]['IDCONSISTE'] == 'S'){
            $parans = array(
                'cnpjempresa'  => $retorno->getCnpjEmpresa(),
                'cdforneced'   => null,
                'nrnotafisc'   => $retorno->getNrnotafisc(),
                'dtvenbolpag'  => $retorno->getDtvenbolpag(),
                'vrfinalbpag'  => $retorno->getVrfinalbpag(),
                'todosvr'      => 'N',
                'idstatusa'    => 'T',
                'idstatusb'    => 'T',
                'idstatusc'    => 'T',
                'tcdforneced'  => 'T',
                'tdtvenbolpag' => 'N',
                'tnrnotafisc'  => 'N'
            );
            
            $boletPagar = $this->inconsistenciasDDAFactory->getBoletos($parans, $this->environment->getNrOrgTrab());
            
            if ($boletPagar){
                $this->existeInconsistencia = true;
                $dsincons = "Fornecedor inconsistente.";
                $row['CDREDUCONT']   = $boletPagar[0]['CDREDUCONT'];
                $row['NRINSJURFILI'] = $boletPagar[0]['NRINSJURFILI'];
                $row['STATUS']       = 'ERRO';
                $retornoDda = $this->inconsistenciasDDAFactory->insereRetornoDDA($retorno, $dsincons, $row);
                $this->entityManager->persist($retornoDda);
                
                $this->existeInconsistencia = true;
            }
        }
        
        return $this->existeInconsistencia;
    }
    
    public function existeDocumentoDivergente($retorno, $row){
        $this->existeInconsistencia = false;
        
        $paramdda = $this->inconsistenciasDDAFactory->getParamdda(8, $this->environment->getNrOrgTrab());
        
        if($paramdda[0]['IDCONSISTE'] == 'S'){
            $parans = array(
                'cnpjempresa'  => $retorno->getCnpjEmpresa(),
                'cdforneced'   => $retorno->getCdforneced(),
                'nrnotafisc'   => null,
                'dtvenbolpag'  => $retorno->getDtvenbolpag(),
                'vrfinalbpag'  => $retorno->getVrfinalbpag(),
                'todosvr'      => 'N',
                'idstatusa'    => 'T',
                'idstatusb'    => 'T',
                'idstatusc'    => 'T',
                'tcdforneced'  => 'N',
                'tdtvenbolpag' => 'N',
                'tnrnotafisc'  => 'T'
            );
            
            $boletPagar = $this->inconsistenciasDDAFactory->getBoletos($parans, $this->environment->getNrOrgTrab());
            
            if ($boletPagar){
                $this->existeInconsistencia = true;
                $dsincons = "Documento inconsistente.";
                $row['CDREDUCONT']   = $boletPagar[0]['CDREDUCONT'];
                $row['NRINSJURFILI'] = $boletPagar[0]['NRINSJURFILI'];
                $row['STATUS']       = 'ERRO';
                $retornoDda = $this->inconsistenciasDDAFactory->insereRetornoDDA($retorno, $dsincons, $row);
                $this->entityManager->persist($retornoDda);
                
                $this->existeInconsistencia = true;
            }
        }
        
        return $this->existeInconsistencia;
    }
    
    public function geraLinhaDigitavelPorCodigoBarras($codigoBarras){
        $linhaDigitavel = '';
        if ($codigoBarras){
            if (substr($codigoBarras, 1, 1) == '8'){
                if ( (substr($codigoBarras, 3, 1) == '6') || (substr($codigoBarras, 3, 1) == '7') ){
                    $linhaDigitavel =
                        //1º BLOCO + DV DO 1º BLOCO
                        substr($codigoBarras, 0, 11) + Mod10_Concessionaria(substr($codigoBarras, 0, 11)) +
                        //2º BLOCO + DV DO 2º BLOCO
                        substr($codigoBarras, 11, 11) + Mod10_Concessionaria(substr($codigoBarras, 11, 11)) +
                        //3º BLOCO + DV DO 3º BLOCO
                        substr($codigoBarras, 22, 11) + Mod10_Concessionaria(substr($codigoBarras, 22, 11)) +
                        //4º BLOCO + DV DO 4º BLOCO
                        substr($codigoBarras, 33, 11) + Mod10_Concessionaria(substr($codigoBarras, 33, 11));
                }else{
                    $linhaDigitavel =
                        //1º BLOCO + DV DO 1º BLOCO
                        substr($codigoBarras, 0, 11) + Mod11_B9(substr($codigoBarras, 0, 11), 'TRIBUTO') +
                        //2º BLOCO + DV DO 2º BLOCO
                        substr($codigoBarras, 11, 11) + Mod11_B9(substr($codigoBarras, 11, 11), 'TRIBUTO') +
                        //3º BLOCO + DV DO 3º BLOCO
                        substr($codigoBarras, 22, 11) + Mod11_B9(substr($codigoBarras, 22, 11), 'TRIBUTO') +
                        //4º BLOCO + DV DO 4º BLOCO
                        substr($codigoBarras, 33, 11) + Mod11_B9(substr($codigoBarras, 33, 11), 'TRIBUTO');
                }
            }
        }
    }

    public function formataData($data)
    {
        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDatadeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }

    public function Mod10_Concessionaria ($codigo){

        $result = '';

        if (strlen($codigo) > 0){

            $base   = 2;
            $soma   = 0;
            $carac  = 0;

            for ($i = (strlen($codigo) - 1); $i >= 0; $i--){

                $carac = $codigo[$i];

                if (($carac * $base) > 9)
                    $soma  = $soma + (($carac * $base) - 10) + 1;
                else $soma += ($carac * $base);

                if ($base == 2)
                    $base -= 1;
                else $base += 1;
            }

            $resto = $soma % 10;

            $result = 10 - $resto;

            if (($resto) == 0)
                $result = '0';
        }

        return $result;
    }

    public function Mod11_B9($codigo, $banco){

        $result = '';

        if (strlen($codigo) > 0) {
            $soma = 0;
            $base = 2;
            $carac = 0;

            $barraaux = $codigo;

            //if (strlen($codigo) >= 44)
            // Retira o dígito verificador do código de barras
                //$barraaux = codigodebarra.substr(0, 4) + codigodebarra.substr(5, (strlen($codigo) - 5));

            for ($i = (strlen($barraaux) - 1); $i >= 0; $i--) {

                $carac = $codigo[$i];

                $soma += ($carac * $base);

                // Incrementa a base até chegar a 9
                $base += 1;

                if ($base > 9)
                    $base = 2;
            }

            $dv = 11 - ($soma % 11);
            $resto = ($soma % 11);

            // BANESTES E NOSSA CAIXA
            if (($banco == '021') || ($banco == '151')) { // Banestes e Nossa Caixa
                if (($resto == 0) || ($resto == 1) || ($resto > 9))
                    $result = '1';
                else $result = $dv;
            } else {
                // MERCANTIL
                if ($banco == '389') { // Mercantil
                    if (($resto == 0) || ($resto == 1)) {
                        if (strlen($codigo) >= 44)
                            $result = '1'; // Dac do Código de barras
                        else $result = '0'; // Dac do Nosso numero
                    } else $result = $dv;
                } else {
                    if ($banco == '422') {
                        if ($resto == 0)
                            $result = '1';
                        else if ($resto == 1)
                            $result = '0';
                        else $result = $dv;
                    }
                    // SICREDI
                    else if ($banco == '748') {
                        if (($resto == 0) || ($resto == 1))
                            $result = '0';
                        else $result = $dv;
                    } else if ($banco == 'TRIBUTO') {
                        if (($resto == 1) || ($resto == 0))
                            $result = '0';
                        else if ($resto == 10)
                            $result = '1';
                        else $result = $dv;
                    }
                    // DEMAIS BANCOS
                    else {
                        /*
                         *Esta regra se aplica para o DV do CdBarra da caixa federal,
                         *mas não para o Nosso Número da caixa. Este é retornado na função MOD11_DV
                         */
                        if (($dv == 0) || ($dv == 1) || ($dv > 9))
                            $result = '1';
                        else $result = $dv;
                    }
                }
            }
        }
        return $result;
    }
}