<?php
namespace Teknisa\FIN\Modules\ContaPagar\Services\Pix\Banco;

use Teknisa\FIN\Modules\ContaPagar\Services\Pix\Util\ApiSicoob;
use Doctrine\ORM\EntityManager;

class GeraPixSicoobService {
    
    function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    /**
     * Inicia Pagamento Pix
     */
    public function iniciaPagamentoPix($cdbanco, $cdclientid, $dsdircertificado, $cdsenhacertif, $chavepix, $dataAgendamento, $nomefornecedor, $inscricao, $valor, $descricao, $homologacao = true){
        try {
            // Criação da Requisição
            $request = [
                "chave" => $chavepix,
                "dataAgendamento" => $dataAgendamento
            ];
            
            // Construção do Objeto API Itau
            $obApiPix = new ApiSicoob($cdclientid, $dsdircertificado, $cdsenhacertif, $homologacao);
            
            // Resposta da Requisição de Criação
            $response = $obApiPix->iniciaPagamento($request);
            
            // Verifica existência de erro
            if(isset($response['detail'])){
                var_dump('Operação bloqueada. Erro na inicialização do pagamento por pix: '.$response['detail']);die;
            }
            
            $endtoendid = $response['endToEndId'];
            
            return $endtoendid;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Confirma Pagamento Pix
     */
    public function confirmaPagamentoPix($endtoendid, $cdbanco, $cdclientid, $dsdircertificado, $cdsenhacertif, $chavepix, $dataAgendamento, $nomefornecedor, $inscricao, $valor, $descricao, $homologacao = true){
        try {
            // Criação da Requisição
            $request = [
                "endToEndId"      => $endtoendid,
                "valor"           => $valor,
                "descricao"       => $descricao,
                "repeticao"       => false,
                "meioIniciacao"   => "CHAVE",
                "dataAgendamento" => $dataAgendamento
            ];
            
            // Construção do Objeto API Itau
            $obApiPix = new ApiSicoob($cdclientid, $dsdircertificado, $cdsenhacertif, $homologacao);
            
            // Resposta da Requisição de Criação
            $response = $obApiPix->confirmaPagamento($request);
            
            // Verifica existência de erro
            if(isset($response['detail'])){
                var_dump('Operação bloqueada. Erro na confirmação do pagamento por pix: '.$response['detail']);die;
            }
            
            return $response;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Consulta Pagamento Pix
     */
    public function consultaPagamentoPix($endtoendid, $cdbanco, $cdclientid, $dsdircertificado, $cdsenhacertif, $chavepix, $dataAgendamento, $nomefornecedor, $inscricao, $valor, $descricao, $homologacao = true){
        try {
            // Criação da Requisição
            $request = [];
            
            // Construção do Objeto API Itau
            $obApiPix = new ApiSicoob($cdclientid, $dsdircertificado, $cdsenhacertif, $homologacao);
            
            // Resposta da Requisição de Criação
            $response = $obApiPix->consultaPagamento($endtoendid, $request);
            
            // Verifica existência de erro
            if(isset($response['detail'])){
                var_dump('Operação bloqueada. Erro na confirmação do pagamento por pix: '.$response['detail']);die;
            }
            
            return $response;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
}