<?php
namespace Teknisa\FIN\Modules\ContaPagar\Services\Remessa\Banco;

use Teknisa\Model\Financeiro\Entities\Cnabpag;
use Fiscal\Entities\Boletpagar;
use Exception;
use Geral\Entities\Municipio;
use Geral\Entities\Banco;
use Geral\Entities\Empresa;
use Geral\Entities\Filial;
use Geral\Entities\Contcorr;
use Geral\Entities\Tipobaixapag;
use Teknisa\Libs\Util\DateUtil;
use Fiscal\Entities\Contapagar;
use Financeiro\Util\Util;
use Teknisa\FIN\Modules\ContaPagar\Services\Remessa\GeraCnabService;

/**
 * Class GeraCnabPadraoFebrabamService
 * @package Teknisa\FIN\Modules\ContaPagar\Services\Remessa\Banco
 */
class GeraCnabPadraoFebrabamService extends GeraCnabService{
    
    const TIPO_INS_JUR_CPF = 1;
    const TIPO_INS_JUR_CNPJ = 2;
    const ID_REMESSA = '1';
    const EXCLUSAO = 'E';
    const INCLUSAO = 'I';
    const BLOQUEIO = 'B';
    
    // Formas de Pagamento fixos no arquivo
    const TIPO_BAIXA_PAG_TRIBUTO_MUNICIPAL = 'N';
    const TIPO_BAIXA_PAG_IPVA = 'V';
    const TIPO_BAIXA_PAG_GARE = 'X';
    const TIPO_BAIXA_PAG_DARJ = 'J';
    const TIPO_BAIXA_PAG_DAE = 'Q';
    const TIPO_BAIXA_PIX = 'F';
    const TIPO_BAIXA_PIX_QRCODE = 'Z';
    
    private $qtdRegiLote = 0;  //Qtde de registro do lote
    private $qtdRegiArq = 0; //Quantidade de registro do arquivo
    private $num_lote = 0; // Sequencial incrementado em 1 para cada lote no arquivo
    private $formapgto;
    
    /**
     * @param Cnabpag $remessa
     * @throws Exception
     */
    public function __construct(Cnabpag $remessa){

        try {
            $this->nrorg = $remessa->getNrorg();

            $this->remessa = $remessa;
            
            $this->cdbanco = $remessa->getCdbanco();
            
            // Carrega os atributos Filial e Empresa
            $this->filial = Filial::findOneByCdfilialAndNrorg($remessa->getCdfilial(), $remessa->getNrorg());
            $this->empresa = Empresa::findOneByCdempresaAndNrorg($remessa->getCdempresa(), $remessa->getNrorg());

            // Busca dados referente a filial se foi informada
            if ($remessa->getCdfilial()) {
                if ($this->filial->getIdtpijurfili() == Filial::TIPO_INSCRICAO_FISICA)
                    $this->tipoinscricao = self::TIPO_INS_JUR_CPF;
                else
                    $this->tipoinscricao = self::TIPO_INS_JUR_CNPJ;

                $this->inscricaojuridica = Util::removeMascaraInscricao($this->filial->getNrinsjurfili());
                $this->nomeempresa = $this->filial->getNmrazsocfili();
                $this->dsEndeEF = $this->filial->getEndefiliprinc()->getDsendefili();
                $this->muniEF = $this->filial->getEndefiliprinc()->getMunicipio()->getNmmunicipio();
                $this->cepEF = $this->filial->getEndefiliprinc()->getNrcepfili();
                $this->estadoEF = $this->filial->getEndefiliprinc()->getSgestado();

            } // Se nao busca dados da Empresa
            else {
                if ($this->empresa->getIdtpijurempr() == Empresa::TIPO_INSCRICAO_FISICA)
                    $this->tipoinscricao = self::TIPO_INS_JUR_CPF;
                else
                    $this->tipoinscricao = self::TIPO_INS_JUR_CNPJ;

                $this->inscricaojuridica = Util::removeMascaraInscricao($this->empresa->getNrinsjurempr());
                $this->nomeempresa = $this->empresa->getNmrazsocempr();
                $this->dsEndeEF = $this->empresa->getDsendeempr();
                $this->muniEF = $this->empresa->getMunicipio()->getNmmunicipio();
                $this->cepEF = $this->empresa->getNrcepempr();
                $this->estadoEF = $this->empresa->getSgestado();
            }
            /** @var contcorr Contcorr */
            $this->contcorr = Contcorr::findOneByCdreducontAndNrorg($remessa->getCdreducont(), $remessa->getNrorg());
        } catch (\Exception $e) {
            throw $e;
        }

    }
    
    /**
     * @param $caminhoArquivo
     * @param $boletos
     * @param bool $autorizaAgendamento - Parámetro utilizando quando o banco for Bradesco
     * @throws \Exception
     * @internal param $remessa
     * @return mixed
     */
    public function geraArquivo($caminhoArquivo, $boletos, $autorizaAgendamento = false, $idopecnabpag = null){
        try{
             $file = fopen($caminhoArquivo.'.REM', "a+");
            $headerarquivo = self::geraHeaderArquivo();
            fprintf($file, "%s",$headerarquivo);

            // Agrupar boletos por tipo de baixa e por banco favorecido
            $boletosAgrup = self::agrupaBoletos($boletos, $file);

            //Percorre as formas de pagamento
            foreach($boletosAgrup as $segmento => $boletosFormaPgo){
                
                if($segmento == 'SEGMENTOAB'){
                    foreach($boletosFormaPgo as $key => $boletosBco){
                        $headerlote  = self::geraHeaderLote($key, $boletosBco);
                        fprintf($file, "%s",$headerlote);

                        foreach($boletosBco as $boleto){
                            $segmentoA = self::geraSegmentoA($boleto, $boleto->getCdbancbolp());
                            fprintf($file, "%s",$segmentoA);
                            $segmentoB = self::geraSegmentoB($boleto, $boleto->getCdbancbolp());
                            fprintf($file, "%s",$segmentoB);
                        }
                        $trailerLote = self::geraTraillerLote();
                        fprintf($file, "%s",$trailerLote);
                    }
                } 
                if($segmento == 'SEGMENTOJ'){
                    foreach($boletosFormaPgo as $key => $boletosBco){
                        //Percorre os Boletos
                        foreach($boletosBco as $boleto){
                             $headerlote  = self::geraHeaderLotePagamentoTitulos($key, $boletosBco);
                             fprintf($file, "%s",$headerlote);
                            /** @var $boleto Boletpagar */
                            $segmentoJ = self::geraSegmentoJ($boleto,$boleto->getCdbancbolp());
                            fprintf($file, "%s",$segmentoJ);
                            /** @var $boleto Boletpagar */
                            $segmentoJ = self::geraSegmentoJ52($boleto,$boleto->getCdbancbolp());
                            fprintf($file, "%s",$segmentoJ);
                        }
                        $trailerLote = self::geraTraillerLote();
                        fprintf($file, "%s",$trailerLote);
                    }
                }
                if($segmento == 'SEGMENTON'){
                    foreach($boletosFormaPgo as $key => $boletosBco){
                        //Percorre os Boletos
                        foreach($boletosBco as $boleto){
                             $headerlote  = self::geraHeaderLote($key, $boletosBco);
                             fprintf($file, "%s",$headerlote);
                            /** @var $boleto Boletpagar */
                            $segmentoN = self::geraSegmentoN($boleto);
                            fprintf($file, "%s",$segmentoN);
                        }
                        $trailerLote = self::geraTraillerLote();
                        fprintf($file, "%s",$trailerLote);
                    }
                }
                if($segmento == 'SEGMENTOO'){
                    foreach($boletosFormaPgo as $key => $boletosBco){
                        //Percorre os Boletos
                        foreach($boletosBco as $boleto){
                             $headerlote  = self::geraHeaderLotePagamentoTributos($key, $boletosBco);
                             fprintf($file, "%s",$headerlote);
                            /** @var $boleto Boletpagar */
                            $segmentoO = self::geraSegmentoO($boleto);
                            fprintf($file, "%s",$segmentoO);
                        }
                        $trailerLote = self::geraTraillerLote();
                        fprintf($file, "%s",$trailerLote);
                    }
                }
                // if($segmento == 'SEGMENTOJPIX'){
                //     //Percorre os bancos
                //     foreach($boletosFormaPgo as $key => $boletosBco){
                //         $headerlote  = self::geraHeaderLote($key, $boletosBco);
                //         fprintf($file, "%s",$headerlote);
                //         foreach($boletosBco as $boleto){
                //             $segmentoJ = self::geraSegmentoJPix($boleto, $boleto->getCdbancbolp());
                //             $segmentoj52 = self::geraSegmentoj52Pix($boleto, $boleto->getCdbancbolp());
                //             fprintf($file, "%s",$segmentoJ);
                //             fprintf($file, "%s",$segmentoj52);
                //         }
                //         $trailerLote = self::geraTraillerLote();
                //         fprintf($file, "%s",$trailerLote);
                //     }
                // }
            }

            $trailerArquivo = self::geraTrailerArquivo();
            fprintf($file, "%s",$trailerArquivo);
            fclose($file);
        }
        catch (\Exception $e){
            fclose($file);
            throw $e;
        }
    }
    
    /** 
     * Retorna array de Boletos Agrupados por 
     * Forma de Pagamento e por código do banco favorecido
     */
    private function agrupaBoletos($boletos){
        $boletosAgroup = array();

        $segmentosAB = array(
            Tipobaixapag::TIPOBAIXA_CNAB,
            Tipobaixapag::TIPOBAIXA_CNAB_DOC,
            Tipobaixapag::TIPOBAIXA_TED_CIP,
            Tipobaixapag::TIPO_BAIXA_TED_STR,
            self::TIPO_BAIXA_PIX,
        );

        $segmentoJ = array(Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO);
        
        //PIX QRCODE
        // $segmentoJPIX = array('Z');

        $segmentoO = array(
            Tipobaixapag::TIPO_BAIXA_PAG_ORDEM_TRIB_COD_BARRAS, 
            Tipobaixapag::TIPO_BAIXA_PAG_DARF_NORMAL, 
            Tipobaixapag::TIPO_BAIXA_FGTS,
            self::TIPO_BAIXA_PAG_TRIBUTO_MUNICIPAL,
            self::TIPO_BAIXA_PAG_DAE
        );
        
        $segmentoN = array(
            Tipobaixapag::TIPO_BAIXA_PAG_DARF_SIMPLES,
            Tipobaixapag::TIPO_BAIXA_GPS,
            self::TIPO_BAIXA_PAG_IPVA,
            self::TIPO_BAIXA_PAG_GARE,
            self::TIPO_BAIXA_PAG_DARJ
        );

        /** @var $boleto Boletpagar */
        foreach ($boletos as $boleto){
            
            /** @var Boletpagar $boleto */
            $boleto = Boletpagar::findOneByNrorgAndCdfilialAndNrlanbolpag(
                $boleto['NRORG'], $boleto['CDFILIAL'], $boleto['NRLANBOLPAG']
            );

            /** @var Tipobaixapag $tipobaixapag */
            $tipobaixapag = Tipobaixapag::findOneByNrorgAndCdtpbaixapag($boleto->getNrorg(), $boleto->getCdtpbaixapag());
            
            $idbaixautilp = $tipobaixapag?$tipobaixapag->getIdbaixautilp():null;
            
            if(in_array($idbaixautilp,$segmentosAB)){

                if(self::getBanco($boleto) == GeraCnabService::CODIGO_COMPENSACAO_BRASIL)
                     $boletosAgroup['SEGMENTOAB'] [GeraCnabService::CODIGO_COMPENSACAO_BRASIL] [] = $boleto;
                else $boletosAgroup['SEGMENTOAB'] ['OUTROS_BANCOS'] [] = $boleto;

            }else{
                if(in_array($idbaixautilp,$segmentoJ)){
                    $cdbanco = self::getBanco($boleto,Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO);

                    if($cdbanco == (GeraCnabService::CODIGO_COMPENSACAO_BRASIL))
                         $boletosAgroup['SEGMENTOJ'] [GeraCnabService::CODIGO_COMPENSACAO_BRASIL] [] = $boleto;
                    else $boletosAgroup['SEGMENTOJ'] ['OUTROS_BANCOS'] [] = $boleto;
                }else{
                    if(in_array($idbaixautilp,$segmentoO)){
                        $cdbanco = self::getBanco($boleto, $idbaixautilp);

                        if($cdbanco == (GeraCnabService::CODIGO_COMPENSACAO_BRASIL))
                             $boletosAgroup['SEGMENTOO'] [GeraCnabService::CODIGO_COMPENSACAO_BRASIL] [] = $boleto;
                        else $boletosAgroup['SEGMENTOO'] ['OUTROS_BANCOS'] [] = $boleto;

                    }else{
                        if(in_array($idbaixautilp,$segmentoN)){
                            $cdbanco = self::getBanco($boleto, $idbaixautilp);
    
                            if($cdbanco == (GeraCnabService::CODIGO_COMPENSACAO_BRASIL))
                                 $boletosAgroup['SEGMENTON'] [GeraCnabService::CODIGO_COMPENSACAO_BRASIL] [] = $boleto;
                            else $boletosAgroup['SEGMENTON'] ['OUTROS_BANCOS'] [] = $boleto;
                            
                        // }elseif(in_array($idbaixautilp,$segmentoJPIX)){
                        //     $cdbanco = self::getBanco($boleto, $idbaixautilp);
                        //     if($cdbanco == (GeraCnabService::CODIGO_COMPENSACAO_BRASIL))
                        //          $boletosAgroup['SEGMENTOJPIX'] [GeraCnabService::CODIGO_COMPENSACAO_BRASIL] [] = $boleto;
                        //     else $boletosAgroup['SEGMENTOJPIX'] ['OUTROS_BANCOS'] [] = $boleto;
                        } else throw new \Exception ('Operação bloqueada. Tipo de Baixa não reconhecido.');
                        
                    }
                }
            }
        }
        
        return $boletosAgroup;
    }

    /**
     * Responsável pegar a forma de pagamento
     * @param $cdbanco
     * @param Boletpagar $boleto
     */
    private function getFormaPagto($cdbanco, $boletos){
        
        /** @var $boleto Boletpagar */
        foreach ($boletos as $boleto){
            $boleto = Boletpagar::findOneByNrorgAndCdfilialAndNrlanbolpag($boleto->getNrorg(), $boleto->getCdfilial(), $boleto->getNrlanbolpag());

            $idbaixa = $boleto->getTpbaixapag()->getIdbaixautilp();
        }

        if($idbaixa == Tipobaixapag::TIPO_BAIXA_CNAB_ORDEM_PAG)
           return '10';
        else if ($idbaixa == Tipobaixapag::TIPO_BAIXA_PAG_ORDEM_TRIB_COD_BARRAS || $idbaixa == self::TIPO_BAIXA_PAG_DAE)
            return '11';
        else if($idbaixa == Tipobaixapag::TIPO_BAIXA_GPS)
            return '17';
        else if($idbaixa == Tipobaixapag::TIPO_BAIXA_PAG_DARF_SIMPLES)
            return '18';
        else if($idbaixa == Tipobaixapag::TIPO_BAIXA_PAG_DARF_NORMAL)
            return '16';
        else if($idbaixa == Tipobaixapag::TIPO_BAIXA_DARJ)
            return '21';
        else if($idbaixa == Tipobaixapag::TIPO_BAIXA_FGTS)
            return '99';
        else if($idbaixa == Tipobaixapag::TIPO_BAIXA_GARE)
            return '22';
        else if($idbaixa == self::TIPO_BAIXA_PIX)
            return '45';
        else{
            if($idbaixa == Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO){
                if($cdbanco == self::CODIGO_COMPENSACAO_BRASIL)
                    return '30'; //Liquidação de Títulos do Próprio Banco
                else
                    return '31'; //Pagamento de Títulos de Outros Bancos
            }else{
                if($cdbanco == self::CODIGO_COMPENSACAO_BRASIL)
                    return '01'; //Crédito em conta corrente
                else{
                    return '03'; //DOC/TED
                }
            }
        }

    }
    
    /**
     * @param $nrlanbolpag
     * @param $cdfilibpag
     * @param $nrorg
     * @return Contapagar
     */
    private function getContaPagar($nrlanbolpag, $cdfilibpag, $nrorg){
        /** @var $contaPagar Contapagar */
        $contaPagar = Contapagar::findOneByNrlanbolpagAndCdfilibpagAndNrorg($nrlanbolpag, $cdfilibpag, $nrorg);
        return $contaPagar;
    }
    
    /**
     * Função Responsável por gerar a linha do Header de Arquivo do arquivo de remessa
     * @param $cdbanco
     * @param $boletos
     * @return String $linha
     */
    public function geraHeaderArquivo()
    {
        $linha = '';
        
        $this->qtdRegiArq++; //Quantidade de registro do arquivo
        
        // CAMPO: Código do Banco na Compensação
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        $linha .= $this->cdbanco;
        
        // CAMPO: Lote de Serviço
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        // DEFAULT: 0000
        $linha .= '0000';
        
        // CAMPO: Tipo de Registro
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        // DEFAULT:  0
        $linha .= '0';
        
        // CAMPO: Uso Exclusivo FEBRABAN / CNAB
        // POSIÇÃO: 009 a 017
        // PICTURE: X(9)
        // DEFAULT:  Brancos
        $linha .= str_repeat(' ', 9);
        
        // CAMPO: Tipo de Inscrição da Empresa
        // POSIÇÃO: 018 a 018
        // PICTURE: 9(1)
        $linha .= $this->tipoinscricao;
        
        // CAMPO: Número de Inscrição da Empresa
        // POSIÇÃO: 019 a 032
        // PICTURE: 9(14)
        $linha .= str_pad(trim($this->inscricaojuridica), 14, '0', STR_PAD_LEFT);
        
        // CAMPO: Código do Convênio no Banco
        // POSIÇÃO: 033 a 052
        // PICTURE: X(20)
        $identpag = self::getIdentPag();
        if($identpag){
            $linha.= str_pad(trim($identpag),20);
        }else{
            $linha .= str_repeat(' ', 20);
        }
        
        // CAMPO: Agência Mantenedora da Conta
        // POSIÇÃO: 053 057 
        // PICTURE: 9(5)
        $banco = Banco::findOneByCdbancoAndNrorg($this->contcorr->getCdbanco(), $this->nrorg);
        $digitoAgencia   = (strpos($banco->getCdpictagen(), "-") > -1)?true:false;
         
        if($digitoAgencia){
            $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,-1),5,'0',STR_PAD_LEFT);
            $digitoAgencia = substr(trim($this->contcorr->getCdagencia()), -1);
        } else {
            $linha.= str_pad($this->contcorr->getCdagencia(), 5, '0', STR_PAD_LEFT);
            $digitoAgencia = ' ';
        }
        
        // CAMPO: Dígito Verificador da Agência 
        // POSIÇÃO: 058 a 058
        // PICTURE: X(1)
        $linha.= $digitoAgencia;
        
        // CAMPO: Número da Conta Corrente
        // POSIÇÃO: 059 a 070
        // PICTURE: 9(12)
        $digitoContaCorrenteFlag   = (strpos($banco->getCdpictcont(), "-") > -1)?true:false;
        
        if($digitoContaCorrenteFlag){
            $linha.= str_pad(Util::campoCnabValido(substr(self::getContaCorrenteSemDigito(), 0, 12)), 12,'0',STR_PAD_LEFT);
            $digitoContaCorrente = substr(trim($this->contcorr->getCdcontcorr()), -1);
        }else{
            $linha.= str_pad(Util::campoCnabValido(substr(self::getContaCorrenteSemDigito(), 0, 12)), 12,'0',STR_PAD_LEFT);
            $digitoContaCorrente = ' ';
        }
        
        // CAMPO: Dígito Verificador da Conta
        // POSIÇÃO: 071 a 071 
        // PICTURE: X(1)
        $linha.= $digitoContaCorrente;
        
        // CAMPO: Dígito Verificador da Ag/Conta
        // POSIÇÃO: 072 a 072
        // PICTURE: X(1)
        $linha.= ' ';
        
        // CAMPO: Nome da Empresa
        // POSIÇÃO: 073 a 102 
        // PICTURE: X(30)
        $linha.= self::preenche_brancos(substr(trim(strtoupper($this->nomeempresa)),0,30),30);
        
        // CAMPO: Nome do Banco
        // POSIÇÃO: 103 a 132 
        // PICTURE: X(30)
        $banco = Banco::findOneByCdbancoAndNrorg($this->contcorr->getCdbanco(), $this->nrorg);
        $linha.= self::preenche_brancos(Util::campoCnabValido($banco->getNmbanco()),30);
        
        // CAMPO: Uso Exclusivo FEBRABAN / CNAB 
        // POSIÇÃO: 133 a 142 
        // PICTURE: X(10)
        // DEFAULT: Brancos
        $linha.= str_repeat(' ', 10);
        
        // CAMPO: Código Remessa / Retorno
        // POSIÇÃO: 143 a 143 
        // PICTURE: 9(1)
        // OBS: '1' = Remessa (Cliente → Banco) '2' = Retorno (Banco → Cliente)
        $linha .= self::ID_REMESSA;
        
        // CAMPO: Data de Geração do Arquivo
        // POSIÇÃO: 144 a 151 
        // PICTURE: 9(8)
        // DEFAULT: Utilizar o formato DDMMAAAA
        $dataAtual = DateUtil::getDataAtual();
        $linha .= $dataAtual->format('dmY');
        
        // CAMPO: Hora de Geração do Arquivo
        // POSIÇÃO: 152 a 157 
        // PICTURE: 9(6)
        // DEFAULT: Utilizar o formato HHMMSS
        $linha .= $dataAtual->format('His');
        
        // CAMPO: Número Seqüencial do Arquivo
        // POSIÇÃO: 158 a 163 
        // PICTURE: 9(6)
        $linha .= substr($this->remessa->getNrseqremessapag(), -6);
        
        // CAMPO: Nº da Versão do Layout do Arquivo
        // POSIÇÃO: 164 a 166  
        // PICTURE: 9(3)
        //DEFAULT: 103
        $linha .= '103';
        
        // CAMPO: Densidade de Gravação do Arquivo
        // POSIÇÃO: 167 a 171 
        // PICTURE: 9(5)
        // DEFAULT: Domínio: 1600 BPI ou 6250 BPI
        $linha .= '00000';
        
        // CAMPO: Para Uso Reservado do Banco 
        // POSIÇÃO: 172 a 191 
        // PICTURE: X(20)
        $linha.= str_repeat(' ',20);
        
        // CAMPO: Para Uso Reservado da Empresa
        // POSIÇÃO: 192 a 211
        // PICTURE: X(20)
        $linha.= str_repeat(' ',20);
        
        // CAMPO: Uso Exclusivo FEBRABAN / CNAB
        // POSIÇÃO: 212 a 40
        // PICTURE: X(29)
        //DEFAULT: Brancos
        $linha.= str_repeat(' ',29);
        
        return $linha."\r\n";
    }

    /**
     * Função Responsável por gerar a linha do Header de Lote do arquivo de remessa
     * @param $cdbanco
     * @param $boletos
     * @return String $linha
     */
    public function geraHeaderLote($cdbanco, $boletos = null)
    {
        
        $this->seqRegLote = 0;
        $this->vrPagtLote = 0;
        $this->qtdRegiLote = 0;
        $this->qtdRegiLote++;
        $this->qtdRegiArq++;
        
        $linha = '';
        
        $this->qtdRegiArq++; //Quantidade de registro do arquivo
        
        // CAMPO: Código do Banco na Compensação
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        $linha .= $this->cdbanco;
        
        // CAMPO: Lote de Serviço
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        $linha .= str_pad(++$this->num_lote,4,'0',STR_PAD_LEFT);
        
        // CAMPO: Tipo de Registro
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        // DEFAULT:  1
        $linha .= '1';
        
        // CAMPO: Tipo da Operação
        // POSIÇÃO: 009 a 009
        // PICTURE: X(1)
        // DEFAULT:  C
        $linha .= 'C';
        
        // CAMPO: Tipo do Serviço
        // POSIÇÃO: 010 a 011
        // PICTURE: 9(2)
        // Obs: '20' = Pagamento Fornecedor
        $linha.= '20';
        
        // CAMPO: Forma de Lançamento
        // POSIÇÃO: 012 a 013
        // PICTURE: 9(2)
        $this->formaPgto = self::getFormaPagto($cdbanco, $boletos);
        $linha.= $this->formaPgto;
        
        // CAMPO: Nº da Versão do Layout do Lote
        // POSIÇÃO: 014 a 016
        // PICTURE: 9(3)
        // DEFAULT: 046
        $linha.= '046';
         
        // CAMPO: Uso Exclusivo da FEBRABAN/CNAB
        // POSIÇÃO: 017 a 017
        // PICTURE: X(1)
        // DEFAULT: Brancos
        $linha.= str_repeat(' ',1);
        
        // CAMPO: Tipo de Inscrição da Empresa
        // POSIÇÃO: 018 a 018
        // PICTURE: 9(1)
        $linha.= $this->tipoinscricao;
        
        // CAMPO: Número de Inscrição da Empresa
        // POSIÇÃO: 019 a 032
        // PICTURE: 9(14)
        $linha.= $this->inscricaojuridica;
        
        // CAMPO: Código do Convênio no Banco
        // POSIÇÃO: 033 a 052
        // PICTURE: X(20)
        $identpag = self::getIdentPag();
        if($identpag){
            $linha.= str_pad(trim($identpag),20);
        }else{
             $linha.= str_repeat(' ',20);
        }
        
        // CAMPO: Agência Mantenedora da Conta
        // POSIÇÃO: 053 a 057
        // PICTURE: 9(5)
        $banco = Banco::findOneByCdbancoAndNrorg($this->contcorr->getCdbanco(), $this->nrorg);
        $digitoAgencia   = (strpos($banco->getCdpictagen(), "-") > -1)?true:false;
         
        if($digitoAgencia){
            $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,-1),5,'0',STR_PAD_LEFT);
            $digitoAgencia = substr(trim($this->contcorr->getCdagencia()), -1);
        } else {
            $linha.= str_pad($this->contcorr->getCdagencia(), 5, '0', STR_PAD_LEFT);
            $digitoAgencia = ' ';
        }
        
        // CAMPO: Dígito Verificador da Agência 
        // POSIÇÃO: 058 a 058
        // PICTURE: X(1)
        $linha.= $digitoAgencia;

        
        // CAMPO: Número da Conta Corrente 
        // POSIÇÃO: 059 a 070 
        // PICTURE: 9(12)
        $digitoContaCorrenteFlag   = (strpos($banco->getCdpictcont(), "-") > -1)?true:false;
        
        if($digitoContaCorrenteFlag){
            $linha.= str_pad(Util::campoCnabValido(substr(self::getContaCorrenteSemDigito(), 0, 12)), 12,'0',STR_PAD_LEFT);
            $digitoContaCorrente = substr(trim($this->contcorr->getCdcontcorr()), -1);
        }else{
            $linha.= str_pad(Util::campoCnabValido(substr(self::getContaCorrenteSemDigito(), 0, 12)), 12,'0',STR_PAD_LEFT);
            $digitoContaCorrente = ' ';
        }
        
        // CAMPO: Dígito Verificador da Conta
        // POSIÇÃO: 071 a 071 
        // PICTURE: X(1)
        $linha.= $digitoContaCorrente;
        
        // CAMPO: Dígito Verificador da Ag/Conta
        // POSIÇÃO: 072 a 072 
        // PICTURE: X(1)
        $linha.= ' ';
        
        // CAMPO: Nome da Empresa
        // POSIÇÃO: 073 a 102 
        // PICTURE: X(30)
        $linha.= self::preenche_brancos(substr(trim(strtoupper($this->nomeempresa)),0,30),30);
        
        // CAMPO: Mensagem
        // POSIÇÃO: 103 a 142 
        // PICTURE: X(40)
        $linha.= str_repeat(' ',40);
        
        // CAMPO: Logradouro
        // POSIÇÃO: 143 a 172 
        // PICTURE: X(30)
        // DEFAULT:  Nome da Rua, Av, Pça, Etc
        $linha.= str_pad(substr(Util::campoCnabValido($this->empresa->getDsendeempr()),0, 30), 30);
        
        // CAMPO: Número do Local
        // POSIÇÃO: 173 a 177
        // PICTURE: 9(5)
        $linha.= str_repeat('0',5);
        
        // CAMPO: Complemento
        // POSIÇÃO: 178 192
        // PICTURE: X(15)
        $linha.= str_repeat(' ',15);
        
        // CAMPO: Nome da Cidade 
        // POSIÇÃO: 193 a 212
        // PICTURE: X(20)
        $municipio = Municipio::findOneBySgestadoAndCdpaisAndCdmunicipio($this->empresa->getSgestado(), $this->empresa->getCdpais(), $this->empresa->getCdmunicipio());
        $linha.= str_pad(substr($municipio->getNmmunicipio(), 0, 20), 20);
        
        // CAMPO: CEP -Complemento do CEP
        // POSIÇÃO: 213 a 220
        // PICTURE: 9(8)
        $linha.= str_pad(Util::campoCnabValido(substr($this->cepEF, 0, 8)), 8,' ',STR_PAD_RIGHT);
        
        // CAMPO: Sigla do Estado
        // POSIÇÃO: 221 a 222
        // PICTURE: X(2)
        $linha.= $this->empresa->getSgestado();
        
        // CAMPO: Indicativo da Forma de Pagamento do Serviço
        // POSIÇÃO: 223 a 224
        // PICTURE: 9(2)
        $linha.= str_repeat('0',2);
        
        // CAMPO: Uso Exclusivo FEBRABAN/CNAB
        // POSIÇÃO: 225 a 230
        // PICTURE: X(6)
        // DEFAULT: Brancos
        $linha.= str_repeat(' ',6);
        
        // CAMPO: Códigos das Ocorrências p/ Retorno
        // POSIÇÃO: 231 a 240
        // PICTURE: X(10)
        // DEFAULT: Brancos
        $linha.= str_repeat(' ',10);
        
        return $linha."\r\n";
    }
    
    /**
     * Função Responsável por gerar a linha do Header de Lote Seguimento J e J52 arquivo de remessa
     * @param $cdbanco
     * @param $boletos
     * @return String $linha
     */
    public function geraHeaderLotePagamentoTitulos($cdbanco, $boletos = null)
    {
        
        $this->seqRegLote = 0;
        $this->vrPagtLote = 0;
        $this->qtdRegiLote = 0;
        $this->qtdRegiLote++;
        $this->qtdRegiArq++;
        
        $linha = '';
        
        $this->qtdRegiArq++; //Quantidade de registro do arquivo
        
        // CAMPO: Código do Banco na Compensação
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        $linha .= $this->cdbanco;
        
        // CAMPO: Lote de Serviço
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        $linha .= str_pad(++$this->num_lote,4,'0',STR_PAD_LEFT);
        
        // CAMPO: Tipo de Registro
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        // DEFAULT:  1
        $linha .= '1';
        
        // CAMPO: Tipo da Operação
        // POSIÇÃO: 009 a 009
        // PICTURE: X(1)
        // DEFAULT:  C
        $linha .= 'C';
        
        // CAMPO: Tipo do Serviço
        // POSIÇÃO: 010 a 011
        // PICTURE: 9(2)
        // Obs: '20' = Pagamento Fornecedor
        $linha.= '20';
        
        // CAMPO: Forma de Lançamento
        // POSIÇÃO: 012 a 013
        // PICTURE: 9(2)
        $this->formaPgto = self::getFormaPagto($cdbanco, $boletos);
        $linha.= $this->formaPgto;
        
        // CAMPO: Nº da Versão do Layout do Lote
        // POSIÇÃO: 014 a 016
        // PICTURE: 9(3)
        // DEFAULT: 040
        $linha.= '040';
         
        // CAMPO: Uso Exclusivo da FEBRABAN/CNAB
        // POSIÇÃO: 017 a 017
        // PICTURE: X(1)
        // DEFAULT: Brancos
        $linha.= str_repeat(' ',1);
        
        // CAMPO: Tipo de Inscrição da Empresa
        // POSIÇÃO: 018 a 018
        // PICTURE: 9(1)
        $linha.= $this->tipoinscricao;
        
        // CAMPO: Número de Inscrição da Empresa
        // POSIÇÃO: 019 a 032
        // PICTURE: 9(14)
        $linha.= $this->inscricaojuridica;
        
        // CAMPO: Código do Convênio no Banco
        // POSIÇÃO: 033 a 052
        // PICTURE: X(20)
        $identpag = self::getIdentPag();
        if($identpag){
            $linha.= str_pad(trim($identpag),20);
        }else{
             $linha.= str_repeat(' ',20);
        }
        
        // CAMPO: Agência Mantenedora da Conta
        // POSIÇÃO: 053 a 057
        // PICTURE: 9(5)
        $banco = Banco::findOneByCdbancoAndNrorg($this->contcorr->getCdbanco(), $this->nrorg);
        $digitoAgencia   = (strpos($banco->getCdpictagen(), "-") > -1)?true:false;
         
        if($digitoAgencia){
            $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,-1),5,'0',STR_PAD_LEFT);
            $digitoAgencia = substr(trim($this->contcorr->getCdagencia()), -1);
        } else {
            $linha.= str_pad($this->contcorr->getCdagencia(), 5, '0', STR_PAD_LEFT);
            $digitoAgencia = ' ';
        }
        
        // CAMPO: Dígito Verificador da Agência 
        // POSIÇÃO: 058 a 058
        // PICTURE: X(1)
        $linha.= $digitoAgencia;

        
        // CAMPO: Número da Conta Corrente 
        // POSIÇÃO: 059 a 070 
        // PICTURE: 9(12)
        $digitoContaCorrenteFlag   = (strpos($banco->getCdpictcont(), "-") > -1)?true:false;
        
        if($digitoContaCorrenteFlag){
            $linha.= str_pad(Util::campoCnabValido(substr(self::getContaCorrenteSemDigito(), 0, 12)), 12,'0',STR_PAD_LEFT);
            $digitoContaCorrente = substr(trim($this->contcorr->getCdcontcorr()), -1);
        }else{
            $linha.= str_pad(Util::campoCnabValido(substr(self::getContaCorrenteSemDigito(), 0, 12)), 12,'0',STR_PAD_LEFT);
            $digitoContaCorrente = ' ';
        }
        
        // CAMPO: Dígito Verificador da Conta
        // POSIÇÃO: 071 a 071 
        // PICTURE: X(1)
        $linha.= $digitoContaCorrente;
        
        // CAMPO: Dígito Verificador da Ag/Conta
        // POSIÇÃO: 072 a 072 
        // PICTURE: X(1)
        $linha.= ' ';
        
        // CAMPO: Nome da Empresa
        // POSIÇÃO: 073 a 102 
        // PICTURE: X(30)
        $linha.= self::preenche_brancos(substr(trim(strtoupper($this->nomeempresa)),0,30),30);
        
        // CAMPO: Mensagem
        // POSIÇÃO: 103 a 142 
        // PICTURE: X(40)
        $linha.= str_repeat(' ',40);
        
        // CAMPO: Logradouro
        // POSIÇÃO: 143 a 172 
        // PICTURE: X(30)
        // DEFAULT:  Nome da Rua, Av, Pça, Etc
        $linha.= str_pad(substr(Util::campoCnabValido($this->empresa->getDsendeempr()),0, 30), 30);
        
        // CAMPO: Número do Local
        // POSIÇÃO: 173 a 177
        // PICTURE: 9(5)
        $linha.= str_repeat('0',5);
        
        // CAMPO: Complemento
        // POSIÇÃO: 178 192
        // PICTURE: X(15)
        $linha.= str_repeat(' ',15);
        
        // CAMPO: Nome da Cidade 
        // POSIÇÃO: 193 a 212
        // PICTURE: X(20)
        $municipio = Municipio::findOneBySgestadoAndCdpaisAndCdmunicipio($this->empresa->getSgestado(), $this->empresa->getCdpais(), $this->empresa->getCdmunicipio());
        $linha.= str_pad(substr($municipio->getNmmunicipio(), 0, 20), 20);
        
        // CAMPO: CEP -Complemento do CEP
        // POSIÇÃO: 213 a 220
        // PICTURE: 9(8)
        $linha.= str_pad(Util::campoCnabValido(substr($this->cepEF, 0, 8)), 8,' ',STR_PAD_RIGHT);
        
        // CAMPO: Sigla do Estado
        // POSIÇÃO: 221 a 222
        // PICTURE: X(2)
        $linha.= $this->empresa->getSgestado();

        // CAMPO: Uso Exclusivo FEBRABAN/CNAB
        // POSIÇÃO: 223 a 230
        // PICTURE: X(8)
        // DEFAULT: Brancos
        $linha.= str_repeat(' ',8);
        
        // CAMPO: Códigos das Ocorrências p/ Retorno
        // POSIÇÃO: 231 a 240
        // PICTURE: X(10)
        // DEFAULT: Brancos
        $linha.= str_repeat(' ',10);
        
        return $linha."\r\n";
    }
    
    /**
     * Função Responsável por gerar a linha do Header de Lote Tributos do arquivo de remessa
     * @param $cdbanco
     * @param $boletos
     * @return String $linha
     */
    public function geraHeaderLotePagamentoTributos($cdbanco, $boletos = null)
    {
        
        $this->seqRegLote = 0;
        $this->vrPagtLote = 0;
        $this->qtdRegiLote = 0;
        $this->qtdRegiLote++;
        $this->qtdRegiArq++;
        
        $linha = '';
        
        $this->qtdRegiArq++; //Quantidade de registro do arquivo
        
        // CAMPO: Código do Banco na Compensação
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        $linha .= $this->cdbanco;
        
        // CAMPO: Lote de Serviço
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        $linha .= str_pad(++$this->num_lote,4,'0',STR_PAD_LEFT);
        
        // CAMPO: Tipo de Registro
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        // DEFAULT:  1
        $linha .= '1';
        
        // CAMPO: Tipo da Operação
        // POSIÇÃO: 009 a 009
        // PICTURE: X(1)
        // DEFAULT:  C
        $linha .= 'C';
        
        // CAMPO: Tipo do Serviço
        // POSIÇÃO: 010 a 011
        // PICTURE: 9(2)
        // Obs: '20' = Pagamento Fornecedor
        if (($this->formaPgto = '16') or ($this->formaPgto = '17') or ($this->formaPgto = '18') or
            ($this->formaPgto = '19') or ($this->formaPgto = '21') or ($this->formaPgto = '22') or
            ($this->formaPgto = '25') or ($this->formaPgto = '27') or ($this->formaPgto = '35') or
            ($this->formaPgto = '91') or ($this->formaPgto = '11') or ($this->formaPgto = '99')){
             $linha.= '22';
          } else {
              $linha.='20';
          }
        
        // CAMPO: Forma de Lançamento
        // POSIÇÃO: 012 a 013
        // PICTURE: 9(2)
        $this->formaPgto = self::getFormaPagto($cdbanco, $boletos);
        if($this->formaPgto != '99'){
            $linha.= $this->formaPgto;
        } else {
            $linha.='11';
        }
        
        // CAMPO: Nº da Versão do Layout do Lote
        // POSIÇÃO: 014 a 016
        // PICTURE: 9(3)
        // DEFAULT: 012
        $linha.= '012';
         
        // CAMPO: Uso Exclusivo da FEBRABAN/CNAB
        // POSIÇÃO: 017 a 017
        // PICTURE: X(1)
        // DEFAULT: Brancos
        $linha.= str_repeat(' ',1);
        
        // CAMPO: Tipo de Inscrição da Empresa
        // POSIÇÃO: 018 a 018
        // PICTURE: 9(1)
        $linha.= $this->tipoinscricao;
        
        // CAMPO: Número de Inscrição da Empresa
        // POSIÇÃO: 019 a 032
        // PICTURE: 9(14)
        $linha.= $this->inscricaojuridica;
        
        // CAMPO: Código do Convênio no Banco
        // POSIÇÃO: 033 a 052
        // PICTURE: X(20)
        $identpag = self::getIdentPag();
        if($identpag){
            $linha.= str_pad(trim($identpag),20);
        }else{
             $linha.= str_repeat(' ',20);
        }
        
        // CAMPO: Agência Mantenedora da Conta
        // POSIÇÃO: 053 a 057
        // PICTURE: 9(5)
        $banco = Banco::findOneByCdbancoAndNrorg($this->contcorr->getCdbanco(), $this->nrorg);
        $digitoAgencia   = (strpos($banco->getCdpictagen(), "-") > -1)?true:false;
         
        if($digitoAgencia){
            $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,-1),5,'0',STR_PAD_LEFT);
            $digitoAgencia = substr(trim($this->contcorr->getCdagencia()), -1);
        } else {
            $linha.= str_pad($this->contcorr->getCdagencia(), 5, '0', STR_PAD_LEFT);
            $digitoAgencia = ' ';
        }
        
        // CAMPO: Dígito Verificador da Agência 
        // POSIÇÃO: 058 a 058
        // PICTURE: X(1)
        $linha.= $digitoAgencia;

        
        // CAMPO: Número da Conta Corrente 
        // POSIÇÃO: 059 a 070 
        // PICTURE: 9(12)
        $digitoContaCorrenteFlag   = (strpos($banco->getCdpictcont(), "-") > -1)?true:false;
        
        if($digitoContaCorrenteFlag){
            $linha.= str_pad(Util::campoCnabValido(substr(self::getContaCorrenteSemDigito(), 0, 12)), 12,'0',STR_PAD_LEFT);
            $digitoContaCorrente = substr(trim($this->contcorr->getCdcontcorr()), -1);
        }else{
            $linha.= str_pad(Util::campoCnabValido(substr(self::getContaCorrenteSemDigito(), 0, 12)), 12,'0',STR_PAD_LEFT);
            $digitoContaCorrente = ' ';
        }
        
        // CAMPO: Dígito Verificador da Conta
        // POSIÇÃO: 071 a 071 
        // PICTURE: X(1)
        $linha.= $digitoContaCorrente;
        
        // CAMPO: Dígito Verificador da Ag/Conta
        // POSIÇÃO: 072 a 072 
        // PICTURE: X(1)
        $linha.= ' ';
        
        // CAMPO: Nome da Empresa
        // POSIÇÃO: 073 a 102 
        // PICTURE: X(30)
        $linha.= self::preenche_brancos(substr(trim(strtoupper($this->nomeempresa)),0,30),30);
        
        // CAMPO: Mensagem
        // POSIÇÃO: 103 a 142 
        // PICTURE: X(40)
        $linha.= str_repeat(' ',40);
        
        // CAMPO: Logradouro
        // POSIÇÃO: 143 a 172 
        // PICTURE: X(30)
        // DEFAULT:  Nome da Rua, Av, Pça, Etc
        $linha.= str_pad(substr(Util::campoCnabValido($this->empresa->getDsendeempr()),0, 30), 30);
        
        // CAMPO: Número do Local
        // POSIÇÃO: 173 a 177
        // PICTURE: 9(5)
        $linha.= str_repeat('0',5);
        
        // CAMPO: Complemento
        // POSIÇÃO: 178 192
        // PICTURE: X(15)
        $linha.= str_repeat(' ',15);
        
        // CAMPO: Nome da Cidade 
        // POSIÇÃO: 193 a 212
        // PICTURE: X(20)
        $municipio = Municipio::findOneBySgestadoAndCdpaisAndCdmunicipio($this->empresa->getSgestado(), $this->empresa->getCdpais(), $this->empresa->getCdmunicipio());
        $linha.= str_pad(substr($municipio->getNmmunicipio(), 0, 20), 20);
        
        // CAMPO: CEP -Complemento do CEP
        // POSIÇÃO: 213 a 220
        // PICTURE: 9(8)
        $linha.= str_pad(Util::campoCnabValido(substr($this->cepEF, 0, 8)), 8,' ',STR_PAD_RIGHT);
        
        // CAMPO: Sigla do Estado
        // POSIÇÃO: 221 a 222
        // PICTURE: X(2)
        $linha.= $this->empresa->getSgestado();

        // CAMPO: Uso Exclusivo FEBRABAN/CNAB
        // POSIÇÃO: 223 a 230
        // PICTURE: X(8)
        // DEFAULT: Brancos
        $linha.= str_repeat(' ',8);
        
        // CAMPO: Códigos das Ocorrências p/ Retorno
        // POSIÇÃO: 231 a 240
        // PICTURE: X(10)
        // DEFAULT: Brancos
        $linha.= str_repeat(' ',10);
        
        return $linha."\r\n";
    }

    /**
     *  Função Responsável por gerar o Seguimento A do arquivo de remessa
     * @param Boletpagar $boleto
     * @param $cdbanco
     * @return string
     */
    private function geraSegmentoA(Boletpagar $boleto, $cdbanco)
    {
        
        $linha = '';
        $this->qtdRegiLote++;//Qtde de registro do lote
        $this->qtdRegiArq++; //Quantidade de registro do arquivo
        
        // CAMPO: Código do Banco na Compensação
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        $linha .= $this->cdbanco;
        
        // CAMPO: Lote de Serviço
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        $linha .= str_pad($this->num_lote, 4, '0', STR_PAD_LEFT);
        
        // CAMPO: Tipo de Registro
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        // DEFAULT:  3
        $linha .= '3';
        
        // CAMPO: Nº Seqüencial do Registro no Lote
        // POSIÇÃO: 009 a 013
        // PICTURE: 9(5)
        $linha .= str_pad($this->seqRegLote, 5, '0', STR_PAD_LEFT);
        
        // CAMPO: Código de Segmento do Reg. Detalhe
        // POSIÇÃO: 014 a 014
        // PICTURE: X(1)
        // DEFAULT:  A
        $linha.= 'A';
        
        // CAMPO: Tipo de Movimento
        // POSIÇÃO: 015 a 015
        // PICTURE: 9(1)
        // DEFAULT:  
        if ($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
            $linha .= '9'; // '9' = Indica EXCLUSÃO
        }
        else{
            $linha.=  '0'; // '0' = Indica INCLUSÃO
            
            //Somatório dos registros com tipo de movimento = 0 (inclusão)
            $this->vrPagtLote += round($boleto->getVrfinalbpag(),2);
        }
        
        // CAMPO: Código da Instrução p/ Movimento
        // POSIÇÃO: 016 a 017
        // PICTURE: 9(2)
        
        if ($this->remessa->getIdopecnabpag() == self::INCLUSAO){
            $linha .= '00';  //'00' = Inclusão de Registro Detalhe Liberado 
        }
        else{
            if($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
                $linha.=  '99';  //'99' = Exclusão do Registro Detalhe Incluído Anteriormente
            }
            else{
                $linha.=  '09'; //'09' = Inclusão do Registro Detalhe Bloqueado
            }
        }
          
        // CAMPO: Código da Câmara Centralizadora
        // POSIÇÃO: 018 a 020
        // PICTURE: 9(3)
        $tipobaixapag = Tipobaixapag::findOneByCdtpbaixapagAndNrorg($boleto->getCdtpbaixapag(), $this->nrorg );
         if($this->formaPgto == '01' or $this->formaPgto == '03' or $this->formaPgto == '45'){
             if(($tipobaixapag->getIdbaixautilp() == self::TIPO_BAIXA_PIX)|| ($tipobaixapag->getIdbaixautilp() == self::TIPO_BAIXA_PIX_QRCODE)){
                 $linha.= '009'; //“009” – PIX (SPI)
             } else if(floatval($boleto->getVrfinalbpag()) > floatval($tipobaixapag->getVrmaxdoccnab())){
                if($tipobaixapag->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO){
                    $linha.= '018'; //'018' = TED (STR,CIP)
                }else {
                    $linha.= '988'; //“988”- TED (STR/CIP)
                }
             } else{
                 $linha.= '700'; //'700' = DOC (COMPE)
             }
         }
            
        // CAMPO: Código do Banco do Favorecido
        // POSIÇÃO: 021 a 023
        // PICTURE: 9(3)
        $linha .= str_pad(substr($cdbanco, 0, 3), 3, '0', STR_PAD_LEFT);
        
        
        if ($boleto->getTpbaixapag()->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_ORDEM_PAG) {
            
            // CAMPO: Ag. Mantenedora da Cta do Favor.
            // POSIÇÃO: 024 a 028
            // PICTURE: 9(5)
            $linha .= str_pad(substr(trim($boleto->getReducont()->getCdagencia()), 0, 4), 5, '0', STR_PAD_LEFT);
            
            // CAMPO: Dígito Verificador da Agência
            // POSIÇÃO: 029 a 029
            // PICTURE: 9(1)
            $linha .= str_pad(substr(trim($boleto->getReducont()->getCdagencia()), -1), 1, ' ', STR_PAD_LEFT);
            
            // CAMPO: Número da Conta Corrente
            // POSIÇÃO: 030 a 041
            // PICTURE: 9(12)
            $linha.= str_repeat('0',12);
            
            // CAMPO: Dígito Verificador da Conta
            // POSIÇÃO: 042 a 042
            // PICTURE: 9(1)
            $linha.= str_repeat('0',1);
            
            // CAMPO: Dígito Verificador da AG/Conta
            // POSIÇÃO: 043 a 043
            // PICTURE: X(1)
            $linha.= str_repeat('0',1);
        } else {
            //Busca a máscara da CONTA e AGÊNCIA do fornecedor cadastrada na tela de Banco
            /** @var $bancoFornecedor Banco */
            $bancoFornecedor = self::getBancoForn($boleto->getCdbancbolp(), $boleto->getNrorg());
            
            if($bancoFornecedor and $bancoFornecedor->getCdpictagen()){
                //Caso na máscara da agência NÃO tiver '-' 
                if(strpos($bancoFornecedor->getCdpictagen(),'-') === false){
                    // CAMPO: Ag. Mantenedora da Cta do Favor.
                    // POSIÇÃO: 024 a 028
                    // PICTURE: 9(5)
                    $linha.= str_pad(substr(trim(Util::campoCnabValido($boleto->getCdagenbolp())),0,5),5,'0',STR_PAD_LEFT);
                    // CAMPO: Dígito Verificador da Agência
                    // POSIÇÃO: 029 a 029
                    // PICTURE: 9(1)
                    $linha.= ' ';
                }
                else{
                    //Caso a máscara TENHA '-'
                    
                    // CAMPO: Ag. Mantenedora da Cta do Favor.
                    // POSIÇÃO: 024 a 028
                    // PICTURE: 9(5)
                    $linha.= str_pad(substr(trim(Util::campoCnabValido($boleto->getCdagenbolp())),0,strpos($bancoFornecedor->getCdpictagen(),'-')),5,'0',STR_PAD_LEFT);

                    /** 
                     * Se foi cadastrada a agencia do favorecido como 9999 na tela de boleto e
                        tempos depois foi modificada a máscara para 9999-9 (e o campo anterior não foi atualizado),
                        qnd for pegar a parte do -9, nao tem e ele fica vazio, este if arruma este problema.
                    */
                    if(strlen(substr(trim(Util::campoCnabValido($boleto->getCdagenbolp())),strpos($bancoFornecedor->getCdpictagen(),'-')))>0){
                        // CAMPO: Dígito Verificador da Agência
                        // POSIÇÃO: 029 a 029
                        // PICTURE: 9(1)
                        $pos = strpos($bancoFornecedor->getCdpictagen(),'-');
                        $linha.= substr(trim(Util::campoCnabValido($boleto->getCdagenbolp())),$pos,1);
                    }
                    else{
                        $linha.= ' ';
                    }
                }
            }
            else{
                //Se o Banco do fornecedor NÃO estiver cadastrado no sistema,NÃO parametrizado com as máscaras, entrará nesse else 
                if(strpos($boleto->getCdagenbolp(),'-')=== false){
                    // { se o que foi digitado no campo de agencia do favorecido da tela de boleto NÃO tem '-' }
                    // CAMPO: Ag. Mantenedora da Cta do Favor.
                    // POSIÇÃO: 024 a 028
                    // PICTURE: 9(5)
                    $linha.= str_pad(substr(trim($boleto->getCdagenbolp()),0,5),5,'0',STR_PAD_LEFT);

                    // CAMPO: Dígito Verificador da Agência
                    // POSIÇÃO: 029 a 029
                    // PICTURE: 9(1)
                    $linha.= ' ';
                }
                else{
                    //Caso o que foi digitado no campo de agencia do favorecido da tela de boleto TENHA '-' 
                    
                    // CAMPO: Ag. Mantenedora da Cta do Favor.
                    // POSIÇÃO: 024 a 028
                    // PICTURE: 9(5)
                    $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdagenbolp(),0,strpos($boleto->getCdagenbolp(),'-')
                    )),5,'0',STR_PAD_LEFT);

                    // CAMPO: Dígito Verificador da Agência
                    // POSIÇÃO: 029 a 029
                    // PICTURE: 9(1)
                    $pos = strpos($boleto->getCdagenbolp(),'-') +1;
                    $linha.= substr(trim(Util::campoCnabValido($boleto->getCdagenbolp())),$pos,1);
                }
            }
            
            // CAMPO: Número da Conta Corrente
            // POSIÇÃO: 030 a 041
            // PICTURE: 9(12)
            if ($boleto->getTpbaixapag()->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_ORDEM_PAG) {
                $linha .= str_repeat('0', 12);
                $digitoConta = '0';
            } else {
                if (strpos($boleto->getCdccorbolp(), '-') === false) {
                    $linha .= str_pad(trim(substr($boleto->getCdccorbolp(), 0, 12)), 12, '0', STR_PAD_LEFT);
                    $digitoConta = ' ';
                } else {
                    $linha .= substr(str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(), 0, strpos($boleto->getCdccorbolp(), '-'))), 12, '0', STR_PAD_LEFT), -12);
                    $digitoConta = trim(substr($boleto->getCdccorbolp(), (strpos($boleto->getCdccorbolp(), '-') + 1), 1));
                }
            }
            
            // CAMPO: Dígito Verificador da Conta
            // POSIÇÃO: 042 a 042
            // PICTURE: 9(1)
            $linha .= $digitoConta;
            
            // CAMPO: Dígito Verificador da AG/Conta
            // POSIÇÃO: 043 a 043
            // PICTURE: X(1)
            $linha .= ' ';
        }
        
        // CAMPO: Nome do Favorecido
        // POSIÇÃO: 044 a 073
        // PICTURE: X(30)
        $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(), 0, 30)), 30,' ',STR_PAD_RIGHT);
        
        // CAMPO: Nº do Docum. Atribuído p/ Empresa - FILIAL+ NRBOLETO
        // POSIÇÃO: 074 a 093
        // PICTURE: X(20)
        if($boleto->getNrlanbolpag()){
            $linha.= str_pad(trim($boleto->getCdfilial()).trim($boleto->getNrlanbolpag()),20,' ', STR_PAD_RIGHT );
        } else {
            $linha.= str_repeat(' ', 20);
        }
        
        // CAMPO: Data do Pagamento
        // POSIÇÃO: 094 a 101
        // PICTURE: 9(8)
        // DEFAULT: DDMMAAAA
        $linha.= $boleto->getDtvenbolpag()->format('dmY');
        
        // CAMPO: Tipo da Moeda
        // POSIÇÃO: 102 a 104
        // PICTURE: X(3)
         $linha.= 'BRL';
        
        // CAMPO: Quantidade da Moeda
        // POSIÇÃO: 105 a 119
        // PICTURE: 9(10) V(5)
        $linha.= str_repeat('0', 15);
        
        // CAMPO: Valor do Pagamento
        // POSIÇÃO: 120 a 134
        // PICTURE: 9(13) V(2)
        $linha.= str_pad(str_replace(array('.',','),'',number_format($boleto->getVrfinalbpag(),2)),15,'0',STR_PAD_LEFT);
            
        // CAMPO: Nº do Docum. Atribuído pelo Banco
        // POSIÇÃO: 135 a 154
        // PICTURE: X(20)
        if($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
            $linha.= str_pad(substr($boleto->getNrretcnabpag(),0,20),20,' ',STR_PAD_RIGHT);
        } else{
            $linha.= self::preenche_brancos(' ',20);
        }
        
        // CAMPO: Data Real da Efetivação Pagto
        // POSIÇÃO: 155 a 162
        // PICTURE: 9(8)
        // DEFAULT: DDMMAAAA
        $linha.= str_repeat('0', 8);
        
        // CAMPO: Valor Real da Efetivação do Pagto
        // POSIÇÃO: 163 a 177
        // PICTURE: 9(13) V(2)
        $linha.= str_repeat('0', 15);
        
        // CAMPO: Informação 2
        // POSIÇÃO: 178 a 217
        // PICTURE: X(40)
        $linha.= str_pad(trim($boleto->getCdfilial()).trim($boleto->getNrlanbolpag()),40,' ', STR_PAD_RIGHT );
        
        // CAMPO: Compl. Tipo Serviço
        // POSIÇÃO: 218 a 219
        // PICTURE: X(2)
        if ($this->formaPgto = '03'){
            $linha.= '03';
        }else {
            $linha.= '99';
        }
        
        // CAMPO: Codigo finalidade da TED
        // POSIÇÃO: 220 a 224
        // PICTURE: X(5)
        if ($this->formaPgto = '41'){
            $linha.= '00005';
        }else {
            $linha.=  str_repeat(' ', 5);
        }
        
        // CAMPO: Complemento de finalidade pagto.
        // POSIÇÃO: 225 a 226
        // PICTURE: X(2)
        if ($this->formaPgto = '41'){
            $linha.= 'CC';
        }else {
            $linha.=  str_repeat(' ', 2);
        }
        
        // CAMPO: Uso Exclusivo FEBRABAN/CNAB
        // POSIÇÃO: 227 a 229
        // PICTURE: X(3)
        // DEFAULT: Brancos
        $linha.= str_repeat(' ',3);
        
        // CAMPO: Aviso ao Favorecido
        // POSIÇÃO: 230 a 230
        // PICTURE: 9(1)      
        $linha.= str_repeat('0',1);
        
        // CAMPO: Códigos das Ocorrências p/ Retorno
        // POSIÇÃO: 231 a 240
        // PICTURE: X(10)
        $linha.= self::preenche_brancos(' ',10);

        return $linha."\r\n";
    }

    /**
     * Função Responsável por gerar o Seguimento B complemenentar do arquivo de remessa
     * @param Boletpagar $boleto
     * @return string
     * @throws Exception
     * @internal param $cdbanco
     */
    private function geraSegmentoB(Boletpagar $boleto){

        $linha = '';

        $this->qtdRegiLote++;
        $this->qtdRegiArq++;
        
        // CAMPO: Código do Banco na Compensação
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        $linha .= $this->cdbanco;
        
        // CAMPO: Lote de Serviço
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        $linha .= str_pad($this->num_lote, 4, '0', STR_PAD_LEFT);
        
        // CAMPO: Tipo do Registro
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        // DEFAULT: 3
        $linha .= '3';
        
        // CAMPO: Nº Seqüencial do Registro no Lote
        // POSIÇÃO: 009 a 013
        // PICTURE: 9(5)
        $linha .= str_pad(++$this->seqRegLote, 5, '0', STR_PAD_LEFT);
        
        // CAMPO: Código de Segmento do Reg. Detalhe
        // POSIÇÃO: 014 a 014
        // PICTURE: X(1)
        // DEFAULT: B
        $linha .= 'B';
        
        // PIX 
        if ($boleto->getIdchavepix()){
            
            // CAMPO: Forma de Iniciação
            // POSIÇÃO: 015 a 017
            // PICTURE: X(3)
            
            /* Para o tipo “forma de lançamento” igual ao PIX, usar o seguinte domínio.
            Domínio:
            o “01” – Chave Pix – tipo Telefone
            o “02” – Chave Pix – tipo Email
            o “03” – Chave Pix – tipo CPF/CNPJ
            o “04” – Chave Aleatória
            o “05” – Dados bancários
            */
            $linha.= str_pad($boleto->getIdchavepix(), 3,'0',STR_PAD_LEFT);

            if($boleto->getIdtpicnabbop()) {
                if ($boleto->getIdtpicnabbop() == 'F'){
                    $tipoInsc = '1'; //CPF
                } else {
                    if ($boleto->getIdtpicnabbop() == 'J'){ 
                        $tipoInsc =  '2'; //CGC
                    } else {
                        $tipoInsc = '3'; //Outros
                    }
                }
            }
            else {
                if ($boleto->getForneced()->getIdtpijurforn() == 'F'){
                    $tipoInsc = '1'; //CPF
                } else {
                    if ($boleto->getForneced()->getIdtpijurforn() == 'J') {
                        $tipoInsc = '2'; //CGC
                    } else {
                        $tipoInsc = '3'; //Outros
                    }
                }
            }
            
            // CAMPO: Tipo de Inscrição do Favorecido
            // POSIÇÃO: 018 a 018
            // PICTURE: 9(1)
            $linha.= $tipoInsc;
            
            // CAMPO: Nº de Inscrição do Favorecido
            // POSIÇÃO: 019 a 032
            // PICTURE: 9(14)
            if ($boleto->getNrinscnabbop()){
                $linha.= str_pad(substr($boleto->getNrinscnabbop(), 0, 14), 14,'0',STR_PAD_LEFT);
            } else {
                $linha.= str_pad(substr($boleto->getForneced()->getNrinsjurforn(), 0, 14), 14,'0',STR_PAD_LEFT);
            }
            
            // CAMPO: Informação 10
            // POSIÇÃO: 033 a 067
            // PICTURE: X(35)
            /* Para o tipo “forma de lançamento” igual ao PIX, usar campo como:
               TX ID (opcional) Posição (33 67) Alfa.
            */
            $linha.= str_repeat(' ', 35);
            
            
            // CAMPO:  Informação 11 
            // POSIÇÃO: 68 a 127 
            // PICTURE: X(60)
            $linha.= str_repeat(' ', 60);
            
            // CAMPO: Informação 12
            // POSIÇÃO: 128 a 226 
            // PICTURE: X(99)
            $linha.= str_pad($boleto->getChavepix(), 99);
            
        } else {
            
            // CAMPO: Forma de Iniciação
            // POSIÇÃO: 015 a 017
            // PICTURE: X(3)
            $linha.= str_repeat(' ', 3);
            
            if($boleto->getIdtpicnabbop()) {
                if ($boleto->getIdtpicnabbop() == 'F')
                    $tipoInsc = '1'; //CPF
                else {
                    if ($boleto->getIdtpicnabbop() == 'J'){
                        $tipoInsc =  '2'; //CGC
                    } else {
                        $tipoInsc = '3'; //Outros
                    }
                }
            } else {
                if ($boleto->getForneced()->getIdtpijurforn() == 'F'){
                    $tipoInsc = '1'; //CPF
                } else {
                    if ($boleto->getForneced()->getIdtpijurforn() == 'J'){
                        $tipoInsc = '2'; //CGC
                    } else {
                        $tipoInsc = '3'; //Outros
                    }
                }
            }
            
            // CAMPO: Tipo de Inscrição do Favorecido
            // POSIÇÃO: 018 a 018
            // PICTURE: 9(1)
            $linha.= $tipoInsc;
    
            // CAMPO: Nº de Inscrição do Favorecido
            // POSIÇÃO: 019 a 032
            // PICTURE: 9(14)
            if ($boleto->getNrinscnabbop()) {
                $linha.= str_pad(substr($boleto->getNrinscnabbop(), 0, 14), 14,'0',STR_PAD_LEFT);
            } else {
                $linha.= str_pad(substr($boleto->getForneced()->getNrinsjurforn(), 0, 14), 14,'0',STR_PAD_LEFT);
            }
            
            
            // CAMPO: Informação 11 
            // POSIÇÃO: 33 a 67
            // PICTURE: X(35)
            // OBSEVAÇÃO MANUAL G101 -  Para os demais quee não seja PIX, usar como:
            // Logradouro do Favorecido: Nome da Rua, Av, Pça, Etc Posição 
             $endforn = $boleto->getForneced()->getEndefornecobr();
            
            if(!$endforn)
                $endforn = $boleto->getForneced()->getEndeforneprinc();
            $linha.= str_pad(substr(Util::campoCnabValido($endforn->getDsendeforn()),0,35),35,' ',STR_PAD_RIGHT);
             
            // CAMPO: Informação 11 - Nº do Local
            // POSIÇÃO: 68 a 72
            // PICTURE: 9(5)
            // OBSEVAÇÃO MANUAL G101 
            $linha.= str_repeat('0', 5);
            
            // CAMPO: Informação 11 - Complemento Casa, Apto, Etc Posição
            // POSIÇÃO: 73 a 87
            // PICTURE: X(15)
            // OBSEVAÇÃO MANUAL G101 
            $linha.= str_repeat(' ', 15);
            
            // CAMPO: Informação 11 - Bairro 
            // POSIÇÃO: 88 a 102
            // PICTURE: X(15)
            // OBSEVAÇÃO MANUAL G101 
            $linha.= str_pad(substr(Util::campoCnabValido($endforn->getNmbairforn()),0,15),15,' ',STR_PAD_RIGHT);

            // CAMPO: Informação 11 - Nome da Cidade Posição
            // POSIÇÃO: 103 a 117
            // PICTURE: X(15)
            // OBSEVAÇÃO MANUAL G101
            $municipio = Municipio::findOneBySgestadoAndCdpaisAndCdmunicipio($endforn->getSgestado(), $endforn->getCdpais(), $endforn->getCdmunicipio());
            $linha.= str_pad(substr(Util::campoCnabValido($municipio->getNmmunicipio()),0,15),15,' ',STR_PAD_RIGHT);
            
            // CAMPO: Informação 11 - CEP
            // POSIÇÃO: 118 a 122
            // PICTURE: 9(5)
            // OBSEVAÇÃO MANUAL G101
            
            // CAMPO: Informação 11 - CEP Complem
            // POSIÇÃO: 123 a 125
            // PICTURE: X(3)
            // OBSEVAÇÃO MANUAL G101
            
            if($endforn->getNrcepforn()){
                $linha.= str_pad(substr(Util::campoCnabValido(trim($endforn->getNrcepforn())),0,8),8,'0');
            }else{
                $linha.= str_repeat('0',8);
            }
            
            // CAMPO: Informação 11 - Sigla do Estado
            // POSIÇÃO: 126 a 127
            // PICTURE: X(2)
            // OBSEVAÇÃO MANUAL G101
            $linha.= str_pad(substr(Util::campoCnabValido($endforn->getSgestado()),0,2),2,' ',STR_PAD_RIGHT);
            
            // CAMPO: Informação 12 - Data do vencimento (Nominal)
            // POSIÇÃO: 128 a 135
            // PICTURE: 9(08)
            // OBSEVAÇÃO MANUAL G101
            $linha.= $boleto->getDtvenbolpag()->format('dmY');
            
            // CAMPO: Valor do documento (Nominal)
            // POSIÇÃO: 136 a 150
            // PICTURE: 9(13)V9(2)
            // OBSEVAÇÃO MANUAL G101
            $linha.= str_pad(str_replace(array('.',','),'',number_format($boleto->getVrfinalbpag(),2)),15,'0',STR_PAD_LEFT);
    
            // CAMPO: Valor do abatimento 
            // POSIÇÃO: 151 a 165 
            // PICTURE: 9(13)V9(2)
            // OBSEVAÇÃO MANUAL G101
            $linha.= str_repeat('0', 15);
    
            // CAMPO:  Valor do desconto 
            // POSIÇÃO: 166 a 180 
            // PICTURE: 9(13)V9(2)
            $linha.= str_repeat('0', 15);
    
            // CAMPO:  Valor da mora
            // POSIÇÃO: 181 a 195 
            // PICTURE: 9(13)V9(2)
            $linha.= str_repeat('0', 15);
    
            // CAMPO:   Valor da multa 
            // POSIÇÃO: 196 a 210 
            // PICTURE: 9(13)V9(2)
            $linha.= str_repeat('0', 15);
    
            // CAMPO: Codigo do Favorecido
            // POSIÇÃO:  211 a 225
            // PICTURE: X(15)
            $linha.= str_repeat(' ', 15);
    
            // CAMPO: Aviso ao Favorecido
            // POSIÇÃO:  226 a 226
            // PICTURE: 9(1)
            $linha.= str_repeat('0', 1);
            
            
        }
        
        // CAMPO: Uso Exclusivo para o SIAPE
        // POSIÇÃO: 227 a 232 
        // PICTURE: 9(6)
        $linha.= str_repeat('0', 6);
        
        // CAMPO: Código ISPB
        // POSIÇÃO: 233 a 240 
        // PICTURE: 9(8)
        $linha.= str_repeat('0', 8);
        return $linha."\r\n";
    }
    
    /**
     * Função Responsável por gerar a linha do seguimento J da remessa no arquivo
     * @param Boletpagar $boleto
     * @param $cdbanco
     * @return string
     */
    private function geraSegmentoJ(Boletpagar $boleto, $cdbanco)
    {

        $linha = '';
        $this->qtdRegiLote++;//Qtde de registro do lote
        $this->qtdRegiArq++; //Quantidade de registro do arquivo
        
        // CAMPO: Código do Banco na Compensação
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        $linha .= $this->cdbanco;
        
        // CAMPO: Lote de Serviço
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        $linha .= str_pad($this->num_lote, 4, '0', STR_PAD_LEFT);
        
        // CAMPO: Tipo de Registro
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        // DEFAULT:  3
        $linha .= '3';
        
        // CAMPO: Nº Seqüencial do Registro no Lote
        // POSIÇÃO: 009 a 013
        // PICTURE: 9(5)
        // DEFAULT:  3
        $linha .= str_pad(++$this->seqRegLote, 5, '0', STR_PAD_LEFT);
        
        // CAMPO: Código de Segmento do Reg. Detalhe
        // POSIÇÃO: 014 a 014
        // PICTURE: X(1)
        // DEFAULT:  J
        $linha.= 'J';
        
        // CAMPO: Tipo de Movimento
        // POSIÇÃO: 015 a 015
        // PICTURE: 9(1)
        if ($this->remessa->getIdopecnabpag() == 'E'){
             $linha.= '9';
        } else{
            $linha.= '0';
            //Somatório dos registros com tipo de movimento = 000 (inclusão)
            $this->vrPagtLote+= round($boleto->getVrfinalbpag(), 2);
        }
        
        // CAMPO: Código da Instrução p/ Movimento
        // POSIÇÃO: 016 a 017
        // PICTURE: 9(2)
        if ($this->remessa->getIdopecnabpag() == 'E'){
            $linha .= '99'; //Exclusão
        } else {
            if ($this->remessa->getIdopecnabpag() == 'I'){
                $linha .= '00'; //Inclusão
            } else {
                $linha .= '09';   // Bloqueio
            }
        }
        
        
        if ($boleto->getCdbarrapag()) {
            $cdbanco = str_pad(substr($boleto->getCdbarrapag(), 0, 3), 3, '0', STR_PAD_LEFT);
            $cdmoeda = str_pad(substr($boleto->getCdbarrapag(), 3, 1), 1, '0', STR_PAD_LEFT);
            $cddigito = str_pad(substr($boleto->getCdbarrapag(), 4, 1), 1, '0', STR_PAD_LEFT);
            $valor = str_pad(substr($boleto->getCdbarrapag(), 5, 14), 14, '0', STR_PAD_LEFT);
            $cdcampo = str_pad(substr($boleto->getCdbarrapag(), 19, 25), 25, '0', STR_PAD_LEFT);
        } else {
            $linhaBarra = '';
            $dslinhabarrapag = $boleto->getDslinhbarpag();
            for ($i = 0; $i < strlen($boleto->getDslinhbarpag()); $i++) {
                if (in_array($dslinhabarrapag[$i], array('0', '1', '2', '3', '4', '5', '6', '7', '8', '9')))
                    $linhaBarra .= $dslinhabarrapag[$i];
            }

            $cdbanco = str_pad(substr($linhaBarra, 0, 3), 3, '0', STR_PAD_LEFT);
            $cdmoeda = str_pad(substr($linhaBarra, 3, 1), 1, '0', STR_PAD_LEFT);
            $cddigito = str_pad(substr($linhaBarra, 32, 1), 1, '0', STR_PAD_LEFT);
            $valor = str_pad(substr($linhaBarra, 33, 14), 14, '0', STR_PAD_LEFT);
            $cdcampo = str_pad(substr($linhaBarra, 33, 14), 14, '0', STR_PAD_LEFT);
        }
        
        // CAMPO: Código de Barras
        // POSIÇÃO: 018 a 061
        // PICTURE: 9(44)
        $linha .= $cdbanco;
        $linha .= $cdmoeda;
        $linha .= $cddigito;
        $linha .= $valor;
        $linha .= $cdcampo;
       
        // CAMPO: Nome do Beneficiário
        // POSIÇÃO: 062 a 091
        // PICTURE: X(30)
        $linha .= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(), 0, 30)), 30, ' ', STR_PAD_RIGHT);
       
        // CAMPO: Data do Vencimento (Nominal)
        // POSIÇÃO: 092 a 099
        // PICTURE: 9(8)
        $linha .= $boleto->getDtvenbolpag()->format('dmY');
        
        // CAMPO: Valor do Título (Nominal)
        // POSIÇÃO: 100 a 114
        // PICTURE: 9(13) V(2)
        if ($boleto->getCdbarrapag()){
            $linha.= str_pad(substr($boleto->getCdbarrapag(), 10, 9), 15,'0',STR_PAD_LEFT);
            $vrCodBarras = (int)(substr($boleto->getCdbarrapag(), 10, 9)) / 100;
            if ($boleto->getVrfinalbpag() - $vrCodBarras > 0) {
                $vrdesconto = 0;
                $vracrescimo = $boleto->getVrfinalbpag() - $vrCodBarras;
              } else {
                  $vrdesconto = $vrCodBarras - $boleto->getVrfinalbpag();
                  $vracrescimo = 0;
              }
           }
           else {
               $vrdesconto = 0;
               $vracrescimo = 0;
               $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),15,'0',STR_PAD_LEFT);
           }
       
        // CAMPO: Valor do Desconto + Abatimento
        // POSIÇÃO: 115 a 129
        // PICTURE: 9(13) V(2)
        $linha.= str_pad(str_replace(array('.',','),'',number_format($vrdesconto,2)),15,'0',STR_PAD_LEFT);
        
        // CAMPO: Valor da Mora + Multa
        // POSIÇÃO: 130 a 144
        // PICTURE: 9(13) V(2)
        $linha.= str_pad(str_replace(array('.',','),'',number_format($vracrescimo,2)),15,'0',STR_PAD_LEFT);
       
        // CAMPO: Data do Pagamento
        // POSIÇÃO: 145 a 152
        // PICTURE: 9(8)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');
               
        // CAMPO: Valor do Pagamento
        // POSIÇÃO: 153 a 167
        // PICTURE: 9(13) V(2)
        $linha.= str_pad(str_replace(array('.',','),'',number_format($boleto->getVrfinalbpag(),2)),15,'0',STR_PAD_LEFT);   
        
        // CAMPO: Quantidade da Moeda
        // POSIÇÃO: 168 a 182
        // PICTURE: 9(10) V(5)
        $linha.= str_repeat('0', 15);
        
        // CAMPO: Nº do Docto Atribuído pela Empresa
        // POSIÇÃO: 183 a 202
        // PICTURE: X(20)
        $linha.= str_pad(trim($boleto->getCdfilial()).trim($boleto->getNrlanbolpag()),20,' ', STR_PAD_RIGHT );  
        
        // CAMPO: Nº do Docto Atribuído pelo Banco
        // POSIÇÃO: 203 a 222
        // PICTURE: X(20)
        if($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getNrretcnabpag(),0,20)),20,' ',STR_PAD_RIGHT);
        } else {
            $linha.= str_repeat(' ', 20); 
        }
        
        // CAMPO: Código de Moeda
        // POSIÇÃO: 223 a 224
        // PICTURE: 9(2)
        $linha.= '09';
        
        // CAMPO: Uso Exclusivo FEBRABAN/CNAB
        // POSIÇÃO: 225 a 230
        // PICTURE: X(6)
        // DEFAULT: Brancos
        $linha.= self::preenche_brancos(' ',6);
        
        // CAMPO: Códigos das Ocorrências p/ Retorno
        // POSIÇÃO: 231 a 240
        // PICTURE: X(10)
        $linha.= self::preenche_brancos(' ',10);
        return $linha."\r\n";
    }
    
    /**
     * Função Responsável por gerar a linha do Seguimento J52 do arquivo de remessa
     * @param Boletpagar $boleto
     * @return string
     * @throws Exception
     * @internal param $cdbanco
     */

    private function geraSegmentoJ52(Boletpagar $boleto,  $cdbanco){

        $linha = '';
        
        $this->qtdRegiLote++;
        $this->qtdRegiArq++;
        
        // CAMPO: Código do Banco na Compensação
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        $linha .= $this->cdbanco;
        
        // CAMPO: Lote de Serviço
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        $linha .= str_pad($this->num_lote, 4, '0', STR_PAD_LEFT);
        
        // CAMPO: Tipo de Registro
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        // DEFAULT:  3
        $linha .= '3';
        
        // CAMPO: Nº Seqüencial do Registro no Lote
        // POSIÇÃO: 009 a 013
        // PICTURE: 9(5)
        // DEFAULT:  3
        $linha .= str_pad(++$this->seqRegLote, 5, '0', STR_PAD_LEFT);
        
        // CAMPO: Código de Segmento do Reg. Detalhe
        // POSIÇÃO: 014 a 014
        // PICTURE: X(1)
        // DEFAULT:  J
        $linha.= 'J';
        
        // CAMPO: Uso Exclusivo FEBRABAN/CNAB
        // POSIÇÃO: 015 a 015
        // PICTURE: X(1)
        // DEFAULT:  Brancos
        $linha.= str_repeat(' ', 1);
        
        // CAMPO: Código de Movimento Remessa
        // POSIÇÃO: 016 a 017
        // PICTURE: 9(2)
        $linha .= '00'; 
        
        // CAMPO: Identificação Registro Opcional
        // POSIÇÃO: 018 a 019
        // PICTURE: 9(2)
        // DEFAULT: J52
        $linha.= '52';
        
        // CAMPO: Tipo de Inscrição
        // POSIÇÃO: 020 a 020
        // PICTURE: 9(1)
        $linha.= $this->tipoinscricao;
        
        // CAMPO: Número de Inscrição
        // POSIÇÃO: 021 a 035
        // PICTURE: 9(15)
        $linha.= str_pad(substr($this->inscricaojuridica,0,14),15,'0', STR_PAD_LEFT);
        
        // CAMPO: Nome
        // POSIÇÃO: 036 a 075
        // PICTURE: X(40)
        $linha.= str_pad(Util::campoCnabValido(substr($this->nomeempresa,0,40)),40);
        
        // CAMPO: Tipo de Inscrição
        // POSIÇÃO: 076 a 076
        // PICTURE: 9(1)
        if(strlen(Util::campoCnabValido($boleto->getForneced()->getNrinsjurforn())) == 11){
            $linha.= '1';
        }else{
            $linha.= '2';
        }
        
        // CAMPO: Número de Inscrição
        // POSIÇÃO: 077 a 091
        // PICTURE: 9(15)
        $linha.= str_pad(substr($boleto->getForneced()->getNrinsjurforn(),0,14),15,'0', STR_PAD_LEFT);
        
        // CAMPO: Nome
        // POSIÇÃO: 092 a 131
        // PICTURE: X(40)
        $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,40)),40);
        
        // CAMPO: Tipo de Inscrição
        // POSIÇÃO: 132 a 132
        // PICTURE: 9(1)
        $linha.= $this->tipoinscricao;
        
        // CAMPO: Número de Inscrição
        // POSIÇÃO: 133 a 147
        // PICTURE: 9(15)
        $linha.= str_pad(substr($this->inscricaojuridica,0,14),15,'0', STR_PAD_LEFT);
        
        // CAMPO:  Nome
        // POSIÇÃO: 148 a 187
        // PICTURE: X(40)
        $linha.= str_pad(Util::campoCnabValido(substr($this->nomeempresa,0,40)),40);
        
        // CAMPO:  Uso Exclusivo FEBRABAN/CNAB
        // POSIÇÃO: 188 a 240
        // PICTURE: X(53)
        // DEFAULT: Brancos
        $linha.= str_repeat(' ',53);
        return $linha . "\r\n";
    }
    
     /**
      * Função Responsável por gerar a linha do Seguimento J52 PIX do arquivo de remessa
     * @param Boletpagar $boleto
     * @return string
     * @throws Exception
     * @internal param $cdbanco
     */
    private function geraSegmentoj52Pix(Boletpagar $boleto){

        $linha = '';
        
        $this->qtdRegiLote++;
        $this->qtdRegiArq++;
        
        // CAMPO: Código do Banco na Compensação
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        $linha .= $this->cdbanco;
        
        // CAMPO: Lote de Serviço
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        $linha .= str_pad($this->num_lote, 4, '0', STR_PAD_LEFT);
        
        // CAMPO: Tipo de Registro
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        // DEFAULT:  3
        $linha .= '3';
        
        // CAMPO: Nº Seqüencial do Registro no Lote
        // POSIÇÃO: 009 a 013
        // PICTURE: 9(5)
        $linha .= str_pad(++$this->seqRegLote, 5, '0', STR_PAD_LEFT);
        
        // CAMPO: Código de Segmento do Reg. Detalhe
        // POSIÇÃO: 014 a 014
        // PICTURE: X(1)
        // DEFAULT:  J
        $linha.= 'J';
        
        // CAMPO: Uso Exclusivo FEBRABAN/CNAB
        // POSIÇÃO: 015 a 015
        // PICTURE: X(1)
        // DEFAULT:  Brancos
        $linha.= str_repeat(' ', 1);
        
        // CAMPO: Código de Movimento Remessa
        // POSIÇÃO: 016 a 017
        // PICTURE: 9(2)
        if ($this->remessa->getIdopecnabpag() == 'E'){
           $linha .= '99'; //Exclusão
        }
        else {
            if ($this->remessa->getIdopecnabpag() == 'I'){
                $linha .= '00'; //Inclusão
            } else{
                $linha .= '09'; // Bloqueio
            }
        }
        
        // CAMPO: Identificação Registro Opcional
        // POSIÇÃO: 018 a 019
        // PICTURE: 9(2)
        // DEFAULT: 52
        $linha.= '52';
        
        // CAMPO: Tipo de Inscrição
        // POSIÇÃO: 020 a 020
        // PICTURE: 9(1)
        $linha.= $this->tipoinscricao;
        
        // CAMPO: Número de Inscrição
        // POSIÇÃO: 021 a 035
        // PICTURE: 9(15)
        $linha.= str_pad(substr($this->inscricaojuridica,0,14),15,'0', STR_PAD_LEFT);
        
        // CAMPO: Nome
        // POSIÇÃO: 036 a 075
        // PICTURE: X(40)
        $linha.= str_pad(Util::campoCnabValido(substr($this->nomeempresa,0,40)),40);
        
        // CAMPO: Tipo de Inscrição
        // POSIÇÃO: 076 a 076
        // PICTURE: 9(1)
        if(strlen(Util::campoCnabValido($boleto->getForneced()->getNrinsjurforn())) == 11){
            $linha.= '1';
        }else{
            $linha.= '2';
        }
        
        // CAMPO: Número de Inscrição
        // POSIÇÃO: 077 a 091
        // PICTURE: 9(15)
        $linha.= str_pad(substr($boleto->getForneced()->getNrinsjurforn(),0,14),15,'0', STR_PAD_LEFT);
        
        // CAMPO: Nome
        // POSIÇÃO: 092 a 131
        // PICTURE: X(40)
        $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,40)),40);
        
        // CAMPO: URL/Chave de endereçamento
        // POSIÇÃO: 132 a 210
        // PICTURE: X(79)
        $linha.= str_pad($boleto->getCodeqrpix(),79);
        
        // CAMPO: Código de Identificação do QR-Code
        // POSIÇÃO: 211 a 240
        // PICTURE: X(30)
        $linha.= str_repeat(' ',30);
    
        return $linha . "\r\n";
    }
    
    /**
     *  Função Responsável por gerar a linha do Seguimento O do arquivo de remessa
     * @param Boletpagar $boleto
     * @return string
     * @throws Exception
     * @internal param $cdbanco
     */
    private function geraSegmentoO(Boletpagar $boleto){

        $linha = '';
        
        $this->qtdRegiLote++;
        $this->qtdRegiArq++;
        
        // CAMPO: Código do Banco na Compensação
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        $linha .= $this->cdbanco;
        
        // CAMPO: Lote de Serviço
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        $linha .= str_pad($this->num_lote, 4, '0', STR_PAD_LEFT);
        
        // CAMPO: Registro Detalhe de Lote
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        // DEFAULT:  3
        $linha .= '3';
        
        // CAMPO: Nº Seqüencial do Registro no Lote
        // POSIÇÃO: 009 a 013
        // PICTURE: 9(5)
        // DEFAULT:  3
        $linha .= str_pad(++$this->seqRegLote, 5, '0', STR_PAD_LEFT);
        
        // CAMPO: Código de Segmento do Reg. Detalhe
        // POSIÇÃO: 014 a 014
        // PICTURE: X(1)
        // DEFAULT:  O
        $linha.= 'O';
        
        // CAMPO: Tipo de Movimento
        // POSIÇÃO: 015 a 015
        // PICTURE: 9(1)
        // DEFAULT:  Brancos
        if ($this->remessa->getIdopecnabpag() == 'E'){
           $linha.= '9'; 
        } else{
            $linha.= '0';
            
            //Somatório dos registros com tipo de movimento = 000 (inclusão)
            $this->vrPagtLote+= round($boleto->getVrfinalbpag(), 2);
        }
        
        // CAMPO: Código da Instrução de Movimento
        // POSIÇÃO: 016 a 017
        // PICTURE: 9(2)
        if ($this->remessa->getIdopecnabpag() == 'E'){
           $linha .= '99'; //Exclusão
        } else {
            if ($this->remessa->getIdopecnabpag() == 'I') {
                $linha .= '00'; //Inclusão
            }
            else {
                $linha .= '09';   // Bloqueio
            }
        }
        
        if ($boleto->getCdbarrapag()) {
            $cdbanco = str_pad(substr($boleto->getCdbarrapag(), 0, 3), 3, '0', STR_PAD_LEFT);
            $cdmoeda = str_pad(substr($boleto->getCdbarrapag(), 3, 1), 1, '0', STR_PAD_LEFT);
            $cddigito = str_pad(substr($boleto->getCdbarrapag(), 4, 1), 1, '0', STR_PAD_LEFT);
            $valor = str_pad(substr($boleto->getCdbarrapag(), 5, 14), 14, '0', STR_PAD_LEFT);
            $cdcampo = str_pad(substr($boleto->getCdbarrapag(), 19, 25), 25, '0', STR_PAD_LEFT);
        } else {
            
            $linhaBarra = '';
            $dslinhabarrapag = $boleto->getDslinhbarpag();
            for ($i = 0; $i < strlen($boleto->getDslinhbarpag()); $i++) {
                if (in_array($dslinhabarrapag[$i], array('0', '1', '2', '3', '4', '5', '6', '7', '8', '9')))
                    $linhaBarra .= $dslinhabarrapag[$i];
            }

            $cdbanco = str_pad(substr($linhaBarra, 0, 3), 3, '0', STR_PAD_LEFT);
            $cdmoeda = str_pad(substr($linhaBarra, 3, 1), 1, '0', STR_PAD_LEFT);
            $cddigito = str_pad(substr($linhaBarra, 32, 1), 1, '0', STR_PAD_LEFT);
            $valor = str_pad(substr($linhaBarra, 33, 14), 14, '0', STR_PAD_LEFT);
            $cdcampo = str_pad(substr($linhaBarra, 33, 14), 14, '0', STR_PAD_LEFT);
        }
        
        // CAMPO: Código de Barras
        // POSIÇÃO: 018 a 061
        // PICTURE: X(44)
        $linha .= $cdbanco;
        $linha .= $cdmoeda;
        $linha .= $cddigito;
        $linha .= $valor;
        $linha .= $cdcampo;
        
        // CAMPO: Nome da Concessionária / Órgão Público
        // POSIÇÃO: 062 a 091
        // PICTURE: X(30)
        $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,30)),30);
        
        // CAMPO: Data do Vencimento (Nominal)
        // POSIÇÃO: 092 a 099
        // PICTURE: 9(8)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');
                
        // CAMPO: Data do Pagamento
        // POSIÇÃO: 100 a 107
        // PICTURE: 9(8)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');
                      
        // CAMPO: Valor do Pagamento
        // POSIÇÃO: 108 a 122
        // PICTURE: 9(13) V(2)
        $linha.= str_pad(str_replace(array('.',','),'',number_format($boleto->getVrfinalbpag(),2)),15,'0',STR_PAD_LEFT);
        
        // CAMPO: Nº do Docto Atribuído pela Empresa
        // POSIÇÃO: 123 a 142
        // PICTURE: X(20)
        $linha.= str_pad(trim($boleto->getCdfilial()).trim($boleto->getNrlanbolpag()),20,' ', STR_PAD_RIGHT );
        
        // CAMPO: Nº do Docto Atribuído pelo Banco
        // POSIÇÃO: 143 a 162
        // PICTURE: X(20)
        $linha.= str_repeat(' ', 20);
        
        // CAMPO: Uso Exclusivo FEBRABAN/CNAB
        // POSIÇÃO: 163 a 230
        // PICTURE: X(68)
        $linha.= str_repeat(' ', 68);
        
        // CAMPO: Códigos das Ocorrências p/ Retorno
        // POSIÇÃO: 230 a 240
        // PICTURE: X(10)
        $linha.= str_repeat(' ', 10);
        
        return $linha . "\r\n";
    }
    
    /**
     *  Função Responsável por gerar a linha do Seguimento N do arquivo de remessa
     * @param Boletpagar $boleto
     * @return string
     * @throws Exception
     * @internal param $cdbanco
     */
    private function geraSegmentoN(Boletpagar $boleto){

        $linha = '';
        
        $this->qtdRegiLote++;
        $this->qtdRegiArq++;
        
        // CAMPO: Código do Banco na Compensação
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        $linha .= $this->cdbanco;
        
        // CAMPO: Lote de Serviço
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        $linha .= str_pad($this->num_lote, 4, '0', STR_PAD_LEFT);
        
        // CAMPO: Registro Detalhe de Lote
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        // DEFAULT:  3
        $linha .= '3';
        
        // CAMPO: Nº Seqüencial do Registro no Lote
        // POSIÇÃO: 009 a 013
        // PICTURE: 9(5)
        // DEFAULT:  3
        $linha .= str_pad(++$this->seqRegLote, 5, '0', STR_PAD_LEFT);
        
        // CAMPO: Código de Segmento no Reg. Detalhe
        // POSIÇÃO: 014 a 014
        // PICTURE: X(1)
        // DEFAULT:  N
        $linha.= 'N';
        
        // CAMPO: Tipo de Movimento
        // POSIÇÃO: 015 a 015
        // PICTURE: 9(1)
        // DEFAULT:  Brancos
        if ($this->remessa->getIdopecnabpag() == 'E'){
           $linha.= '9'; 
        } else{
            $linha.= '0';
            
            //Somatório dos registros com tipo de movimento = 000 (inclusão)
            $this->vrPagtLote+= round($boleto->getVrfinalbpag(), 2);
        }
        
        // CAMPO: Código da Instrução de Movimento
        // POSIÇÃO: 016 a 017
        // PICTURE: 9(2)
        if ($this->remessa->getIdopecnabpag() == 'E'){
           $linha .= '99'; //Exclusão
        } else {
            if ($this->remessa->getIdopecnabpag() == 'I') {
                $linha .= '00'; //Inclusão
            }
            else {
                $linha .= '09';   // Bloqueio
            }
        }
        
        // CAMPO: Nº do Docto Atribuído pela Empresa
        // POSIÇÃO: 018 a 037
        // PICTURE: X(20)
        $linha.= str_pad(trim($boleto->getCdfilial()).trim($boleto->getNrlanbolpag()),20,' ', STR_PAD_RIGHT );
        
        // CAMPO: Nº do Docto Atribuído pelo Banco
        // POSIÇÃO: 038 a 57
        // PICTURE: X(20)
        if($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getNrretcnabpag(),0,20)),20,' ',STR_PAD_RIGHT);
        } else {
            $linha.= str_repeat(' ', 20);
        }
        
        // CAMPO: Nome do Contribuinte
        // POSIÇÃO: 058 a 087
        // PICTURE: X(30)
        $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(), 0, 30)), 30,' ',STR_PAD_RIGHT);
        
        // CAMPO: Nome do Contribuinte
        // POSIÇÃO: 088 a 095
        // PICTURE: 9(8)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');
        
        // CAMPO: Valor do Total do Pagamento
        // POSIÇÃO: 096 a 230
        // PICTURE: 9(8)
        $linha .= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),15,STR_PAD_LEFT);
        
                
        // CAMPO: Informações Complementares de acordo com o respectivo tributo
        // POSIÇÃO: 111 a 230
        // PICTURE: X(120)
         if ($this->formaPgto == '16'){
            $linha = self::darfNormal($linha, $boleto);
         }elseif($this->formaPgto == '17'){
            $linha = self::GPS($linha, $boleto);
         } elseif ($this->formaPgto == '18'){
            $linha = self::darfSimples($linha, $boleto);
         } elseif ($this->formaPgto == '21'){
             $linha = self::DARJ($linha, $boleto);
         }elseif ($this->formaPgto == '22') {
            $linha = self::GARE($linha, $boleto);
         }
        
        // CAMPO: Códigos das Ocorrências p/ Retorno
        // POSIÇÃO: 231 a 240
        // PICTURE: X(10)
        $linha.= str_repeat(' ', 10);
        
        
        return $linha . "\r\n";
    }
    
    
    /**
     * @param Boletpagar $boleto
     * @return string
     * @throws Exception
     * @internal param $cdbanco
     */
    private function darfSimples($linha, $boleto){
        
        // CAMPO: Código da Receita do Tributo
        // POSIÇÃO: 111 a 116
        // PICTURE: X(6)
        $cdreceitaguia = $boleto->getCdreceitaguia();
        if ($cdreceitaguia){
             $linha.= str_pad($cdreceitaguia,6,'0',STR_PAD_LEFT);
        } else { 
            $linha.= str_repeat('0', 6);
        }
        
        // CAMPO: Tipo de Identificação do Contribuinte
        // POSIÇÃO: 117 a 118
        // PICTURE: 9(2)
        if ($this->empresa->getIdtpijurempr() == 'F'){
            $linha.= '01'; //CPF
        } else{
            $linha.= '02'; //CNPJ
        }
        
        // CAMPO: Identificação do Contribuinte
        // POSIÇÃO: 119 a 132
        // PICTURE: 9(14)
        $linha.= str_pad(substr($this->inscricaojuridica,0,14),14,'0', STR_PAD_LEFT);
        
        // CAMPO: Código de Identificação do Tributo
        // POSIÇÃO: 133 a 134
        // PICTURE: X(2)
        $linha.= $this->formaPgto;
        
        // CAMPO: Período de Apuração
        // POSIÇÃO: 135 a 142
        // PICTURE: 9(8)
        if ($boleto->getDtapuracguia()){
            $linha.= $boleto->getDtapuracguia()->format('dmY');
        } else {
            $linha.= str_repeat('0', 8);
        }
        
        // CAMPO: Valor da Receita Bruta Acumulada
        // POSIÇÃO: 143 a 157
        // PICTURE: 9(15)V(2)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVroutrentguia(),2)),15,'0',STR_PAD_LEFT);
        
        // CAMPO: Percentual sobre a Receita Bruta Acumulada
        // POSIÇÃO: 158 a 164
        // PICTURE: 9(5)V(2)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrpercenttrib(),2)),7,'0',STR_PAD_LEFT);
        
        // CAMPO: Valor Principal
        // POSIÇÃO: 165 a 179
        // PICTURE: 9(13)V(2)
        if($boleto->getVrpercenttrib() != 0 && $boleto->getVroutrentguia() != 0 ){
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVroutrentguia() * $boleto->getVrpercenttrib(),2)),15,STR_PAD_LEFT);
        } else {
             $linha .= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),15,STR_PAD_LEFT);
        }
        // CAMPO: Valor da Multa
        // POSIÇÃO: 180 a 194
        // PICTURE: 9(13)V(2)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmultabpag(),2)),15,'0',STR_PAD_LEFT);
        
        // CAMPO: Valor dos Juros / Encargos
        // POSIÇÃO: 195 a 209
        // PICTURE: 9(13)V(2)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmordiabpag(),2)),15,'0',STR_PAD_LEFT);
        
        // CAMPO: Uso Exclusivo FEBRABAN/CNAB
        // POSIÇÃO: 210 a 230
        // PICTURE: X(21)
        $linha = str_pad($linha, 230);
        return $linha;
    
    }
    
    /**
     * @param Boletpagar $boleto
     * @return string
     * @throws Exception
     * @internal param $cdbanco
     */
    private function darfNormal($linha, $boleto){
        
        // CAMPO: Código da Receita do Tributo
        // POSIÇÃO: 111 a 116
        // PICTURE: X(6)
        $cdreceitaguia = $boleto->getCdreceitaguia();
        if ($cdreceitaguia){
             $linha.= str_pad($cdreceitaguia,6,'0',STR_PAD_LEFT);
        } else { 
            $linha.= str_repeat('0', 6);
        }
        
        // CAMPO: Tipo de Identificação do Contribuinte
        // POSIÇÃO: 117 a 118
        // PICTURE: 9(2)
        if ($this->empresa->getIdtpijurempr() == 'F'){
            $linha.= '01'; //CPF
        } else{
            $linha.= '02'; //CNPJ
        }
        
        // CAMPO: Identificação do Contribuinte
        // POSIÇÃO: 119 a 132
        // PICTURE: 9(14)
        $linha.= str_pad(substr($this->inscricaojuridica,0,14),14,'0', STR_PAD_LEFT);
        
        // CAMPO: Código de Identificação do Tributo
        // POSIÇÃO: 133 a 134
        // PICTURE: X(2)
        $linha.= $this->formaPgto;
        
        // CAMPO: Período de Apuração
        // POSIÇÃO: 135 a 142
        // PICTURE: 9(8)
        if ($boleto->getDtapuracguia()){
            $linha.= $boleto->getDtapuracguia()->format('dmY');
        } else {
            $linha.= str_repeat('0', 8);
        }
        
        // CAMPO: Número de Referência
        // POSIÇÃO: 143 a 159
        // PICTURE: 9(17)
        $nrrefeguia = $boleto->getNrrefeguia();
        if (!$nrrefeguia){
            $linha.= str_pad($nrrefeguia,17,'0',STR_PAD_LEFT);
        } else {
            $linha.= str_repeat('0', 17);
        }
        
        // CAMPO: Valor Principal
        // POSIÇÃO: 160 a 174
        // PICTURE: 9(13)V(2)
        $linha.= str_pad(str_replace(array('.',','),'',number_format($boleto->getVrfinalbpag(),2)),15,'0',STR_PAD_LEFT);
        
        // CAMPO: Valor da Multa
        // POSIÇÃO: 175 a 189
        // PICTURE: 9(13)V(2)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmultabpag(),2)),15,'0',STR_PAD_LEFT);
        
        // CAMPO: Valor dos Juros / Encargos
        // POSIÇÃO: 190 a 204
        // PICTURE: 9(13)V(2)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmordiabpag(),2)),15,'0',STR_PAD_LEFT);
        
        // CAMPO: Data de Vencimento
        // POSIÇÃO: 205 a 212
        // PICTURE: 9(8)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');
        
        // CAMPO: Uso Exclusivo FEBRABAN/CNAB
        // POSIÇÃO: 213 a 240
        // PICTURE: X(18)
        $linha = str_pad($linha, 230);

        return $linha;
    
    }
    
    /**
     * @param Boletpagar $boleto
     * @return string
     * @throws Exception
     * @internal param $cdbanco
     */
     private function GPS($linha, $boleto){
        
        // CAMPO: Código da Receita do Tributo
        // POSIÇÃO: 111 a 116
        // PICTURE: X(6)
        $cdreceitaguia = $boleto->getCdreceitaguia();
        if ($cdreceitaguia){
             $linha.= str_pad($cdreceitaguia,6,'0',STR_PAD_LEFT);
        } else { 
            $linha.= str_repeat(' ', 6);
        }
        
                
        // CAMPO: Tipo de Identificação do Contribuinte
        // POSIÇÃO: 117 a 118
        // PICTURE: 9(2)
        if ($this->empresa->getIdtpijurempr() == 'F'){
            $linha.= '01'; //CPF
        } else{
            $linha.= '02'; //CNPJ
        }
        
        // CAMPO: Identificação do Contribuinte
        // POSIÇÃO: 119 a 132
        // PICTURE: 9(14)
        $linha.= str_pad(substr($this->inscricaojuridica,0,14),14,'0', STR_PAD_LEFT);
        
        // CAMPO: Código de Identificação do Tributo
        // POSIÇÃO: 133 a 134
        // PICTURE: X(2)
        $linha.= $this->formaPgto;
        
        // CAMPO: Mês e ano de competência
        // POSIÇÃO: 135 a 140
        // PICTURE: 9(6)
        if ($boleto->getDtapuracguia()){
            $linha.= $boleto->getDtapuracguia()->format('dmy');
        } else {
            $linha.= str_repeat('0', 6);
        }
        
        // CAMPO: Valor previsto do pagamento do INSS
        // POSIÇÃO: 141 a 155
        // PICTURE: 9(13)V(2)
         $vrprevisto = $boleto->getVrfinalbpag() - ($boleto->getVroutrentguia() + $boleto->getVrmultabpag() + $boleto->getVrmordiabpag());
        $linha.= str_pad(str_replace(array('.',','),'', number_format($vrprevisto,2)),15,'0',STR_PAD_LEFT);
        
        
        // CAMPO: Valor de Outras Entidades
        // POSIÇÃO: 156 a 170
        // PICTURE: 9(13)V(2)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVroutrentguia(),2)),15,'0',STR_PAD_LEFT);
        
        // CAMPO: Atualização Monetária
        // POSIÇÃO: 171 a 185
        // PICTURE: 9(13)V(2)
        $vratualizacao = $boleto->getVrmultabpag() + $boleto->getVrmordiabpag();
        $linha.= str_pad(str_replace(array('.',','),'', number_format($vratualizacao,2)),15,'0',STR_PAD_LEFT);
        
        // CAMPO: Uso Exclusivo FEBRABAN/CNAB
        // POSIÇÃO: 186 a 230
        // PICTURE: X(45)
        $linha = str_pad($linha, 230);
        
        return $linha;
    }
    
    /**
     * @param Boletpagar $boleto
     * @return string
     * @throws Exception
     * @internal param $cdbanco
     */
     private function GARE($linha, $boleto){
         // CAMPO: Código da Receita do Tributo
         // POSIÇÃO: 111 a 116
         // PICTURE: X(6)
         $cdreceitaguia = $boleto->getCdreceitaguia();
         if ($cdreceitaguia){
             $linha.= str_pad($cdreceitaguia,6,'0',STR_PAD_LEFT);
         } else { 
            $linha.= str_repeat('0', 6);
         }
         
         // CAMPO: Tipo de Identificação do Contribuinte
         // POSIÇÃO: 117 a 118
         // PICTURE: 9(2)
         if ($this->empresa->getIdtpijurempr() == 'F'){
             $linha.= '01'; //CPF
         } else {
            $linha.= '02'; //CNPJ
         }
         
         // CAMPO: Identificação do Contribuinte
         // POSIÇÃO: 119 a 132
         // PICTURE: 9(14)
         $linha.= str_pad(substr($this->inscricaojuridica,0,14),14,'0', STR_PAD_LEFT);
         
         // CAMPO: Código de Identificação do Tributo
         // POSIÇÃO: 133 a 134
         // PICTURE: X(2)
         $linha.= $this->formaPgto;
         
         // CAMPO: Data de Vencimento
         // POSIÇÃO: 135 a 142
         // PICTURE: 9(8)
         $linha.= $boleto->getDtvenbolpag()->format('dmY');
         
         // CAMPO: Inscrição Estadual / Código do Município / Número Declaração
         // POSIÇÃO: 143 a 154
         // PICTURE: 9(12)
         $empresa = empresa::findOneByNrorgAndCdempresa($this->nrorg, $this->remessa->getCdempresa());
         $linha.= str_pad($empresa->getCdinscesta(),12, '0', STR_PAD_LEFT);
        
         // CAMPO: Dívida Ativa / N. Etiqueta
         // POSIÇÃO: 155 a 167
         // PICTURE: 9(13)
        $nrrefeguia = $boleto->getNrrefeguia();
        if (!$nrrefeguia){
            $linha.= str_pad($nrrefeguia,13,'0',STR_PAD_LEFT);
        } else {
            $linha.= str_repeat('0', 13);
        }
         // CAMPO: Período de Referência
         // POSIÇÃO: 168 a 173
         // PICTURE: 9(6)
         if ($boleto->getDtapuracguia()){
            $linha.= $boleto->getDtapuracguia()->format('dmy');
         } else {
            $linha.= str_repeat('0', 6);
         }
         
         // CAMPO: Número da Parcela / Notificação
         // POSIÇÃO: 174 a 186
         // PICTURE: 9(13)
         $linha.= str_repeat('0', 13);
         
         // CAMPO: Valor da Receita
         // POSIÇÃO: 187 a 201
         // PICTURE: 9(13)V(2)
         $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVroutrentguia(),2)),15,'0',STR_PAD_LEFT);
         
         // CAMPO: Valor dos Juros / Encargos
         // POSIÇÃO: 202 a 215
         // PICTURE: 9(12)V(2)
          $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmordiabpag(),2)),14,'0',STR_PAD_LEFT);
         
         // CAMPO: Valor da Multa
         // POSIÇÃO: 216 a 229
         // PICTURE: 9(12)V(2)
         $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmultabpag(),2)),14,'0',STR_PAD_LEFT);
         
         // CAMPO: Uso Exclusivo FEBRABAN/CNAB
         // POSIÇÃO: 230 a 230
         // PICTURE: X(1)
         $linha.= str_repeat(' ', 1);
         
        return $linha;
    }
    
     /**
     * @param Boletpagar $boleto
     * @return string
     * @throws Exception
     * @internal param $cdbanco
     */
     private function DARJ($linha, $boleto){
         
        // CAMPO: Código da Receita do Tributo
        // POSIÇÃO: 111 a 116
        // PICTURE: X(6)
        $cdreceitaguia = $boleto->getCdreceitaguia();
        if ($cdreceitaguia){
             $linha.= str_pad($cdreceitaguia,6,'0',STR_PAD_LEFT);
        } else { 
            $linha.= str_repeat('0', 6);
        }
        
        // CAMPO: Tipo de Identificação do Contribuinte
        // POSIÇÃO: 117 a 118
        // PICTURE: 9(2)
        if ($this->empresa->getIdtpijurempr() == 'F'){
            $linha.= '01'; 
        }
        else{
            $linha.= '02'; 
        }
        
        // CAMPO: Identificação do Contribuinte
        // POSIÇÃO: 119 a 132
        // PICTURE: X(14)
        $linha.= str_pad(substr($this->inscricaojuridica,0,14),14,'0', STR_PAD_LEFT);
        
        // CAMPO: Inscrição Estadual / Código do Município / Número Declaração
        // POSIÇÃO: 133 a 140
        // PICTURE: X(8)
        $empresa = empresa::findOneByNrorgAndCdempresa($this->nrorg, $this->remessa->getCdempresa());
        $linha.= str_pad($empresa->getCdinscesta(),8, '0', STR_PAD_LEFT);
        
        
        // CAMPO: Número do Documento Origem
        // POSIÇÃO: 141 a 156
        // PICTURE: 9(16)
        $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(), $boleto->getCdfilial(), $boleto->getNrorg());
        $linha.= str_pad(trim($contaPagar->getCdfilibpag().$boleto->getNrlanbolpag()),16,'0',STR_PAD_LEFT);
        
        // CAMPO: Valor Principal
        // POSIÇÃO: 157 a 171
        // PICTURE: 9(13)V(2)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag() - $boleto->getVrmordiabpag() + $boleto->getVrmultabpag(),2)),15,STR_PAD_LEFT);
        
        // CAMPO: Valor da Atualização Monetária
        // POSIÇÃO: 172 a 186
        // PICTURE: 9(13)V(2)
        $linha.= str_repeat('0', 15);
        
        // CAMPO: Valor da Mora
        // POSIÇÃO: 187 a 201
        // PICTURE: 9(13)V(2)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmordiabpag(),2)),15,'0',STR_PAD_LEFT);
        
        // CAMPO: Valor da Multa
        // POSIÇÃO: 202 a 216
        // PICTURE: 9(13)V(2)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmultabpag(),2)),15,'0',STR_PAD_LEFT);
        
        // CAMPO: Data de Vencimento
        // POSIÇÃO: 217 a 224
        // PICTURE: 9(8)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');
        
        // CAMPO: Período de Referência ou número da parcela
        // POSIÇÃO: 225 a 230
        // PICTURE: 9(6)
        if ($boleto->getDtapuracguia() != 0) {
            $linha.= $boleto->getDtvenbolpag()->format('mY');
        }
        else {
            $linha.= str_repeat('0', 6);
        }
        return $linha;
    
    }
    
    /**
     * Função Responsável por gerar a linha do trailler de lote da remessa
     */
    public function geraTraillerLote($somavalores = null)
    {
        $linha = '';
        $this->qtdRegiLote++; //Qtde de registro do lote
        $this->qtdRegiArq++; //Quantidade de registro do arquivo
        
        // CAMPO: Código do Banco na Compensação
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        $linha .= $this->cdbanco;
        
        // CAMPO: Lote de Serviço
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        $linha.= str_pad(($this->num_lote), 4,'0',STR_PAD_LEFT);
        
        // CAMPO: Tipo de Registro
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        // DEFAULT:  5
        $linha .= '5';
        
        // CAMPO: Uso Exclusivo FEBRABAN/CNAB
        // POSIÇÃO: 009 a 017
        // PICTURE: X(1)
        // DEFAULT:  Brancos
        $linha.= str_repeat(' ', 9);
        
        // CAMPO: Quantidade de Registros do Lote
        // POSIÇÃO: 018 a 023
        // PICTURE: 9(6)
        $linha.= str_pad($this->qtdRegiLote, 6, '0', STR_PAD_LEFT);
        
        // CAMPO: Somatória dos Valores
        // POSIÇÃO: 024 a 041
        // PICTURE: 9(16) V(2)
        $linha.= str_pad(str_replace(array('.',','),'',number_format($this->vrPagtLote,2)),18,'0',STR_PAD_LEFT);
        
        // CAMPO: Somatória de Quantidade de Moedas
        // POSIÇÃO: 042 a 059
        // PICTURE: 9(13) V(5)
        $linha.= str_repeat('0', 18);
        
        // CAMPO: Número Aviso de Débito
        // POSIÇÃO: 060 a 065
        // PICTURE: 9(6)
        $linha.= str_repeat('0', 6);  
        
        // CAMPO: Uso Exclusivo FEBRABAN/CNAB
        // POSIÇÃO: 066 a 230
        // PICTURE: X(165)
        // DEFAULT:  Brancos
        $linha.= str_repeat(' ', 165);
        
        // CAMPO: Códigos das Ocorrências para Retorno
        // POSIÇÃO: 231 a 240
        // PICTURE: X(10)
        $linha.= str_repeat(' ', 10);
        
        return $linha."\r\n";

    }
    
     /**
      * Função Responsável por gerar a linha do trailler de arquivo da remessa
     */
    public function geraTrailerArquivo()
    {
        
        // CAMPO: Código do Banco na Compensação
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        $linha = $this->cdbanco;
        
        // CAMPO: Lote de Serviço
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        // DEFAULT: 9999
        $linha .= '9999';
        
        // CAMPO: Tipo de Registro
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        // DEFAULT:  9
        $linha .= '9';
        
        // CAMPO: Uso Exclusivo FEBRABAN / CNAB
        // POSIÇÃO: 009 a 017
        // PICTURE: X(9)
        // DEFAULT:  Brancos
        $linha .= str_repeat(' ', 9);
        
        // CAMPO: Quantidade de Lotes do Arquivo
        // POSIÇÃO: 018 a 023
        // PICTURE: 9(6)
        $linha.= str_pad($this->num_lote, 6,'0',STR_PAD_LEFT);
        
        // CAMPO: Quantidade de Registros do Arquivo
        // POSIÇÃO: 024 a 029
        // PICTURE: 9(6)
        $linha.= str_pad($this->qtdRegiArq, 6,'0',STR_PAD_LEFT);
        
        // CAMPO: Qtde de Contas p/ Conc. (Lotes)
        // POSIÇÃO: 030 a 035
        // PICTURE: 9(6)
        $linha.= str_repeat('0', 6);
        
        // CAMPO: Uso Exclusivo FEBRABAN / CNAB
        // POSIÇÃO: 036 a 240
        // PICTURE: X(205)
        // DEFAULT:  Brancos
        $linha.= str_repeat(' ', 205);
        
        return $linha."\r\n";
    }
    
    /**
     * @param $valor - valor a ser alterado
     * @param $tamanho - tamanho total da string desejado
     * @return string
     */
    private function preenche_brancos($valor, $tamanho){
        return str_pad($valor, $tamanho);
    }
}