<?php
namespace Teknisa\FIN\Modules\ContaPagar\Services\Remessa\Banco;

use Exception;
use Teknisa\Model\Financeiro\Entities\Cnabpag;
use Fiscal\Entities\Boletpagar;
use Geral\Entities\Banco;
use Geral\Entities\Empresa;
use Geral\Entities\Fornecedor;
use Geral\Entities\Municipio;
use Geral\Entities\Filial;
use Geral\Entities\Contcorr;
use Geral\Entities\Tipobaixapag;
use Teknisa\Libs\Util\DateUtil;
use Fiscal\Entities\Contapagar;
use Financeiro\Util\Util;
use Teknisa\FIN\Modules\ContaPagar\Services\Remessa\GeraCnabService;

/**
 * Class GeraCnabSafraService
 * @package Teknisa\FIN\Modules\ContaReceber\Services\Remessa\Banco
 */
class GeraCnabSafraService extends GeraCnabService{
  
  
  CONST REMESSA = '1';
  CONST RETORNO = '2';
  CONST TIPO_INS_JUR_CPF = 1;
  CONST TIPO_INS_JUR_CNPJ = 2;
  CONST ID_TP_ARQUIVO_HEADER_ARQUIVO = '0';
  CONST NOME_BANCO = 'BANCO SAFRA S/A';
  CONST CDBANCO = '422';
  CONST MOEDA = 'REAL';
  CONST ID_TP_ARQUIVO_TRAILER = '9';
  
  //Quantidade de registro do arquivo
  private $qtdRegiArq = 0;
  //Somatório dos valores de pagamentos
  private $valorTotalPgto = 0;
  
  public function __construct(Cnabpag $remessa){
    try {
        $this->nrorg = $remessa->getNrorg();

        $this->remessa = $remessa;

        // Carrega os atributos Filial e Empresa
        $this->filial = Filial::findOneByCdfilialAndNrorg($remessa->getCdfilial(), $remessa->getNrorg());
        $this->empresa = Empresa::findOneByCdempresaAndNrorg($remessa->getCdempresa(), $remessa->getNrorg());

        // Busca dados referente a filial se foi informada
        if ($remessa->getCdfilial()) {
            if ($this->filial->getIdtpijurfili() == Filial::TIPO_INSCRICAO_FISICA)
                $this->tipoinscricao = self::TIPO_INS_JUR_CPF;
            else
                $this->tipoinscricao = self::TIPO_INS_JUR_CNPJ;

            $this->inscricaojuridica = $this->filial->getNrinsjurfili();
            $this->nomeempresa = $this->filial->getNmfilial();
        } // Se nao busca dados da Empresa
        else {
            if ($this->empresa->getIdtpijurempr() == Empresa::TIPO_INSCRICAO_FISICA)
                $this->tipoinscricao = self::TIPO_INS_JUR_CPF;
            else
                $this->tipoinscricao = self::TIPO_INS_JUR_CNPJ;

            $this->inscricaojuridica = $this->empresa->getNrinsjurempr();
            $this->nomeempresa = $this->empresa->getNmempresa();
        }
        /** @var contcorr Contcorr */
        $this->contcorr = Contcorr::findOneByCdreducontAndNrorg($remessa->getCdreducont(), $remessa->getNrorg());
    } catch (\Exception $e) {
        throw $e;
    }
  }
    
   /**
   * @param $nrlanbolpag
   * @param $cdfilibpag
   * @param $nrorg
   * @return Contapagar
   */
  private function getContaPagar($nrlanbolpag, $cdfilibpag, $nrorg){
      /** @var $contaPagar Contapagar */
      $contaPagar = Contapagar::findOneByNrlanbolpagAndCdfilibpagAndNrorg($nrlanbolpag, $cdfilibpag, $nrorg);
      return $contaPagar;
  }
  
  /** Retorna Conta Corrente sem a ultima posição (dígito)
   * @return string
   */
  protected function getContaCorrenteSemDigito(){
      $contacorrente = $this->contcorr->getCdcontcorr();
      return substr($contacorrente,0,-1);
  }
  
   /**
   * @param $valor - valor a ser alterado
   * @param $tamanho - tamanho total da string desejado
   * @return string
   */
  private function preenche_brancos($valor, $tamanho){
      return str_pad($valor, $tamanho);
  }
  
   /**
   * retorna a data atual no formato ddmmYYYY
   * @return string
   */
  private function getDataGeracao(){
      $dataatual = DateUtil::getDataAtual();
      
      return $dataatual->format('d').$dataatual->format('m').$dataatual->format('y');
  }
    
  /**
   * @param $caminhoArquivo
   * @param $boletos
   * @throws \Exception
   * @internal param $remessa
   * @return mixed
   */
   public function geraArquivo($caminhoArquivo, $boletos, $autorizaAgendamento = false, $idopecnabpag = null){
        try{
            $file = fopen($caminhoArquivo.'.REM', "a+");
            $headerarquivo = self::geraHeaderArquivo();
            fprintf($file, "%s",$headerarquivo);
            foreach($boletos as $boleto){
                $boleto = Boletpagar::findOneByNrorgAndCdfilialAndNrlanbolpag($boleto['NRORG'], $boleto['CDFILIAL'], $boleto['NRLANBOLPAG']);
                $this->valorTotalPgto = $this->valorTotalPgto + $boleto->getVrfinalbpag();
                $idbaixautilp = $boleto->getTpbaixapag()->getIdbaixautilp();
                if($idbaixautilp == 'B'){
                  $detalhePgtoBoleto = self::geraDetalhePgtoBoletoBancario($boleto);
                  fprintf($file, "%s",$detalhePgtoBoleto);
                } else {
                  $detalhePgtoDocTedCredConta = self::geraDetalhePgtoDocTedCredConta($boleto);
                  fprintf($file, "%s",$detalhePgtoDocTedCredConta);
                }
            }
            $trailler = self::geraTrailerArquivo();
            fprintf($file, "%s",$trailler);
            fclose($file);
        }
        catch (\Exception $e){
            fclose($file);
            throw $e;
        }
    }
   
   
   //#################################################################//
   //############### REGISTRO HEADER DE ARQUIVO ######################//
   //#################################################################//
   
    public function geraHeaderArquivo()
    {
        try {
          
          $linha = '';
          $this->qtdRegiArq++;
         // CAMPO: Identificação do registro header
         // POSIÇÃO: 1 a 1
         // PICTURE: 9(01)
         // CONTEÚDO: Obrigatório - fixo "zero" (0)
         
         $linha.= self::ID_TP_ARQUIVO_HEADER_ARQUIVO;
         
         // CAMPO: Identificação arquivo remessa
         // POSIÇÃO: 2 a 2
         // PICTURE: 9(01)
         // CONTEÚDO: Obrigatório - fixo (1)
         
         $linha.= self::REMESSA;

         // CAMPO: Identificação do arquivo remessa p/ extenso
         // POSIÇÃO: 3 a 9
         // PICTURE: X(07)
         // CONTEÚDO: Obrigatório - fixo "REMESSA"
         
         $linha.= 'REMESSA';
         
         // CAMPO: Identificação do serviço
         // POSIÇÃO: 10 a 11
         // PICTURE: 9(02)
         // CONTEÚDO: Obrigatório - fixo (11)
         
         $linha.= '11';
         
         // CAMPO: Identificação do serviço p/ extenso
         // POSIÇÃO: 12 a 26
         // PICTURE: X(15)
         // CONTEÚDO: Obrigatório - fixo "PAGTOS FORNECED"
         
         $linha.= 'PAGTOS FORNECED';
         
         // CAMPO: Identificação da empresa no banco
         // POSIÇÃO: 27 a 34
         // PICTURE: 9(08)
         // CONTEÚDO: Número da Conta
         
         $cCorrente = self::getContaCorrenteSemDigito();
         $digito = self::getDigitoContaCorrente();
         
         $cCorrente.= $digito;
         $linha.= str_pad($cCorrente,8,'0',STR_PAD_LEFT);
         
         // CAMPO: Validação do Trailler
         // POSIÇÃO: 35 a 35
         // PICTURE: X(01)
         // CONTEÚDO: Opcional - fixo "N" ou "S"
         
         $linha.= 'N';
         
         // CAMPO: Brancos
         // POSIÇÃO: 36 a 37
         // PICTURE: X(02)
         
         $linha.= str_repeat(' ', 2);
         
         // CAMPO: Código da agência do cliente
         // POSIÇÃO: 38 a 44
         // PICTURE: 9(07)
         
         $banco = Banco::findOneByCdbancoAndNrorg($this->contcorr->getCdbanco(), $this->nrorg);
         $digitoAgencia   = (strpos($banco->getCdpictagen(), "-") > -1)?true:false;
         
         if($digitoAgencia){
            $linha.= str_pad($this->contcorr->getCdagencia(), 7, '0', STR_PAD_LEFT);
         } else {
            $linha.= str_pad($this->contcorr->getCdagencia(), 7, '0', STR_PAD_LEFT);
         }
         
         // CAMPO: Brancos
         // POSIÇÃO: 45 a 46
         // PICTURE: X(02)
         
         $linha.= str_repeat(' ', 2);
         
         // CAMPO: Nome do cliente
         // POSIÇÃO: 47 a 76
         // PICTURE: X(30)
         
         $linha.= self::preenche_brancos(substr(trim(Util::campoCnabValido($this->nomeempresa)),0,30),30);
         
         // CAMPO: Código do banco
         // POSIÇÃO: 77 a 79
         // PICTURE: 9(03)
         // CONTEÚDO: Obrigatório - fixo "422"
         
         $linha.= self::CDBANCO;
         
         // CAMPO: Nome do banco
         // POSIÇÃO: 80 a 94
         // PICTURE: X(15)
         // CONTEÚDO: Obrigatório - fixo "BANCO SAFRA S/A"
         
         $linha.= self::NOME_BANCO;
         
         // CAMPO: Data da gravação do arquivo remessa
         // POSIÇÃO: 95 a 100
         // PICTURE: 9(06)
         // CONTEÚDO: Data gravação(ddmmaa)
         
         $linha.=  self::getDataGeracao();
         
         // CAMPO: Identif. da origem do arquivo
         // POSIÇÃO: 101 a 101
         // PICTURE: 9(01)
         // CONTEÚDO: Obrigatório - fixo "zero"
         
         $linha.= '0';
         
         // CAMPO: Identif. da origem do terceiro
         // POSIÇÃO: 102 a 103
         // PICTURE: 9(02)
         // CONTEÚDO: Obrigatório - fixo "zero"
         
         $linha.= '00';
         
         // CAMPO: Brancos
         // POSIÇÃO: 104 a 387
         // PICTURE: X(284)
         
         $linha.= str_repeat(' ', 284);
         
         // CAMPO: Valida CNPJ/CPF no registro detalhe
         // POSIÇÃO: 388 a 388
         // PICTURE: X(1)
         
         $linha.= 'S';
         
         // CAMPO: Número Seqüencial
         // POSIÇÃO: 389 a 394
         // PICTURE: 9(6)
         
         $linha.= substr($this->remessa->getNrseqremessapag(),-6);

         // CAMPO: Nº sequenc. de registro de arquivo
         // POSIÇÃO: 395 a 400
         // PICTURE: 9(06)
         // CONTEÚDO: Obrigatório - fixo (000001)

          $linha.= str_pad($this->qtdRegiArq,6,'0',STR_PAD_LEFT);
         
          return $linha."\r\n";
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $cdbanco
     * @param $boletos
     * @throws \Exception
     * @internal param $formaLancamento
     * @return string
     */
    public function geraHeaderLote($cdbanco, $boletos = null){
    }
    
    /**
     * @internal param $cdbanco
     * @throws Exception
     * @return string
     * @internal param $caminho
     * @internal param $nrorg
     */
    public function geraTraillerLote($somavalores = null){

    }

    //#################################################################//
    //#################  Regitro Trailler de arquivo  #################//
    //#################################################################//
    
    /**
     * @return string
     * @throws \Exception
     */
    public function geraTrailerArquivo(){
      try { 
        
        $linha = '';
        $this->qtdRegiArq++;
        
        // CAMPO: Identificação do registro
        // POSIÇÃO: 1 a 1
        // PICTURE: 9(1)
        // CONTEÚDO: FIXO "9"
        
         $linha.= '9';
         
         // CAMPO: Brancos
         // POSIÇÃO: 2 a 124
         // PICTURE: X(123)
         
        $linha.= str_repeat(' ', 123);
        
        // CAMPO: Somatório do campo 'Valor' dos compromissos
        // POSIÇÃO: 125 a 139
        // PICTURE: 9(15)
        // CONTEÚDO: Obrigatório - fixo "zeros"
        
        $linha.= str_pad(str_replace(array('.',','),'', number_format($this->valorTotalPgto,2)),15,'0',STR_PAD_LEFT);

        // CAMPO: Brancos
        // POSIÇÃO: 140 a 248
        // PICTURE: X(109)
       
        $linha.= str_repeat(' ', 109);
        
        // CAMPO: Somatório do campo 'Abatimento' dos compromissos
        // POSIÇÃO: 249 a 263
        // PICTURE: 9(15)
        // CONTEÚDO: Obrigatório - fixo "zeros"
        
        $linha.= str_repeat('0',15);
        
        // CAMPO: Brancos
        // POSIÇÃO: 264 a 345
        // PICTURE: X(82)
        
        $linha.= str_repeat(' ', 82);
        
        // CAMPO: Somatório do campo 'Juros de mora multa' dos compromissos
        // POSIÇÃO: 346 a 360
        // PICTURE: 9(15)
        // CONTEÚDO: Obrigatório - fixo "zeros"
        
        $linha.= str_repeat('0', 15);
        
        // CAMPO: Brancos
        // POSIÇÃO: 361 a 364
        // PICTURE: X(04)
        
        $linha.= str_repeat(' ', 4);
        
        // CAMPO: Somatório do campo 'Valor autorizado p/ pagamento' dos compromissos
        // POSIÇÃO: 365 a 379
        // PICTURE: 9(15)
        // CONTEÚDO: Obrigatório - fixo "zeros"

        $linha.= str_pad(str_replace(array('.',','),'', number_format($this->valorTotalPgto,2)),15,'0',STR_PAD_LEFT);
        
        // CAMPO: Brancos
        // POSIÇÃO: 380 a 394
        // PICTURE: X(15)
        
        $linha.= str_repeat(' ', 15);
                
        // CAMPO: Nº sequencial de registro
        // POSIÇÃO: 395 a 400
        // PICTURE: 9(06)
        
        $linha.= str_pad($this->qtdRegiArq,6,'0',STR_PAD_LEFT);

        return $linha."\r\n";      
        
    } catch (\Exception $e) {
        throw $e;
    }
  }
  
  
    //####################################################################################################//
    //################  REGISTRO DETALHE DE PAGAMENTO DE DOC / TED / CRÉDITO EM CONTA  ###################//
    //####################################################################################################//
    
    public function geraDetalhePgtoDocTedCredConta(Boletpagar $boleto){
      
      $linha = '';
      $tipoInsc = '';
      $nrinsjur = '';
      $this->qtdRegiArq++;
      
      // CAMPO: Identificação registro compromisso
      // Posição: 1 a 1
      // Picture: 9(1)
      // Conteúdo: FIXO "1"
      
      $linha .= '1';
      
      /**
       * Tipo da Incrição empresa
       * Caso tenha Filial informadam passa o tipo de inscrição da Filial se não da empresa
      **/
      
      if($this->remessa->getCdfilial()){
        
       if ($this->filial->getIdtpijurfili() == 'F'){
             $tipoInsc .= '02'; //CPF
             $nrinsjurfili = trim($this->filial->getNrinsjurfili());
             $nrinsjur = str_pad($nrinsjurfili, 14, '0', STR_PAD_LEFT);
        }else{
            $tipoInsc .= '01'; //CNPJ
            $nrinsjur = trim($this->filial->getNrinsjurfili());
        }
      } else {
        if ($this->empresa->getIdtpijurempr() == 'F'){
            $tipoInsc .= '02'; //CPF
            $nrinsjuremp = trim($this->empresa->getNrinsjurempr());
            $nrinsjur = str_pad($nrinsjuremp, 14, '0', STR_PAD_LEFT);
            
        }else{
            $tipoInsc .= '01'; //CNPJ
            $nrinsjur = trim($this->empresa->getNrinsjurempr());
        }
      }
      
      // CAMPO :Tipo de incrição da empresa (01 = CNPJ / 02 = CPF)
      // Posição: 2 a 3
      // Picture: 9(02)
      // Conteúdo: 01 = CNPJ / 02 = CPF
      
      $linha .= $tipoInsc;
      
      // CAMPO: Número de inscrição
      // Posição: 4 a 17
      // Picture: 9(14)
      
      $linha .= $nrinsjur;  
      
      // CAMPO: Identificação da empresa no banco
      // Posição: 18 a 25
      // Picture: 9(08)
      // Conteúdo: Número da Conta
      
      $linha .= str_pad($this->contcorr->getCdcontcorr(),8,'0',STR_PAD_LEFT);
      
      // CAMPO: Brancos
      // Posição: 26 a 28
      // Picture: X(03)
      
      $linha.= str_repeat(' ', 3);
      
      // CAMPO: Código da agência
      // Posição: 29 a 35
      // Picture: 9(07)
      
      $linha.= str_pad($this->contcorr->getCdagencia(), 7, '0', STR_PAD_LEFT);

      // CAMPO: Brancos
      // Posição: 36 a 37
      // Picture: X(02)
      
      $linha.= str_repeat(' ', 2);
      
      // CAMPO: Uso exclusivo da empresa
      // Posição: 38 a 62
      // Picture: X(25)
      
      $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(), $boleto->getCdfilial(), $boleto->getNrorg());
      $linha.= str_pad(trim($contaPagar->getCdfilibpag().$boleto->getNrlanbolpag()),25,' ',STR_PAD_RIGHT);
      
      // CAMPO: Código de inscrição do fornecedor
      // Posição: 63 a 76
      // Picture: 9(14)
      
      if($boleto->getNrinscnabbop()){
        if(strlen(Util::campoCnabValido($boleto->getNrinscnabbop())) == 11){
        $linha.= str_pad(substr($boleto->getNrinscnabbop(),0,14),14,'0', STR_PAD_LEFT);
        } else{
          $linha.= $boleto->getNrinscnabbop();
        }
        
      } else {
        if(strlen(Util::campoCnabValido($boleto->getForneced()->getNrinsjurforn())) == 11){
          $linha.= str_pad(substr($boleto->getForneced()->getNrinsjurforn(),0,14),14,'0', STR_PAD_LEFT);
        } else {
          $linha.= $boleto->getForneced()->getNrinsjurforn();
        }
      }
      
      // CAMPO: Tipo de documento
      // POSIÇÃO: 77 a 79
      // PICTURE: X(03)
      // CONTEÚDO: Obrigatório - fixo "DUP"
      
      $linha.= 'DUP';
      
      // CAMPO: Nº do compromisso no banco
      // Posição: 80 a 89
      // Picture: X(10)
      
      $linha.= $boleto->getNrlanbolpag();
      
      // CAMPO: Sequencia de compromisso
      // POSIÇÃO: 90 a 90
      // PICTURE: X(01)
      
      $linha.= str_repeat(' ', 1);
      
      // CAMPO: Brancos
      // Posição: 91 a 106
      // Picture: X(16)
      
      $linha.= str_repeat(' ', 16);
      
      // CAMPO: Identificação do tipo de pessoa
      // POSIÇÃO: 107 A 107
      // PICTURE: X(01)
      // CONTEÚDO: “1” – p. Física, “2” - p. Jurídica
      
      if($boleto->getIdtpicnabbop()) {
            if ($boleto->getIdtpicnabbop() == 'F')
                $tipoInsc = '2'; //CPF
            else {
                $tipoInsc =  '1'; //CNPJ
            }
      } else {
          if ($boleto->getForneced()->getIdtpijurforn() == 'F')
              $tipoInsc = '2'; //CPF
          else {
              $tipoInsc = '1'; //CNPJ
          }
      }
      
      $linha.= $tipoInsc;
        
      // CAMPO: Código de operação
      // POSIÇÃO: 108 a 108
      // PICTURE: X(01)
      // CONTEÚDO: Obrigatório - fixo "C"
       
      $linha.= 'C';
      
      // CAMPO: Código da ocorrência
      // Posição: 109 a 110
      // Picture: 9(02)
      // CONTEÚDO: 01 – inclusão;   02 – alteração;
      //           03 – exclusão;   04 – autorização;
      //           05 – bloqueio;   06 – desbloqueio
      
      if ($this->remessa->getIdopecnabpag() == 'E'){
        $linha .= '03'; //Exclusão
      }elseif($this->remessa->getIdopecnabpag() == 'I'){
        $linha .= '01'; // Inclusão
      }elseif($this->remessa->getIdopecnabpag() == 'B'){
        $linha .= '05'; // Bloqueio
      }elseif($this->remessa->getIdopecnabpag() == 'A'){
        $linha .= '02'; // Autorização
      }elseif($this->remessa->getIdopecnabpag() == 'D'){
        $linha .= '06'; // Desbloqueio
      }
      
      // CAMPO: Nº compromisso p/ cliente
      // Posição: 111 a 120
      // Picture: X(10)
      
      $linha.= $boleto->getNrlanbolpag();
      
      // CAMPO: Data de vencimento
      // Posição: 121 a 126
      // Picture: 9(06)
      // Conteúdo: Vencimento(ddmmaa)
      
      $linha.= $boleto->getDtvenbolpag()->format('dmy');
      
      // CAMPO: Valor do pagto
      // Posição: 127 a 139
      // Picture: 9(11)V9(2)
      
      $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),13,'0',STR_PAD_LEFT);
      
      // CAMPO: Tipo de pagamento
      // POSIÇÃO: 140 a 142
      // PICTURE: X(03)
      // CONTEÚDO: Obrigatório  - Fixos: "DOC", "CC", "TED"
      $tipobaixapag = Tipobaixapag::findOneByNrorgAndCdtpbaixapag($boleto->getNrorg(), $boleto->getCdtpbaixapag());
      
      if($tipobaixapag->getIdbaixautilp() == "C"){
        $linha.= ' CC';
      } elseif($tipobaixapag->getIdbaixautilp() == "D"){
        $linha.= 'DOC';
      } elseif($tipobaixapag->getIdbaixautilp() == "T" || $tipobaixapag->getIdbaixautilp() == "S"){
        $linha.= 'TED';
      }
      
      // CAMPO: Banco destino
      // Posição: 143 a 145
      // Picture: 9(03)
      
      $linha.= $boleto->getCdbancbolp();
      
      // CAMPO: Agência destino
      // Posição: 146 a 152
      // Picture: 9(07)
      
      $linha.= str_pad($boleto->getCdagenbolp(), 7, '0', STR_PAD_LEFT);
      
      // CAMPO: Zeros
      // Posição: 153 a 155
      // Picture: 9(3)
      
      $linha.= str_repeat('0', 3);
      
      // CAMPO: Conta corrente destino
      // Posição: 156 a 165
      // Picture: 9(10)
      //Obs: A 2 últimas posições da C/C é res. para o Dig. Verif.
      
      $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),-10)),10,'0',STR_PAD_LEFT);
      
      // CAMPO: Nome agência destino
      // Posição: 166 a 185
      // Picture: X(20)
      
      $linha.= str_repeat(' ', 20);
      
      // CAMPO: Agência p/ pagamento
      // Posição: 186 a 192
      // Picture: 9(07)
      $linha .= str_pad($this->contcorr->getCdagencia(), 7, '0', STR_PAD_LEFT);
  
      // CAMPO: Brancos
      // Posição: 193 a 247
      // Picture: X(55)
      
      $linha.= str_repeat(' ', 55);
      
      // CAMPO: Zeros
      // Posição: 248 a 263
      // Picture: 9(16)
      
      $linha.= str_repeat('0', 16);
      
      // CAMPO: Nome do fornecedor
      // Posição: 264 a 293
      // Picture: X(30)
      
      $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,30)),30);
      
      // CAMPO: Brancos
      // Posição: 294 a 347
      // Picture: X(54)
      
      $linha.= str_repeat(' ', 54);
      
      // CAMPO: Zeros
      // Posição: 348 a 360
      // Picture: 9(13)
      
      $linha.= str_repeat('0', 13);
      
      // CAMPO: Data pagamento
      // Posição: 361 a 366
      // Picture: 9(06)
      // Conteúdo: Data pagamento(ddmmaa)
      
      $linha.= $boleto->getDtvenbolpag()->format('dmy');
      
      // CAMPO: Valor autorizado p/ pagto
      // Posição: 367 a 379
      // Picture: 9(11)V9(2)
      
      $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),13,'0',STR_PAD_LEFT);
       
      // CAMPO: Código da meoda
      // POSIÇÃO: 380 a 383
      // PICTURE: X(04)
      // CONTEÚDO: Obrigatório - fixo "REAL"
       
      $linha.= self::MOEDA;
       
      // CAMPO: Brancos
      // POSIÇÃO: 384 a 391
      // PICTURE: X(08)
       
      $linha.= str_repeat(' ', 8);
      
      // CAMPO: Digito da agência destino (opcional)
      // POSIÇÃO: 392 a 392
      // PICTURE: X(01)
      
      $linha.= ' ';
      
      // CAMPO: BRANCOS
      // POSIÇÃO: 393 a 394
      // PICTURE: X(02)
      
      $linha.= str_repeat(' ', 2);
      
      // CAMPO: Nº sequencial do registro
      // POSIÇÃO: 395 a 400
      // PICTURE: 9(06)
      
      $linha.= str_pad($this->qtdRegiArq,6,'0',STR_PAD_LEFT);

      return $linha."\r\n";
    }
    
    
    //#########################################################################################//
    //################  REGISTRO DETALHE DE PAGAMENTO DE BOLETO BANCÁRIO  #####################//
    //#########################################################################################//
    
    public function geraDetalhePgtoBoletoBancario(Boletpagar $boleto){
      
      $linha = '';
      $tipoInsc = '';
      $this->qtdRegiArq++;
      
      // CAMPO: Identificação registro compromisso
      // POSIÇÃO: 1 a 1
      // PICTURE: 9(1)
      // CONTEÚDO: FIXO "1"
      
      $linha.= '1';
      
      // Tipo da Incrição empresaF
      // POSIÇÃO: 2 a 3
      // PICTURE: 9(02)
      // CONTEÚDO: (01 = CNPJ / 02 = CPF)
      
      if($this->remessa->getCdfilial()){
        
       if ($this->filial->getIdtpijurfili() == 'F'){
             $tipoInsc .= '02'; //CPF
             $nrinsjurfili = trim($this->filial->getNrinsjurfili());
             $nrinsjur = str_pad($nrinsjurfili, 14, '0', STR_PAD_LEFT);
        }else{
            $tipoInsc .= '01'; //CNPJ
            $nrinsjur = trim($this->filial->getNrinsjurfili());
        }
      } else {
        if ($this->empresa->getIdtpijurempr() == 'F'){
            $tipoInsc .= '02'; //CPF
            $nrinsjuremp = trim($this->empresa->getNrinsjurempr());
            $nrinsjur = str_pad($nrinsjuremp, 14, '0', STR_PAD_LEFT);
            
        }else{
            $tipoInsc .= '01'; //CNPJ
            $nrinsjur = trim($this->empresa->getNrinsjurempr());
        }
      }
      
      $linha .= $tipoInsc;
      
      // CAMPO: Número de inscrição da empresa
      // POSIÇÃO: 4 a 17
      // PICTURE: 9(14)
      
      $linha .= $nrinsjur;
      
      
      // CAMPO: Identificação da empresa no banco
      // POSIÇÃO: 18 a 25
      // PICTURE: 9(08)
      // CONTEÚDO: Número da Conta
      
      $linha .= str_pad($this->contcorr->getCdcontcorr(),8,'0',STR_PAD_LEFT);
      
      // CAMPO: Brancos
      // POSIÇÃO: 26 a 28
      // PICTURE: X(03)
      
      $linha.= str_repeat(' ', 3);
      
      // CAMPO: Código da agência
      // POSIÇÃO: 29 a 35
      // PICTURE: 9(07)
      
      $linha.= str_pad($this->contcorr->getCdagencia(), 7, '0', STR_PAD_LEFT);

      // CAMPO: Brancos
      // POSIÇÃO: 36 a 37
      // PICTURE: X(02)
      
      $linha.= str_repeat(' ', 2);
      
      // CAMPO: Uso exclusivo da empresa
      // POSIÇÃO: 38 a 62
      // PICTURE: X(25)
      
      $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(), $boleto->getCdfilial(), $boleto->getNrorg());
      $linha.= str_pad(trim($contaPagar->getCdfilibpag().$boleto->getNrlanbolpag()),25,' ',STR_PAD_RIGHT);
      
      // CAMPO: Código de inscrição do fornecedor
      // Posição: 63 a 76
      // Picture: 9(14)
      
      if($boleto->getNrinscnabbop()){
        
        if(strlen(Util::campoCnabValido($boleto->getNrinscnabbop())) == 11){
          $linha.= str_pad(substr($boleto->getNrinscnabbop(),0,14),14,'0', STR_PAD_LEFT);
        } else{
          $linha.= $boleto->getNrinscnabbop();
        }
        
      } else {
        
        if(strlen(Util::campoCnabValido($boleto->getForneced()->getNrinsjurforn())) == 11){
          $linha.= str_pad(substr($boleto->getForneced()->getNrinsjurforn(),0,14),14,'0', STR_PAD_LEFT);
        } else {
          $linha.= $boleto->getForneced()->getNrinsjurforn();
        }
      }
      
      // CAMPO: Tipo de documento
      // POSIÇÃO: 77 a 79
      // PICTURE: X(03)
      // CONTEÚDO: Obrigatório - fixo "DUP"
      
      $linha.= 'DUP';
      
      // CAMPO: Nº do compromisso
      // POSIÇÃO: 80 a 89
      // PICTURE: X(10)
      
      $linha.= $boleto->getNrlanbolpag();
      
      // CAMPO: Sequencia de compromisso
      // POSIÇÃO: 90 a 90
      // PICTURE: X(01)
      
      $linha.= str_repeat('0', 1);
      
      // CAMPO: Brancos
      // POSIÇÃO: 91 a 107
      // PICTURE: X(17)
      
      $linha.= str_repeat(' ', 17);

      // CAMPO: Código de operação
      // POSIÇÃO: 108 a 108
      // PICTURE: X(01)
      // CONTEÚDO: Obrigatório - fixo "C"
      
      $linha.= 'C';
      
      // CAMPO: Código da ocorrência
      // Posição: 109 a 110
      // Picture: 9(02)
      // CONTEÚDO: 01 – inclusão;   02 – alteração;
      //           03 – exclusão;   04 – autorização;
      //           05 – bloqueio;   06 – desbloqueio
      
      if ($this->remessa->getIdopecnabpag() == 'E'){
        $linha .= '03'; //Exclusão
      }elseif($this->remessa->getIdopecnabpag() == 'I'){
        $linha .= '01'; // Inclusão
      }elseif($this->remessa->getIdopecnabpag() == 'B'){
        $linha .= '05'; // Bloqueio
      }elseif($this->remessa->getIdopecnabpag() == 'A'){
        $linha .= '02'; // Autorização
      }elseif($this->remessa->getIdopecnabpag() == 'D'){
        $linha .= '06'; // Desbloqueio
      }
      
      // CAMPO: Nº compromisso p/ cliente
      // Posição: 111 a 120
      // Picture: X(10)
      
      $linha.= $boleto->getNrlanbolpag();
      
      // CAMPO: Data de vencimento
      // Posição: 121 a 126
      // Picture: 9(06)
      // Conteúdo: Vencimento(ddmmaa)
      
      $linha.= $boleto->getDtvenbolpag()->format('dmy');
      
      // CAMPO: Valor do pagto
      // Posição: 127 a 139
      // Picture: 9(11)V9(2)
      
      $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),13,'0',STR_PAD_LEFT);
      
      // CAMPO: Tipo de pagamento
      // POSIÇÃO: 140 a 142
      // PICTURE: X(03)
      // CONTEÚDO: Obrigatório
      
      $linha.= 'COB'; //BOLETO BANCÁRIO
      
      // CAMPO: Banco destino
      // POSIÇÃO: 143 a 145
      // PICTURE: 9(03)
      $linha.=str_pad(substr($boleto->getCdbarrapag(), 0, 3), 3, '0', STR_PAD_LEFT);
      
      // CAMPO: Agência destino (Opcional)
      // POSIÇÃO: 146 a 152
      // PICTURE: 9(07)
      
      $linha.= str_pad($boleto->getCdagenbolp(), 7, '0', STR_PAD_LEFT);

      // CAMPO: Zeros
      // Posição: 153 a 155
      // Picture: 9(3)
      $linha.= str_repeat('0', 3);

      // CAMPO: Conta corrente destino (Opcional)
      // POSIÇÃO: 156 a 165
      // PICTURE: 9(10)
      //Obs: A 2 últimas posição da C/C é res. para o Dig. Verif.
      
      $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,-1)),10,'0',STR_PAD_LEFT);
      
      // CAMPO: Brancos
      // POSIÇÃO: 166 a 185
      // PICTURE: X(20)

      $linha.= str_repeat(' ', 20);
      
      
      // CAMPO: Agência p/ pagamento
      // POSIÇÃO: 186 a 192
      // PICTURE: 9(07)
      
      $linha.= str_pad($this->contcorr->getCdagencia(), 7, '0', STR_PAD_LEFT);
      
      // CAMPO: Nosso Número (Opcional)
      // POSIÇÃO: 193 a 208
      // PICTURE: X(16)
      
      $linha.= str_repeat(' ', 16); //Não coloca-se Nosso Número em CNAB PAG
      
      // CAMPO: Brancos
      // POSIÇÃO: 209 a 247
      // PICTURE: X(39)
      
      $linha.= str_repeat(' ', 39);
      
      // CAMPO: Banco portador Bloquete
      // POSIÇÃO: 248 a 250
      // PICTURE: 9(03)
      
      $linha .= str_pad(substr($boleto->getCdbarrapag(), 0, 3), 3, '0', STR_PAD_LEFT);
      
      // CAMPO: Valor Abatimento
      // POSIÇÃO: 251 a 263
      // PICTURE: 9(13)
      
      $linha.= str_repeat('0', 13);
      
      // CAMPO: Nome Fornecedor
      // POSIÇÃO: 264 a 293
      // PICTURE: X(30)
      
      $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,30)),30);
      
      // CAMPO: Brancos
      // POSIÇÃO: 294 a 303
      // PICTURE: X(10)
      
      $linha.= str_repeat(' ', 10);
      
      // CAMPO: Código de Barras
      // POSIÇÃO: 304 a 347
      // PICTURE: X(44)
      
      $linha.= $boleto->getCdbarrapag();
      
      // CAMPO: Valor de juros mora/multa
      // POSIÇÃO: 348 a 360
      // PICTURE: 9(13)
      
      $linha.= str_repeat('0', 13);
      
      // CAMPO: Data pagamento
      // POSIÇÃO: 361 a 366
      // PICTURE: 9(06)
      // CONTEÚDO: DDMMAA
      
      $linha.= $boleto->getDtvenbolpag()->format('dmy');
      
      // CAMPO: Valor autorizado p/ pagto
      // Posição: 367 a 379
      // Picture: 9(11)V9(2)
      
      $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),13,'0',STR_PAD_LEFT);
      
      
      // CAMPO: Código da meoda
      // POSIÇÃO: 380 a 383
      // PICTURE: X(04)
      // CONTEÚDO: Obrigatório - fixo "REAL"
      
      $linha.= self::MOEDA;
      
      // CAMPO: Tipo de Carteira (Opcional)
      // POSIÇÃO: 384 a  387
      // PICTURE: X(04)
      
      $linha.= str_repeat(' ',4);
      
      // CAMPO: Espécie de documento (opcional)
      // POSIÇÃO: 388 a 390
      // PICTURE: X(03)
      
      $linha.= str_repeat(' ',3);
              
      // CAMPO: Brancos
      // POSIÇÃO: 391 a 394
      // PICTURE: X(04)
      
      $linha.= str_repeat(' ',4);
      
      // CAMPO: Nº sequencial do registro
      // POSIÇÃO: 395 a 400
      // PICTURE: 9(06)
      
      $linha.= str_pad($this->qtdRegiArq,6,'0',STR_PAD_LEFT);

      return $linha."\r\n";
    }
}