<?php

namespace Teknisa\FIN\Modules\ContaPagar\Services\Retorno;

use Fiscal\Entities\Boletpagar;
use Geral\Entities\Empresa;

/**
 * Class LeitorCnabSafra
 * @package Financeiro\Logic\ContasPagar\Cnab\Retorno
 */
class LeitorCnabSafra extends LeitorCnab{

    CONST ID_REGISTRO_HEADER_ARQUIVO = '0';
    CONST ID_REGISTRO_DETALHE = '1';
    CONST ID_REGISTRO_TRAILLER_ARQUIVO = '9';
    CONST TIPO_INS_JUR_CPF = '02';
    CONST TIPO_INS_JUR_CNPJ = '01';
    CONST RETORNO = 2;
    CONST CODIGO_COMPENSACAO_SAFRA = '422';

    private $cdagencia;
    private $cdconta;
    private $ocorrenciaDetalhamento;
    private $contHeaderLote = 0;
    private $cdfilial;
    private $nrlanbolpag;
    private $vencimento;

    /**
     * @param $arquivo
     * @param $cdempresa
     * @param $cdbanco
     * @throws \Exception
     */
    public function __construct($arquivo, $cdempresa, $cdbanco){
        try {
            $this->cdbanco = $cdbanco;
            $this->arquivo = $arquivo;
            $this->cdempresa = $cdempresa;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Busca Estreutura Arquivo
     * @return array
     */
     public function getEstruturaArquivo(){
        return array(
            self::ID_REGISTRO_HEADER_ARQUIVO,
            self::ID_REGISTRO_DETALHE,
            self::ID_REGISTRO_TRAILLER_ARQUIVO
        );
    }

    /**
     * Valida Arquivo CNAB
     * @param $linha
     * @param $estruturaArquivo
     * @return mixed
     */
    private function validaArquivo($linha, $estruturaArquivo){
	
        parent::setStatusArquivo(true);
        
        // CAMPO: Tipo de Registro
        // POSIÇÃO: 2 a 2
        // PICTURE: 9(1)
        $idReg = substr($linha,0,1);
        
        if($this->contlinha == 1 and $idReg != self::ID_REGISTRO_HEADER_ARQUIVO){
            self::gravaErros($this->contlinha, "Erro de identificação do registro header.",true);
            parent::setStatusArquivo(false);
        }
        
        
        //Estrutura Disponível (Header/Detalhe/Trailler de Aquivo)
        if($this->contlinha > 1 and (!in_array($idReg, $estruturaArquivo))){
            self::gravaErros($this->contlinha,"Identificação do registro não é válida",true);
            parent::setStatusArquivo(false);
        }

        return parent::getStatusArquivo();
    }

    /**
     * Lê Arquivo CNAB
     * @param $nrorg
     * @throws \Exception
     */
    public function leArquivo($nrorg){
        try {
            foreach ($this->arquivo as $linha) {
                $this->contlinha++;
                if($linha){

                    $this->retorno = new Retorno();

                    // Busca o Tipo de Registro do Arquivo
                    $idRegistro = substr($linha,0,1);
                    
                    if(self::validaArquivo($linha, self::getEstruturaArquivo())){
                        switch ($idRegistro) {
                            case(self::ID_REGISTRO_HEADER_ARQUIVO):
                                self::leHeaderArquivo($linha, $nrorg);
                                break;

                            case(self::ID_REGISTRO_DETALHE):
                                self::leDetalhe($linha, $nrorg);
                                break;

                            case(self::ID_REGISTRO_TRAILLER_ARQUIVO):
                                break;
                        }
                    }
                    $this->retornos[] = $this->retorno;
                }
            }
            self::preparaOcorrencias($this->cdbanco, $nrorg);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Busca Inscrição Empresa
     * @param $cdempresa
     * @param $nrorg
     * @return mixed
     * @throws \Exception
     */
    private function getInscricaoEmpresa($cdempresa, $nrorg){
        /** @var Empresa $empresa */
        $empresa = Empresa::findOneByCdempresaAndNrorg($cdempresa, $nrorg);
        if($empresa)
            return $empresa->getNrinsjurempr();
        else throw new \Exception ('Empresa não Encontrada.');
    }

    /**
     * // =====================================================================  //
     * // ====================  REGISTRO HEADER  DE LOTE   ====================  //
     * // =====================================================================  //
     * @param $header
     * @param $nrorg
     */
    private function leHeaderArquivo($header, $nrorg){
        
        // CAMPO: Identificação arquivo retorno
        // POSIÇÃO: 2 a 2
        // PICTURE: 9(1)
        // DEFAULT: 2
        if(substr($header,1,1) != self::RETORNO){
            self::gravaErros($this->contlinha,'Código de Retorno Inválido.', true);
            parent::setStatusArquivo(false);
        }
        
        // CAMPO: Ident. Empresa no banco
        // POSIÇÃO: 27 a 34
        // PICTURE: 9(8)
        // NOTA: Número da Conta
        $this->cdconta = substr($header,26,8);

        // CAMPO: Código da agência do cliente
        // POSIÇÃO: 38 a 44
        // PICTURE: 9(7)
        $this->cdagencia = substr($header,37,7);
        
        // CAMPO: Código Banco
        // POSIÇÃO: 77 a 79
        // PICTURE: 9(3)
        // DEFAULT: "422"
        
        if(substr($header,76,3) != self::CODIGO_COMPENSACAO_SAFRA){
            self::gravaErros($this->contlinha,'Código do banco inválido.', true);
            parent::setStatusArquivo(false);
        }
                        
    }


    /**
     * // =====================================================================  //
     * // ====================      REGISTRO DETALHE       ====================  //
     * // =====================================================================  //
     * @param $detalhe
     */
    private function leDetalhe($detalhe, $nrorg){
        
        $this->retorno->setCdlinha(str_pad($this->contlinha,10,'0',STR_PAD_LEFT));
        
        // CAMPO: Código Registro
        // POSIÇÃO: 01 a 01
        // PICTURE: 9(1)
        // DEFAULT: "1"

        // CAMPO: Código Inscrição
        // POSIÇÃO: 02 a 03
        // PICTURE: 9(2)
        $tipoInscricao = substr($detalhe,1,2);
        if(empty($tipoInscricao)){
            self::gravaErros($this->contlinha,'Tipo de Inscrição da Empresa não Informado.', true);
            parent::setStatusArquivo(false);
        }
        
        // CAMPO: Número Inscrição
        // POSIÇÃO: 04 a 17
        // PICTURE: 9(14)
        $nrinsjurempr = self::getInscricaoEmpresa($this->cdempresa, $nrorg);
        
        if($tipoInscricao == self::TIPO_INS_JUR_CPF){
            $inscricao = substr($detalhe,3,11);
        }else{
            $inscricao = substr($detalhe,3,14);
        }
        
        if($nrinsjurempr <> $inscricao){
            self::gravaErros($this->contlinha,'Inscrição da empresa não cadastrada.', true);
            parent::setStatusArquivo(false);
        }
        
        // CAMPO: Número conta
        // POSIÇÃO: 18 a 25
        // PICTURE: 9(8)
        $this->retorno->setCdconta($this->cdconta);
        
        // CAMPO: Brancos
        // POSIÇÃO: 26 a 28
        // PICTURE: X(3)
        
        // CAMPO: Código Agência
        // POSIÇÃO: 29 a 35
        // PICTURE: 9(7)
        $this->retorno->setCdagencia($this->cdagencia);
       
        // CAMPO: Brancos
        // POSIÇÃO: 36 a 37
        // PICTURE: X(2)
                
        // CAMPO: Uso exclusivo
        // POSIÇÃO: 38 a 62
        // PICTURE: X(25)
        // FILIAL+ NRLANBOLPAG
        $this->retorno->setCdfilial(substr($detalhe,37,4));
        $this->cdfilial = substr($detalhe,37,4);
        $this->retorno->setNrlanbolpag(substr($detalhe,41,10));
        $this->nrlanbolpag = substr($detalhe,41,10);
        $this->retorno->setBoleto('BOLETO');
      
                
        // CAMPO: Código Fornecedor
        // POSIÇÃO: 63 a 76
        // PICTURE: 9(14)
           
        // CAMPO: Tipo de Documento
        // POSIÇÃO: 77 a 79
        // PICTURE: X(03)
                
        // CAMPO: Num. Compromisso
        // POSIÇÃO: 80 a 89
        // PICTURE: X(10)
        
        // CAMPO: Seq. Compromisso
        // POSIÇÃO: 90 a 90
        // PICTURE: X(1)
                
        // CAMPO: Data de Pagamento
        // POSIÇÃO: 91 a 98
        // PICTURE: X(8)
        //DEFAULT: DDMMAAAA
        $this->retorno->setDtpagamento(substr($detalhe,90,8)); 
        
        // CAMPO: Data de Vencimento
        // POSIÇÃO: 99 a 106
        // PICTURE: X(8)
        //DEFAULT: DDMMAAAA
        $this->retorno->setDtvencimento(substr($detalhe,98,8));

        
        // CAMPO: Negociação Compromisso
        // POSIÇÃO: 107 a 107
        // PICTURE: X(1)
                
        // CAMPO: Código Operação
        // POSIÇÃO: 108 a 108
        // PICTURE: X(1)
                
        // CAMPO: Código Ocorrência
        // POSIÇÃO: 109 a 110
        // PICTURE: 9(2)
        $this->ocorrenciaDetalhamento = substr($detalhe,108,2);
        
        
        //Insere Situação do Cnab
        $this->retorno->setIdsitucnabp(self::situacaoCnab($this->ocorrenciaDetalhamento));
        
        // CAMPO: Data Ocorrência
        // POSIÇÃO: 111 a 116
        // PICTURE: 9(6)
        //DEFAULT: DDMMAA
                
        // CAMPO: Seu Número Compromisso
        // POSIÇÃO: 117 a 126
        // PICTURE: X(10)
        
        // CAMPO: Banco destino
        // POSIÇÃO: 127 a 129
        // PICTURE: 9(3)
        
        // CAMPO: Número do Lote
        // POSIÇÃO: 130 a 135
        // PICTURE: 9(06)
        
        // CAMPO: Sequência do Lote
        // POSIÇÃO: 136 a 139
        // PICTURE: 9(04)
        
        // CAMPO: Tipo de pagto
        // POSIÇÃO: 140 a 142
        // PICTURE: X(03)
        
        // CAMPO: Brancos
        // POSIÇÃO: 143 a 146
        // PICTURE: X(04)
        
        // CAMPO: Data de Vencimento
        // POSIÇÃO: 147 a 152
        // PICTURE: 9(06)
        // DEFAULT: DDMMAA
        
        // CAMPO: Valor do Compromisso
        // POSIÇÃO: 153 a 165
        // PICTURE: 9(13)
        
        // CAMPO: Código Rejeição
        // POSIÇÃO: 166 a 168
        // PICTURE: X(03)
        $ocorrencia = substr($detalhe,165,3);
        if(self::situacaoCnab($this->ocorrenciaDetalhamento) == '01'){
            $this->retorno->setOcorrencia(substr($detalhe,165,3));
            if(!$ocorrencia or $ocorrencia == ' '){
                self::gravaErros($this->contlinha,'Ocorrência não informada.');
                parent::setStatusArquivo(false);
            }
        }
        
        // CAMPO: Código Rejeição
        // POSIÇÃO: 169 a 273
        // PICTURE: X(105)
        // NOTA: 35 ocorrências com tamanho de x(03)
        
        // CAMPO: Brancos
        // POSIÇÃO: 274 a 285
        // PICTURE: X(02)
        
        // CAMPO: Brancos
        // POSIÇÃO: 274 a 285
        // PICTURE: X(02)
        
        // CAMPO: Código ISPB
        // POSIÇÃO: 286 a 293
        // PICTURE: 9(08)
        
        // CAMPO: Brancos
        // POSIÇÃO: 294 a 295
        // PICTURE: X(02)
        
        // CAMPO: Nome do Fornecedor
        // POSIÇÃO: 296 a 325
        // PICTURE: X(30)
        $this->retorno->setNmrazsocforn(substr($detalhe,295,30));
        
        // CAMPO: Sacador Avalista
        // POSIÇÃO: 326 a 345
        // PICTURE: X(20)
        
        // CAMPO: Agência Destino
        // POSIÇÃO: 346 a 352
        // PICTURE: X(07)
        
        // CAMPO: Conta Destino
        // POSIÇÃO: 353 a 362
        // PICTURE: X(10)
        
        // CAMPO: Aceite DDA
        // POSIÇÃO: 363 a 363
        // PICTURE: X(10)
        
        // CAMPO: Tipo Pessoa DDA
        // POSIÇÃO: 364 a 364
        // PICTURE: X(01)
        
        // CAMPO: CNPJ Compart. DDA
        // POSIÇÃO: 365 a 379
        // PICTURE: X(15)
        
        // CAMPO: Seu Número DDA
        // POSIÇÃO: 380 a 394
        // PICTURE: X(15)
        
        // CAMPO: Número Sequencial registro
        // POSIÇÃO: 395 a 400
        // PICTURE: 9(06)
        
        $this->retorno->setCdtpbaixapag(self::getTipoBaixaPagamento($this->cdfilial, $this->nrlanbolpag, $nrorg));
        $this->retorno->setCdreducont(self::getReducont($this->cdfilial, $this->nrlanbolpag, $nrorg));
        $boletpagar = Boletpagar::findOneByCdfilialAndNrlanbolpagAndNrorg($this->cdfilial, $this->nrlanbolpag, $nrorg);
        $valorFinalPagoBoleto = ($boletpagar) ? $boletpagar->getVrfinalbpag() : 0;
        $this->retorno->setVrfinalpago($valorFinalPagoBoleto);

    }
    

	
    /**
     * Lê Trailer de Arquivo
     * @param $trailerlote
     */
    public function leTrailerArquivo($trailerlote){
        // CAMPO: Identificação do registro
        // POSIÇÃO: 1 a 1
        // PICTURE: 9(01)
        // DEFAULT: 9
        
        // CAMPO: Brancos
        // POSIÇÃO: 2 a 394
        // PICTURE: X(395)
        
        // CAMPO: Número sequencial de registro
        // POSIÇÃO: 395 a 400
        // PICTURE: 9(06)
    }

    /**
	 * Verifica Situação CNAB
     * @param $cdocorrencia
     * @return string
     */
    private function situacaoCnab($cdocorrencia){
         if ($cdocorrencia){
             if ((strpos($cdocorrencia,'11') === false) and
               (strpos($cdocorrencia,'12') === false) and
               (strpos($cdocorrencia,'14') === false) and
               (strpos($cdocorrencia,'15') === false) and
               (strpos($cdocorrencia,'16') === false) and
               (strpos($cdocorrencia,'17') === false) and
               (strpos($cdocorrencia,'20') === false) and
               (strpos($cdocorrencia,'21') === false))
               return '01'; // Retorno com erro
            if ((strpos($cdocorrencia,'01') === false) and
               (strpos($cdocorrencia,'02') === false) and
               (strpos($cdocorrencia,'03') === false) and
               (strpos($cdocorrencia,'04') === false) and
               (strpos($cdocorrencia,'05') === false) and
               (strpos($cdocorrencia,'06') === false))
               return '20'; // Retorno Ok (Agendado)
        }
        return '10'; //Sem retorno
    }

}