<?php

namespace Teknisa\FIN\Modules\ContaPagar\Services\Retorno;

use Financeiro\Entities\Cnabpag;
use Fiscal\Entities\Boletpagar;
use Geral\Entities\Empresa;

class MultiPagBradesco extends LeitorCnab {
    CONST ID_HEADER_ARQUIVO = '0';
    CONST ID_HEADER_LOTE = '1';
    CONST ID_REGISTRO_DETALHE = '3';
    CONST ID_REGISTRO_TRAILLER_ARQUIVO = '9';
    CONST ID_REGISTRO_TRAILLER_LOTE = '5';
    CONST INSCRICAO_FISICA = '1';
    CONST DARF_NORMAL = '16';
    CONST GPS = '17';
    CONST DARF_SIMPLES = '18';
    CONST PREVIDENCIA_SOCIAL = '22';
    CONST CODIGO_COMPENSACAO_BRADESCO = '237';

    private $cdagencia;
    private $cdconta;
    private $ocorrenciaHeader;
    private $contHeaderLote = 0;
    private $nrlanbolpag = 0;
    private $cdfilial = 0;
    private $boletoValido;
    /**
     * @param $arquivo
     * @param $cdempresa
     * @param $cdbanco
     * @throws \Exception
     */
    public function __construct($arquivo, $cdempresa, $cdbanco){
        $this->cdbanco = $cdbanco;
        $this->arquivo = $arquivo;
        $this->cdempresa = $cdempresa;
    }

    private function validaArquivo($linha, $estruturaArquivo){

        parent::setStatusArquivo(true);
        $idReg = substr($linha,7,1);
        
        if($this->contlinha == 1 and $idReg != self::ID_HEADER_ARQUIVO ){
            self::gravaErros($this->contlinha, "Erro de identificação do registro header.",true);
            throw new \Exception("Erro de identificação do registro header.", 1);
            parent::setStatusArquivo(false);
        }
        if($this->contlinha > 1 and (! in_array($idReg, $estruturaArquivo))){
            self::gravaErros($this->contlinha,"Identificação do registro não é válida",true);
            throw new \Exception("Identificação do registro não é válida.", 1);
            parent::setStatusArquivo(false);
        }

        return parent::getStatusArquivo();
    }

    public function getEstruturaArquivo(){
        return array(
                self::ID_HEADER_ARQUIVO,
                self::ID_HEADER_LOTE,
                self::ID_REGISTRO_DETALHE,
                self::ID_REGISTRO_TRAILLER_ARQUIVO,
                self::ID_REGISTRO_TRAILLER_LOTE
            );
    }

    public function leArquivo($nrorg)
    {
        try {
            foreach ($this->arquivo as $linha) {
                $this->contlinha++;
                if($linha){
                    $this->retorno = new Retorno();

                    // Busca o Tipo de Registro do Arquivo
                    $idRegistro = substr($linha,7,1);
                    if(self::validaArquivo($linha, $this->getEstruturaArquivo())){
                        switch ($idRegistro) {
                            case(self::ID_HEADER_ARQUIVO):
                                self::leHeaderArquivo($linha, $nrorg);
                                break;
                            case(self::ID_HEADER_LOTE):
                                self::leHeaderLote($linha);
                                break;
                            case(self::ID_REGISTRO_DETALHE):
                                self::leRegistroDetalhe($linha, $nrorg);
                                break;
                            case(self::ID_REGISTRO_TRAILLER_LOTE):
                                self::leRegistroTraillerLote($linha, $nrorg);
                                break;
                        }
                    }
                    $this->retornos[] = $this->retorno;
                }
            }
            parent::preparaOcorrencias($this->cdbanco, $nrorg);

        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * @param
     * Retona 3 caso o registro não tenha nenhuma inconsistência,
     * e 2 caso possua alguma inconsistência.(Ver Layout p/ mais informações)
     * @return int
     */
    private function nivel($codigo){

        $conterro = 0;

        //Percorre as informações do retorno de duas em duas posições
        while($conterro < 10 ){

            $cderro = trim(substr($codigo,$conterro,2));
            //Verifica se a informação não está vazia
            if(! empty($cderro)){
                //Verifica se a informação está inconsistente
                if(! in_array($cderro,array('BD','BT','BU','BV','BW','FS',
                                            'GS','GT','GU','GV','GX','JB','HA'))){
                    return 2; //INCONSISTÊNCIA INVALIDA O REGISTRO
                }
            }
            $conterro+=2;
        }
        return 3; //CONSISTÊNCIA A TAREFA FOI EXECUTADA
    }

    private function getIncricaoEmpresa($cdempresa, $nrorg){
        $empresa = Empresa::findOneByCdempresaAndNrorg($cdempresa, $nrorg);
        if($empresa)
           return $empresa->getNrinsjurempr();
        else
            throw new \Exception ('Empresa não Encontrada.');
    }

    private function leHeaderArquivo($header, $nrorg){
        
        // CAMPO: Código do Banco na Compensação
        // POSIÇÃO: 1 a 3
        // PICTURE: 9(3)
        if(substr($header,0,3) != self::CODIGO_COMPENSACAO_BRADESCO)
            self::gravaErros($this->contlinha,'Banco não Cadastrado.', true);
            
        $nrinsjurempr = self::getIncricaoEmpresa($this->cdempresa, $nrorg);
        
        // CAMPO: Tipo de Inscrição da Empresa
        // POSIÇÃO: 18 a 18
        // PICTURE: 9(1)
        $tipoInscricao = substr($header,17,1);
        
        if(empty($tipoInscricao))
            self::gravaErros($this->contlinha,'Tipo de Inscrição da Empresa não Informado.', true);
        
        // CAMPO: Número de Inscrição da Empresa
        // POSIÇÃO: 19 a 32
        // PICTURE: 9(14)
        if ($tipoInscricao == self::INSCRICAO_FISICA){
            $codigoInscricao = substr($header,21,11);
        }else{
            $codigoInscricao = substr($header,18,14);
        }

        if ($nrinsjurempr <> $codigoInscricao){
            self::gravaErros($this->contlinha, 'Inscrição informada no retorno diverge da inscrição cadastrada no sistema.',true);
        }
        
        // CAMPO: Código de Remessa/Retorno
        // POSIÇÃO: 143 a 143
        // PICTURE: 9(1)
        $tipoRegistroRetorno = substr($header,142,1);
        if ($tipoRegistroRetorno <> '2'){
            self::gravaErros($this->contlinha, 'Código de retorno inválido.',true);
            
        }
        
    }

    private function leHeaderLote($header){
        
        // CAMPO: Agência Mantenedora da Conta
        // POSIÇÃO: 53 a 57
        // PICTURE: 9(5)
        $this->cdagencia = substr($header,52,5);
        
        // CAMPO: Número da Conta Corrente
        // POSIÇÃO: 59 a 70
        // PICTURE: 9(12)
        $this->cdconta = substr($header,58, 12);
        
        // CAMPO: Códigos das Ocorrências p/ Retorno
        // POSIÇÃO: 231 a 240
        // PICTURE: X(10)
        $this->ocorrenciaHeader = substr($header, 230, 10);
        
    }

    private function leRegistroDetalhe($registro, $nrorg){
        $this->boletoValido = false;
        
        // CAMPO: Código de Segmento do Reg. Detalhe
        // POSIÇÃO: 14 a 14
        // PICTURE: X(1)
        $segmento = substr($registro, 13, 1);

        switch($segmento){
            case 'A': $this->segmentoA($registro);
                break;
            case 'B': $this->segmentoB($registro);
                break;
            case 'J' :
                if($segmento == "J" && substr($registro, 17,2) != "52"){
                   $this->segmentoJ($registro); 
                }
                break;
            case 'O': $this->segmentoO($registro);
                break;
            case 'N': $this->segmentoN($registro);
                break;
        }

        $idtitulopag = substr($registro,414,9);
        if ($this->boletoValido){
            $this->retorno->setCdagencia($this->cdagencia);
            $this->retorno->setCdconta($this->cdconta);
            $this->retorno->setBoleto('BOLETO');
            $this->retorno->setNrlanbolpag($this->nrlanbolpag);
            $this->retorno->setCdfilial($this->cdfilial);
            $this->retorno->setCdlinha(str_pad($this->contlinha,'0',STR_PAD_LEFT));
            $this->retorno->setCdtpbaixapag(self::getTipoBaixaPagamento($this->cdfilial, $this->nrlanbolpag, $nrorg));
            $this->retorno->setCdreducont(self::getReducont($this->cdfilial, $this->nrlanbolpag, $nrorg));
            $boletpagar = Boletpagar::findOneByCdfilialAndNrlanbolpagAndNrorg($this->cdfilial, $this->nrlanbolpag, $nrorg);
            $valorFinalPagoBoleto = ($boletpagar) ? $boletpagar->getVrfinalbpag() : 0;
            $this->retorno->setVrfinalpago($valorFinalPagoBoleto);
            
            // CAMPO: Tipo de movimento   [0 - Inclusão  9 - Exclusão]
            // POSIÇÃO: 15 a 15
            // PICTURE: 9(1)
            $this->retorno->getTpmovimento(substr($registro, 14, 1));
            
        }
    }

    private function segmentoA($registro){
        $this->boletoValido = true;
        
        // CAMPO: Nº do Docum. Atribuído p/ Empresa (CDFILIAL+ NRLANBOLREC)
        // POSIÇÃO: 74 a 93
        // PICTURE: 9(20)
        $this->cdfilial = substr($registro, 73, 4);
        $this->nrlanbolpag = substr($registro, 77, 10);
        
        // CAMPO: Nome do Favorecido
        // POSIÇÃO: 44 a 73
        // PICTURE: 9(30)
        $this->retorno->setNmrazsocforn(substr($detalhe,43,30));
        
        // CAMPO: Nº do Docum. Atribuído pelo Banco - Nosso Número 
        // POSIÇÃO: 135 a 154 
        // PICTURE: X(20)
        $nossoNumero = substr($registro, 134, 20);
        $this->retorno->setNossonumero($nossoNumero);
        
        // CAMPO: Data Real da Efetivação Pagto 
        // POSIÇÃO: 155 a 162
        // PICTURE: 9(8)
        $this->retorno->setDtpagamento(substr($registro, 154, 8));
        
        // CAMPO: Valor Real da Efetivação do Pagto  
        // POSIÇÃO: 163 a 177
        // PICTURE: 9(13)
        $vrpago = floatval(substr($registro, 119, 15));
        $this->retorno->setVrpago(round($vrpago / 100, 2));
    }

    private function segmentoB($registro){
        $registroAnterior = count($this->retornos) - 1;
        $this->cdfilial = $this->retornos[$registroAnterior]->getCdfilial();
        $this->nrlanbolpag = $this->retornos[$registroAnterior]->getNrlanbolpag();
        $this->retorno->setNossonumero(substr($registro, 215, 15));
        $this->retornos[$registroAnterior]->setDtvencimento(substr($registro, 127, 8));
        $vrabatimento = floatval(substr($registro, 150, 13));
        $vrdesconto = floatval(substr($registro, 165, 13));
        $this->retornos[$registroAnterior]->setVrdesconto(round($vrdesconto / 100, 2));
        $vracrescimo = floatval(substr($registro, 180, 13));
        $vrmulta = floatval(substr($registro, 195, 13));
        $this->retornos[$registroAnterior]->setVracrescimo($vracrescimo);
    }

    private function segmentoJ($registro){
        $this->boletoValido = true;
        
        $registroComplemetar = count($this->retornos) + 1;
        
        //CAMPO: Nome do Cedente
        // POSIÇÃO: 062 a 091
        // PICTURE: X(30)
        $this->retorno->setNmrazsocforn(substr($registro,61,30));
        
        
        // CAMPO: Nº do Docto Atribuído pela Empresa  (CDFILIAL+ NRLANBOLREC)
        // POSIÇÃO: 183 a 202
        // PICTURE: X(20)
        $this->cdfilial = substr($registro, 182, 4);
        $this->nrlanbolpag = substr($registro, 186, 10);
        
        // CAMPO: Data do Vencimento (Nominal)
        // POSIÇÃO: 92 a 99
        // PICTURE: 9(8)
        $this->retorno->setDtvencimento(substr($registro, 91, 8));
        
        // CAMPO: Nosso Número  - Nº do Docto Atribuído pelo Banco
        // POSIÇÃO: 203 a 222 
        // PICTURE: X(20)
        $this->retorno->setNossonumero(substr($registro, 202, 20));
        
        // CAMPO: Valor do Desconto + Abatimento
        // POSIÇÃO: 115 a 129 
        // PICTURE: 9(15)
        $vrdesconto = floatval(substr($registro, 114, 15));
        $this->retorno->setVrdesconto(round($vrdesconto / 100, 2));
        
        // CAMPO: Acréscimos Valor da Mora + Multa
        // POSIÇÃO: 130 a 144
        // PICTURE: 9(15)
        $vracrescimo = floatval(substr($registro, 129, 15));
        $this->retorno->setVracrescimo($vracrescimo);
        
        // CAMPO: Data do Pagamento
        // POSIÇÃO: 145 a 152
        // PICTURE: 9(8)
        $this->retorno->setDtpagamento(substr($registro, 144, 8));
        
        // CAMPO: Valor do Pagamento
        // POSIÇÃO: 153 a 167
        // PICTURE: 9(15)
        $vrpago = floatval(substr($registro, 152, 15));
        $this->retorno->setVrpago(round($vrpago / 100, 2));
        
        // CAMPO: Nosso Número  - Nº do Docto Atribuído pelo Banco
        // POSIÇÃO: 135 a 154
        // PICTURE: X(20)
        $nrRetorno = trim(substr($registro, 202, 20));
        $this->retorno->setNossonumero($nrRetorno);
        // CAMPO: Códigos das Ocorrências p/ Retorno
        // POSIÇÃO: 231 a 240
        // PICTURE: X(10)
        $this->retorno->setOcorrencia(substr($registro, 230, 10));
        $ocorrencia = trim(substr($registro, 230, 10));
        $this->retorno->setIdsitucnabp(self::situacaoCnab($this->retorno->getOcorrencia().$ocorrencia.$this->ocorrenciaHeader));
    }
    
    private function segmentoJ52($registro){
         $registroAnterior = count($this->retornos) - 1;
        $this->cdfilial = $this->retornos[$registroAnterior]->getCdfilial();
        $this->nrlanbolpag = $this->retornos[$registroAnterior]->getNrlanbolpag();
        //$this->retorno->setNossonumero(substr($registro, 215, 15));
        $this->retorno->setDtvencimento(substr($registro, 127, 8));
        $vrabatimento = floatval(substr($registro, 150, 13));
        $vrdesconto = floatval(substr($registro, 165, 13));
        $this->retorno->setVrdesconto(round($vrdesconto / 100, 2));
        $vracrescimo = floatval(substr($registro, 180, 13));
        $vrmulta = floatval(substr($registro, 195, 13));
        $this->retorno->setVracrescimo($vracrescimo);
    }

    private function segmentoN($registro){
        // CAMPO: Data do Pagamento
        // POSIÇÃO: 88 a 95
        // PICTURE: 9(8)
        $this->retorno->setDtpagamento(substr($registro, 87, 8));
        
        // CAMPO: Valor do Total do Pagamento
        // POSIÇÃO: 96 a 110
        // PICTURE: 9(13) V9(2)
        $vrpago = floatval(substr($registro, 95, 15));
        $this->retorno->setVrpago(round($vrpago / 100, 2));
        
        // CAMPO: Código de Identificação do Tributo
        // POSIÇÃO: 133 a 134
        // PICTURE: X(2)
        $tipoTributo = substr($registro, 132, 2);
        
        if ($tipoTributo == self::DARF_NORMAL){
            // CAMPO: Valor da Multa
            // POSIÇÃO: 175 a 189
            // PICTURE: 9(13) V9(2)
            $vrmulta = floatval(substr($registro, 174, 15));
            // CAMPO: Valor dos Juros / Encargos
            // POSIÇÃO: 190 a 204
            // PICTURE: 9(13) V9(2)
            $juros = floatval(substr($registro, 189, 15));
            $vracrescimo = $vrmulta + $juros;
            $this->retorno->setVracrescimo($vracrescimo);
        }elseif ($tipoTributo == self::GPS){
            // CAMPO: Atualização Monetária 
            // POSIÇÃO: 171 a 185
            // PICTURE: 9(13) V9(2)
            $vracrescimo = floatval(substr($registro, 170, 15));
            $this->retorno->setVracrescimo($vracrescimo);
        }elseif ($tipoTributo == self::DARF_SIMPLES){
            // CAMPO: Valor da Multa 
            // POSIÇÃO: 180 a 194
            // PICTURE: 9(13) V9(2)
            $vrmulta = floatval(substr($registro, 179, 15));
            
            // CAMPO: Valor dos Juros / Encargos
            // POSIÇÃO: 195 a 209
            // PICTURE: 9(13) V9(2)
            $juros = floatval(substr($registro, 194, 15));
            $vracrescimo = $vrmulta + $juros;
            $this->retorno->setVracrescimo($vracrescimo);
        }elseif ($tipoTributo == self::GARE){
            // CAMPO: Valor dos Juros / Encargos
            // POSIÇÃO: 202 a 215
            // PICTURE: 9(13) V9(2)
            $vrmulta = floatval(substr($registro, 201, 15));
            
            // CAMPO: Valor da Multa
            // POSIÇÃO: 216 a 229
            // PICTURE: 9(13) V9(2)
            $juros = floatval(substr($registro, 215, 15));
            $vracrescimo = $vrmulta + $juros;
            
            // CAMPO: Data de Vencimento
            // POSIÇÃO: 135 a 142
            // PICTURE: 9(8)
            $this->retorno->setDtvencimento(substr($registro, 135, 8));
        }else{
            // CAMPO: Acréscimo/Atualização Monetária
            // POSIÇÃO: 171 a 185
            // PICTURE: 9(13)V9(2)
            $vracrescimo = floatval(substr($registro, 170, 13));
        }
        
        // CAMPO:  Nº do Docto Atribuído pelo Banco
        // POSIÇÃO: 38 a 57
        // PICTURE: X(20)
        $nrRetorno = trim(substr($registro, 37, 20));
        if (empty($nrRetorno)){
            self::gravaErros($this->contlinha, 'Número do retorno inválido.', false);
            parent::setStatusArquivo(false);
        }
        
        // CAMPO: Códigos das Ocorrências p/ Retorno
        // POSIÇÃO: 231 a 240
        // PICTURE: X(10)
        $this->retorno->setOcorrencia(substr($registro, 230, 10));
        $ocorrencia = trim(substr($registro, 230, 10));
        $this->retorno->setIdsitucnabp(self::situacaoCnab($this->retorno->getOcorrencia().$ocorrencia.$this->ocorrenciaHeader));
    }

    private function segmentoO($registro){
        // CAMPO: Data do Vencimento (Nominal)
        // POSIÇÃO: 92 a 99
        // PICTURE: 9(8)
        $this->retorno->setDtvencimento(substr($registro, 91, 8));
        
        // CAMPO: Data do Pagamento
        // POSIÇÃO: 100 a 107
        // PICTURE: 9(8)
        $this->retorno->setDtpagamento(substr($registro, 99, 8));
        
        // CAMPO: Valor do Pagamento
        // POSIÇÃO: 108 a 122
        // PICTURE: 9(13) V9(2)
        $vrpago = floatval(substr($registro, 107, 15));
        $this->retorno->setVrpago(round($vrpago / 100, 2));
        
        // CAMPO: Nº do Docto Atribuído pelo Banco
        // POSIÇÃO: 143 162
        // PICTURE: X20) 
        $nrRetorno = trim(substr($registro, 142, 20));
        if (empty($nrRetorno)){
            self::gravaErros($this->contlinha, 'Número do retorno inválido.', false);
            parent::setStatusArquivo(false);
        }
        
        // CAMPO: Códigos das Ocorrências p/ Retorno
        // POSIÇÃO: 231 a 240
        // PICTURE: X(10)
        $this->retorno->setOcorrencia(substr($registro, 230, 10));
        $ocorrencia = trim(substr($registro, 230, 10));
        $this->retorno->setIdsitucnabp(self::situacaoCnab($this->retorno->getOcorrencia().$ocorrencia.$this->ocorrenciaHeader));
    }
    
    private function leRegistroTraillerLote($registro){
        
        // CAMPO: Ocorrências
        // POSIÇÃO: 231 a 240
        // PICTURE: X(10)
        $ocorrTraillerLote = trim(substr($registro, 230, 10));
        
        $this->retorno->setIdsitucnabp(self::situacaoCnab($this->retorno->getOcorrencia().$ocorrTraillerLote.$this->ocorrenciaHeader));
        
        //Adiciona as ocorrencias do trailler e do header.
        $this->retorno->setOcorrencia($this->retorno->getOcorrencia().$ocorrTraillerLote.$this->ocorrenciaHeader);
    }
    
    /**
     * @param $cdocorrencia
     * @return string
     */
    private function situacaoCnab($cdocorrencia){
      if ($cdocorrencia){
            if (strpos($cdocorrencia,'BD') !== false)
               return '20'; // Retorno Ok (Agendado)
            if (strpos($cdocorrencia,'02') !== false)
                return '00'; // Não Agendado
            if (strpos($cdocorrencia,'00') !== false)
                return '90'; // Baixado via CNAB
            if ((strpos($cdocorrencia,'BD') === false) and
               (strpos($cdocorrencia,'00') === false) and
               (strpos($cdocorrencia,'02') === false))
               return '01'; // Retorno com erro
        }
        return '10'; //Sem retorno
    }

}