<?php

namespace Teknisa\FIN\Modules\ContaPagar\Services\Retorno;


use Financeiro\Entities\Cnabpag;
use Fiscal\Entities\Boletpagar;

class PagFornBradesco extends LeitorCnab {

    CONST ID_REGISTRO_HEADER = '0';
    CONST ID_REGISTRO_TRANSACAO = '1';
    CONST ID_REGISTRO_TRAILLER = '9';

    CONST TIPO_MOVIMENTO_INCLUSAO = 0;
    CONST TIPO_MOVIMENTO_ALTERACAO = 5;
    CONST TIPO_MOVIMENTO_EXCLUSAO = 9;
    private $tpProcessamento;
    public function __construct($arquivo, $cdempresa, $cdbanco){
        try {

            $this->cdbanco = $cdbanco;
            $this->arquivo = $arquivo;
            $this->cdempresa = $cdempresa;

        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function validaArquivo($linha, $estruturaArquivo){

        parent::setStatusArquivo(true);
        $idReg = substr($linha,0,1);
        if($this->contlinha == 1 and $idReg != self::ID_REGISTRO_HEADER ){
            throw new \Exception("Erro de identificação do registro header.", 1);
            self::gravaErros($this->contlinha, "Erro de identificação do registro header.",true);
            parent::setStatusArquivo(false);
        }
        if($this->contlinha > 1 and (! in_array($idReg, $estruturaArquivo))){
            throw new \Exception("Identificação do registro não é válida.", 1);
            self::gravaErros($this->contlinha,"Identificação do registro não é válida",true);
            parent::setStatusArquivo(false);
        }

        return parent::getStatusArquivo();
    }

    public function getEstruturaArquivo(){
        return array(
            self::ID_REGISTRO_HEADER,
            self::ID_REGISTRO_TRANSACAO,
            self::ID_REGISTRO_TRAILLER
            );
    }

    public function leArquivo($nrorg)
    {
        try {
            foreach ($this->arquivo as $linha) {
                $this->contlinha++;
                if($linha){
                    $this->retorno = new Retorno();

                    // CAMPO: Identificação do registro
                    // POSIÇÃO: 0001 a 0001
                    // PICTURE: 9(1)
                    // DEFAULT:  0 - Header / 1- Transação / 9-Trailer
                    $idRegistro = substr($linha,0,1);

                    if(self::validaArquivo($linha, $this->getEstruturaArquivo())){
                        switch ($idRegistro) {
                            case(self::ID_REGISTRO_HEADER):
                                self::leHeaderArquivo($linha);
                                break;
                            case(self::ID_REGISTRO_TRANSACAO):
                                self::leRegistroTransacao($linha, $nrorg);
                                break;
                        }
                    }
                    $this->retornos[] = $this->retorno;
                }
            }
            self::preparaOcorrencias($this->cdbanco, $nrorg);

        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * @param
     * Retona 3 caso o registro não tenha nenhuma inconsistência,
     * e 2 caso possua alguma inconsistência.(Ver Layout p/ mais informações)
     * @return int
     */
    private function nivel($codigo){

        $conterro = 0;

        //Percorre as informações do retorno de duas em duas posições
        while($conterro < 10 ){

            $cderro = trim(substr($codigo,$conterro,2));
            //Verifica se a informação não está vazia
            if(! empty($cderro)){
                //Verifica se a informação está inconsistente
                if(! in_array($cderro,array('BD','BT','BU','BV','BW','FS',
                                            'GS','GT','GU','GV','GX','JB','HA'))){
                    return 2; //INCONSISTÊNCIA INVALIDA O REGISTRO
                }
            }
            $conterro+=2;
        }
        return 3; //CONSISTÊNCIA A TAREFA FOI EXECUTADA
    }


    // =====================================================================  //
    // ====================      REGISTRO HEADER        ====================  //
    // =====================================================================  //
    /**
     * @param $header
     */
    private function leHeaderArquivo($header){

        // CAMPO: Tipo de Processamento
        // POSIÇÃO: 106 a 106
        // PICTURE: 9(1)
        // DEFAULT:  1 -  Rastreamento da Cobrança Bradesco/ Rastreamento DDA/ Cheque estornado e doc COMPE devolvido
        // 2 - Confirmação de Agendamento/Inconsistência
        // 3 - Confirmação de Pagamento/Pagamento não efetuado
        $this->tpProcessamento = substr($header, 105, 1);
        if(empty($this->tpProcessamento) or (!in_array($this->tpProcessamento,array(1,2,3)))){
            throw new \Exception("Tipo de Processamento inválido.", 1);
            self::gravaErros($this->contlinha,"Tipo de Processamento inválido.",true);
            parent::setStatusArquivo(false);
        }
    }

    // =====================================================================  //
    // ====================      REGISTRO TRANSAÇÃO       ====================  //
    // =====================================================================  //
    /**
     * @param $registro
     */
    private function leRegistroTransacao($registro, $nrorg){
        
        // CAMPO: Nome do Forecedor
        // POSIÇÃO: 18 a 47
        // PICTURE: 9(30)
        $this->retorno->setNmrazsocforn(substr($registro,17,30));
        
        // CAMPO: Reservado a Empresa (Chave do Boleto)
        // POSIÇÃO: 120 a 135
        // PICTURE: (14) 
        //OBS: 4 Posições Filial + 10 Posições Nrlanbolrec
        $cdfilial = trim(substr($registro,119,4));
        $this->retorno->setCdfilial($cdfilial);
        
        $nrlanbolpag = trim(substr($registro,123,10));
        $this->retorno->setNrlanbolpag($nrlanbolpag);
        
        // CAMPO: Nosso Numero
        // POSIÇÃO: 139 a 150
        // PICTURE: (12) 
        $idtitulopag =  trim(substr($registro,138,12));
        $this->retorno->setNossonumero($idtitulopag);
        
        $boleto = false;
        if(!empty($cdfilial) && !empty($nrlanbolpag)){
            $boletpagar = Boletpagar::findOneByCdfilialAndNrlanbolpagAndNrorg($cdfilial, $nrlanbolpag, $nrorg);
            $this->retorno->setCdagencia('');
            $this->retorno->setCdconta('');
            $this->retorno->setBoleto('BOLETO');
        }
        
        $this->retorno->setCdlinha(str_pad($this->contlinha,'0',STR_PAD_LEFT));
        
        
        // CAMPO: Data de Vencimento
        // POSIÇÃO: 166 a 173
        // PICTURE: (8)
        $this->retorno->setDtvencimento(substr($registro, 171, 2) . substr($registro, 169, 2) . substr($registro, 165, 4));
        
        if($boletpagar instanceof Boletpagar){
            $this->retorno->setCdreducont($boletpagar->getCdreducont());
            $this->retorno->setCdtpbaixapag($boletpagar->getCdtpbaixapag());
            $this->retorno->setVrfinalpago($boletpagar->getVrfinalbpag());
        }else{
            $this->retorno->setVrfinalpago(0);
        }
        
        // CAMPO: Valor do pagamento
        // POSIÇÃO: 205 a 219
        // PICTURE: 9(15)
        $varpago = floatval(substr($registro,204,15));
        $this->retorno->setVrpago(round($varpago/100,2));

        // CAMPO: Valor do desconto
        // POSIÇÃO: 220 a 234
        // PICTURE: 9(15)
        $vardesconto= floatval(substr($registro,219,15));
        $this->retorno->setVrdesconto(round($vardesconto/100,2));

        // CAMPO: Valor do acréscimo
        // POSIÇÃO: 235 a 249
        // PICTURE: 9(15)
        $valoracrescimo = floatval(substr($registro,234,15));
        $this->retorno->setVracrescimo(round($valoracrescimo/100,2));

        // CAMPO: Data da efetivação do pagamento
        // POSIÇÃO: 266 a 273
        // PICTURE: 9(8)
        $this->retorno->setDtpagamento(substr($registro, 271, 2) . substr($registro, 269, 2) . substr($registro, 265, 4));
        
        // CAMPO: Informação do retorno
        // POSIÇÃO: 279 a 288
        // PICTURE: X(10)
        $infretorno = trim(substr($registro,278,10));
        if(empty($infretorno)){
            self::gravaErros($this->contlinha,'Código de ocorrência do retorno não foi informado');
            parent::setStatusArquivo(false);
        }

        // CAMPO: Tipo de movimento   [0 - Inclusão  9 - Exclusão]
        // POSIÇÃO: 289 a 289
        // PICTURE: X(1)
        self::tipoMovimento(substr($registro,288,1));

        $nivelInformacao = self::nivel(substr($registro,278,10));
        if($this->retorno->getTpmovimento() != self::TIPO_MOVIMENTO_EXCLUSAO){
            if($nivelInformacao == '3'){
                if($this->tpProcessamento == '3'){
                    $this->retorno->setIdsitucnabp('90'); // boleto pode ser baixado
                }
                else{
                    if($this->tpProcessamento == '2')
                        $this->retorno->setIdsitucnabp('20'); //boleto agendado
                    else{
                        $this->retorno->setIdsitucnabp('00'); //boleto nao agendado
                    }
                $this->retorno->setOcorrencia(substr($registro,278,10));
                }
            }
            else{
                $this->retorno->setIdsitucnabp('01');
                $this->retorno->setOcorrencia(substr($registro,278,10));
            }

        }
        else
            $this->retorno->setIdsitucnabp(CnabPag::SITUAACAO_NAO_ENVIADO);
    }

}