<?php

namespace Teknisa\FIN\Modules\ContaReceber\Factory;

use Doctrine\ORM\EntityManager;

class  Rec22000HistoricoRenegociacaoFinanceiraFactory{

    protected $entityManager;

    /**
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }
    
    public function verificaContaBaixada($nrorg, $cdfilialnf, $nrlanctonf){
        $params = array();

        $params['P_NRORG']      = $nrorg;
        $params['P_CDFILIALNF'] = $cdfilialnf;
        $params['P_NRLANCTONF'] = $nrlanctonf;

        $sql = "SELECT NRSEQVENREC
                  FROM CONTARECEBER
                  WHERE NRORG         = :P_NRORG
                       AND CDFILIAL   = :P_CDFILIALNF
                       AND NRLANCTONF = :P_NRLANCTONF
                       AND IDSTATREC <> 'A'
                  ORDER BY DTATUAVENREC, NRSEQVENREC, NRPARCELAREC, VRATUAREC";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function verificaBoletoAssociado($nrorg, $cdfilialnf, $nrlanctonf){
        $params = array();

        $params['P_NRORG']      = $nrorg;
        $params['P_CDFILIALNF'] = $cdfilialnf;
        $params['P_NRLANCTONF'] = $nrlanctonf;

        $sql = "SELECT BR.CDFILIAL, BR.NRLANBOLREC, COUNT(1) AS QTD
                  FROM BOLETRECEBER BR, CONTARECEBER CR
                  WHERE BR.CDFILIAL = CR.CDFILIBOLREC
                       AND BR.NRLANBOLREC = CR.NRLANBOLREC
                       AND EXISTS (SELECT 1
                            FROM CONTARECEBER CT
                            WHERE CT.NRORG       = :P_NRORG
                              AND CT.CDFILIAL    = :P_CDFILIALNF
                              AND CT.NRLANCTONF  = :P_NRLANCTONF
                              AND BR.CDFILIAL    = CT.CDFILIBOLREC
                              AND BR.NRLANBOLREC = CT.NRLANBOLREC)
                        GROUP BY BR.CDFILIAL, BR.NRLANBOLREC HAVING COUNT(1) > 1";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function buscaDocOrigReneg( $cdfilialnf, $nrlanctonf){
        $params = array();

        $params['P_CDFILIALNF'] = $cdfilialnf;
        $params['P_NRLANCTONF'] = $nrlanctonf;
        
        $sql = "SELECT CR.CDFILIAL, CR.NRLANCTONF, CR.NRSEQVENREC, CR.NRPARCELAREC,
                       CR.CDEMPRESA, CR.CDCLASSFINA, CR.DTATUAVENREC, CR.CDFIMOVFLREC,
                       CR.VRATUAREC, CR.CDFILIBOLREC, CR.NRLANBOLREC, NF.NRLANCTBDOC, CR.VRORIGREC
                  FROM CONTARECEBER CR, NOTAFISCAL NF
                  WHERE CR.CDFILIAL   = :P_CDFILIALNF
                        AND CR.NRLANCTONF = :P_NRLANCTONF
                   AND CR.CDFILIAL   = NF.CDFILIAL
                   AND CR.NRLANCTONF = NF.NRLANCTONF";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function excluirRenegociacaoLanccont( $params){
        try{
            $sql = "DELETE FROM LANCCONT WHERE CDEMPRESA = :CDEMPRESA AND NRLANCCONT = :NRLANCCONT";
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
     /* busca os ajustes da conta*/
     
     public function buscaAjusteConta($dadosDocs){
        
        $sql = "SELECT AJ.CDEMMFAJCNTR, AJ.CDFIMFAJCNTR, AJ.CDCLASSFINA, AJ.DTMOVFLUXO,  AJ.VRAJUSTEREC, TA.IDAJUCTREC, AJ.NRLCTBAJREC
                    FROM AJCTRECEBER AJ, TIPOAJRECEBER TA
                    WHERE AJ.CDFILIAL = :P_CDFILIAL
                        AND AJ.NRLANCTONF = :P_NRLANCTONF
                        AND AJ.NRSEQVENREC = :P_NRSEQVENREC
                        AND AJ.NRPARCELAREC = :P_NRPARCELAREC
                        AND AJ.CDAJUCTREC = TA.CDAJUCTREC";

        $std = $this->connection->prepare($sql);
        $std->execute($dadosDocs);
        return $std->fetchAll();
    }
    
    public function excluirAjusteRec( $params){
        try{
            
            $sql = "DELETE FROM AJCTRECEBER
                    WHERE CDFILIAL = :P_CDFILIAL
                       AND NRLANCTONF = :P_NRLANCTONF
                       AND NRSEQVENREC = :P_NRSEQVENREC
                       AND NRPARCELAREC = :P_NRPARCELAREC";
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function excluirContaRec( $params){
        try{
            
            $sql = "DELETE FROM CONTARECEBER
                    WHERE CDFILIAL = :P_CDFILIAL
                       AND NRLANCTONF = :P_NRLANCTONF
                       AND NRSEQVENREC = :P_NRSEQVENREC
                       AND NRPARCELAREC = :P_NRPARCELAREC";
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function excluirBoletoRec( $params){
        try{
            $sql = "DELETE FROM BOLETRECEBER
                        WHERE CDFILIAL = :P_CDFILIAL
                        AND NRLANBOLREC = :P_NRLANBOLREC";
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function excluirNotaFiscal( $params){
        try{
            
            $sql = "DELETE FROM NOTAFISCAL
                    WHERE CDFILIAL = :P_CDFILIAL
                       AND NRLANCTONF = :P_NRLANCTONF";
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function excluirOrcamentosEventos( $params){
        try{
            $sql = "DELETE FROM NFRELACORC
                    WHERE CDFILIALNF = :P_CDFILIAL
                       AND NRLANCTONF = :P_NRLANCTONF";
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    /*busca a baixa da negociação para atualizar o status do boleto e da conta */
    public function buscaBaixaNegociacao($params){
        
        $sql = "SELECT CDFILIAL, NRBAIXAREC, CDEMPRESA, NRLCTBBREC
                  FROM BAIXARECEBIDA
                  WHERE NRSEQHISTRENEGO = :P_NRSEQHISTRENEGO
                    AND CDCLIENTERENEGO = :P_CDCLIENTERENEGO";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /* busca todos os boletos da baixa de renegociação*/
    public function buscaBoletoBaixaRenegociacao($params){
        
        $sql = "SELECT BO.CDFILIAL, BO.NRLANBOLREC , BO.CDBARRAREC
                  FROM BOLETRECEBER BO, CONTARECEBER CR
                  WHERE BO.CDFILIAL = CR.CDFILIBOLREC
                   AND BO.NRLANBOLREC = CR.NRLANBOLREC
                   AND BO.CDFILIBAIREC = :P_CDFILIBAIREC
                   AND BO.NRBAIXAREC = :P_NRBAIXAREC";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /* busca o ajuste da renegociação*/
    public function buscaAjusteRenegociacao($params){
        
        $sql = "SELECT SUM(DECODE(TA.IDAJUCTREC, 'D', -VRAJBOLREC, VRAJBOLREC)) VRAJBOLREC
                 FROM BOLETRECEBER BO, AJBOLREC AJ, TIPOAJRECEBER TA
                    WHERE AJ.CDFILIAL = BO.CDFILIAL
                       AND AJ.NRLANBOLREC = BO.NRLANBOLREC
                       AND AJ.CDAJUCTREC = TA.CDAJUCTREC
                       AND BO.CDFILIAL = :P_CDFILIAL
                       AND BO.NRLANBOLREC = :P_NRLANBOLREC
                       AND ((AJ.IDORIGAJBREC = '2') OR (AJ.IDORIGAJBREC = '3'))";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function excluirAjusteRenegociacao( $params){
        try{
            $sql = "DELETE FROM AJBOLREC
                    WHERE CDFILIAL = :P_CDFILIAL
                             AND NRLANBOLREC = :P_NRLANBOLREC
                      AND ((IDORIGAJBREC = '2') OR (IDORIGAJBREC = '3'))";
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    /* atualiza os boletos da baixa de renegociação*/
    public function updateBoletoBaixaNegociacao( $params){
        
        $sql = "UPDATE BOLETRECEBER
                   SET CDFILIBAIREC = NULL,
                        NRBAIXAREC = NULL,
                        VRFINALBREC = VRFINALBREC - NVL(:P_VRAJUSTE, 0),
                        IDSTATBREC = :P_IDSTATBREC,
                        DTLIQBOLREC = NULL
                   WHERE CDFILIAL = :P_CDFILIAL
                        AND NRLANBOLREC = :P_NRLANBOLREC";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /* atualiza as contas da baixa de renegociação*/
    public function updateContaBaixaNegociacao( $params){
        
        $sql = "UPDATE CONTARECEBER
                   SET IDSTATREC = 'A',
                       NRSEQHISTRENEGO = null,
                       CDCLIENTERENEGO = null
                   WHERE CDFILIBOLREC = :P_CDFILIBOLREC
                       AND NRLANBOLREC = :P_NRLANBOLREC";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /* deleta lançamento contábil da baixa*/
    public function excluiLancContabilBaixa( $params){
        try{
            $sql = "DELETE FROM LANCCONT WHERE CDEMPRESA = :CDEMPRESA AND NRLANCCONT = :NRLANCCONT";
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    /*deleta a baixa da negociação*/
    public function excluiBaixaNegociacao( $params){
        try{
            $sql = "DELETE FROM BAIXARECEBIDA WHERE CDFILIAL = :P_CDFILIAL AND NRBAIXAREC = :P_NRBAIXAREC ";
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function excluiHistorico( $params){
        try{
            $sql = "DELETE HISTRENEGOCFIN WHERE CDCLIENTE = :CDCLIENTE AND NRSEQHISTRENEGO = :NRSEQHISTRENEGO";
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
}