<?php

namespace Teknisa\FIN\Modules\ContaReceber\Factory;

use Teknisa\Model\Geral\Entities\Municipio;
use Teknisa\Model\Geral\Entities\Endefili;
use Teknisa\Model\Geral\Entities\Cliente;
use Teknisa\Model\Geral\Entities\Filial;
use Teknisa\Model\Geral\Entities\Empresa;
use Teknisa\Model\Geral\Entities\Histpadr;
use Teknisa\Model\Geral\Entities\Contcorr;
use Teknisa\Model\Geral\Entities\Inscrestad;
use Teknisa\Model\Fiscal\Entities\Numerorps;
use Teknisa\Model\Fiscal\Entities\Notafiscal;
use Teknisa\Model\Financeiro\Entities\Tipobaixarec;
use Teknisa\Model\Financeiro\Entities\Devolucaorec;
use Teknisa\Model\Financeiro\Entities\Boletreceber;
use Teknisa\Model\Financeiro\Entities\Tipoctreceber;
use Teknisa\Model\Financeiro\Entities\Ratctreceber;
use Teknisa\Model\Financeiro\Entities\Rattpctr;
use Teknisa\Model\Financeiro\Entities\Ajbolrec;
use Teknisa\Model\Financeiro\Entities\Histrenegocfin;
use Teknisa\Model\Geral\Entities\Centcustfil;
use Teknisa\Model\Financeiro\Entities\Emprbanc;
use Teknisa\Model\Financeiro\Entities\Baixarecebida;
use Teknisa\Model\Financeiro\Entities\Cnabrec;
use Teknisa\Model\Financeiro\Entities\Tpdebibanc;
use Teknisa\Model\Financeiro\Entities\Movbancario;
use Teknisa\Model\Financeiro\Entities\Adiantareceb;
use Teknisa\Model\Contabilidade\Entities\Lanccont;
use Teknisa\Model\Financeiro\Entities\Contareceber;
use Teknisa\Model\Financeiro\Entities\Ajctreceber;
use Teknisa\Model\Financeiro\Entities\Tipoajreceber;
use Teknisa\Model\Financeiro\Entities\Tpcredbanc;
use Teknisa\Model\Financeiro\Entities\Movcredbanc;
use Teknisa\Model\Financeiro\Entities\Movdebbanc;
use Teknisa\Model\Fiscal\Entities\Impbaixarec;
use Doctrine\ORM\EntityManager;
use Indeterminado\Entities\Logboletoremrec;
use Teknisa\Libs\Util\DateUtil;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Teknisa\Model\Financeiro\Entities\GeralRec;
use Teknisa\Model\Eventos\Entities\Nfrelacorc;

/**
 * Class TitulosAReceberFactory
 * @package Teknisa\FIN\Modules\ContaReceber\Factory
 */
class TitulosAReceberFactory {

    protected $entityManager;

    /**
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    /**
     * Busca Filial
     * @param $nrorg
     * @param $cdfiliemit
     * @return Filial
     */
    public function buscaFilial($nrorg, $cdfiliemit){
        /** @var $params */
        $params = array();
        $params['nrorg']      = $nrorg;
        $params['cdfiliemit'] = $cdfiliemit;

        /** @var Filial $filial */
        $filial = $this->entityManager->getRepository(Filial::class)->findOneBy($params);

        return $filial;
    }

    /**
     * Busca Cliente
     * @param $nrorg
     * @param $cdcliente
     * @return Cliente
     */
    public function buscaCliente($nrorg, $cdcliente){
        /** @var $params */
        $params = array();
        $params['nrorg']     = $nrorg;
        $params['cdcliente'] = $cdcliente;

        /** @var Cliente $cliente */
        $cliente = $this->entityManager->getRepository(Cliente::class)->findOneBy($params);

        return $cliente;
    }

    /**
     * Busca Empresa
     * @param $nrorg
     * @param $cdempresa
     * @return Empresa
     */
    public function buscaEmpresa($nrorg, $cdempresa){
        /** @var $params */
        $params = array();
        $params['nrorg']     = $nrorg;
        $params['cdempresa'] = $cdempresa;

        /** @var Empresa $empresa */
        $empresa = $this->entityManager->getRepository(Empresa::class)->findOneBy($params);

        return $empresa;
    }

    /**
     * Busca Empresa Banco
     * @param $nrorg
     * @param $cdempresa
     * @param $cdbanco
     * @return Empresa
     */
    public function buscaEmprbanc($nrorg, $cdempresa, $cdbanco){
        /** @var $params */
        $params = array();
        $params['nrorg']     = $nrorg;
        $params['cdempresa'] = $cdempresa;
        $params['cdbanco']   = $cdbanco;

        /** @var Empresa $emprbanc */
        $emprbanc = $this->entityManager->getRepository(Emprbanc::class)->findOneBy($params);

        return $emprbanc;
    }

    /**
     * Busca Nota Fiscal Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @return Notafiscal
     */
    public function buscaNotaFiscalReceber($nrorg, $cdfilial, $nrlanctonf){
        /** @var $params */
        $params = array();
        $params['nrorg']      = $nrorg;
        $params['cdfilial']   = $cdfilial;
        $params['nrlanctonf'] = $nrlanctonf;

        /** @var Notafiscal $notafiscal */
        $notafiscal = $this->entityManager->getRepository(Notafiscal::class)->findOneBy($params);

        return $notafiscal;
    }

    /**
     * Busca Banco Preferecial Dados Boleto
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @param $cdreducont
     * @return array
     */
    public function buscaBancoPrefDadosBoleto($nrorg, $cdcliente, $cdempresa, $cdreducont){
        /** @var $params */
        $params = array();

        $params['P_CDCLIENTE']    = $cdcliente;
        $params['P_CDEMPRESA'] = $cdempresa;
        $params['P_CDREDUCONT'] = $cdreducont;
        $params['P_NRORG']       = $nrorg;
        
        $sql = "SELECT
                BP.CDBANCO,
                BP.CDREDUCONT,
                EB.CDCARTCOBR,
                EB.QTDIAPROTEST,
                EB.TXMENSBOLETO,
                EB.IDIMPRBOLETO,
                NVL(BP.CDINS1COBRCL,EB.CDINS1COBR) CDINS1COBRCL,
                NVL(BP.CDINS2COBRCL,EB.CDINS2COBR) CDINS2COBRCL,
                EB.IDCARTCOBR
             FROM
                 BANCOPREF BP,
                 CONTCORR CC,
                 EMPRBANC EB
             WHERE
                 ( BP.NRORG = CC.NRORG )
                 AND ( BP.CDCLIENTE =:P_CDCLIENTE )
                 AND ( BP.CDREDUCONT = CC.CDREDUCONT )
                 AND ( CC.CDEMPRESA =:P_CDEMPRESA )
                 AND ( BP.IDPRIORBPREF = 'S' )
                 AND ( EB.NRORG (+) = CC.NRORG )
                 AND ( EB.CDBANCO (+) = CC.CDBANCO )
                 AND ( EB.CDEMPRESA (+) = CC.CDEMPRESA )
                 AND ( EB.NRORG = :P_NRORG )
                 AND ('T' = NVL(:P_CDREDUCONT, 'T'))
            UNION ALL
            SELECT
                CC.CDBANCO,
                CC.CDREDUCONT,
                EB.CDCARTCOBR,
                EB.QTDIAPROTEST,
                EB.TXMENSBOLETO,
                EB.IDIMPRBOLETO,
                EB.CDINS1COBR CDINS1COBRCL,
                EB.CDINS2COBR CDINS2COBRCL,
                EB.IDCARTCOBR
             FROM
                 CONTCORR CC,
                 EMPRBANC EB
             WHERE
                     ( CC.CDEMPRESA = :P_CDEMPRESA )
                 AND ( EB.NRORG (+) = CC.NRORG )
                 AND ( EB.CDBANCO (+) = CC.CDBANCO )
                 AND ( EB.CDEMPRESA (+) = CC.CDEMPRESA )
                 AND ( CC.CDREDUCONT = :P_CDREDUCONT)
                 AND ( EB.NRORG =:P_NRORG )";
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetch();
    }

    /**
     * Busca Boleto a Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @return Boletreceber
     */
    public function buscaBoletoReceber($nrorg, $cdfilial, $nrlanbolrec){
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        $params['cdfilial']    = $cdfilial;
        $params['nrlanbolrec'] = $nrlanbolrec;

        /** @var Boletreceber $boletreceber */
        $boletreceber = $this->entityManager->getRepository(Boletreceber::class)->findOneBy($params);

        return $boletreceber;
    }

    /**
     * Busca Nota Fiscal Relacionada ao Orçamento de Eventos
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @return $nrlanctonf
     */
    public function buscaNfrelacorc($nrorg, $cdfilial, $nrlanctonf){
        /** @var $params */
        $params = array();
        $params['nrorg']      = $nrorg;
        $params['cdfilial']   = $cdfilial;
        $params['nrlanctonf'] = $nrlanctonf;

        /** @var Nfrelacorc $nfrelacorc */
        $nfrelacorc = $this->entityManager->getRepository(Nfrelacorc::class)->findOneBy($params);

        return $nfrelacorc;
    }

    /**
     * Busca Boleto Receber - Mivmento de Crédito
     * @param $nrorg
     * @param $cdreducont
     * @param $nrmovicred
     * @return Boletreceber
     */
    public function buscaBoletosReceberMovCred($nrorg, $cdreducont, $nrmovicred){
        /** @var $params */
        $params = array();
        $params['nrorg']      = $nrorg;
        $params['cdreducont'] = $cdreducont;
        $params['nrmovicred'] = $nrmovicred;

        /** @var Boletreceber $boletreceber */
        $boletreceber = $this->entityManager->getRepository(Boletreceber::class)->findBy($params);

        return $boletreceber;
    }

    /**
     * Busca Ajuste Conta Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenrec
     * @param $nrparcelarec
     * @return Ajctreceber
     */
    public function buscaAjctreceber($nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdfilial']     = $cdfilial;
        $params['nrlanctonf']   = $nrlanctonf;
        $params['nrseqvenrec']  = $nrseqvenrec;
        $params['nrparcelarec'] = $nrparcelarec;

        /** @var Ajctreceber $ajctreceber */
        $ajctreceber = $this->entityManager->getRepository(Ajctreceber::class)->findBy($params);

        return $ajctreceber;
    }

    /**
     * Busca Tipo de Ajuste a Receber
     * @param $nrorg
     * @param $cdajuctrec
     * @return Tipoajreceber
     */
    public function buscaTipoajreceber($nrorg, $cdajuctrec){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdajuctrec']   = $cdajuctrec;

        /** @var Tipoajreceber $tipoajreceber */
        $tipoajreceber = $this->entityManager->getRepository(Tipoajreceber::class)->findOneBy($params);

        return $tipoajreceber;
    }

    /**
     *
     * @param $nrorg
     * @param $cdtpctreceb
     * @param $idutlanccrec
     * @return Tipoctreceber
     */
    public function buscaTipoContaReceber($nrorg, $cdtpctreceb, $idutlanccrec){
        /** @var $params */
        $params = array();
        $params['nrorg']         = $nrorg;
        $params['$cdtpctreceb']  = $cdtpctreceb;
        $params['$idutlanccrec'] = $idutlanccrec;

        /** @var Tipoctreceber $tipoctreceber */
        $tipoctreceber = $this->entityManager->getRepository(Tipoctreceber::class)->findOneBy($params);

        return $tipoctreceber;
    }

    /**
     * Busca Rateio Conta Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenrec
     * @param $nrparcelarec
     * @return Ratctreceber
     */
    public function buscaRatsctreceber($nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdfilial']     = $cdfilial;
        $params['nrlanctonf']   = $nrlanctonf;
        $params['nrseqvenrec']  = $nrseqvenrec;
        $params['nrparcelarec'] = $nrparcelarec;

        /** @var Ratctreceber $ratctreceber */
        $ratctreceber = $this->entityManager->getRepository(Ratctreceber::class)->findBy($params);

        return $ratctreceber;
    }

    /**
     * Busca Rateio Conta Receber
     * @param $nrorg
     * @param $nrlanctonf
     * @param $cdfilial
     * @param $nrseqvenrec
     * @param $nrparcelarec
     * @param $cdfiratctrec
     * @return Ratctreceber
     */
    public function buscaRatCtReceber($nrorg, $nrlanctonf, $cdfilial, $nrseqvenrec, $nrparcelarec, $cdfiratctrec){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdfilial']     = $cdfilial;
        $params['nrseqvenrec']  = $nrseqvenrec;
        $params['nrparcelarec'] = $nrparcelarec;
        $params['cdfiratctrec'] = $cdfiratctrec;
        $params['nrlanctonf']   = $nrlanctonf;

        /** @var Ratctreceber $ratctreceber */
        $ratctreceber = $this->entityManager->getRepository(Ratctreceber::class)->findBy($params);

        return $ratctreceber;
    }

    /**
     * Busca Rateio Tipo de Conta Receber
     * @param $nrorg
     * @param $cdtpctreceb
     * @return Rattpctr
     */
    public function buscaRatTpCtR($nrorg, $cdtpctreceb){
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        $params['cdtpctreceb'] = $cdtpctreceb;

        /** @var Rattpctr $rattpctr */
        $rattpctr = $this->entityManager->getRepository(Rattpctr::class)->findBy($params);

        return $rattpctr;
    }


    /**
     * Busca Ajuste Boleto Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @param string $idorigajbrec
     * @return Ajbolrec
     */
    public function buscaAjusteBoletoReceber($nrorg, $cdfilial, $nrlanbolrec, $idorigajbrec = '4')
    {
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdfilial']     = $cdfilial;
        $params['nrlanbolrec']  = $nrlanbolrec;
        $params['idorigajbrec'] = $idorigajbrec;

        /** @var Ajbolrec $ajbolrec */
        $ajbolrec = $this->entityManager->getRepository(Ajbolrec::class)->findBy($params);

        return $ajbolrec;
    }

    /**
     * Busca Centro de Custo Filial
     * @param $nrorg
     * @param $cdfiliemit
     * @return Centcustfil
     */
    public function buscaCentCustFil($nrorg, $cdfiliemit){
        /** @var $params */
        $params = array();
        $params['nrorg']      = $nrorg;
        $params['cdfiliemit'] = $cdfiliemit;

        /** @var Centcustfil $centcustfil */
        $centcustfil = $this->entityManager->getRepository(Centcustfil::class)->findOneBy($params);

        return $centcustfil;
    }

    /**
     * Busca Conta Receber
     * @param $nrorg
     * @param $cdfilibolrec
     * @param $nrlanbolrec
     * @return Contareceber
     */
    public function buscaContareceber($nrorg, $cdfilibolrec, $nrlanbolrec){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdfilibolrec'] = $cdfilibolrec;
        $params['nrlanbolrec' ] = $nrlanbolrec;

        /** @var Contareceber $contareceber */
        $contareceber = $this->entityManager->getRepository(Contareceber::class)->findOneBy($params);

        return $contareceber;
    }

    /**
     * Busca Conta Receber
     * @param $nrorg
     * @param $cdfilibolrec
     * @param $nrlanbolrec
     * @return Contareceber
     */
    public function buscaContareceberDuplicata($nrorg, $cdfilial, $cdfilibolrec, $nrlanctonf, $nrlanbolrec, $nrlanctodup){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdfilial']     = $cdfilial;
        $params['cdfilibolrec'] = $cdfilibolrec;
        $params['nrlanctonf' ] = $nrlanctonf;
        $params['nrlanbolrec' ] = $nrlanbolrec;
        $params['nrlanctodup' ] = $nrlanctodup;

        /** @var Contareceber $contasreceber */
        $contasreceber = $this->entityManager->getRepository(Contareceber::class)->findBy($params);

        return $contasreceber;
    }

    /**
     * Conta Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrparcelarec
     * @param $nrseqvenrec
     * @return Contareceber
     */
    public function buscaContareceberParcela($nrorg, $cdfilial, $nrlanctonf, $nrparcelarec, $nrseqvenrec){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdfilial']     = $cdfilial;
        $params['nrlanctonf']   = $nrlanctonf;
        $params['nrparcelarec'] = $nrparcelarec;
        $params['nrseqvenrec' ] = $nrseqvenrec;

        /** @var Contareceber $contareceber */
        $contareceber = $this->entityManager->getRepository(Contareceber::class)->findOneBy($params);

        return $contareceber;
    }

    /**
     * Busca Conta Receber Renegociadas
     * @param $nrorg
     * @param $nrseqhistrenego
     * @param $cdclienterenego
     * @return Contareceber
     */
    public function buscaContasreceberRenegoc($nrorg, $nrseqhistrenego, $cdclienterenego){
        /** @var $params */
        $params = array();
        $params['nrorg']            = $nrorg;
        $params['nrseqhistrenego']  = $nrseqhistrenego;
        $params['cdclienterenego' ] = $cdclienterenego;

        /** @var Contareceber $contareceber */
        $contareceber = $this->entityManager->getRepository(Contareceber::class)->findBy($params);

        return $contareceber;
    }

    /**
     * Busca Ajuste Conta Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @param $cdajuctrec
     * @return Ajbolrec
     */
    public function buscaAjbolrecCod($nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec){
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        $params['cdfilial']    = $cdfilial;
        $params['nrlanbolrec'] = $nrlanbolrec;
        $params['cdajuctrec']  = $cdajuctrec;

        /** @var Ajbolrec $ajbolrec */
        $ajbolrec = $this->entityManager->getRepository(Ajbolrec::class)->findOneBy($params);

        return $ajbolrec;
    }

    /**
     * Busca Ajuste Conta Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @return Ajbolrec
     */
    public function buscaAjbolrec($nrorg, $cdfilial, $nrlanbolrec){
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        $params['cdfilial']    = $cdfilial;
        $params['nrlanbolrec'] = $nrlanbolrec;

        /** @var Ajbolrec $ajbolrec */
        $ajbolrec = $this->entityManager->getRepository(Ajbolrec::class)->findBy($params);

        return $ajbolrec;
    }

    /**
     * Busca Histórico Renegociação Financeira
     * @param $nrorg
     * @param $nrseqhistrenego
     * @param $cdcliente
     * @return Histrenegocfin
     */
    public function buscaHistrenegocfin($nrorg, $nrseqhistrenego, $cdcliente){
        /** @var $params */
        $params = array();
        $params['nrorg']           = $nrorg;
        $params['nrseqhistrenego'] = $nrseqhistrenego;
        $params['cdcliente']       = $cdcliente;

        /** @var Histrenegocfin $histrenegocfin */
        $histrenegocfin = $this->entityManager->getRepository(Histrenegocfin::class)->findOneBy($params);

        return $histrenegocfin;
    }

    /**
     * Busca Nota Fiscal
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @return Notafiscal
     */
    public function buscaNotafiscal($nrorg, $cdfilial, $nrlanctonf){
        /** @var $params */
        $params = array();
        $params['nrorg']      = $nrorg;
        $params['cdfilial']   = $cdfilial;
        $params['nrlanctonf'] = $nrlanctonf;

        /** @var Notafiscal $notafiscal */
        $notafiscal = $this->entityManager->getRepository(Notafiscal::class)->findOneBy($params);

        return $notafiscal;
    }

    /**
     * Busca Inscrição Estadual
     * @param $nrorg
     * @param $cdinscesta
     * @param $cdempresa
     * @return Inscrestad
     */
    public function buscaInscrestad($nrorg, $cdinscesta, $cdempresa){
        /** @var $params */
        $params = array();
        $params['nrorg']      = $nrorg;
        $params['cdinscesta'] = $cdinscesta;
        $params['cdempresa']  = $cdempresa;

        /** @var Inscrestad $inscrestad */
        $inscrestad = $this->entityManager->getRepository(Inscrestad::class)->findOneBy($params);

        return $inscrestad;
    }

    /**
     * Busca Número RPS
     * @param $nrorg
     * @param $nrrps
     * @param $cdserierps
     * @param $cdemploterps
     * @param $nrinsjuriest
     * @return Numerorps
     */
    public function buscaNumerorps($nrorg, $nrrps, $cdserierps, $cdemploterps, $nrinsjuriest){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['nrrps']        = $nrrps;
        $params['cdserierps']   = $cdserierps;
        $params['cdemploterps'] = $cdemploterps;
        $params['nrinsjuriest'] = $nrinsjuriest;

        /** @var Numerorps $numerorps */
        $numerorps = $this->entityManager->getRepository(Numerorps::class)->findOneBy($params);

        return $numerorps;
    }

    /**
     * Busca Contas Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @return Contareceber
     */
    public function buscaContasreceber($nrorg, $cdfilial, $nrlanctonf){
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        $params['cdfilial']    = $cdfilial;
        $params['nrlanctonf' ] = $nrlanctonf;

        /** @var Centcustfil $centcustfil */
        $centcustfil = $this->entityManager->getRepository(Contareceber::class)->findBy($params);

        return $centcustfil;
    }

    /**
     * Busca Contas Receber
     * @param $nrorg
     * @param $cdfilibolrec
     * @param $nrlanbolrec
     * @return Contareceber
     */
    public function buscaContasreceberBol($nrorg, $cdfilibolrec, $nrlanbolrec){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdfilibolrec'] = $cdfilibolrec;
        $params['nrlanbolrec' ] = $nrlanbolrec;

        /** @var Contareceber $contareceber */
        $contareceber = $this->entityManager->getRepository(Contareceber::class)->findBy($params);

        return $contareceber;
    }


    /**
     * Busca Lançamento Contabil
     * @param $nrorg
     * @param $cdempresa
     * @param $nrlanccont
     * @return Lanccont
     */
    public function buscaLanccont($nrorg, $cdempresa, $nrlanccont){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdempresa'] = $cdempresa;
        $params['nrlanccont'] = $nrlanccont;

        /** @var Lanccont $lanccont */
        $lanccont = $this->entityManager->getRepository(Lanccont::class)->findBy($params);

        return $lanccont;
    }

    /**
     * busca Nota Fical
     * @param nrorg
     * @param cdfilirece
     * @param nrnotafisc
     * @param cdserieentr
     * @param nrinsjurnf
     * @param vrnotafisc
     * @return Notafical
     */
    public function buscaNotaFical($cdfilirece, $nrnotafisc, $cdserieentr, $nrinsjurnf, $vrnotafisc, $nrorg){
        /** @var $params */
        $params = array();
        $params['nrorg'] = $nrorg;
        $params['cdfilirece'] = $cdfilirece;
        $params['nrnotafisc'] = $nrnotafisc;
        $params['cdserieentr'] = $cdserieentr;
        $params['nrinsjurnf'] = $nrinsjurnf;
        $params['vrnotafisc'] = $vrnotafisc;
        $params['idsistema'] = 'REC';
        $params['identrsaidop'] = 'S';

        /** @var Notafical $notaFiscal */
        $notaFiscal = $this->entityManager->getRepository(Notafiscal::class)->findOneBy($params);
        return $notaFiscal;
    }

    /**
     * Busca Conta Corrente
     * @param $nrorg
     * @param $cdreducont
     * @return Contcorr
     */
    public function buscaContcorr($nrorg, $cdreducont){
        /** @var $params */
        $params = array();
        $params['nrorg'] = $nrorg;
        $params['cdreducont'] = $cdreducont;

        /** @var Contcorr $contcorr */
        $contcorr = $this->entityManager->getRepository(Contcorr::class)->findOneBy($params);
        return $contcorr;
    }

    /**
     * Busca Tipo de Crédito Bancário
     * @param $nrorg
     * @param $cdtpcredbanc
     * @return Tpcredbanc
     */
    public function buscaTpcredbanc($nrorg, $cdtpcredbanc){
        /** @var $params */
        $params = array();
        $params['nrorg'] = $nrorg;
        $params['cdtpcredbanc'] = $cdtpcredbanc;

        /** @var Tpcredbanc $tpcredbanc */
        $tpcredbanc = $this->entityManager->getRepository(Tpcredbanc::class)->findOneBy($params);
        return $tpcredbanc;
    }

    /**
     * Busca Movimentação de Crédito Bancário
     * @param $nrorg
     * @param $cdreducont
     * @param $nrmovicred
     * @return Movcredbanc
     */
    public function buscaMovcredbanc($nrorg, $cdreducont, $nrmovicred){
        /** @var $params */
        $params = array();
        $params['nrorg'] = $nrorg;
        $params['cdreducont'] = $cdreducont;
        $params['nrmovicred'] = $nrmovicred;

        /** @var Movcredbanc $movcredbanc */
        $movcredbanc = $this->entityManager->getRepository(Movcredbanc::class)->findOneBy($params);
        return $movcredbanc;
    }

    /**
     * Busca Movimentação de Crédito Bancário
     * @param $nrorg
     * @param $cdreducont
     * @param $nrmovbanc
     * @return Movcredbanc
     */
    public function buscaMovbancario($nrorg, $cdreducont,$nrmovbanc){
        /** @var $params */
        $params = array();
        $params['nrorg']      = $nrorg;
        $params['cdreducont'] = $cdreducont;
        $params['nrmovbanc']  = $nrmovbanc;

        /** @var Movbancario $movbancario */
        $movbancario = $this->entityManager->getRepository(Movbancario::class)->findOneBy($params);
        return $movbancario;
    }
    
    /**
     * Busca Boleto a Receber Originais
     * @param $nrorg
     * @param $cdfilial
     * @param $nrorigbolrec
     * @return Boletreceber
     */
    public function getBoletosOriginais($nrorg, $cdfilial, $nrorigbolrec){
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        $params['cdfilial']    = $cdfilial;
        $params['nrorigbolrec'] = $nrorigbolrec;

        /** @var Boletreceber $boletreceber */
        $boletreceber = $this->entityManager->getRepository(Boletreceber::class)->findBy($params);

        return $boletreceber;
    }
    
    /**
     * Busca Boleto a Receber Parcelado
     * @param $nrorg
     * @param $cdfilial
     * @param $nrorigbolrec
     * @param $nrparcbolrec
     * @return Boletreceber
     */
    public function buscaBoletoParcelado($nrorg, $cdfilial, $nrorigbolrec, $nrparcbolrec){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdfilial']     = $cdfilial;
        $params['nrorigbolrec'] = $nrorigbolrec;
        $params['nrparcbolrec'] = $nrparcbolrec;

        /** @var Boletreceber $boletreceber */
        $boletreceber = $this->entityManager->getRepository(Boletreceber::class)->findOneBy($params);

        return $boletreceber;
    }

    /**
     * Busca Histórico Padrão
     * @param $nrorg
     * @param $cdhistpadr
     * @return Histpadr
     */
    public function buscaHistpadr($nrorg, $cdhistpadr){
        /** @var $params */
        $params = array();
        $params['nrorg'] = $nrorg;
        $params['cdhistpadr'] = $cdhistpadr;

        /** @var Histpadr $histpadr */
        $histpadr = $this->entityManager->getRepository(Histpadr::class)->findOneBy($params);
        return $histpadr;
    }

    /**
     * Busca Baixa Recebida
     * @param $nrorg
     * @param $cdfilial
     * @param $nrbaixarec
     * @return Baixarecebida
     */
    public function buscaBaixarecebida($nrorg, $cdfilial, $nrbaixarec){
        /** @var $params */
        $params = array();
        $params['nrorg'] = $nrorg;
        $params['cdfilial'] = $cdfilial;
        $params['nrbaixarec'] = $nrbaixarec;

        /** @var Baixarecebida $baixarecebida */
        $baixarecebida = $this->entityManager->getRepository(Baixarecebida::class)->findOneBy($params);
        return $baixarecebida;
    }

    /**
     * Busca Tipo de Baixa Recebida
     * @param $nrorg
     * @param $cdtpbaixarec
     * @return Tipobaixarec
     */
    public function buscaTipobaixarec($nrorg, $cdtpbaixarec){
        /** @var $params */
        $params = array();
        $params['nrorg'] = $nrorg;
        $params['cdtpbaixarec'] = $cdtpbaixarec;

        /** @var Tipobaixarec $tipobaixarec */
        $tipobaixarec = $this->entityManager->getRepository(Tipobaixarec::class)->findOneBy($params);
        return $tipobaixarec;
    }

    /**
     * Busca Boletos Baixa
     * @param $nrorg
     * @param $cdfilibairec
     * @param $nrbaixarec
     * @return Boletreceber
     */
    public function buscaBoletosBaixa($nrorg, $cdfilibairec, $nrbaixarec){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdfilibairec'] = $cdfilibairec;
        $params['nrbaixarec']   = $nrbaixarec;

        /** @var Boletreceber $boletreceber */
        $boletreceber = $this->entityManager->getRepository(Boletreceber::class)->findBy($params);
        return $boletreceber;
    }

    /**
     * busca parcela
     * @param nrorg
     * @param cdfilirece
     * @param nrnotafisc
     * @param cdserieentr
     * @param nrinsjurnf
     * @param vrnotafisc
     * @return array
     */
    public function buscaParcela($cdfiliemit, $nrnotafisc, $cdserieentr, $nrinsjurnf, $vrnotafisc, $dtemissao, $nrorg){
        $params = array();
        $params['P_CDFILIEMIT'] = $cdfiliemit;
        $params['P_NRNOTAFISC'] = $nrnotafisc;
        $params['P_CDSERIE'] = $cdserieentr;
        $params['P_NRINSJURNF'] = $nrinsjurnf;
        $params['P_VRNOTAFISC'] = $vrnotafisc;
        $params['P_DTEMISSAO'] = $dtemissao;
        $params['P_NRORG'] = $nrorg;
        $sql = "SELECT CDFILIAL, NRLANCTONF
                  FROM NOTAFISCAL
                 WHERE CDFILIEMIT = :P_CDFILIEMIT
                   AND NRNOTAFISC = :P_NRNOTAFISC
                   AND CDSERIE = :P_CDSERIE
                   AND NRINSJURNF = :P_NRINSJURNF
                   AND VRNOTAFISC = :P_VRNOTAFISC
                   AND DTEMISSAO = :P_DTEMISSAO
                   AND NRORG = :P_NRORG
                   AND IDSISTEMA = 'REC'
                   AND IDENTRSAIDOP = 'S'";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }

    /**
     * Insere Movimento Bancario
     * @param $vrmovbanc
     * @param $cdreducont
     * @param $nrdocmbanc
     * @param $dshismbanc
     * @param $nrorg
     * @param $dataatual
     * @param $dtmovicred
     * @param $nrmovbanc
     * @return Movbancario
     */
    public function insereMovbancarioDescontado(
        $vrmovbanc, $cdreducont, $nrdocmbanc, $dshismbanc, $nrorg, $dataatual, $dtmovicred, $nrmovbanc, $idmovbanc = 'C'
    ){
        $movbancario = new Movbancario();
        $movbancario->setNrorg($nrorg);
        $movbancario->setCdreducont($cdreducont);
        $movbancario->setDtultatu($dataatual);
        $movbancario->setDtmovbanc($dtmovicred);
        $movbancario->setDshismbanc($dshismbanc);
        $movbancario->setNrdocmbanc($nrdocmbanc);
        $movbancario->setIdmovbanc($idmovbanc);
        $movbancario->setVrmovbanc($vrmovbanc);
        $movbancario->setIdorigmbanc('C');
        $movbancario->setIdstconext('0');
        $movbancario->setNrmovbanc($nrmovbanc);
        $movbancario->setDtinclusao($dataatual);

        return $movbancario;
    }

     /**
     * Insere Histórico de Renegociação Financeira
     * @param $nrorg
     * @param $cdempresa
     * @param $cdcliente
     * @param $nrseqhistrenego
     * @param $dthistrenegofin
     * @param $cdoperador
     * @param $dshistrenegofin
     * @param $cdfilialnf
     * @param $nrlanctonf
     * @return Histrenegocfin
     */
    public function insereHistrenegocfin(
        $nrorg, $cdempresa, $cdcliente, $nrseqhistrenego, $dthistrenegofin, $cdoperador, $dshistrenegofin, $cdfilialnf, $nrlanctonf
    ){
        $histrenegocfin = new Histrenegocfin();
        $histrenegocfin->setNrorg($nrorg);
        $histrenegocfin->setCdempresa($cdempresa);
        $histrenegocfin->setCdcliente($cdcliente);
        $histrenegocfin->setNrseqhistrenego($nrseqhistrenego);
        $histrenegocfin->setDthistrenegofin($dthistrenegofin);
        $histrenegocfin->setCdoperador($cdoperador);
        $histrenegocfin->setDshistrenegofin($dshistrenegofin);
        $histrenegocfin->setCdfilialnf($cdfilialnf);
        $histrenegocfin->setNrlanctonf($nrlanctonf);
        $histrenegocfin->setCdoperinclusao($cdoperador);
        $histrenegocfin->setCdoperultatu($cdoperador);
        $histrenegocfin->setDtinclusao(DateUtil::getDataAtual());
        $histrenegocfin->setDtultatu(DateUtil::getDataAtual());

        return $histrenegocfin;
    }

    /**
     * Insere Movimentação de Crédito Bancário
     * @param $cdreducont
     * @param $cdclassfina
     * @param $cdempresa
     * @param $cdfilial
     * @param $cdtpcredbanc
     * @param $dsmovicred
     * @param $vrmovicred
     * @param $nrlctbmcred
     * @param $cdfililancre
     * @param $nrorg
     * @param $dataatual
     * @param $nrmovbanc
     * @param $dtmovicred
     * @param $nrmovicred
     * @return mixed
     */
    public static function insereMovcredbanc(
        $cdreducont, $cdclassfina, $cdempresa, $cdfilial, $cdtpcredbanc, $dsmovicred, $vrmovicred, $nrlctbmcred,
        $cdfililancre, $nrorg, $dataatual, $nrmovbanc, $dtmovicred, $nrmovicred
    ){
        $movcredbanc = new Movcredbanc();
        $movcredbanc->setCdreducont($cdreducont);
        $movcredbanc->setNrmovicred($nrmovicred);
        $movcredbanc->setCdclassfina($cdclassfina);
        $movcredbanc->setCdempresa($cdempresa);
        $movcredbanc->setCdfilial($cdfilial);
        $movcredbanc->setDtmovfluxo($dataatual);
        $movcredbanc->setNrmovbanc($nrmovbanc);
        $movcredbanc->setCdtpcredbanc($cdtpcredbanc);
        $movcredbanc->setDtmovicred($dtmovicred);
        $movcredbanc->setDsmovicred($dsmovicred);
        $movcredbanc->setVrmovicred($vrmovicred);
        $movcredbanc->setCdectbmcred($cdempresa);
        $movcredbanc->setNrlctbmcred($nrlctbmcred);
        $movcredbanc->setCdfililancre($cdfililancre);
        $movcredbanc->setIdlancomcontrec('N');
        $movcredbanc->setIdativo('S');
        $movcredbanc->setNrorg($nrorg);
        $movcredbanc->setDtinclusao($dataatual);
        $movcredbanc->setDtultatu($dataatual);

        return $movcredbanc;

    }

    /**
     * Salvar Nota Fiscal Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $cdempresa
     * @param $cdinscesta
     * @param $cdinscmuni
     * @param $cdfiliemit
     * @param $idtpijurnf
     * @param $nrinsjurnf
     * @param $cdcliente
     * @param $idbloqueado
     * @param $iddocfiscal
     * @param $iddocfinanc
     * @param $vrprodutos
     * @param $vrnotafisc
     * @param $nrnotafisc
     * @param $dtemissao
     * @param $dtentrsaid
     * @param $identrsaidop
     * @param $cdtipooper
     * @param $cdserie
     * @param $iddoccontab
     * @param $idstatus
     * @param $iddocfiscent
     * @param $iddocfiscsai
     * @param $iddocestoque
     * @param $idsistema
     * @param $nrlicencauso
     * @param $cdvendedor
     * @param $dtlancamennf
     * @return Notafiscal
     */
    public function insereNotaFiscalReceber(
        $nrorg, $cdfilial, $nrlanctonf, $cdempresa, $cdinscesta, $cdinscmuni, $cdfiliemit,
        $idtpijurnf, $nrinsjurnf, $cdcliente, $idbloqueado, $iddocfiscal, $iddocfinanc,
        $vrprodutos, $vrnotafisc, $nrnotafisc, $dtemissao, $dtentrsaid, $identrsaidop,
        $cdtipooper, $cdserie, $iddoccontab, $idstatus, $iddocfiscent,
        $iddocfiscsai, $iddocestoque, $idsistema, $nrlicencauso, $cdvendedor, $dtlancamennf, $nrlanctbdoc
    ){
        /** @var Notafiscal $notafiscal */
        $notafiscal = new Notafiscal();
        $notafiscal->setNrorg($nrorg);
        $notafiscal->setCdfilial($cdfilial);
        $notafiscal->setNrlanctonf($nrlanctonf);
        $notafiscal->setCdempresa($cdempresa);
        $notafiscal->setCdinscesta($cdinscesta);
        $notafiscal->setCdinscmuni($cdinscmuni);
        $notafiscal->setCdfiliemit($cdfiliemit);
        $notafiscal->setIdtpijurnf($idtpijurnf);
        $notafiscal->setNrinsjurnf($nrinsjurnf);
        $notafiscal->setCdcliente($cdcliente);
        $notafiscal->setIdbloqueado($idbloqueado);
        $notafiscal->setIddocfiscal($iddocfiscal);
        $notafiscal->setIddocfinanc($iddocfinanc);
        $notafiscal->setVrprodutos($vrprodutos);
        $notafiscal->setVrnotafisc($vrnotafisc);
        $notafiscal->setNrnotafisc($nrnotafisc);
        $notafiscal->setDtemissao($dtemissao);
        $notafiscal->setDtentrsaid($dtentrsaid);
        $notafiscal->setIdentrsaidop($identrsaidop);
        $notafiscal->setCdtipooper($cdtipooper);
        $notafiscal->setCdserie($cdserie);
        $notafiscal->setIddoccontab($iddoccontab);
        $notafiscal->setIdstatus($idstatus);
        $notafiscal->setIddocfiscent($iddocfiscent);
        $notafiscal->setIddocfiscsai($iddocfiscsai);
        $notafiscal->setIddocestoque($iddocestoque);
        $notafiscal->setIdsistema($idsistema);
        $notafiscal->setNrlicencauso($nrlicencauso);
        $notafiscal->setCdvendedor($cdvendedor);
        $notafiscal->setDtlancamennf($dtlancamennf);
        $notafiscal->setDtinclusao(DateUtil::getDataAtual());
        $notafiscal->setNrlanctbdoc($nrlanctbdoc);
        return $notafiscal;
    }

    /**
     * Insere Boleto Receber
     * @param $nrorg
     * @param $cdfiliemit
     * @param $nrlanbolrec
     * @param $cdcliente
     * @param $cdempresa
     * @param $vrbolrec
     * @param $vrfinalbrec
     * @param $dtvenbolrec
     * @param $vrpemulbrec
     * @param $vrmultabrec
     * @param $vrpemorbrec
     * @param $vrmordiabrec
     * @param $cdreducont
     * @param $cdcartcobbol
     * @param $qtdiaprotbol
     * @param $txmensbolbol
     * @param $idimprbolbol
     * @param $cdins1cobbol
     * @param $cdins2cobbol
     * @dsboletrec
     * @cdnosnumrec
     * @cdbarrarec
     * @dslinhabarrec
     * @dscontarec
     * @cdtpbaixarec
     * @$cdoperinclusao
     * @return Boletreceber
     */
    public function insereBoletoReceber(
        $nrorg, $cdfiliemit, $nrlanbolrec, $cdcliente, $cdempresa, $vrbolrec, $vrfinalbrec,
        $dtvenbolrec, $vrpemulbrec, $vrmultabrec , $vrpemorbrec , $vrmordiabrec, $cdreducont,
        $cdcartcobbol, $idcartcobr, $qtdiaprotbol, $txmensbolbol, $idimprbolbol, $cdins1cobbol, $cdins2cobbol, $cddiaprotbol, $dsboletrec,
        $cdnosnumrec, $cdbarrarec, $dslinhabarrec, $dscontarec, $cdtpbaixarec, $cdoperinclusao
    ){
        /** @var Boletreceber $boletreceber */
        $boletreceber = new Boletreceber();
        $boletreceber->setNrorg($nrorg);
        $boletreceber->setCdfilial($cdfiliemit);
        $boletreceber->setNrlanbolrec($nrlanbolrec);
        $boletreceber->setCdcliente($cdcliente);
        $boletreceber->setCdempresa($cdempresa);
        $boletreceber->setVrbolrec($vrbolrec);
        $boletreceber->setVrfinalbrec($vrfinalbrec);
        $boletreceber->setDtvenbolrec($dtvenbolrec);
        $boletreceber->setVrpemulbrec($vrpemulbrec);
        $boletreceber->setVrmultabrec($vrmultabrec);
        $boletreceber->setVrpemorbrec($vrpemorbrec);
        $boletreceber->setVrmordiabrec($vrmordiabrec);
        $boletreceber->setCdreducont($cdreducont);
        $boletreceber->setCdcartcobbol($cdcartcobbol);
        $boletreceber->setIdcartcobrec($idcartcobr);
        $boletreceber->setQtdiaprotbol($qtdiaprotbol);
        $boletreceber->setTxmensbolbol($txmensbolbol);
        $boletreceber->setIdimprbolbol($idimprbolbol);
        $boletreceber->setCdins1cobbol($cdins1cobbol);
        $boletreceber->setCdins2cobbol($cdins2cobbol);
        $boletreceber->setCddiaprotbol($cddiaprotbol);
        $boletreceber->setDsboletrec($dsboletrec);
        $boletreceber->setCdnosnumrec($cdnosnumrec);
        $boletreceber->setCdbarrarec($cdbarrarec);
        $boletreceber->setDslinhabarrec($dslinhabarrec);
        $boletreceber->setCdtpbaixarec($cdtpbaixarec);
        $boletreceber->setCdoperinclusao($cdoperinclusao);
        $boletreceber->setDtinclusao(DateUtil::getDataAtual());
        $boletreceber->setDtultatu(DateUtil::getDataAtual());
        $boletreceber->setIdativo('S');

        return $boletreceber;
    }

    /**
     * Insere Boleto a Receber Associado
     * @param $nrorg
     * @param $nrlanbolrec
     * @param $cdfilial
     * @param $vrbolrec
     * @param $vrfinalbrec
     * @param $datavencimento
     * @param $cdreducont
     * @param $cdtpbaixarec
     * @param $cdcliente
     * @param $nrlicencauso
     * @param $cdempresa
     * @param $idprotestrec
     * @param $idbolrecalter
     * @param $idformcadbrec
     * @param $idstatbrec
     * @param $idsitucnabr
     * @param $nrparcbolrec
     * @param $cdoperinclusao
     * @return Boletreceber
     */
    public function insereBoletoRecAssociado(
        $nrorg, $nrlanbolrec, $cdfilial, $vrbolrec, $vrfinalbrec, $datavencimento, $cdreducont, $cdtpbaixarec, $cdcliente, 
        $nrlicencauso, $cdempresa, $idbolrecalter, $idstatbrec, $idsitucnabr, $nrparcbolrec, $cdoperinclusao,
        $cdcartcobbol, $idcartcobrec, $idimprbolbol, $idformcadbrec, $qtdiaprotbol,
        $cddiaprotbol, $cdins1cobbol, $cdins2cobbol, $vrmultabrec, $vrmordiabrec, $vrpemulbrec, $vrpemorbrec
    ){
        /** @var Boletreceber $boletreceber */
        $boletreceber = new Boletreceber();
        $boletreceber->setNrorg($nrorg);
        $boletreceber->setNrlanbolrec($nrlanbolrec);
        $boletreceber->setCdfilial($cdfilial);
        $boletreceber->setVrbolrec($vrbolrec);
        $boletreceber->setVrfinalbrec($vrfinalbrec);
        $boletreceber->setDtvenbolrec($datavencimento);
        $boletreceber->setCdreducont($cdreducont);
        $boletreceber->setCdtpbaixarec($cdtpbaixarec);
        $boletreceber->setCdcliente($cdcliente);
        $boletreceber->setNrlicencauso($nrlicencauso);
        $boletreceber->setCdempresa($cdempresa);
        $boletreceber->setIdbolrecalter($idbolrecalter);
        $boletreceber->setIdstatbrec($idstatbrec);
        $boletreceber->setIdsitucnabr($idsitucnabr);
        $boletreceber->setNrparcbolrec($nrparcbolrec);
        $boletreceber->setCdoperinclusao($cdoperinclusao);
        $boletreceber->setDtinclusao(DateUtil::getDataAtual());
        $boletreceber->setDtultatu(DateUtil::getDataAtual());
        $boletreceber->setIdativo('S');
        
        $boletreceber->setCdcartcobbol($cdcartcobbol);
        $boletreceber->setIdcartcobrec($idcartcobrec);
        $boletreceber->setIdimprbolbol($idimprbolbol);
        $boletreceber->setIdformcadbrec($idformcadbrec);
        $boletreceber->setQtdiaprotbol($qtdiaprotbol);
        $boletreceber->setCddiaprotbol($cddiaprotbol);
        $boletreceber->setCdins1cobbol($cdins1cobbol);
        $boletreceber->setCdins2cobbol($cdins2cobbol);
        
        $boletreceber->setVrmultabrec($vrmultabrec);
        $boletreceber->setVrmordiabrec($vrmordiabrec);
        $boletreceber->setVrpemulbrec($vrpemulbrec);
        $boletreceber->setVrpemorbrec($vrpemorbrec);

        return $boletreceber;
    }

    /**
     * Salvar Conta a Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenrec
     * @param $nrparcelarec
     * @param $dtorigvenrec
     * @param $vratuarec
     * @param $cdtpctreceb
     * @param $dtatuavenrec
     * @param $vrorigrec
     * @param $cdfilibolrec
     * @param $nrlanbolrec
     * @param $idstatrec
     * @param $qtdiasvenrec
     * @param $cdclassfina
     * @param $cdempresa
     * @param $dtmovfluxo
     * @param $idmovfluxo
     * @param $cdfimovflrec
     * @param $dscontarec
     * @return Contareceber
     */
    public function insereContaReceber(
        $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $dtorigvenrec,
        $vratuarec, $cdtpctreceb, $dtatuavenrec, $vrorigrec, $cdfilibolrec, $nrlanbolrec,
        $idstatrec, $qtdiasvenrec, $cdclassfina, $cdempresa, $dtmovfluxo,
        $idmovfluxo, $cdfimovflrec, $dscontarec
    ){
        /** @var Contareceber $contareceber */
        $contareceber = new Contareceber();
        $contareceber->setNrorg($nrorg);
        $contareceber->setCdfilial($cdfilial);
        $contareceber->setNrlanctonf($nrlanctonf);
        $contareceber->setNrseqvenrec($nrseqvenrec);
        $contareceber->setNrparcelarec($nrparcelarec);
        $contareceber->setDtorigvenrec($dtorigvenrec);
        $contareceber->setVratuarec($vratuarec);
        $contareceber->setCdtpctreceb($cdtpctreceb);
        $contareceber->setDtatuavenrec($dtatuavenrec);
        $contareceber->setVrorigrec($vrorigrec);
        $contareceber->setCdfilibolrec($cdfilibolrec);
        $contareceber->setNrlanbolrec($nrlanbolrec);
        $contareceber->setIdstatrec($idstatrec);
        $contareceber->setQtdiasvenrec($qtdiasvenrec);
        $contareceber->setDscontarec($dscontarec);
        $contareceber->setCdclassfina($cdclassfina);
        $contareceber->setCdempresa($cdempresa);
        $contareceber->setDtmovfluxo($dtmovfluxo);
        $contareceber->setIdmovfluxo($idmovfluxo);
        $contareceber->setCdfimovflrec($cdfimovflrec);
        $contareceber->setDtinclusao(DateUtil::getDataAtual());
        $contareceber->setIdrepfoscnc("M");

        return $contareceber;
    }

    /**
     * Insere Rateio Tipo Conta Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $cdfiliemit
     * @param $cdcentcust
     * @param $vrperattpctr
     * @return Rattpctr
     */
    public static function insereRatTpCtR(
        $nrorg, $cdfilial, $cdfiliemit, $cdcentcust, $vrperattpctr
    ){
        /** @var Rattpctr $rattpctr */
        $rattpctr = New Rattpctr();
        $rattpctr->setNrorg($nrorg);
        $rattpctr->setCdfilial($cdfilial);
        $rattpctr->setCdfirattpctr($cdfiliemit);
        $rattpctr->setCdcentcust($cdcentcust);
        $rattpctr->setVrperattpctr($vrperattpctr);
        $rattpctr->setDtinclusao(DateUtil::getDataAtual());
        return $rattpctr;
    }

    /**
     * Ajusta Boleto a Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @param $cdajuctrec
     * @param $vrajbolrec
     * @return Ajbolrec
     */
    public static function insereAjbolrec(
        $nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec, $vrajbolrec
    ){
        /** @var Ajbolrec $ajbolrec */
        $ajbolrec = new Ajbolrec();
        $ajbolrec->setNrorg($nrorg);
        $ajbolrec->setCdfilial($cdfilial);
        $ajbolrec->setNrlanbolrec($nrlanbolrec);
        $ajbolrec->setCdajuctrec($cdajuctrec);
        $ajbolrec->setVrajbolrec($vrajbolrec);
        $ajbolrec->setDtinclusao(DateUtil::getDataAtual());

        return $ajbolrec;
    }

    /**
     * Insere Rateio Conta Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenrec
     * @param $nrparcelarec
     * @param $cdfiratctrec
     * @param $cdcentcust
     * @param $vrratccctrec
     * @return Ratctreceber
     */
    public static function insereRatCtReceber(
        $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $cdcentcust, $cdfiratctrec, $vrratccctrec
    ){
        /** @var Ratctreceber $rateioreceber */
        $rateioreceber = new Ratctreceber();
        $rateioreceber->setNrorg($nrorg);
        $rateioreceber->setCdfilial($cdfilial);
        $rateioreceber->setNrlanctonf($nrlanctonf);
        $rateioreceber->setNrseqvenrec($nrseqvenrec);
        $rateioreceber->setNrparcelarec($nrparcelarec);
        $rateioreceber->setCdfiratctrec($cdfiratctrec);
        $rateioreceber->setCdcentcust($cdcentcust);
        $rateioreceber->setVrratccctrec($vrratccctrec);
        $rateioreceber->setDtinclusao(DateUtil::getDataAtual());
        return $rateioreceber;
    }

    /**
     * Insere Movimentação Bancária
     * @param $nrorg
     * @param $nrmovbanc
     * @param $cdreducont
     * @param $dtmovbanc
     * @param $vrmovbanc
     * @param $dshismbanc
     * @return Movbancario
     */
    public static function insereMovBancario(
        $nrorg, $nrmovbanc, $cdreducont, $dtmovbanc, $vrmovbanc, $dshismbanc = 'bxRec', $idmovbanc = 'C', $nrmovbancorig = null, $idorigmbanc = 'R'
    ){
        /** @var Movbancario $movbancario */
        $movbancario = new Movbancario;
        $movbancario->setNrorg($nrorg);
        $movbancario->setCdreducont($cdreducont);
        $movbancario->setNrmovbanc($nrmovbanc);
        $movbancario->setDtmovbanc($dtmovbanc);
        $movbancario->setVrmovbanc($vrmovbanc);
        $movbancario->setIdativo('S');
        $movbancario->setIdmovbanc($idmovbanc);
        $movbancario->setIdorigmbanc($idorigmbanc);
        $movbancario->setIdstatusctbbanc('P');
        $movbancario->setDshismbanc($dshismbanc);
        $movbancario->setDtinclusao(DateUtil::getDataAtual());
        $movbancario->setNrmovbancorig($nrmovbancorig);

        return $movbancario;
    }

    /**
     * Insere Baixa Recebida
     * @param $nrorg
     * @param $cdfilial
     * @param $cdempresa
     * @param $cdoperador
     * @param $cdtpbaixarec
     * @param $cdreducont
     * @param $dtlancbaixar
     * @param $nrmovbanc
     * @param $nrbaixarec
     * @param $idstatusbrec
     * @param $dsbaixarec
     * @param $dtbaixarec
     * @param $vrbaixarec
     * @return Baixarecebida|null
     * @throws \Exception
     */
    public static function insereBaixaRecebida(
        $nrorg, $cdfilial, $cdempresa, $cdoperador, $cdtpbaixarec, $cdreducont, $dtlancbaixar, $nrmovbanc, $nrbaixarec,
        $idstatusbrec, $dsbaixarec, $dtbaixarec, $vrbaixarec, $nrlctbbrec = null, $cdclienterenego = null,  $nrseqhistrenego = null
    ){
        /** @var Baixarecebida $baixarecebida */
        $baixarecebida = new Baixarecebida();
        $baixarecebida->setNrorg($nrorg);
        $baixarecebida->setCdfilial($cdfilial);
        $baixarecebida->setNrorginclusao($nrorg);
        $baixarecebida->setCdempresa($cdempresa);
        $baixarecebida->setNrmovbanc($nrmovbanc);
        $baixarecebida->setNrbaixarec($nrbaixarec);
        $baixarecebida->setDtbaixarec($dtbaixarec);
        $baixarecebida->setCdreducont($cdreducont);
        $baixarecebida->setDsbaixarec($dsbaixarec);
        $baixarecebida->setVrbaixarec($vrbaixarec);
        $baixarecebida->setCdoperador($cdoperador);
        $baixarecebida->setDtlancbaixar($dtlancbaixar);
        $baixarecebida->setIdstatusbrec($idstatusbrec);
        $baixarecebida->setCdtpbaixarec($cdtpbaixarec);
        $baixarecebida->setCdoperinclusao($cdoperador);
        $baixarecebida->setDtinclusao(DateUtil::getDataAtual());
        $baixarecebida->setNrlctbbrec($nrlctbbrec);
        $baixarecebida->setCdclienterenego($cdclienterenego);
        $baixarecebida->setNrseqhistrenego($nrseqhistrenego);
        
        return $baixarecebida;
    }

    /**
     * Insere Log Boleto Remessa
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @param $nrcnabrec
     * @param $cdempresa
     * @param $cdoperador
     * @return Logboletoremrec
     */
    public static function insereLogBoletoRem(
        $nrorg, $cdfilial, $nrlanbolrec, $nrcnabrec, $cdempresa, $cdoperador
    ){
        /** @var Logboletoremrec $logboletoremrec */
        $logboletoremrec = New Logboletoremrec();
        $logboletoremrec->setNrorg($nrorg);
        $logboletoremrec->setCdfiliallogrec($cdfilial);
        $logboletoremrec->setNrlanbolreclog($nrlanbolrec);
        $logboletoremrec->setNrcnabrecretlog($nrcnabrec);
        $logboletoremrec->setCdempresalogrec($cdempresa);
        $logboletoremrec->setCdopelogrec($cdoperador);
        $logboletoremrec->setDtlogboletrrec(DateUtil::getDataAtual());
        $logboletoremrec->setDtinclusao(DateUtil::getDataAtual());

        return $logboletoremrec;
    }

    /**
     * Insere Ajuste Conta Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $dtajusterec
     * @param $cdajuctrec
     * @param $dsajusterec
     * @param $vrajusterec
     * @param $nrseqvenrec
     * @param $nrparcelarec
     * @param $cdclassfina
     * @param $cdempresa
     * @return Ajctreceber
     */
    public static function insereAjctreceber(
        $nrorg, $cdfilial, $nrlanctonf, $dtajusterec, $cdajuctrec, $dsajusterec, $vrajusterec,
        $nrseqvenrec, $nrparcelarec, $cdclassfina, $cdempresa, $dtmovfluxo
    ){
        /** @var Ajctreceber $ajctreceber */
        $ajctreceber = New Ajctreceber();
        $ajctreceber->setNrorg($nrorg);
        $ajctreceber->setCdfilial($cdfilial);
        $ajctreceber->setNrlanctonf($nrlanctonf);
        $ajctreceber->setDtajusterec($dtajusterec);
        $ajctreceber->setCdajuctrec($cdajuctrec);
        $ajctreceber->setDsajusterec($dsajusterec);
        $ajctreceber->setVrajusterec($vrajusterec);
        $ajctreceber->setNrseqvenrec($nrseqvenrec);
        $ajctreceber->setNrparcelarec($nrparcelarec);
        $ajctreceber->setCdclassfina($cdclassfina);
        $ajctreceber->setCdemmfajcntr($cdempresa);
        $ajctreceber->setCdfimfajcntr($cdfilial);
        $ajctreceber->setDtinclusao(DateUtil::getDataAtual());
        $ajctreceber->setIdmovfluxo('E');
        $ajctreceber->setDtmovfluxo($dtmovfluxo);
        return $ajctreceber;
    }

    /**
     * Insere CNAB Receber
     * @param $nrorg
     * @param $cdempresa
     * @param $cdoperador
     * @param $dtcnabrec
     * @param $idtipcnabrec
     * @param $cdreducont
     * @param $cdbanco
     * @param $idopecnabrec
     * @return Cnabrec
     */
    public static function insereCnabRec(
        $nrorg, $cdempresa, $nrcnabrec, $cdoperador, $dtcnabrec, $idtipcnabrec,
        $cdreducont, $cdbanco, $cdreducont1, $idopecnabrec, $qtbolenvcnabrec,
        $vrtotbolcnabrec
    ){
        /** @var Cnabrec $cnabrec */
        $cnabrec = New Cnabrec();
        $cnabrec->setNrorg($nrorg);
        $cnabrec->setCdempresa($cdempresa);
        $cnabrec->setNrcnabrec($nrcnabrec);
        $cnabrec->setCdoperador($cdoperador);
        $cnabrec->setDtcnabrec($dtcnabrec);
        $cnabrec->setIdtipcnabrec($idtipcnabrec);
        $cnabrec->setCdreducont($cdreducont);
        $cnabrec->setCdbanco($cdbanco);
        $cnabrec->setCdreducontcobr($cdreducont);
        $cnabrec->setIdopecnabrec($idopecnabrec);
        $cnabrec->setQtbolenvcnabrec($qtbolenvcnabrec);
        $cnabrec->setVrtotbolcnabrec($vrtotbolcnabrec);
        $cnabrec->setDtinclusao(DateUtil::getDataAtual());
        $cnabrec->setDtultatu(DateUtil::getDataAtual());

        return $cnabrec;
    }

    /**
     * Insere Adiantamento Receber
     * @param $nradiantareceb
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrlanbolrec
     * @param $nrseqvenrec
     * @param $nrparcelarec
     * @param $vrdesconto
     * @param $nrorg
     * @param $cdajuctrec
     * @param $dtadiantarec
     * @return Adiantareceb
     */
    public static function insereAdiantaReceb(
        $nradiantareceb, $cdfilial, $cdfiliajbolr, $nrlanctonf, $nrlanbolrec,
        $nrseqvenrec, $nrparcelarec, $vrdesconto, $nrorg, $cdajuctrec, $dtadiantarec, $cdoperador
    ){
        $adiantareceb = new Adiantareceb();
        $adiantareceb->setNrorg($nrorg);
        $adiantareceb->setNradiantareceb($nradiantareceb);
        $adiantareceb->setCdfilial($cdfilial);
        $adiantareceb->setCdfiliajbolr($cdfiliajbolr);
        $adiantareceb->setNrlanctonf($nrlanctonf);
        $adiantareceb->setNrlanbolrec($nrlanbolrec);
        $adiantareceb->setNrseqvenrec($nrseqvenrec);
        $adiantareceb->setNrparcelarec($nrparcelarec);
        $adiantareceb->setVradiantarec($vrdesconto);
        $adiantareceb->setCdajuctrec($cdajuctrec);
        $adiantareceb->setDtadiantarec(DateUtil::truncateData($dtadiantarec));
        $adiantareceb->setDtinclusao(DateUtil::getDataAtual());
        $adiantareceb->setCdoperinclusao($cdoperador);

        return $adiantareceb;
    }

    /**
     * Insere Devolução Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $cdfilibolrec
     * @param $nrlanbolrec
     * @param $nrdevolucaorec
     * @param $dtdevolucaorec
     * @param $nrlanctonf
     * @param $cdajuctrec
     * @param $vrdevolucaorec
     * @return Devolucaorec
     */
    public static function insereDevolucaoRec(
        $nrorg, $cdfilial, $cdfilibolrec, $nrlanbolrec, $nrdevolucaorec,
        $dtdevolucaorec, $nrlanctonf, $cdajuctrec, $vrdevolucaorec
    ){
        /** @var Devolucaorec $devolucaorec */
        $devolucaorec = New Devolucaorec();
        $devolucaorec->setNrorg($nrorg);
        $devolucaorec->setCdfilial($cdfilial);
        $devolucaorec->setCdfilibolrec($cdfilibolrec);
        $devolucaorec->setNrlanbolrec($nrlanbolrec);
        $devolucaorec->setNrdevolucaorec($nrdevolucaorec);
        $devolucaorec->setDtdevolucaorec($dtdevolucaorec);
        $devolucaorec->setNrlanctonf($nrlanctonf);
        $devolucaorec->setCdajuctrec($cdajuctrec);
        $devolucaorec->setVrdevolucaorec($vrdevolucaorec);
        $devolucaorec->setDtinclusao(DateUtil::getDataAtual());

        return $devolucaorec;
    }

    /**
     * Insere Lançamento Contabil
     * @param $nrorg
     * @param $cdempresa
     * @param $nrlanccont
     * @param $nrseqlanc
     * @param $cdcontctbl
     * @param $cdhistpadr
     * @param $dtlanccont
     * @param $cddocucont
     * @param $idtipolanc
     * @param $dshistlanc
     * @param $vrlancament
     * @param $idlanccome
     * @param $cdcontcome
     * @param $idlancsaldo
     * @param $idlancinte
     * @param $idlancmani
     * @param $cdoperins
     * @param $dtultatulan
     * @return Lanccont
     */
    public static function insereLanccont(
        $nrorg, $cdempresa, $nrlanccont, $nrseqlanc, $cdcontctbl, $cdhistpadr,
        $dtlanccont, $cddocucont, $idtipolanc, $dshistlanc, $vrlancament, $idlanccome,
        $cdcontcome, $idlancsaldo, $idlancinte, $idlancmani, $cdoperins, $dtultatulan
    ){
        $lanccont = new Lanccont();
        $lanccont->setNrorg($nrorg);
        $lanccont->setCdempresa($cdempresa);
        $lanccont->setNrlanccont($nrlanccont);
        $lanccont->setNrseqlanc($nrseqlanc);
        $lanccont->setCdcontctbl($cdcontctbl);
        $lanccont->setCdhistpadr($cdhistpadr);
        $lanccont->setDtlanccont($dtlanccont);
        $lanccont->setCddocucont($cddocucont);
        $lanccont->setIdtipolanc($idtipolanc);
        $lanccont->setDshistlanc($dshistlanc);
        $lanccont->setVrlancament($vrlancament);
        $lanccont->setIdlanccome($idlanccome);
        $lanccont->setCdcontcome($cdcontcome);
        $lanccont->setIdlancsaldo($idlancsaldo);
        $lanccont->setIdlancinte($idlancinte);
        $lanccont->setIdlancmani($idlancmani);
        $lanccont->setCdoperins($cdoperins);
        $lanccont->setDtultatulan($dtultatulan);
        $lanccont->setDtinclusao(DateUtil::getDataAtual());

        return $lanccont;
    }

    /**
     * Atualiza Boleto Receber
     * @param Boletreceber $boletreceber
     * @param $nrcnabrecrem
     * @param $idsitucnabr
     * @return Boletreceber
     */
    public static function atualizaBoletoReceber(
        Boletreceber $boletreceber, $nrcnabrecrem, $idsitucnabr
    ){
        $boletreceber->setNrcnabrecrem($nrcnabrecrem);
        $boletreceber->setIdsitucnabr($idsitucnabr);

        return $boletreceber;
    }
    
    /**
     * Atualiza Titulo Baixado
     * @param Boletreceber $boletreceber
     * @param $cdfilial
     * @param $idstatbrec
     * @param $idstatbrec
     * @param $idstatbrec
     * @param $idstatbrec
     * @return Boletreceber
     */
    public static function atualizaTituloBaixado(
        Boletreceber $boletreceber, $cdfilial, $idstatbrec, $dtliqbolrec, $nrbaixarec, 
        $cdtpbaixarec, $cdreducont
    ){
        $boletreceber->setCdfilibairec($cdfilial);
        $boletreceber->setIdstatbrec($idstatbrec);
        $boletreceber->setDtliqbolrec($dtliqbolrec);
        $boletreceber->setNrbaixarec($nrbaixarec);
        $boletreceber->setCdtpbaixarec($cdtpbaixarec);
        $boletreceber->setCdreducont($cdreducont);

        return $boletreceber;
    }

    /**
     * Atualiza Boleto Receber Renegociado
     * @param Boletreceber $boletreceber
     * @param $vrfinalbrec
     * @return Boletreceber
     */
    public static function atualizaBoletreceberReneg(
        Boletreceber $boletreceber, $vrfinalbrec
    ){
        $boletreceber->setVrfinalbrec($vrfinalbrec);

        return $boletreceber;
    }

    /**
     * Atualiza Boleto da Baixa
     * @param Boletreceber $boletreceber
     * @param $cdfilibairec
     * @param $nrbaixarec
     * @return Boletreceber
     */
    public static function atualizaBoletoBaixa(
        Boletreceber $boletreceber, $cdfilibairec, $nrbaixarec
    ){
        $boletreceber->setCdfilibairec($cdfilibairec);
        $boletreceber->setNrbaixarec($nrbaixarec);

        return $boletreceber;
    }

    /**
     * Atualiza Nota Fiscal
     * @param Notafiscal $notafiscal
     * @param $vrdevolnfutl
     * @return Notafiscal
     */
    public static function atualizaNotaFiscalDev(
        Notafiscal $notafiscal, $vrdevolnfutl
    ){
        $notafiscal->setVrdevolnfutl($vrdevolnfutl);

        return $notafiscal;
    }

    /**
     * @param Notafiscal $notafiscal
     * @param $nrnotafisc
     * @param $dtemissao
     * @param $dtentrsaid
     * @param $cdserie
     * @param $idstatus
     * @param $nrlanccont
     * @return Notafiscal
     */
    public static function atualizaNotaFiscal(
        Notafiscal $notafiscal, $nrnotafisc, $dtemissao, $dtentrsaid, $cdserie, $idstatus, $nrlanccont
    ){
        $notafiscal->setNrnotafisc($nrnotafisc);
        $notafiscal->setDtemissao($dtemissao);
        $notafiscal->setDtentrsaid($dtentrsaid);
        $notafiscal->setCdserie($cdserie);
        $notafiscal->setIdstatus($idstatus);
        $notafiscal->setNrlanccont($nrlanccont);

        return $notafiscal;
    }

    /**
     * @param Contareceber $contareceber
     * @param $cdtpctreceb
     * @param $dtatuavenrec
     * @param $cdclassfina
     * @param $dtmovfluxo
     * @param $idmovfluxo
     * @param $dscontarec
     * @return Contareceber
     */
    public static function atualizaContaReceberClassfina(
        Contareceber $contareceber, $cdtpctreceb, $dtatuavenrec, $cdclassfina, $dtmovfluxo, $idmovfluxo, $dscontarec
    ){
        $contareceber->setCdtpctreceb($cdtpctreceb);
        $contareceber->setDtatuavenrec($dtatuavenrec);
        $contareceber->setCdclassfina($cdclassfina);
        $contareceber->setDtmovfluxo($dtmovfluxo);
        $contareceber->setIdmovfluxo($idmovfluxo);
        $contareceber->setDscontarec($dscontarec);

        return $contareceber;
    }

    /**
     * Atualiza Conta a Receber
     * @param Contareceber $contareceber
     * @param $idstatrec
     * @param $idmovfluxo
     * @param $dtmovfluxo
     * @return Contareceber
     */
    public static function atualizaContaReceber(
        Contareceber $contareceber, $idstatrec, $idmovfluxo, $dtmovfluxo
    ){
        $contareceber->setIdstatrec($idstatrec);
        $contareceber->setIdmovfluxo($idmovfluxo);
        $contareceber->setDtmovfluxo($dtmovfluxo);

        return $contareceber;
    }

    /**
     * Atualiza Conta a Receber Renegociada
     * @param Contareceber $contareceber
     * @param $nrseqhistrenego
     * @param $cdclienterenego
     * @return Contareceber
     */
    public static function atualizaContareceberReneg(
        Contareceber $contareceber, $nrseqhistrenego, $cdclienterenego
    ){
        $contareceber->setNrseqhistrenego($nrseqhistrenego);
        $contareceber->setCdclienterenego($cdclienterenego);

        return $contareceber;
    }

    /**
     * Atualiza Conta Receber Associada
     * @param Contareceber $contareceber
     * @param $nrlanbolrec
     * @param $cdfilibolrec
     * @param $dtatuavenrec
     * @param $dtmovfluxo
     * @param $cdmotipror
     * @return Contareceber
     */
    public static function atualizaContaRecAssoc(
        Contareceber $contareceber, $nrlanbolrec, $cdfilibolrec, $dtatuavenrec, $dtmovfluxo, $cdmotipror
    ){
        $contareceber->setNrlanbolrec($nrlanbolrec);
        $contareceber->setCdfilibolrec($cdfilibolrec);
        $contareceber->setDtatuavenrec($dtatuavenrec);
        $contareceber->setDtmovfluxo($dtmovfluxo);
        $contareceber->setCdmotipror($cdmotipror);

        return $contareceber;
    }

    /**
     * @param Ratctreceber $ratctreceber
     * @param $vrratccctrec
     * @return Ratctreceber
     */
    public static function atualizaRatCtReceber(
        Ratctreceber $ratctreceber, $vrratccctrec
    ){
        $ratctreceber->setVrratccctrec($vrratccctrec);

        return $ratctreceber;
    }

    /**
     * Atualiza Titulo Descontado
     * @param Boletreceber $boletreceber
     * @param $idstatbrec
     * @return Boletreceber
     */
    public function atualizaTituloDescontado(
        Boletreceber $boletreceber, $idstatbrec
    ){
        $boletreceber->setIdcartcobrec(null);
        $boletreceber->setCdfilibairec(null);
        $boletreceber->setNrbaixarec(null);
        $boletreceber->setNrmovicred(null);
        $boletreceber->setIdstatbrec($idstatbrec);

        return $boletreceber;
    }
    
    /**
     * Atualiza Boleto Receber
     * @param Boletreceber $boletreceber
     * @param $nrcnabrecrem
     * @param $idsitucnabr
     * @return Boletreceber
     */
    public static function atualizaTituloPreparado(
        Boletreceber $boletreceber, $cdtpbaixarec, $cdreducont, $cdcartcobbol, $idcartcobrec, $idimprbolbol, $idformcadbrec, 
        $qtdiaprotbol, $cddiaprotbol, $cdins1cobbol, $cdins2cobbol
    ){
        $boletreceber->setCdtpbaixarec($cdtpbaixarec);
        $boletreceber->setCdreducont($cdreducont);
        $boletreceber->setCdcartcobbol($cdcartcobbol);
        $boletreceber->setIdcartcobrec($idcartcobrec);
        $boletreceber->setIdimprbolbol($idimprbolbol);
        $boletreceber->setIdformcadbrec($idformcadbrec);
        $boletreceber->setQtdiaprotbol($qtdiaprotbol);
        $boletreceber->setCddiaprotbol($cddiaprotbol);
        $boletreceber->setCdins1cobbol($cdins1cobbol);
        $boletreceber->setCdins2cobbol($cdins2cobbol);

        return $boletreceber;
    }

    /**
     * Valida Adiantamento Utilizado
     * @param $nrorg
     * @param $cdfilial
     * @param $nrbaixarec
     * @return array
     */
    public function validaAdiantamentoUtilizado(
        $nrorg, $cdfilial, $nrbaixarec
    ){

        $params = array();
        $params['P_NRORG'     ] = $nrorg;
        $params['P_CDFILIAL'  ] = $cdfilial;
        $params['P_NRBAIXAREC'] = $nrbaixarec;

        $sql = "SELECT A.NRBAIXAREC FROM (SELECT MAX(BO.NRBAIXAREC) NRBAIXAREC
				  FROM BOLETRECEBER  BO, CONTARECEBER CR, ADIANTARECEB AR, TIPOCTRECEBER TR
				 WHERE (BO.NRORG         = :P_NRORG       ) AND
				       (BO.CDFILIBAIREC  = :P_CDFILIAL    ) AND
    			           (BO.NRBAIXAREC    = :P_NRBAIXAREC  ) AND
					   (CR.CDFILIBOLREC  = BO.CDFILIAL    ) AND
			    	   (CR.NRLANBOLREC   = BO.NRLANBOLREC ) AND
					   (CR.IDSTATREC     = 'B'            ) AND
			      	   (CR.CDFILIAL      = AR.CDFILIAL    ) AND
					   (CR.NRLANCTONF    = AR.NRLANCTONF  ) AND
			      	   (CR.NRSEQVENREC   = AR.NRSEQVENREC ) AND
					   (CR.NRPARCELAREC  = AR.NRPARCELAREC) AND
			      	   (TR.CDTPCTRECEB   = CR.CDTPCTRECEB ) AND
					   (TR.IDTPCTRECEBER = '2'            ))A";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Valida Adiantamento Utilizado Quando Foi selecionado mais de um título
     * @param $nrorg
     * @param $cdfilial
     * @param $nrbaixarec
     * @return array
     */
    public function validaAdiantamentoUtilizadoMultiploBol(
        $nrorg, $cdfilial, $nrlanbolrec
    ){

        $params = array();
        $params['P_NRORG'     ] = $nrorg;
        $params['P_CDFILIAL'  ] = $cdfilial;
        $params['P_NRLANBOLREC'] = $nrlanbolrec;

        $sql = "SELECT
                    A.NRBAIXAREC
                    FROM
                    (
                        SELECT
                            MAX(BO.NRBAIXAREC) NRBAIXAREC
                        FROM
                            BOLETRECEBER  BO,
                            CONTARECEBER  CR,
                            ADIANTARECEB  AR,
                            TIPOCTRECEBER TR,
                            BAIXARECEBIDA BR
                        WHERE
                            ( BO.NRORG = :P_NRORG )
                            AND ( BO.CDFILIBAIREC = BR.CDFILIAL )
                            AND ( BO.NRBAIXAREC = BR.NRBAIXAREC )
                            AND (BO.CDFILIAL = :P_CDFILIAL)
                            AND (BO.NRLANBOLREC = :P_NRLANBOLREC)
                            AND ( CR.CDFILIBOLREC = BO.CDFILIAL )
                            AND ( CR.NRLANBOLREC = BO.NRLANBOLREC )
                            AND ( CR.IDSTATREC = 'B' )
                            AND ( CR.CDFILIAL = AR.CDFILIAL )
                            AND ( CR.NRLANCTONF = AR.NRLANCTONF )
                            AND ( CR.NRSEQVENREC = AR.NRSEQVENREC )
                            AND ( CR.NRPARCELAREC = AR.NRPARCELAREC )
                            AND ( TR.CDTPCTRECEB = CR.CDTPCTRECEB )
                            AND ( TR.IDTPCTRECEBER = '2' )
                    ) A";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    /**
     * Busca Centro de Custo Filial
     * @param $cdfilial
     * @param $cdempresa
     * @return array
     */
    public function buscaCentCustFili(
        $cdfilial, $cdempresa
    ){

        /** @var $params */
        $params = array();
        $params['P_CDFILIAL' ] = $cdfilial;
        $params['P_CDEMPRESA'] = $cdempresa;

        $sql = "SELECT F.CDFILIAL, NVL(F.CDCENTCUST, E.CDCENTCUST) AS CDCENTCUST
                  FROM FILIAL F, EMPRESA E
                 WHERE F.CDFILIAL   = :P_CDFILIAL  AND
                       E.CDEMPRESA  = :P_CDEMPRESA AND
                       (F.CDEMPRESA = E.CDEMPRESA)
                 UNION
                SELECT F.CDFILIAL, NVL(F.CDCENTCUST, E.CDCENTCUST) AS CDCENTCUST
                  FROM FILIAL F, EMPRESA E, EMPRCNTRFIL EC
                 WHERE EC.CDFILIAL  = :P_CDFILIAL  AND
                       EC.CDEMPRESA = :P_CDEMPRESA AND
                       E.CDEMPRESA  = EC.CDEMPRESA AND
                       (F.CDFILIAL  = EC.CDFILIAL)";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Busca Todos Ajustes Boletos
     * @param $cdFilialLanBolRec
     * @return array
     */
    public function buscaTodosAjustesBoletosRec($cdFilialLanBolRec) {
        $sql = "SELECT 'X' IDOPERACAO,
          AJ.CDAJUCTREC,
          TA.DSAJUCTREC,
          TA.IDAJUCTREC,
          AJ.VRAJBOLREC,
          AJ.NRORG,
          AJ.CDFILIAL,
          AJ.NRLANBOLREC,
          AJ.IDORIGAJBREC
        FROM AJBOLREC AJ,
          TIPOAJRECEBER TA
        WHERE AJ.NRORG    = TA.NRORG
        AND AJ.CDAJUCTREC = TA.CDAJUCTREC
        AND AJ.CDFILIAL||AJ.NRLANBOLREC IN $cdFilialLanBolRec";

        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    }

    /**
     * Busca Ajuste Titulo
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @param $cdajuctrec
     * @return Ajbolrec
     */
    public function buscaAjusteTitulo($nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec)
    {
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        $params['cdfilial']    = $cdfilial;
        $params['nrlanbolrec'] = $nrlanbolrec;
        $params['cdajuctrec']  = $cdajuctrec;

        /** @var Ajbolrec $ajbolrec */
        $ajbolrec = $this->entityManager->getRepository(Ajbolrec::class)->findOneBy($params);

        return $ajbolrec;
    }

    /**
     * Consulta Numero Duplicata
     * @param $nrlanbolrec
     * @param $cdfilial
     * @param $nrorg
     * @return array
     */
    public function consultaNumeroDuplicata(
        $nrlanbolrec, $cdfilial, $nrorg
    ){

        /** @var $params */
        $params = array();
        $params['P_NRLANBOLREC' ] = $nrlanbolrec;
        $params['P_CDFILIAL']     = $cdfilial;
        $params['P_NRORG']        = $nrorg;

        $sql = "SELECT DECODE(CR.NRPARCELAREC, '00', CR.NRSEQVENREC, CR.NRPARCELAREC) AS NRPARCELA,
                       NVL(DP.NRDUPLICATA, NF.NRNOTAFISC) DUPLICATA,
                       NF.DTEMISSAO
                  FROM DUPLICATA DP, CONTARECEBER CR, NOTAFISCAL NF
                 WHERE (NF.CDFILIAL     = CR.CDFILIAL      ) AND
                       (NF.NRLANCTONF   = CR.NRLANCTONF    ) AND
                       (CR.CDFILIALDUP  = DP.CDFILIAL   (+)) AND
                       (CR.NRLANCTODUP  = DP.NRLANCTODUP(+)) AND
                       (CR.NRLANBOLREC  = :P_NRLANBOLREC   ) AND
                       (CR.CDFILIBOLREC = :P_CDFILIAL      ) AND
                       (CR.NRORG        = :P_NRORG         )";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    /**
     * Executa Procedure CTB_REC_LANC
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrparcelarec
     * @param $nrseqvenrec
     * @param $dtajusterec
     * @param $cdajuctrec
     * @param $idtipo
     * @return mixed
     */
    public function execProcCtbRecLanc(
        $cdfilial, $nrlanctonf, $nrparcelarec, $nrseqvenrec, $dtajusterec, $cdajuctrec, $idtipo
    ){
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CTB_REC_LANC');
        $procedure->addParam(new Param('P_CDFILIAL'    , Param::PARAM_INPUT , $cdfilial    , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF'  , Param::PARAM_INPUT , $nrlanctonf  , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRPARCELAREC', Param::PARAM_INPUT , $nrparcelarec, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRSEQVENREC' , Param::PARAM_INPUT , $nrseqvenrec , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTAJUSTEREC' , Param::PARAM_INPUT , $dtajusterec , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDAJUCTREC'  , Param::PARAM_INPUT , $cdajuctrec  , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDTIPO'      , Param::PARAM_INPUT , $idtipo      , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCCTB'   , Param::PARAM_OUTPUT, null         , Param::PARAM_TYPE_STR, 12));
        $nrlancctb = $procedure->execute();

        return $nrlancctb['P_NRLANCCTB'];
    }

    /**
     * Executa Procedure GERA_BOLETOS
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $cdfililanc
     * @param $prxboleto
     * @param $cdtpctreceb
     * @param $idexcbolrec
     * @param $cdfilibolrec
     * @param $nrlanbolrec
     */
    public function execGeraBoletos(
        $cdfilial, $nrlanctonf, $cdfililanc, $prxboleto, $cdtpctreceb, $idexcbolrec, $cdfilibolrec, $nrlanbolrec
    ){
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'GERA_BOLETOS');
        $procedure->addParam(new Param('P_CDFILIAL'   , Param::PARAM_INPUT, $cdfilial    , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF' , Param::PARAM_INPUT, $nrlanctonf  , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILILANC' , Param::PARAM_INPUT, $cdfililanc  , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANBOLREC', Param::PARAM_INPUT, $prxboleto   , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDTPCTRECEB', Param::PARAM_INPUT, $cdtpctreceb , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDEXCBOLREC', Param::PARAM_INPUT, $idexcbolrec , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILBOLORI', Param::PARAM_INPUT, $cdfilibolrec, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANBOLORI', Param::PARAM_INPUT, $nrlanbolrec , Param::PARAM_TYPE_STR));

        $procedure->execute();
    }
    
    /**
     * Executa Procedure GRAVA_BAIXA_IMPOSTO
     * @param $cdfilial
     * @param $nrbaixa
     * @param $cdreducont
     * @param $nrmovidebi
     * @param $idoperacao
     */
    public function execGravaBaixaImposto(
        $cdfilial, $nrbaixa, $cdreducont, $nrmovidebi, $idoperacao
    ){
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'GRAVA_BAIXA_IMPOSTO');
        $procedure->addParam(new Param('P_CDFILIAL'  , Param::PARAM_INPUT, $cdfilial   , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRBAIXA'   , Param::PARAM_INPUT, $nrbaixa    , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDREDUCONT', Param::PARAM_INPUT, $cdreducont , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRMOVIDEBI', Param::PARAM_INPUT, $nrmovidebi , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDOPERACAO', Param::PARAM_INPUT, $idoperacao , Param::PARAM_TYPE_STR));

        $procedure->execute();
    }
    
    /**
     * Busca Impbaixarec Baixa
     * @param $nrorg
     * @param $cdfilial
     * @param $nrbaixarec
     * @return Empresa
     */
    public function buscaImpbaixarecBaixa($nrorg, $cdfilial, $nrbaixarec){
        /** @var $params */
        $params = array();
        
        $params['nrorg']       = $nrorg;
        $params['cdfilibaixa'] = $cdfilial;
        $params['nrbaixarec']  = $nrbaixarec;

        /** @var Impbaixarec $impbaixarec */
        $impbaixarec = $this->entityManager->getRepository(Impbaixarec::class)->findBy($params);

        return $impbaixarec;
    }
    
    /**
     * Busca Impbaixarec Titulo
     * @param $nrorg
     * @param $cdfilial
     * @param $nrbaixarec
     * @param $cdfilibolrec
     * @param $nrlanbolrec
     * @return Empresa
     */
    public function buscaImpbaixarecTitulo($nrorg, $cdfilial, $nrbaixarec, $cdfilibolrec, $nrlanbolrec){
        /** @var $params */
        $params = array();
        
        $params['nrorg']        = $nrorg;
        $params['cdfilibaixa']  = $cdfilial;
        $params['nrbaixarec']   = $nrbaixarec;
        $params['cdfilibolrec'] = $cdfilibolrec;
        $params['nrlanbolrec']  = $nrlanbolrec;

        /** @var Impbaixarec $impbaixarec */
        $impbaixarec = $this->entityManager->getRepository(Impbaixarec::class)->findBy($params);

        return $impbaixarec;
    }

    /**
     * Get Sequencial Nota Fiscal
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrorg
     * @return count
     */
    public function getSequencialNotaFiscal($cdfilial, $nrlanctonf, $nrorg){
        $params = array();
        $params['P_CDFILIAL'] = $cdfilial;
        $params['P_NRLANCTONF'] = $nrlanctonf;
        $params['P_NRORG'] = $nrorg;
        $sql = "SELECT COUNT(*) as QUANT FROM CONTARECEBER WHERE CDFILIAL = :P_CDFILIAL AND NRLANCTONF = :P_NRLANCTONF AND NRORG = :P_NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }

    /**
     * Valida Filial Logada por Empresa Filtrada
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrparcelarec
     * @param $nrseqvenrec
     * @param $dtajusterec
     * @param $cdajuctrec
     * @param $idtipo
     * @return mixed
     */
    public function validaFilialLogadaPorEmpresaFiltrada($cdempresa, $cdfilial, $nrorg){
        $params = array();
        $params['CDFILIAL']  = $cdfilial;
        $params['CDEMPRESA'] = $cdempresa;
        $params['NRORG']     = $nrorg;
        $sql = "SELECT FI.NRORG,
                       FI.CDFILIAL,
                       FI.NMFILIAL,
                       FI.CDFILIAL CDFILIEMIT,
                       FI.NMFILIAL NMFILIEMIT,
                       FI.CDEMPRESA ,
                       FI.CDINSCESTA,
                       FI.CDINSCMUNI,
                       FI.CDSERIE,
                       SE.DSSERIE,
                       NVL(PF.NRTAMNUMNFUT,0) AS NRTAMNUMNFUT,
                       FI.CDCENTCUST
                  FROM PARAMFILIAL PF,
                       FILIAL FI,
                       SERIENF SE
                 WHERE (FI.CDFILIAL   = :CDFILIAL       )
                   AND (FI.CDEMPRESA  = :CDEMPRESA      )
                   AND (FI.NRORG      = :NRORG          )
                   AND (PF.CDFILIAL   = FI.CDFILIAL     )
                   AND (PF.NRORG      = FI.NRORG        )
                   AND (FI.CDINSCMUNI = SE.CDINSCMUNI(+))
                   AND (FI.CDINSCESTA = SE.CDINSCESTA(+))
                   AND (FI.CDEMPRESA  = SE.CDEMPRESA(+) )
                   AND (FI.CDSERIE    = SE.CDSERIE(+)   )
                   AND (FI.IDCNTRRESC = 'N'          )
                 UNION
                SELECT FI.NRORG,
                       EF.CDFILIAL ,
                       FI.NMFILIAL ,
                       EF.CDFILIAL CDFILIEMIT,
                       FI.NMFILIAL NMFILIEMIT,
                       EF.CDEMPRESA ,
                       EF.CDINSCESTA,
                       EF.CDINSCMUNI,
                       FI.CDSERIE,
                       SE.DSSERIE,
                       NVL(PF.NRTAMNUMNFUT,0) AS NRTAMNUMNFUT,
                       FI.CDCENTCUST
                  FROM PARAMFILIAL PF,
                       FILIAL FI,
                       EMPRCNTRFIL EF,
                       SERIENF SE
                 WHERE (FI.CDFILIAL   = :CDFILIAL       )
                   AND (EF.CDEMPRESA  = :CDEMPRESA      )
                   AND (EF.NRORG      = :NRORG          )
                  AND (FI.CDINSCMUNI  = EF.CDINSCMUNI   )
                   AND (FI.CDINSCESTA = EF.CDINSCESTA   )
                   AND (FI.CDFILIAL   = EF.CDFILIAL     )
                   AND (FI.CDINSCMUNI = SE.CDINSCMUNI(+))
                   AND (FI.CDINSCESTA = SE.CDINSCESTA(+))
                   AND (FI.CDEMPRESA  = SE.CDEMPRESA(+) )
                   AND (FI.CDSERIE    = SE.CDSERIE(+)   )
                   AND (PF.CDFILIAL   = FI.CDFILIAL     )
                   AND (PF.NRORG      = FI.NRORG        )
                   AND (FI.IDCNTRRESC = 'N'             )
                 ORDER BY CDFILIAL";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    /**
     * Busca Movimento Indice
     * @param $nrlanbolrec
     * @param $cdfilial
     * @param $nrorg
     * @return array
     */
    public function buscaMovindice(
        $cdindice, $dtvencbol, $dtprevisao
    ){
        /** @var $params */
        $params = array();
        
        $params['P_CDINDICE' ]   = $cdindice;
        $params['P_DTVENBOLPAG'] = $dtvencbol;
        $params['P_DTREFE']      = $dtprevisao;

        $sql = "SELECT M.DTREFIND, M.VRINDICE, I.SGINDICE
                  FROM MOVINDICE M, INDICE I
                 WHERE (M.CDTPINDICE = '2')          AND
                       (M.IDPERVALOR = '1')          AND
                       (M.CDINDICE   = :P_CDINDICE)  AND
                       (M.DTREFIND BETWEEN LAST_DAY(ADD_MONTHS(:P_DTVENBOLPAG, -1)) + 1 AND LAST_DAY(:P_DTREFE)) AND
                       (M.CDTPINDICE = I.CDTPINDICE) AND
                       (M.IDPERVALOR = I.IDPERVALOR) AND
                       (M.CDINDICE   = I.CDINDICE)
              ORDER BY DTREFIND";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }
    
    public function salvaAnexoTitulo($row){
        $params = array(
            'NRANEXOLANCBOL' => $row['NRANEXOLANCBOL'],
            'CDTELA'         => $row['CDTELA'],
            'DTANEXOLANCBOL' => $row['DTANEXOLANCBOL'],
            'NMANEXOLANCBOL' => $row['NMANEXOLANCBOL'][0]["name"],
            'NMPATHANEXO'    => $row['NMPATHANEXO'],
            'DSANEXOLANCBOL' => isset($row['DSANEXOLANCBOL']) ? $row['DSANEXOLANCBOL'] : null,
            'DSINFOANEXO'    => $row['DSINFOANEXO'],
            'NRORG'          => $row['NRORG']
        );
        
        $sql = "INSERT INTO ANEXOLANCBOL 
            (
                NRANEXOLANCBOL,
                CDTELA,
                DTANEXOLANCBOL,
                NMANEXOLANCBOL,
                NMPATHANEXO,
                DSANEXOLANCBOL,
                DSINFOANEXO,
                NRORG
            ) VALUES (
                :NRANEXOLANCBOL,
                :CDTELA,
                :DTANEXOLANCBOL,
                :NMANEXOLANCBOL,
                :NMPATHANEXO,
                :DSANEXOLANCBOL,
                :DSINFOANEXO,
                :NRORG
            )";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);

    }
    
    public function salvaAnexoGrid($row){
        $params = array(
            'NRANEXOLANCBOL' => $row['NRANEXOLANCBOL'],
            'NRLANBOLREC'    => $row['NRLANBOLREC'],
            'CDFILIAL'       => $row['CDFILIAL'],
            'CDTELA'         => $row['CDTELA'],
            'DTANEXOLANCBOL' => $row['DTANEXOLANCBOL'],
            'NMANEXOLANCBOL' => $row['NMANEXOLANCBOL'][0]["name"],
            'NMPATHANEXO'    => $row['NMPATHANEXO'],
            'DSANEXOLANCBOL' => isset($row['DSANEXOLANCBOL']) ? $row['DSANEXOLANCBOL'] : null,
            'DSINFOANEXO'    => $row['DSINFOANEXO'],
            'NRORG'          => $row['NRORG']
        );
        
        $sql = "INSERT INTO ANEXOLANCBOL 
            (
                NRANEXOLANCBOL,
                NRLANBOLREC,
                CDFILIAL,
                CDTELA,
                DTANEXOLANCBOL,
                NMANEXOLANCBOL,
                NMPATHANEXO,
                DSANEXOLANCBOL,
                DSINFOANEXO,
                NRORG
            ) VALUES (
                :NRANEXOLANCBOL,
                :NRLANBOLREC,
                :CDFILIAL,
                :CDTELA,
                :DTANEXOLANCBOL,
                :NMANEXOLANCBOL,
                :NMPATHANEXO,
                :DSANEXOLANCBOL,
                :DSINFOANEXO,
                :NRORG
            )";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);

    }
    
    public function deletaAnexoTitulo($row){
        $params = array(
            'NRANEXOLANCBOL' => $row['NRANEXOLANCBOL'],
            'CDTELA'         => $row['CDTELA']
        );
        
        $sql = "DELETE FROM ANEXOLANCBOL 
            WHERE    
                NRANEXOLANCBOL = :NRANEXOLANCBOL
                AND CDTELA = :CDTELA";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);

    }

    public function selectDeleteAnexoIncompletos(){
        $params = array(
            'CDTELA' => 'REC00001'
        );
        
        $sql = "SELECT * FROM ANEXOLANCBOL 
            WHERE CDTELA = :CDTELA
            AND NRLANBOLREC IS NULL
            AND CDFILIAL IS NULL";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function deleteAnexoIncompletos($row){
        $params = array(
            'CDTELA'         => 'REC00001'
        );
        
        $sql = "DELETE FROM ANEXOLANCBOL 
            WHERE CDTELA = :CDTELA
            AND NRLANBOLREC IS NULL
            AND CDFILIAL IS NULL";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function uptadeAnexos($row){
        $params = array(
            'CDTELA'         => 'REC00001',
            'NRLANBOLREC'    => $row['NRLANBOLREC'],
            'CDFILIAL'       => $row['CDFILIAL'],
            'NMPATHANEXO'    => $row['NMPATHANEXO'],
            'NRANEXOLANCBOL' => $row['NRANEXOLANCBOL']
        );
        
        $sql = "UPDATE ANEXOLANCBOL 
            SET NRLANBOLREC = :NRLANBOLREC,
            CDFILIAL = :CDFILIAL,
            NMPATHANEXO = :NMPATHANEXO
            WHERE CDTELA = :CDTELA
            AND NRANEXOLANCBOL = :NRANEXOLANCBOL";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);

    }
    
    /**
     * Busca GeralRec
     * @param $nrorg
     * @return GeralRec
     */
    public function buscaGeralRec($nrorg){
        /** @var $params */
        $params = array();
        $params['nrorg'] = $nrorg;

        /** @var GeralRec $geralrec */
        $geralrec = $this->entityManager->getRepository(GeralRec::class)->findOneBy($params);

        return $geralrec;
    }
    
    /**
     * Busca Endereço Filial
     * @param $nrorg
     * @param $cdfilial
     * @param $idtpendefili
     * @return Endefili
     */
    public function buscaEndefili($nrorg, $cdfilial, $idtpendefili){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdfilial']     = $cdfilial;
        $params['idtpendefili'] = $idtpendefili;

        /** @var Endefili $endefili */
        $endefili = $this->entityManager->getRepository(Endefili::class)->findOneBy($params);

        return $endefili;
    }
    
    /**
     * Busca Municipio
     * @param $nrorg
     * @param $cdmunicipio
     * @return Municipio
     */
    public function buscaMunicipio($nrorg, $cdmunicipio){
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        $params['cdmunicipio'] = $cdmunicipio;

        /** @var Municipio $municipio */
        $municipio = $this->entityManager->getRepository(Municipio::class)->findOneBy($params);

        return $municipio;
    }
    
    
    
    public function buscaContaCorrenteVinculada($cdReduCont) {
        $sql = "SELECT CC.CDFILIAL, CC.CDREDUCONT, CC.DSCONTCORR
                    FROM CONTCORR CC
                WHERE CC.CDREDUCONT = '{$cdReduCont}'";

        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    }
    
    public function getLoteNotas($nrlanctonfs, $cdempresa) {
        $sql = "SELECT LOTE.CDEMPLOTERPS,
              LOTE.NRSEQLOTERPS,
              LOTE.NRINSJURIEST,
              EMP.NMRAZSOCEMPR,
              LOTE.CDINSCMUNI,
              LOTE.CDINSCESTA,
              LOTE.NRPROTOCOLORPS,
              LOTE.IDSTATUSLOTERPS,
              LOTE.DTENVIOLOTERPS,
              LOTE.NRORG,
              RPS.CDSERIERPS,
              COUNT(RPS.NRRPS) AS QTDERPS
            FROM LOTERPS LOTE
            INNER JOIN NUMERORPS RPS
            ON LOTE.CDEMPLOTERPS  = RPS.CDEMPLOTERPS
            AND LOTE.NRINSJURIEST = RPS.NRINSJURIEST
            AND LOTE.NRSEQLOTERPS = RPS.NRSEQLOTERPS
            AND LOTE.NRORG        = RPS.NRORG
            INNER JOIN EMPRESA EMP
            ON LOTE.CDEMPLOTERPS = EMP.CDEMPRESA
            AND LOTE.NRORG       = EMP.NRORG
            AND LOTE.NRSEQLOTERPS IN
          (SELECT DISTINCT NRSEQLOTERPS
          FROM NOTAFISCAL
          WHERE NRLANCTONF IN (:P_NRLANCTONF))
                AND LOTE.CDEMPLOTERPS = :P_CDEMPRESA
            GROUP BY LOTE.CDEMPLOTERPS,
              LOTE.NRSEQLOTERPS,
              LOTE.NRINSJURIEST,
              EMP.NMRAZSOCEMPR,
              LOTE.CDINSCMUNI,
              LOTE.CDINSCESTA,
              LOTE.NRPROTOCOLORPS,
              LOTE.IDSTATUSLOTERPS,
              LOTE.DTENVIOLOTERPS,
              LOTE.NRORG,
              RPS.CDSERIERPS
            ORDER BY IDSTATUSLOTERPS ASC,
              DTENVIOLOTERPS DESC";
            
            $params = array(
                'P_CDEMPRESA' => $cdempresa,
                'P_NRLANCTONF' => $nrlanctonfs
            );

            $std = $this->connection->prepare($sql);
            $std->execute($params);
            
            return $std->fetchAll();

    }
    
     public function buscaModeloBoleto($cdBanco) {

         $sql = "SELECT DSARQBOLCOMPMOBI
                        FROM BANCO
                        WHERE CDBANCO = $cdBanco";

        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    } 
    
    public function verificaDevolucaoDisponivel($params) {
        $sql = "SELECT NF.CDFILIAL, NF.NRLANCTONF, (NVL(NF.VRNOTAFISC,0) - D.VRDEVOLUCAOREC) AS VRDISP
                  FROM DEVOLUCAOREC D, NOTAFISCAL NF
                WHERE D.NRLANCTONF = :NRLANCTONF
                  AND D.CDFILIAL   = :CDFILIAL
                  AND D.CDFILIAL = NF.CDFILIAL
                  AND D.NRLANCTONF = NF.NRLANCTONF";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
            
        return $std->fetchAll();
    }
    
    /**
     * Valida Periodo Contabil
     * @param $nrorg
     * @param $cdfilial
     * @param $dtperiodo
     * @return array
     */
    public function validaPeriodoContabil(
        $nrorg, $cdfilial, $dtperiodo
    ){
        /** @var $params */
        $params = array();
        $params['P_NRORG'    ] = $nrorg;
        $params['P_CDEMPRESA'] = $cdfilial;
        $params['P_DTPERIODO'] = $dtperiodo;

        $sql = "SELECT CO.CDEMPRESA, CO.DTINICCTBL, CO.DTFINACCTBL
                  FROM CONTOPECTBL CO
                 WHERE CO.NRORG     = :P_NRORG     AND
                       CO.CDEMPRESA = :P_CDEMPRESA AND
                       :P_DTPERIODO BETWEEN CO.DTINICCTBL AND CO.DTFINACCTBL";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function atualizaStatusBoletrecassina($nrorg, $cdfilial, $nrlanbolrec, $idstatus){
        $params = array(
            'NRORG'       => $nrorg,
            'CDFILIAL'    => $cdfilial,
            'NRLANBOLREC' => $nrlanbolrec,
            'IDSTATUS'    => $idstatus
        );
        
        $sql = "UPDATE BOLETRECASSINA 
                   SET IDSTATUS    = :IDSTATUS
                 WHERE NRORG       = :NRORG
                   AND CDFILIAL    = :CDFILIAL
                   AND NRLANBOLREC = :NRLANBOLREC";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);

    }
    
    
    public function atualizaCodigoBarraBoletrecassina($nrorg, $cdfilial, $nrlanbolrec, $idstatus, $cdbarrarec, $dslinhabarrec){
        $params = array(
            'NRORG'       => $nrorg,
            'CDFILIAL'    => $cdfilial,
            'NRLANBOLREC' => $nrlanbolrec,
            'IDSTATUS'    => $idstatus,
            'CDBARRA'     => $cdbarrarec,
            'DSLINHADIGI' => $dslinhabarrec
        );
        
        $sql = "UPDATE BOLETRECASSINA 
                   SET IDSTATUS    = :IDSTATUS,
                       CDBARRA     = :CDBARRA,
                       DSLINHADIGI = :DSLINHADIGI
                 WHERE NRORG       = :NRORG
                   AND CDFILIAL    = :CDFILIAL
                   AND NRLANBOLREC = :NRLANBOLREC";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);

    }
    
    
    /**
     * Busca Dados SMTP Unidade
     * @param $nrorg
     * @param $cdfilial
     * @return array
     */
    public function getSmtp(
        $cdFilial, $idtpendefili, $nrorg
    ){
        /** @var $params */
        $params = array();
        $params['P_CDFILIAL'    ] = $cdFilial;
        $params['P_IDTPENDEFILI'] = $idtpendefili;
        $params['P_NRORG'] = $nrorg;
        $sql = "SELECT 
                  DSLOGINSMTP, 
                  CDSENHEMAILF, 
                  NVL(
                    DECODE(
                      CDPORTASMTP, NULL, DSSMTPEMAILF, DSSMTPEMAILF || ':' || CDPORTASMTP 
                    ), 
                    DSSMTPEMAILF
                  ) AS DSSMTPEMAILF, 
                  IDUTSSLEMAIL, 
                  IDUTTLSEMAIL 
                FROM 
                  ENDEFILI 
                WHERE 
                  CDFILIAL = : P_CDFILIAL 
                  AND IDTPENDEFILI = : P_IDTPENDEFILI 
                  AND NRORG = : P_NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Insert referente ao campo 'padrao de rateio' de quando vai lançar um novo titulo
     */
    public function insereNovoRateio(
		$cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $cdcentcust, $cdfiratctrec, $vrratccctrec, $nrorg
	) {
		$sql = "INSERT INTO RATCTRECEBER
				  (CDFILIAL, NRLANCTONF, NRSEQVENREC, NRPARCELAREC, CDCENTCUST, CDFIRATCTREC, VRRATCCCTREC, NRORG)
				VALUES
				  (:CDFILIAL, :NRLANCTONF, :NRSEQVENREC, :NRPARCELAREC, :CDCENTCUST, :CDFIRATCTREC,
					:VRRATCCCTREC, :NRORG)";

		$params = array("CDFILIAL"=>$cdfilial,
						"NRLANCTONF"=>$nrlanctonf,
						"NRSEQVENREC"=>$nrseqvenrec,
						"NRPARCELAREC"=>$nrparcelarec,
						"CDCENTCUST"=>$cdcentcust,
						"CDFIRATCTREC"=>$cdfiratctrec,
						"VRRATCCCTREC"=>$vrratccctrec,
						"NRORG"=>$nrorg);
						
		$query = $this->connection->prepare($sql);
		$query->execute($params);
    }
	
	/**
	 * Busca Padrão de Rateio 
	 */
    public function getPadraoDeRateio($params) {
        $sql = "SELECT I.NRORG, I.CDRATEIO, I.CDFILIAL AS CDFIRATCTREC, F.NMFILIAL, I.CDCENTCUST, C.NMCENTCUST, I.VRRATEIO AS VRPERATEIO FROM ITEMRATEIO I, FILIAL F, CENTCUST C WHERE I.CDRATEIO = :CDRATEIO AND I.CDFILIAL = F.CDFILIAL AND I.CDCENTCUST = C.CDCENTCUST";
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Busca dados do Título para contrução do Histórico
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @return array
     */
    public function buscaDadosTituloHistorico(
        $nrorg, $cdfilial, $nrlanbolrec
    ){

        $params = array();
        $params['P_NRORG'     ]  = $nrorg;
        $params['P_CDFILIAL'  ]  = $cdfilial;
        $params['P_NRLANBOLREC'] = $nrlanbolrec;

        $sql = "SELECT NF.CDFILIEMIT,
                      NF.NRNOTAFISC,
                      NF.NRLANCTONF,
                      TP.DSTPCTRECEB,
                      NVL(BO.DSBOLETREC, CO.DSCONTAREC) DSBOLETREC,
                      CL.NRINSJURCLIE,
                      CL.NMFANTCLIE,
                      CL.NMRAZSOCCLIE
                    FROM BOLETRECEBER BO,
                      CONTARECEBER CO,
                      NOTAFISCAL NF,
                      TIPOCTRECEBER TP,
                      CLIENTE CL
                    WHERE BO.NRORG     = CO.NRORG
                    AND BO.CDFILIAL    = CO.CDFILIBOLREC
                    AND BO.NRLANBOLREC = CO.NRLANBOLREC
                    AND CO.NRORG       = NF.NRORG
                    AND CO.CDFILIAL    = NF.CDFILIAL
                    AND CO.NRLANCTONF  = NF.NRLANCTONF
                    AND CO.NRORG       = TP.NRORG
                    AND CO.CDTPCTRECEB = TP.CDTPCTRECEB
                    AND BO.NRORG       = CL.NRORG
                    AND BO.CDCLIENTE   = CL.CDCLIENTE
                    AND BO.NRORG       = :P_NRORG
                    AND BO.CDFILIAL    = :P_CDFILIAL
                    AND BO.NRLANBOLREC = :P_NRLANBOLREC";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Busca Historico Padrao
     * @param $nrorg
     * @param $cdtpbaixarec
     * @param $cdoperacao
     * @return array
     */
    public function buscaHistoricoPadraoTipoBaixa(
        $nrorg, $cdtpbaixarec, $cdoperacao
    ){

        $params = array();
        $params['P_NRORG'       ] = $nrorg;
        $params['P_CDTPBAIXAREC'] = $cdtpbaixarec;
        $params['P_CDOPERACAO'  ] = $cdoperacao;

        $sql = "SELECT HI.NRORG,
                       HI.CDHISTPADR,
                       HP.DSHISTPADR
                  FROM
                      ( 
                          SELECT NRORG, CDHISTPADR FROM TIPOBAIXAREC WHERE CDTPBAIXAREC = :P_CDTPBAIXAREC AND NRORG = :P_NRORG
                          UNION ALL
                          SELECT NRORG, CDHISTPADR FROM HISTINTEGRA WHERE CDOPERACAO = :P_CDOPERACAO AND NRORG = :P_NRORG
                      ) HI,
                      HISTPADR HP
                    WHERE HI.CDHISTPADR = HP.CDHISTPADR
                      AND HI.NRORG      = HP.NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    /**
     * Busca Tipo de Débito Bancário
     * @param $nrorg
     * @param $cdtpdebibanc
     * @return Tpdebibanc
     */
    public function buscaTpdebibanc($nrorg, $cdtpdebibanc){
        /** @var $params */
        $params = array();
        $params['nrorg'] = $nrorg;
        $params['cdtpdebibanc'] = $cdtpdebibanc;

        /** @var Tpdebibanc $tpdebibanc */
        $tpdebibanc = $this->entityManager->getRepository(Tpdebibanc::class)->findOneBy($params);
        return $tpdebibanc;
    }
    
    /**
     * Insere Movimento de Débito Bancário
     * @param $cdreducont
     * @param nrmovidebi
     * @param cdclassfina
     * @param cdempresa
     * @param cdfilial
     * @param dtmovfluxo
     * @param idmovfluxo
     * @param nrmovbanc
     * @param cdtpdebibanc
     * @param dtmovidebi
     * @param dsmovidebi
     * @param dsmovidebi
     * @param vrmovidebi
     * @param cdectbmdeb
     * @param nrlctbmdeb
     * @param cdfililandeb
     * @return Movdebbanc
     */
    public function insereMovdebbanc(
        $nrorg, $cdreducont, $nrmovidebi, $cdclassfina, $cdempresa, $cdfilial, $dtmovfluxo, $idmovfluxo, $nrmovbanc, $cdtpdebibanc, 
        $dtmovidebi, $dsmovidebi, $vrmovidebi, $cdectbmdeb, $nrlctbmdeb, $cdfililandeb
    ){
        $movdebbanc = new Movdebbanc();
        
        $movdebbanc->setNrorg($nrorg);
        $movdebbanc->setCdreducont($cdreducont);
        $movdebbanc->setNrmovidebi($nrmovidebi);
        $movdebbanc->setCdclassfina($cdclassfina);
        $movdebbanc->setCdempresa($cdempresa);
        $movdebbanc->setCdfilial($cdfilial);
        $movdebbanc->setDtmovfluxo($dtmovfluxo);
        $movdebbanc->setIdmovfluxo($idmovfluxo);
        $movdebbanc->setNrmovbanc($nrmovbanc);
        $movdebbanc->setCdtpdebibanc($cdtpdebibanc);
        $movdebbanc->setDtmovidebi($dtmovidebi);
        $movdebbanc->setDsmovidebi($dsmovidebi);
        $movdebbanc->setVrmovidebi($vrmovidebi);
        $movdebbanc->setCdectbmdeb($cdectbmdeb);
        $movdebbanc->setNrlctbmdeb($nrlctbmdeb);
        $movdebbanc->setCdfililandeb($cdfililandeb);

        return $movdebbanc;
    }
    
    public function insereAjusteRenegociacao($params){
        
        
        $sql = "INSERT INTO AJBOLREC
                  (NRORG,CDFILIAL, NRLANBOLREC, CDAJUCTREC,
                   VRAJBOLREC, IDORIGAJBREC, IDTPAJBOLREC)
                values
                  (:NRORG, :CDFILIAL, :NRLANBOLREC, :CDAJUCTREC,
                   :VRAJBOLREC,:IDORIGAJBREC, :IDTPAJBOLREC)";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);

    }
    
     /* atualiza os boletos da baixa de renegociação*/
    public function updateBoletoBaixaNegociacao($params){
         $sql = "UPDATE BOLETRECEBER
                   SET CDFILIBAIREC = :CDFILIBAIREC,
                        NRBAIXAREC = :NRBAIXAREC,
                        VRFINALBREC = VRFINALBREC - NVL(:VRAJUSTE, 0),
                        IDSTATBREC = :IDSTATBREC,
                        DTLIQBOLREC = :DTLIQBOLREC,
                        CDTPBAIXAREC = :CDTPBAIXAREC,
                        CDREDUCONT = :CDREDUCONT
                   WHERE CDFILIAL = :CDFILIAL
                        AND NRLANBOLREC = :NRLANBOLREC
                        AND NRORG = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Valida Vinculo com Renegociacao Financeira 
     * @param $nrorg
     * @param $cdempresa
     * @param $cdcliente
     * @param $nrlanctonf
     * @param $cdfilialnf
    */
    public function validaVinculoRenegociacao($params){
        
        $sql = "SELECT NRSEQHISTRENEGO 
                    FROM HISTRENEGOCFIN HN, CLIENTE CL, EMPRESA EM
                     WHERE HN.CDCLIENTE = CL.CDCLIENTE
                       AND HN.CDEMPRESA = EM.CDEMPRESA
                       AND  HN.CDEMPRESA =:CDEMPRESA
                       AND  HN.CDCLIENTE =:CDCLIENTE
                       AND  HN.NRLANCTONF =:NRLANCTONF
                       AND  HN.CDFILIALNF =:CDFILIALNF
                       AND HN.NRORG =:NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Busca Boletos Descontados vinculados ao mesmo numero de movimentação de Crédito
     * @param $nrorg
     * @param $cdfilial
     * @param $nrmovicred
     */
    public function validaBoletoDescontadoVinculado($nrorg, $cdfilial, $nrmovicred, $cdreducont){
        
        /** @var $params */
        $params = array();
        $params['P_NRORG']       = $nrorg;
        $params['P_CDFILIAL']    = $cdfilial;
        $params['P_NRMOVICRED']  = $nrmovicred;
        $params['P_CDREDUCONT']  = $cdreducont;

         $sql = "SELECT * FROM BOLETRECEBER
                    WHERE NRMOVICRED = :P_NRMOVICRED
                    AND CDFILIAL = :P_CDFILIAL
                    AND NRORG = :P_NRORG
                    AND CDREDUCONT =: P_CDREDUCONT";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Busca o valor total de movimentacao de crédito
     * @param $nrorg
     * @param $nrmovicred
     * @param $nrmovicred
     */
    public function buscaTotalMovCredito($cdreducont,$nrmovicred){
        
        /** @var $params */
        $params = array();
        $params['P_NRMOVICRED']  = $nrmovicred;
        $params['P_CDREDUCONT']  = $cdreducont;

         $sql = "SELECT SUM(VRMOVICRED) AS TOTCRED 
                     FROM MOVCREDBANC
                            WHERE NRMOVICRED = :P_NRMOVICRED
                            AND CDREDUCONT = :P_CDREDUCONT";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function insereMovDebitoBancario($params){
        $sql = "INSERT INTO MOVDEBBANC (
                    CDREDUCONT, NRMOVIDEBI, CDCLASSFINA, 
                    CDEMPRESA, CDFILIAL, DTMOVFLUXO, 
                    IDMOVFLUXO, NRMOVBANC, CDTPDEBIBANC, 
                    DTMOVIDEBI, DSMOVIDEBI, VRMOVIDEBI, 
                    CDECTBMDEB, NRLCTBMDEB, CDFILILANDEB, DTINCLUSAO
                ) 
                VALUES 
                  (
                    :CDREDUCONT, :NRMOVIDEBI, :CDCLASSFINA, 
                    :CDEMPRESA, :CDFILIAL, :DTMOVFLUXO, 
                    :IDMOVFLUXO, :NRMOVBANC, :CDTPDEBIBANC, 
                    :DTMOVIDEBI, :DSMOVIDEBI, :VRMOVIDEBI, 
                    :CDECTBMDEB, :NRLCTBMDEB, :CDFILILANDEB ,SYSDATE
                  )";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);

    }
    
     /* atualiza os dados da conta corrente cancelamento de titulo descontado*/
    public function updateContaCancelamentoDescontado($params){
         $sql = "UPDATE CONTCORR
                    SET VRUTLCOBDESC = NVL(VRUTLCOBDESC, 0) + :VRMOVIDEBI
                    WHERE
                       CDREDUCONT = :CDREDUCONT";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
}