<?php
namespace Teknisa\FIN\Modules\ContaReceber\Services\Emissao\Banco;

use Teknisa\Libs\Util\DateUtil;
use Teknisa\FIN\Modules\ContaReceber\Services\Emissao\GeraEmissaoBoletoService;

/**
 * Class GeraEmissaoBoletoBrasilService
 * @package Teknisa\FIN\Modules\ContaReceber\Services\Emissao\Banco
 */
class GeraEmissaoBoletoBrasilService extends GeraEmissaoBoletoService
{

    /**
     * Tamanho do operador da linha digitavel de nosso numero.
     */
    const TAMANHO_NOSSO_NUMERO                   = 10;
    const MODULO_DIGITO_VERIFICADOR_NOSSONUM     = 11;
    const MODULO_DIGITO_VERIFICADOR_LINHADIGIT   = 10;
    const FATOR_VENCIMENTO                       = '07/10/1997';
    const FATOR_VENCIMENTO_NOVO                  = '03/07/2000';
    
    /**
     *  Calcula novo fator de vencimento 
     */
    public function getCalcFatorVencimento(){
            // data fixa 03/07/2000
            $dataFatorVencimento = DateUtil::truncateData(DateUtil::getDataDeString(self::FATOR_VENCIMENTO_NOVO));
            
            $datavenc    = DateUtil::truncateData($this->getDataVencimento());
            
            // Calcula a diferença entre as datas
            $intervalo = $datavenc->diff($dataFatorVencimento);
            
            // Converte o intervalo para dias
            $diasDiferenca = $intervalo->days;
            
            // Aplica o MOD 9000
            $resultadoMod = $diasDiferenca % 9000;
            
            // Soma 1000 ao resultado
            $resultadoFinal = $resultadoMod + 1000;
            
            $fatorDeVencimento   = str_pad($resultadoFinal, 4, "0", STR_PAD_LEFT);
            
            return $resultadoFinal;
    }

    public function getNossoNumeroComDigito()
    {
        $codigoReduzido = $this->getCodigoReduzido();
        $cdcontador      = self::PREFIXO_CDCONTADOR.$codigoReduzido;
        $cdcedente =  $this->getCdcedenteban();
        $nossoNumero     = \Geral\Util\NovoCodigo::geraCodigo (
            $this->boleto->getNrorg(), $cdcontador, 1, self::TAMANHO_NOSSO_NUMERO
        );
        
        if(strlen($this->getCdcedenteban()) == '6'){
            $nossoNumero = substr($nossoNumero, -5);
            $digito = $this->getDigitoVerificadorNossoNumeroBrasil($cdcedente.$nossoNumero);
            $nossoNumero = $cdcedente.$nossoNumero.$digito;
        } elseif(strlen($this->getCdcedenteban()) == '4'){
            $nossoNumero = substr($nossoNumero, -7);
            $digito = $this->getDigitoVerificadorNossoNumeroBrasil($cdcedente.$nossoNumero);
            $nossoNumero = $cdcedente.$nossoNumero.$digito;
        } elseif(strlen($this->getCdcedenteban()) == '7') {
            $sequencia = substr($nossoNumero, -10);
            $nossoNumero = $cdcedente.$sequencia;
        }
        
        
        return $nossoNumero;
    }


    /** Função que monta a Linha Digitável do boleto de acordo com o Layout do Banco Bradesco.
     * @param $nossoNumero
     * @return string
     */
    public function getLinhaDigitavel($nossoNumero)
    {   

        if (($this->boleto->getCdcartcobbol() == '16') or ($this->boleto->getCdcartcobbol() == '18')) {
            $campoLivre = substr(str_pad($this->getCdcedenteban(), 6, 0, STR_PAD_LEFT), 6) .
                substr(str_pad($this->getCdcedenteban(), 17, 0, STR_PAD_LEFT), 17) . '21';
        } else {
            if (strlen($this->getCdcedenteban()) < 7) {
                if (strlen($this->getCdcedenteban() == 4))
                    $nossoNumero = $this->getCdcedenteban() . substr($nossoNumero, -7);
                else if (strlen($this->getCdcedenteban() == 6))
                    $nossoNumero = $this->getCdcedenteban() . substr($nossoNumero, -5);
                    $tamanho = strlen($this->getCdcontcorr());
                    $campoLivre = substr(str_pad(substr($nossoNumero, 0, 11), 11, 0, STR_PAD_LEFT), 0, 11) .
                    substr(str_pad($this->getCdAgencia(), 4, 0, STR_PAD_LEFT), 0, 4) .
                    substr(str_pad(substr($this->getCdcontcorr(), 0,$tamanho -1), 8, 0, STR_PAD_LEFT), 0, 8) .
                    substr($this->boleto->getCdcartcobbol(), -2);
            } else if (strlen($this->getCdcedenteban()) == 7) {
                $campoLivre = '000000' .
                substr(str_pad($this->getCdcedenteban(), 7, 0, STR_PAD_LEFT), -7) .
                substr(str_pad($nossoNumero, 10, 0, STR_PAD_LEFT), -10) .
                substr(str_pad($this->boleto->getCdcartcobbol(), 2, 0, STR_PAD_LEFT), -2);
            } else {
               throw new \Exception('Código do Cedente Inválido!');
            }
        }
        
        $grupo1 = $this->getIdentificadorBanco() . '9' . substr($campoLivre, 0, 5) . self::getVerificadorGruposLinhaDigitavel($this->getIdentificadorBanco() . '9' . substr($campoLivre, 0, 5));
        $grupo2 = substr($campoLivre, 5, 10) . self::getVerificadorGruposLinhaDigitavel(substr($campoLivre, 5, 10));
        $grupo3 = substr($campoLivre, 15, 10) . self::getVerificadorGruposLinhaDigitavel(substr($campoLivre, 15, 10));
        $grupo4 = $this->getDigitoCodigoBarras();
        $grupo5 = $this->getCalcFatorVencimento() . $this->getValorNominal();
        return self::concatLinhaDigitavel($grupo1, $grupo2, $grupo3, $grupo4, $grupo5);

    }

    public function getCodigoDeBarras($nossoNumero)
    {
        $codigoDeBarras  = $this->getIdentificadorBanco();
        $codigoDeBarras .= self::MOEDA_REAL;
        //Esta string será substituída pelo código do dígito verificador
        $codigoDeBarras .= "_SUBSTITUICAO_DV_";
        $codigoDeBarras .= $this->getCalcFatorVencimento();
        $codigoDeBarras .= $this->getValorNominal();
        
        if(strlen($this->getCdcedenteban()) == 7){
            $nossoNumero = str_pad(substr($nossoNumero, 0, 17), 17, 0, STR_PAD_LEFT);
        } else {
            $nossoNumero = str_pad(substr($nossoNumero, 0, 11), 11, 0, STR_PAD_LEFT);
        }
        
		if (strlen($this->getCdcedenteban()) == 4){
			$codigoDeBarras .= $nossoNumero;
			$codigoDeBarras .= str_pad(substr($this->getCdAgencia(), 0, 4), 4, "0", STR_PAD_LEFT);
			$tamanho = strlen($this->getCdcontcorr());
			$codigoDeBarras .= str_pad(substr($this->getCdcontcorr(),0,$tamanho -1), 8, "0", STR_PAD_LEFT);
			$codigoDeBarras .= substr($this->boleto->getCdcartcobbol(), -2);
		}
		else
            if (substr($this->boleto->getCdcartcobbol(), 1, 2) == 16 || substr($this->boleto->getCdcartcobbol(), 1, 2) == 18){
                $codigoDeBarras .= $this->getCdcedenteban();
                $codigoDeBarras .= substr($nossoNumero, 0, 17);
                $codigoDeBarras .= '21';
            }
            else
                if (strlen($this->getCdcedenteban()) == 6){
                    $codigoDeBarras .= $nossoNumero;
                    $codigoDeBarras .= str_pad(substr($this->getCdAgencia(), 0, 4), 4, "0", STR_PAD_LEFT);
                    $tamanho = strlen($this->getCdcontcorr());
                    $codigoDeBarras .= str_pad(substr($this->getCdcontcorr(),0,$tamanho -1), 8, "0", STR_PAD_LEFT);
                    $codigoDeBarras .= substr($this->boleto->getCdcartcobbol(), -2);
                }
                else
                    if (strlen($this->getCdcedenteban()) == 7){
                        $codigoDeBarras .= '000000';
                        $codigoDeBarras .= substr($nossoNumero, 0, 17);
                        $codigoDeBarras .= substr($this->boleto->getCdcartcobbol(), - 2);
                    }
        $digito = $this->getDigitoVerificador(str_replace("_SUBSTITUICAO_DV_", "", $codigoDeBarras));
        $codigoDeBarras = str_replace("_SUBSTITUICAO_DV_", $digito, $codigoDeBarras);

        return $codigoDeBarras;
    }

    private function getValorNominal()
    {
        $valorBoleto = (string)$this->getValorBoleto();
        $valorBoleto = str_replace(',','.',$valorBoleto);
        $valorBoleto = number_format($valorBoleto, 2, '.', '');
        $arrayExclusao = array('.', ',');
        $valorBoleto = str_replace($arrayExclusao, "", $valorBoleto);
        $valorBoleto = str_pad($valorBoleto, 10, "0", STR_PAD_LEFT);
        return $valorBoleto;
    }


    private function getCodigoCedente()
    {
        if ($this->getCdcedenteban() == false) {
            $codigoCedente = '0000';
        } else {
            $codigoCedente = substr($this->getCdcedenteban(), 0, 4);
        }

        return $codigoCedente;
    }


    private function getIdentificadorBanco()
    {
        $identificadorBanco = $this->getCdbanccnab();
        return $identificadorBanco;
    }

    private function getFatorVencimento()
    {
        $datavenc            = DateUtil::truncateData($this->getDataVencimento());
        $dataFatorVencimento = DateUtil::truncateData(DateUtil::getDataDeString(self::FATOR_VENCIMENTO));
        $fatorDeVencimento   = DateUtil::getQtdeDiasEntre($dataFatorVencimento, $datavenc);
        $fatorDeVencimento   = str_pad($fatorDeVencimento, 4, "0", STR_PAD_LEFT);
        return $fatorDeVencimento;
    }

    private function getVerificadorGruposLinhaDigitavel($linhaDigitavel)
    {
        $linhaDigitavel = strrev($linhaDigitavel);
        $peso = 2;
        $total = 0;
		$multiplo = 0;
        for($i = 0; $i<=strlen($linhaDigitavel);$i++){
            $result = (int) substr($linhaDigitavel,$i,1)* $peso;
            if ($result > 9){
                $total += (int)substr((string)$result, 0, 1) + (int)substr((string)$result, 1,1);
            }else{
                $total += ((int) substr($linhaDigitavel,$i,1)* $peso);
            }
            $peso = $this->incrementaValorPeso($peso, 1, 2 );

       }

        $digito = $total % 10;
	    while($multiplo < $digito){
			$multiplo = $multiplo + 10;
		}

        return $multiplo - $digito;
    }

    public function getRelatorioBoleto($nrorg, $cdreducont){

        /** @var $contcorr \Geral\Entities\Contcorr */
        $contcorr = \Geral\Entities\Contcorr::findOneByNrorgAndCdreducont($nrorg,$cdreducont);
        $contcorr->getcdbanco();

        /** @var $dsarqbolcomp Banco*/
        $dsarqbolcomp = \Geral\Entities\Banco::findOneByCdbanco($contcorr->getcdbanco());
        $string = $dsarqbolcomp->getDsarqbolcomp();
        $tam = strlen($string);
        $tam_b = strrpos($string,'\\');
        $ext = -($tam - strrpos($string,'.'));
        if($tam_b === false){
            return NULL;
        }
        else
            return ($string_p = substr($string, ++$tam_b, $ext));
    }

	private function concatLinhaDigitavel($grupo1,$grupo2,$grupo3,$grupo4,$grupo5){

       	$linhadigitavel = substr($grupo1,0,5).'.'.substr($grupo1, 5).' ';

       	$linhadigitavel .= substr($grupo2,0, 5).'.'.substr($grupo2, 5).' ';

       	$linhadigitavel .= substr($grupo3,0, 5).'.'.substr($grupo3, 5).' ';

       	$linhadigitavel .= $grupo4.' ';

       	$linhadigitavel .= $grupo5;

       	return $linhadigitavel;
    }

    public function chamaRelatorio(){
        $nomeArquivo = $this->getCdfilial().$this->getNrlanbolrec();
        $arquivo = APP_PATH . DIRECTORY_SEPARATOR . "vendor" . DIRECTORY_SEPARATOR . "setforms" . DIRECTORY_SEPARATOR . "libraries" . DIRECTORY_SEPARATOR . "lib". DIRECTORY_SEPARATOR . "dompdf" . DIRECTORY_SEPARATOR . $nomeArquivo . '.html';

        ob_start();

        $params = $this->handleBoleto();

        $html = ob_get_clean();

        $htmlFile = fopen($arquivo, "w");
        fwrite($htmlFile, $html);
        fclose($htmlFile);

        require_once(APP_PATH . DIRECTORY_SEPARATOR . "vendor" . DIRECTORY_SEPARATOR . "setforms" . DIRECTORY_SEPARATOR . "libraries" . DIRECTORY_SEPARATOR . "lib". DIRECTORY_SEPARATOR . "dompdf" . DIRECTORY_SEPARATOR . "dompdf_config.inc.php");
        $dompdf = new \DOMPDF();
        $dompdf->load_html_file($arquivo);
        $dompdf->render();
        unlink($arquivo);
        $outfile = APP_PATH . DIRECTORY_SEPARATOR . "public" . DIRECTORY_SEPARATOR . "reports" . DIRECTORY_SEPARATOR . $nomeArquivo . '.pdf';
        file_put_contents($outfile, $dompdf->output( array("compress" => 0) ));

        return $params;
    }

    public function handleBoleto(){

        // DADOS DO BOLETO PARA O SEU CLIENTE
        $dias_de_prazo_para_pagamento = 5;
        $taxa_boleto = 0;
        $data_venc = $this->getDataVencimento()->format('d/m/Y'); // Prazo de X dias OU informe data: "13/04/2006";
        $valor_cobrado = $this->getValorBoleto(); // Valor - REGRA: Sem pontos na milhar e tanto faz com "." ou "," ou com 1 ou 2 ou sem casa decimal
        $valor_cobrado = str_replace(",", ".",$valor_cobrado);
        $valor_boleto=number_format($valor_cobrado+$taxa_boleto, 2, ',', '');

        $dadosboleto["nosso_numero"] = $this->getNossoNumeroComDigito();
        $dadosboleto["numero_documento"] = $this->getNrnotafisc();	// Num do pedido ou do documento
        $dadosboleto["data_vencimento"] = $data_venc; // Data de Vencimento do Boleto - REGRA: Formato DD/MM/AAAA
        $dadosboleto["data_documento"] = date("d/m/Y"); // Data de emiss�o do Boleto
        $dadosboleto["data_processamento"] = date("d/m/Y"); // Data de processamento do boleto (opcional)
        $dadosboleto["valor_boleto"] = $valor_boleto; 	// Valor do Boleto - REGRA: Com v�rgula e sempre com duas casas depois da virgula

        // DADOS DO SEU CLIENTE
        $dadosboleto["sacado"] = $this->getCliente();
        $dadosboleto["endereco1"] = $this->getDsendeclie();
        $dadosboleto["endereco2"] = $this->getMunicipioCliente() . " - " . $this->getEstadoCliente() . " - " . " CEP: " . $this->getCepCliente();

        // INSTRU��ES PARA O CAIXA
        $dadosboleto["instrucoes1"] = $this->getInstrucao1();
        $dadosboleto["instrucoes2"] = $this->getInstrucao2();

        // DADOS OPCIONAIS DE ACORDO COM O BANCO OU CLIENTE
        $dadosboleto["quantidade"] = " ";
        $dadosboleto["valor_unitario"] = " ";
        $dadosboleto["aceite"] = "N";
        $dadosboleto["especie"] = "R$";
        $dadosboleto["especie_doc"] = "DM";


        // ---------------------- DADOS FIXOS DE CONFIGURA��O DO SEU BOLETO --------------- //


        // DADOS DA SUA CONTA - BANCO DO BRASIL
        if(strlen($this->getCdAgencia()) > 4)
            $dadosboleto["agencia"] = substr($this->getCdAgencia(), 0, 4); // Num da agencia, sem digito
        else
            $dadosboleto["agencia"] = $this->getCdAgencia();

        if(strlen($this->getCdcontcorr()) > 7)
            $dadosboleto["conta"] = substr($this->getCdcontcorr(), 0, 7);
        else
            $dadosboleto["conta"] = $this->getCdcontcorr(); // Num da conta, sem digito

        // DADOS PERSONALIZADOS - BANCO DO BRASIL
        $dadosboleto["convenio"] = $this->getCdcedenteban();  // Num do conv�nio - REGRA: 6 ou 7 ou 8 d�gitos
        $dadosboleto["contrato"] = ""; // Num do seu contrato

        if(strlen($this->getCarteira()) > 2)
            $dadosboleto["carteira"] = substr($this->getCarteira(), 1, 2);
        else if(strlen($this->getCarteira()) < 2)
            $dadosboleto["carteira"] = str_pad($this->getCarteira(), 2, 0, STR_PAD_LEFT);
        else
            $dadosboleto["carteira"] = $this->getCarteira();
        $dadosboleto["variacao_carteira"] = "";  // Varia��o da Carteira, com tra�o (opcional)

        // TIPO DO BOLETO
        $dadosboleto["formatacao_convenio"] = strlen($this->getCdcedenteban()); // REGRA: 8 p/ Conv�nio c/ 8 d�gitos, 7 p/ Conv�nio c/ 7 d�gitos, ou 6 se Conv�nio c/ 6 d�gitos
        $dadosboleto["formatacao_nosso_numero"] = "2"; // REGRA: Usado apenas p/ Conv�nio c/ 6 d�gitos: informe 1 se for NossoN�mero de at� 5 d�gitos ou 2 para op��o de at� 17 d�gitos

        // SEUS DADOS
        $dadosboleto["cedente"] = $this->getNmepresa();
        $dadosboleto["cpf_cnpj"] = $this->getNrinsjurempr();

        include_once("BoletoPhp/include/funcoes_bb.php");
        include("BoletoPhp/include/variaveis_bb.php");
        include("BoletoPhp/include/layout_bb.php");

        return array(
            "CDBARRAREC" => $dadosboleto['codigo_barras'],
            "DSLINHABARREC" => $dadosboleto["linha_digitavel"],
            "CDNOSNUMREC" => $dadosboleto["nosso_numero"]
        );
    }
    
     private function getDigitoVerificadorNossoNumeroBrasil($sequencia){
        $numeros = str_split($sequencia);
        $pesos = array(7, 8, 9, 2, 3, 4, 5, 6, 7, 8, 9, 2, 3, 4, 5, 6, 7, 8, 9);
        $soma = 0;
    
        foreach ($numeros as $index => $numero) {
            $multiplicador = $pesos[$index];
            $resultado = $numero * $multiplicador;
            $soma += $resultado;
        }
    
        $resto = $soma % 11;
        $digitoVerificador = $resto;
        return $digitoVerificador;
    }
}
?>