<?php
namespace Teknisa\FIN\Modules\ContaReceber\Services\Emissao\Banco;

use Teknisa\Libs\Util\DateUtil;
use Teknisa\FIN\Modules\ContaReceber\Services\Emissao\GeraEmissaoBoletoService;

/**
 * Class GeraEmissaoBoletoItauService
 * @package Teknisa\FIN\Modules\ContaReceber\Services\Emissao\Banco
 */
class GeraEmissaoBoletoItauService extends GeraEmissaoBoletoService
{

    /**
     * Tamanho do operador da linha digitavel de nosso numero.
     */
    const TAMANHO_NOSSO_NUMERO                   = 8;
    const MODULO_DIGITO_VERIFICADOR_NOSSONUM     = 10;
    const MODULO_DIGITO_VERIFICADOR_LINHADIGIT   = 10;
    const FATOR_VENCIMENTO                       = '07/10/1997';
    const FATOR_VENCIMENTO_NOVO                  = '03/07/2000';
    const MODULO_DIGITO_VERIFICADOR = 10;

    /**
     *  Calcula o fator de vencimento conforme documentação do banco itaú  
     */
    public function getCalcFatorVencimento(){
            // data fixa 03/07/2000
            $dataFatorVencimento = DateUtil::truncateData(DateUtil::getDataDeString(self::FATOR_VENCIMENTO_NOVO));
            
            $datavenc    = DateUtil::truncateData($this->getDataVencimento());
            
            // Calcula a diferença entre as datas
            $intervalo = $datavenc->diff($dataFatorVencimento);
            
            // Converte o intervalo para dias
            $diasDiferenca = $intervalo->days;
            
            // Aplica o MOD 9000
            $resultadoMod = $diasDiferenca % 9000;
            
            // Soma 1000 ao resultado
            $resultadoFinal = $resultadoMod + 1000;
            
            $fatorDeVencimento   = str_pad($resultadoFinal, 4, "0", STR_PAD_LEFT);
            
            return $resultadoFinal;
    }
    
    
    
    public function getNossoNumeroComDigito()
    {
        $codigoReduzido = $this->getCodigoReduzido();
        $cdcontador      = self::PREFIXO_CDCONTADOR.$codigoReduzido;
        $nossoNumero     = \Geral\Util\NovoCodigo::geraCodigo (
            $this->boleto->getNrorg(), $cdcontador, 1, self::TAMANHO_NOSSO_NUMERO
        );
        
        if(in_array($this->getCarteira(),array('126','131','146','150','168')))
        
            /*  MONTAGEM DE MEMORIA DE CALCULO PARA '126','131','146','150','168'
                CARTEIRA 3 - POSIÇÕES
                NOSSO NUMERO 8 POSICOES
            */
            $nossoNumerocalc =  str_pad($this->getCarteira(), 3, "0", STR_PAD_LEFT).$nossoNumero;
            
        else
        
            /*  MONTAGEM DE MEMORIA DE CALCULO PARA CARTEIRAS NORMAIS
                AGENCIA - 4 POSICOES
                CONTA CORRENTE EM O DIGITO  - 5 POSIÇÕES
                CARTEIRA -  3 POSIÇÕES
                SEQUENCAL DO NOSSO NUMERO - 8 POSICOES
            */
    
            $nossoNumerocalc = str_pad(substr($this->getCdAgencia(),0,4),4,'0') .
                str_pad(substr($this->getCdcontcorr(),0,5),5,"0") .// ALTERADO 19/07/2016
                str_pad($this->getCarteira(), 3, "0", STR_PAD_LEFT).
                $nossoNumero;
        
        return $nossoNumero.$this->getDigitoVerificadorNossoNumero($nossoNumerocalc);
    }


    /** Função que monta a Linha Digitável do boleto de acordo com o Layout do Banco Itaú.
     * @param $nossoNumero
     * @return string
     */

    public function getLinhaDigitavel($nossoNumero)
    {
        // MEMORIA DE CALCULO DA LINHA DIGITAVEL
        
        $cdagencia = substr($this->getCdAgencia(), 0, 4);
        $cdcontcorr = substr($this->getCdcontcorr(), 0, 5);
        $nossoNumero = str_pad(substr($nossoNumero, 0, 8), 8, "0", STR_PAD_LEFT);
        $cdcartcobbol = str_pad(substr($this->boleto->getCdcartcobbol(),0,3), 3, "0", STR_PAD_LEFT);
        /** CAMPO 1: 
         * BANCO 3 POSICOES
         * MOEDA 1 POSICAO
         * CARTEIRA 3 POSICOES
         * NOSSO NUMERO 2 PRIMEIRAS POSICOES DO NOSSO NUMERO
         * DAC
        */
        
        $grupo1  = $this->getIdentificadorBanco().self::MOEDA_REAL.$cdcartcobbol;
        $grupo1 .= substr($nossoNumero,0,2);
        $grupo1 .= $this->getVerificadorGruposLinhaDigitavel($grupo1);
        
        /** CAMPO 2:
         * RESTANTE NOSSO NUMERO SEM O DV - 6 POSICOES
         * DAC DO CAMPO (AGENCIA/CONTACORRENTE/NOSSONUMERO)
         * TRES PRIMEIROS DIGITOS DA AGENCIA
         * DAC CALCULADO
        */
        
        $grupo2  = substr($nossoNumero,2,6);
        $grupo2 .= $this->getVerificadorGruposLinhaDigitavel($cdagencia.$cdcontcorr.$cdcartcobbol.$nossoNumero);
        $grupo2 .= substr($this->getCdAgencia(), 0, 3);
        $grupo2 .= $this->getVerificadorGruposLinhaDigitavel($grupo2);
        
        /** CAMPO 3:
         * RESTANTE DA AGENCIA - 1 POSIÇÃO
         * NUMERO DA CONTA CORRENTE + DAC
         * 3 POSICAO - ZERO FIXO
         * DAC
        */
        $grupo3  = substr($this->getCdAgencia(), 3, 1);
        $grupo3 .= str_pad(substr($this->getCdcontcorr(), 0, 6), 6, "0", STR_PAD_LEFT);
        $grupo3 .= "000";
        $grupo3 .= $this->getVerificadorGruposLinhaDigitavel($grupo3);
        
        /** CAMPO 4:
         * DAC DO CÓDIGO DE BARRAS - 1 POSICAO
        */
        $grupo4 = $this->getDigitoCodigoBarras();
        
        /** CAMPO 5:
         * FATOR DE VENCIMENTO - 4 POSICOES
         * VENCIMENTO - 10 POSICOES
        */
        $grupo5 = $this->getCalcFatorVencimento().$this->getValorNominal();
        return self::concatLinhaDigitavel($grupo1,$grupo2,$grupo3,$grupo4,$grupo5);
    }

    public function getCodigoDeBarras($nossoNumero)
    {
        
        /*
            MEMORIA DE CALCULO DAC CODIGO DE BARRAS
            BANCO - 3 POSICOES
            COD MOEDA - 1  POSICAO FIXO 9
            DAC COD BARRAS CALCULADO - 1  POSICAO
            FATOR DE VENCIMENTO - 4 POSICOES
            VALOR -  10 POSICOES
            CARTEIRA - 3 POSICOES
            NOSSO NUMERO + DAC - 9 POSICOES
            AGENCIA 4 POSICOES
            CONTA CORRENTE + DAC  6 POSICOES
            POSICOES LIVRE - 3 POSICOES FIXO 000
        */
        $composicao  = $this->getIdentificadorBanco();
        $composicao .= self::MOEDA_REAL;
        $composicao .= "_SUBSTITUICAO_DV_";
        $composicao .= $this->getCalcFatorVencimento();
        $composicao .= $this->getValorNominal();
        $composicao .= str_pad($this->boleto->getCdcartcobbol(), 3, "0", STR_PAD_LEFT);
        $composicao .= $nossoNumero;
        $composicao .= substr($this->getCdAgencia(), 0, 4);
        $composicao .= substr($this->getCdcontcorr(), 0, 6);
        $composicao .= '000'; 
        $dacCodBarras = $this->getDigitoVerificadorItau(str_replace("_SUBSTITUICAO_DV_", "", $composicao));
        $codigoDeBarras = str_replace("_SUBSTITUICAO_DV_", $dacCodBarras, $composicao);
        return $codigoDeBarras;
    }

    private function getValorNominal()
    {
        $valorBoleto = (string)$this->getValorBoleto();
        $valorBoleto = str_replace(',','.',$valorBoleto);
        $valorBoleto = number_format($valorBoleto, 2, '.', '');
        $arrayExclusao = array('.', ',');
        $valorBoleto = str_replace($arrayExclusao, "", $valorBoleto);
        $valorBoleto = str_pad($valorBoleto, 10, "0", STR_PAD_LEFT);
        return $valorBoleto;
    }

    private function getCodigoCedente()
    {
        if ($this->getCdcedenteban() == false) {
            $codigoCedente = '0000';
        } else {
            $codigoCedente = substr($this->getCdcedenteban(), 0, 4);
        }

        return $codigoCedente;
    }

    private function getIdentificadorBanco()
    {
        $identificadorBanco = $this->getCdbanccnab();
        return $identificadorBanco;
    }

    private function getFatorVencimento()
    {
        $datavenc            = DateUtil::truncateData($this->getDataVencimento());
        $dataFatorVencimento = DateUtil::truncateData(DateUtil::getDataDeString(self::FATOR_VENCIMENTO));
        $fatorDeVencimento   = DateUtil::getQtdeDiasEntre($dataFatorVencimento, $datavenc);
        $fatorDeVencimento   = str_pad($fatorDeVencimento, 4, "0", STR_PAD_LEFT);
        return $fatorDeVencimento;
    }

    private function getVerificadorGruposLinhaDigitavel($linhaDigitavel)
    {
        $linhaDigitavel = strrev($linhaDigitavel);
        $peso = 2;
        $total = 0;
        for($i = 0; $i<=strlen($linhaDigitavel);$i++){
            $result = (int) substr($linhaDigitavel,$i,1)* $peso;
            if ($result > 9){
                $total += (int)substr((string)$result, 0, 1) + (int)substr((string)$result, 1,1);
            }else{
                $total += ((int) substr($linhaDigitavel,$i,1)* $peso);
            }
            $peso = $this->incrementaValorPeso($peso, 1, 2);

        }
        $mod = $total % 10;
        $dac = 10 - $mod;
        
        if($dac > 9){
            $dac = '0';
        }

        return $dac;
    }
    
    private function getDacAgenciaConta($linhaDigitavel)
    {
        $linhaDigitavel = strrev($linhaDigitavel);
        $peso = 2;
        $total = 0;
        for($i = 0; $i<=strlen($linhaDigitavel);$i++){
            $result = (int) substr($linhaDigitavel,$i,1)* $peso;
            if ($result > 9){
                $total += (int)substr((string)$result, 0, 1) + (int)substr((string)$result, 1,1);
            }else{
                $total += ((int) substr($linhaDigitavel,$i,1)* $peso);
            }
            $peso = $this->incrementaValorPeso($peso, 1, 2);

        }
        $digito = $total % 10;
        
        $dac = 10 - $digito;
        
        if($dac === 10){
            return '0';
        } else {
            return $dac;
        }

    }

    public function getRelatorioBoleto($nrorg, $cdreducont){

        /** @var $contcorr \Geral\Entities\Contcorr */
        $contcorr = \Geral\Entities\Contcorr::findOneByNrorgAndCdreducont($nrorg,$cdreducont);
        $contcorr->getcdbanco();

        /** @var $dsarqbolcomp Banco*/
        $dsarqbolcomp = \Geral\Entities\Banco::findOneByCdbanco($contcorr->getcdbanco());
        $string = $dsarqbolcomp->getDsarqbolcomp();
        $tam = strlen($string);
        $tam_b = strrpos($string,'\\');
        $ext = -($tam - strrpos($string,'.'));
        if($tam_b === false){
            return NULL;
        }
        else
            return ($string_p = substr($string, ++$tam_b, $ext));
    }

    private function concatLinhaDigitavel($grupo1,$grupo2,$grupo3,$grupo4,$grupo5){

        $linhadigitavel = substr($grupo1,0,5) . '.' . substr($grupo1, 5) . ' ';

        $linhadigitavel .= substr($grupo2,0, 5) . '.' . substr($grupo2, 5) . ' ';

        $linhadigitavel .= substr($grupo3,0, 5) . '.' . substr($grupo3, 5) . ' ';

        $linhadigitavel .= $grupo4 . ' ';

        $linhadigitavel .= $grupo5;

        return $linhadigitavel;
    }

    private function getDigitoVerificadorNossoNumero($sequencia){

        // Inverte o nosso numero.
        $sequencia   = strrev($sequencia);
        $peso = 2;
        $acumulador = 0;

        // Este loop percorrera todos os caracteres da variavel $nossonumero
        for ($i = 0; $i < strlen($sequencia); $i++) {
            $result = ((int)substr($sequencia, $i, 1) * $peso);
            $acumulador += $result > 9 ? (int)substr((string)$result, 0, 1) + (int)substr((string)$result, 1,1) : $result;// ALTERADO 19/07/2016
            $peso = $this->incrementaValorPeso($peso, 1, 2);
        }

        $digito = $acumulador % 10;
        $digito = $this->avaliarDigitoRetornoNossoNumero($digito);

        $this->setDigitoCodigoBarras($digito);

        return $digito;
    }

    protected function avaliarDigitoRetornoNossoNumero($digito)
    {
        $result = self::MODULO_DIGITO_VERIFICADOR - $digito;
        if ($result == 10) return 0;
        else return $result;
    }
    
    protected function avaliarDacLinha($digito)
    {
        if ($digito == 10) return 1;
        else return $digito;
    }

    public function chamaRelatorio()
    {
        // TODO: Implement chamaRelatorio() method.
    }
}
