<?php
namespace Teknisa\FIN\Modules\ContaReceber\Services\Pix\Banco;

use Teknisa\FIN\Modules\ContaReceber\Services\Pix\Util\Api;
use Teknisa\FIN\Modules\ContaReceber\Services\Pix\Util\ApiItau;
use Teknisa\FIN\Modules\ContaReceber\Services\Pix\Util\Payload;
use Doctrine\ORM\EntityManager;

use Mpdf\QrCode\QrCode;
use Mpdf\QrCode\Output;

class GeraPixEfiService {
    
    function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }

    /**
     * Autenticação Acesso Pix
     * 
     * Antes de fazer qualquer requisição à API do PIX,
     * você precisará se autenticar obtendo um token de acesso.
     */
    public function autenticaAcessoPix($clientID, $clientSecret) {
        $authEndpoint = 'https://api-pix-h.gerencianet.com.br/oauth/token';
        
        $data = array(
            'grant_type' => 'client_credentials',
            'client_id' => $clientID,
            'client_secret' => $clientSecret
        );
    
        $options = array(
            'http' => array(
                'method'  => 'POST',
                'header'  => 'Content-type: application/x-www-form-urlencoded',
                'content' => http_build_query($data),
            ),
        );
        
        $context  = stream_context_create($options);
        $response = file_get_contents($authEndpoint, false, $context);
        $result = json_decode($response, true);
    
        return $result['access_token'];
    }
    
    /**
     * Cria Cobrança Pix
     * 
     * Aqui você pode tratar o resultado da criação da cobrança,
     * como exibir o QR Code para o pagador.
     */
    public function criaCobrancaPix(){
        try {
            
            
            
            $obApiPix = new Api('https://api-pix.gerencianet.com.br',
                                'Client_Id_a1294f4244491e2088a1cfb04270ca9b28c70472',
                                'Client_Secret_0a640a407192cdab6b029686c86bc7b426fc13b6',
                                __DIR__.'/files/certificates/certificado_producao.pem');
                       
            // TO DO - Criar o campo CLIENTID na Conta Corrente         
            // TO DO - Criar o campo CLIENTSECRET na Conta Corrente
            // TO DO - Criar o campo CERTIFICADO na Conta Corrente        
            // TO DO - Criar o campo CHAVEPIX na Conta Corrente
            $chavePix  = "b793987d-6cbd-4825-87d1-3e79ed2da114";
            
            //CORPO DA REQUISIÇÃO
            $request = [
              'calendario' => [
                'expiracao' => 3600
              ],
              'devedor' => [
                'cpf' => '12345678909',
                'nome' => 'Fulano de Tal'
              ],
              'valor' => [
                'original' => '10.00'
              ],
              'chave' => $chavePix,
              'solicitacaoPagador' => 'Pagamento do pedido 123'
            ];
            
            //RESPOSTA DA REQUISIÇÃO DE CRIAÇÃO
            $response = $obApiPix->createCob('WDEV1234123412340000000103', $request);
            
            //VERIFICA A EXISTÊNCIA DO ITEM 'LOCATION'
            if(!isset($response['location'])){
              echo 'Problemas ao gerar Pix dinâmico';
              echo "<pre>";
              print_r($response);
              echo "</pre>"; exit;
            }
            
            //INSTANCIA PRINCIPAL DO PAYLOAD PIX
            $obPayload = (new Payload)->setMerchantName('Fulano de Tal')
                                      ->setMerchantCity('SAO PAULO')
                                      ->setAmount($response['valor']['original'])
                                      ->setTxid('***')
                                      ->setUrl($response['location'])
                                      ->setUniquePayment(true);
            
            //CÓDIGO DE PAGAMENTO PIX
            $payloadQrCode = $obPayload->getPayload();
            
            return $payloadQrCode;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Cria Cobrança Vencimento Pix
     * 
     * Aqui você pode tratar o resultado da criação da cobrança,
     * como exibir o QR Code para o pagador.
     */
    public function criaCobrancaVPix($cdbanco, $cdclientid, $cdclientsecret, $dsdircertificado, $chavepix, $txtid, $vencimento, $validade, $cliente, $inscricao, $valor, $descricao){
        try {
            
            // URL BANCARIA + AUTENTIFICAÇÃO
            if($cdbanco == '033'){
                $urlbanco = 'https://pix.santander.com.br';
                $obApiPix = new ApiSantander($urlbanco, $cdclientid, $cdclientsecret, $dsdircertificado);
            }elseif($cdbanco == '341'){
                $urlbanco = 'https://sandbox.devportal.itau.com.br';
                $obApiPix = new ApiItau($urlbanco, $cdclientid, $cdclientsecret, $dsdircertificado);
                
                $request = "{
                      calendario: {
                        dataDeVencimento: '2023-12-31'
                      },
                      devedor: {
                        cpf: '12345678909',
                        nome: 'Francisco da Silva'
                      },
                      valor: {
                        original: '22.50'
                      },
                      chave: '5f84a4c5-c5cb-4599-9f13-7eb4d419dacc'
                }";
                
                
                
            }else{
                $urlbanco = 'https://api-pix.gerencianet.com.br';
                $obApiPix = new Api($urlbanco, $cdclientid, $cdclientsecret, $dsdircertificado);
            }
            
            // // CORPO DA REQUISIÇÃO
            // $request = '{
            //   "calendario": {
            //     "dataDeVencimento": "2023-12-31"
            //   },
            //   "devedor": {
            //     "cpf": "12345678909",
            //     "nome": "Francisco da Silva"
            //   },
            //   "valor": {
            //     "original": "22.50"
            //   },
            //   "chave": "5f84a4c5-c5cb-4599-9f13-7eb4d419dacc"
            // }';
            
            
            // $request = [
            // 	"calendario" => [
            // 		"dataDeVencimento" => $vencimento,
            // 		"validadeAposVencimento" => $validade
            // 	],
            // 	"devedor" => [
            // 		"nome" => $cliente,
            // 		"cpf" => $inscricao//,
            // 		// "cnpj" => "12345678000100"
            // 		// "email" => "emaildocliente@email.com.br",
            // 		// "logradouro" => "Alameda Souza, Numero 80, Bairro Braz",
            // 		// "cidade" => "Recife",
            // 		// "uf" => "PE",
            // 		// "cep" => "70011750"
            // 	],
            // 	"valor" => [
            // 		"original" => $valor//,
            // // 		"multa" => [
            // // 			"modalidade" => 2,
            // // 			"valorPerc" => "2.00"
            // // 		],
            // // 		"juros" => [
            // // 			"modalidade" => 2,
            // // 			"valorPerc" => "0.30"
            // // 		],
            // // 		"desconto" => [
            // // 			"modalidade" => 1,
            // // 			"descontoDataFixa" => [
            // // 				[
            // // 					"data" => "2022-10-15",
            // // 					"valorPerc" => "30.00"
            // // 				],
            // // 				[
            // // 					"data" => "2022-11-15",
            // // 					"valorPerc" => "15.00"
            // // 				],
            // // 				[
            // // 					"data" => "2022-10-15",
            // // 					"valorPerc" => "5.00"
            // // 				]
            // // 			]
            // // 		]
            // 	],
            // 	"chave" => $chavepix,
            // 	"solicitacaoPagador" => $descricao//,
            // //	"infoAdicionais" => [
            // // 		[
            // // 			"nome" => "Campo 1",
            // // 			"valor" => "Informação Adicional1"
            // // 		],
            // // 		[
            // // 			"nome" => "Campo 2",
            // // 			"valor" => "Informação Adicional2"
            // // 		]
            // //	]
            // ];

            // RESPOSTA DA REQUISIÇÃO DE CRIAÇÃO
            //$response = $obApiPix->createCobV($txtid, $request);
            $response = $obApiPix->getCobV($txtid, $request);
            
            // VERIFICA VIOLAÇÃO
            if(isset($response['violacoes'][0]['razao'])){
                var_dump('Operação bloqueada. Erro na geração do QR Code Pix: '.$response['violacoes'][0]['razao']);die;
            }
            
            // VERIFICA A EXISTÊNCIA DO ITEM 'LOCATION'
            if(!isset($response['loc']['location'])){
              echo 'Problemas ao gerar Pix dinâmico';
              echo "<pre>";
              print_r($response);
              echo "</pre>"; exit;
            }
            
            // INSTANCIA PRINCIPAL DO PAYLOAD PIX
            $obPayload = (new Payload)->setMerchantName('TEKNISA SERVICE')//$empresa
                                      ->setMerchantCity('BELO HORIZONTE')//$cidade
                                      ->setAmount($response['valor']['original'])
                                      ->setTxid('***')
                                      ->setUrl($response['loc']['location'])
                                      ->setUniquePayment(true);
            
            // CÓDIGO DE PAGAMENTO PIX
            $payloadQrCode = $obPayload->getPayload();           
            var_dump($payloadQrCode);die;
            
            return $payloadQrCode;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Consulta Cobrança Pix
     * 
     * Aqui você pode tratar o resultado da consulta da cobrança,
     * como exibir o status da transação.
     */
    public function consultaCobrancaPix($nrorg, $cdfilial, $nrlanbolrec){
        try {
            // Substitua 'SEU_CLIENT_ID' e 'SEU_CLIENT_SECRET' pelas suas credenciais
            $accessToken = $this->autenticaAcessoPix('SEU_CLIENT_ID', 'SEU_CLIENT_SECRET');
            
            $cobID = 'ID_DA_COBRANCA_PIX';
            $retrieveChargeEndpoint = 'https://api-pix.gerencianet.com.br/v2/cob/' . $cobID;
            
            $options = array(
                'http' => array(
                    'method'  => 'GET',
                    'header'  => 'Authorization: Bearer ' . $accessToken,
                ),
            );
            
            $context  = stream_context_create($options);
            $response = file_get_contents($retrieveChargeEndpoint, false, $context);
            $result = json_decode($response, true);
            
            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Cancela Cobrança Pix
     * 
     * Aqui você pode tratar o resultado do cancelamento da cobrança, 
     * como exibir a confirmação de cancelamento. 
     */
    public function cancelaCobrancaPix($nrorg, $cdfilial, $nrlanbolrec){
        try {
            // Substitua 'SEU_CLIENT_ID' e 'SEU_CLIENT_SECRET' pelas suas credenciais
            $accessToken = $this->autenticaAcessoPix('SEU_CLIENT_ID', 'SEU_CLIENT_SECRET');
            
            $cancelChargeEndpoint = 'https://api-pix.gerencianet.com.br/v2/cob/' . $cobID . '/cancel';
            
            $options = array(
                'http' => array(
                    'method'  => 'PUT',
                    'header'  => 'Authorization: Bearer ' . $accessToken,
                ),
            );
            
            $context  = stream_context_create($options);
            $response = file_get_contents($cancelChargeEndpoint, false, $context);
            $result = json_decode($response, true);
            
            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    
    
    /**
     * Cria Cobrança Pix
     * 
     * Aqui você pode tratar o resultado da criação da cobrança,
     * como exibir o QR Code para o pagador.
     */
    public function criaCobrancaPixSantander(){
        
        // Dados da cobrança
        $valor = 1.00; // Valor em reais
        $descricao = "Exemplo de pagamento Pix";
        $chavePix = "b793987d-6cbd-4825-87d1-3e79ed2da114"; // Chave Pix registrada no Banco Central
        $txId = uniqid(); // Identificador único da transação (pode ser um UUID, por exemplo)
        
        // Montar o payload da cobrança
        $payload = [
            "calendario" => [
                "expiracao" => 3600 // Tempo de expiração da cobrança em segundos (1 hora)
            ],
            "valor" => [
                "original" => (string)$valor
            ],
            "chave" => $chavePix,
            "solicitacaoPagador" => $descricao,
            "txid" => $txId
        ];
        
        // Converter o payload para JSON
        $payloadJson = json_encode($payload);
        
        // URL da API do Pix (exemplo, essa URL pode variar dependendo do banco)
        $apiUrl = "https://pix.santander.com.br/cob/txid";
        
        // Cabeçalho da requisição
        $headers = [
            "Content-Type: application/json",
            "Authorization: Bearer 123" // Substitua pelo token de autenticação fornecido pelo banco
        ];
        
        // Inicializar o cURL e configurar a requisição POST
/*        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $apiUrl);
        curl_setopt($curl, CURLOPT_POST, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $payloadJson);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        
        */
        $curl = curl_init();

        curl_setopt_array($curl, [
          CURLOPT_URL => "https://pix.santander.com.br/cob/TESTE123",
          CURLOPT_RETURNTRANSFER => true,
          CURLOPT_ENCODING => "",
          CURLOPT_MAXREDIRS => 10,
          CURLOPT_TIMEOUT => 30,
          CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
          CURLOPT_CUSTOMREQUEST => "GET",
          CURLOPT_HTTPHEADER => [
            "Authorization: ",
            "Content-Type: application/json"
          ],
        ]);

        // Executar a requisição
$response = curl_exec($curl);

        // Verificar se ocorreu algum erro
        if (curl_errno($curl)) {
            echo "Erro na requisição: " . curl_error($curl);
        } else {
            // Tratar a resposta
            $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
            if ($httpCode == 200) {
                // Sucesso: a cobrança foi criada com sucesso
                $responseData = json_decode($response, true);
                echo "Cobrança criada com sucesso! ID da cobrança: " . $responseData['loc']['id'];
            } else {
                // Erro: a API retornou um status diferente de 200
                echo "Erro na API: " . $httpCode . " - " . $response;
            }
        }
        
        // Fechar o cURL
        curl_close($curl);
    }
    
}