<?php

namespace Teknisa\FIN\Modules\ContaReceber\Services\Pix\Util;

class ApiItau{

  /**
   * URL base do PSP
   * @var string
   */
  private $baseUrl;

  /**
   * Client ID do oAuth2 do PSP
   * @var string
   */
  private $clientId;

  /**
   * Client secret do oAuht2 do PSP
   * @var string
   */
  private $clientSecret;

  /**
   * Caminho absoluto até o arquivo do certificado
   * @var string
   */
  private $certificate;

  /**
   * Define os dados iniciais da classe
   * @param string $baseUrl
   * @param string $clientId
   * @param string $clientSecret
   * @param string $certificate
   */
  public function __construct($clientId, $clientSecret, $certificate, $key, $homologacao = true){
    if($homologacao){
      $baseUrlAccessToken = 'https://sandbox.devportal.itau.com.br';
      $baseUrl = 'https://sandbox.devportal.itau.com.br';
    }else{
      $baseUrlAccessToken = 'https://sts.itau.com.br';
      $baseUrl = 'https://secure.api.itau';
    } 
    
    $this->baseUrlAccessToken = $baseUrlAccessToken;
    $this->baseUrl            = $baseUrl;
    $this->clientId           = $clientId;
    $this->clientSecret       = $clientSecret;
    $this->certificate        = $certificate;
    $this->certificatekey     = $key;
  }

  /**
   * Método responsável por criar uma cobrança imediata
   * @param  string $txid
   * @param  array $request
   * @return array
   */
  public function createCob($txid,$request){
    return $this->send('PUT','/pix_recebimentos/v2/cob/'.$txid,$request);
  }


  /**
   * Método responsável por criar uma cobrança imediata
   * @param  string $txid
   * @param  array $request
   * @return array
   */
  public function createCobV($txid,$request){
    return $this->send('PUT','/pix_recebimentos/v2/cobv/'.$txid, $request);
  }


  /**
   * Método responsável por criar uma cobrança imediata
   * @param  string $txid
   * @param  array $request
   * @return array
   */
  public function updateCobV($txid,$request){
    return $this->send('PATCH','/pix_recebimentos/v2/cobv/'.$txid, $request);
  }

  /**
   * Método responsável por criar uma cobrança imediata
   * @param  string $txid
   * @param  array $request
   * @return array
   */
  public function getCobV($txid,$request){
    return $this->send('GET','/pix_recebimentos/v2/cobv/'.$txid, $request);
  }

  /**
   * Método responsável por consultar uma cobrança imediata
   * @param  string $txid
   * @return array
   */
  public function consultCob($txid){
    return $this->send('GET','/pix');
  }


  /**
   * Método responsável por obter o token de acesso às APIs Pix
   * @return string
   */
  private function getAccessToken(){
    //ENDPOINT COMPLETO
    $endpoint = $this->baseUrlAccessToken.'/api/oauth/token';
    
    //HEADERS
    $headers = [
        'Content-Type: application/x-www-form-urlencoded',
        'User-Agent: insomnia/2023.5.5'
    ];

    //CORPO DA REQUISIÇÃO
    $request = 'grant_type=client_credentials&client_id='.$this->clientId.'&client_secret='.$this->clientSecret;
    
    //URL CERTIFICADO/KEY
    $url_certificado = $this->certificate;//sys_get_temp_dir().'/certificate.crt';
    $url_private_key = $this->certificatekey;//sys_get_temp_dir().'/ARQUIVO_CHAVE_PRIVADA.key';
    
    //PATH CERTIFICADO/KEY
    $path_certificado = sys_get_temp_dir().DIRECTORY_SEPARATOR.$this->clientId.'certificate.crt';
    $path_private_key = sys_get_temp_dir().DIRECTORY_SEPARATOR.$this->clientId.'ARQUIVO_CHAVE_PRIVADA.key';

    //INCLUI CERTIFICADO NA TMP(.crt)
    file_put_contents(
        $path_certificado,
        file_get_contents($url_certificado)
    );
    
    //INCLUI KEY NA TMP(.key)
    file_put_contents(
        $path_private_key,
        file_get_contents($url_private_key)
    );
    
    //CONFIGURAÇÃO DO CURL
    $curl = curl_init();
    curl_setopt_array($curl, array(
      CURLOPT_URL => $endpoint,
      CURLOPT_RETURNTRANSFER => true,
      CURLOPT_ENCODING => '',
      CURLOPT_MAXREDIRS => 10,
      CURLOPT_TIMEOUT => 0,
      CURLOPT_FOLLOWLOCATION => true,
      CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
      CURLOPT_CUSTOMREQUEST => 'POST',
      CURLOPT_POSTFIELDS => $request,
      CURLOPT_HTTPHEADER => $headers,
      
      CURLOPT_SSLCERT => $path_certificado,
      CURLOPT_SSLKEY => $path_private_key
    ));
    
    //EXECUTA O CURL
    $response = curl_exec($curl);
    curl_close($curl);
    
    //REMOVE CERTIFICADO/KEY DA TMP
    unlink($path_certificado);
    unlink($path_private_key);
    
    //RESPONSE EM ARRAY
    $responseArray = json_decode($response,true);
    
    //RETORNA O ACCESS TOKEN
    return $responseArray['access_token'] ?? '';
  }


  /**
   * Método responsável por enviar requisições para o PSP
   * @param  string $method
   * @param  string $resource
   * @param  array  $request
   * @return array
   */
  private function send($method,$resource,$request = []){
    //ENDPOINT
    $endpoint = $this->baseUrl.$resource;
    
    //HEADERS
    $headers = [
        'x-itau-apikey: '.$this->clientId,
        'x-itau-correlationID: 2',
        'Content-type: application/json',
        'Accept: application/json',
        'Authorization: Bearer '.$this->getAccessToken()
    ];
    
    //URL CERTIFICADO/KEY
    $url_certificado = $this->certificate;//sys_get_temp_dir().'/certificate.crt';
    $url_private_key = $this->certificatekey;//sys_get_temp_dir().'/ARQUIVO_CHAVE_PRIVADA.key';
    
    //PATH CERTIFICADO/KEY
    $path_certificado = sys_get_temp_dir().DIRECTORY_SEPARATOR.$this->clientId.'certificate.crt';
    $path_private_key = sys_get_temp_dir().DIRECTORY_SEPARATOR.$this->clientId.'ARQUIVO_CHAVE_PRIVADA.key';

    //INCLUI CERTIFICADO NA TMP(.crt)
    file_put_contents(
        $path_certificado,
        file_get_contents($url_certificado)
    );
    
    //INCLUI KEY NA TMP(.key)
    file_put_contents(
        $path_private_key,
        file_get_contents($url_private_key)
    );
    
    //CONFIGURAÇÃO DO CURL
    $curl = curl_init();
    curl_setopt_array($curl, array(
        CURLOPT_URL => $endpoint,
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => '',
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 0,
        CURLOPT_FOLLOWLOCATION => true,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST  => $method,
        CURLOPT_HTTPHEADER => $headers,
        CURLOPT_SSLCERT => $path_certificado,
        CURLOPT_SSLKEY => $path_private_key
    ));
    
    switch ($method) {
      case 'POST':
      case 'PUT':
      case 'PATCH':
        curl_setopt($curl,CURLOPT_POSTFIELDS,json_encode($request));
        break;
    }

    //EXECUTA O CURL
    $response = curl_exec($curl);
    curl_close($curl);
    
    //REMOVE CERTIFICADO/KEY DA TMP
    unlink($path_certificado);
    unlink($path_private_key);
    
    //RESPONSE EM ARRAY
    $responseArray = json_decode($response,true);
    
    //RETORNA O ARRAY DA RESPOSTA
    return $responseArray;
  }

}