<?php

namespace Teknisa\FIN\Modules\ContaReceber\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Teknisa\FIN\Modules\ContaReceber\Factory\Rec22000HistoricoRenegociacaoFinanceiraFactory;

class Rec22000HistoricoRenegociacaoFinanceiraService{
    
    protected $entityManager;
    protected $environment;
    protected $factory;
 
    public function __construct( EntityManager $entityManager, Environment $environment,Rec22000HistoricoRenegociacaoFinanceiraFactory $factory) {
        $this->entityManager = $entityManager;
        $this->environment   = $environment;
        $this->factory       = $factory;
        $this->nrorg = $this->environment->getNrOrgTrab();
    }
    
    public function getRenegociacao($row, $cdfilialnf, $nrlanctonf){
        try{
            // Verifica se tem conta baixada
            $verificaContaBaixada = $this->factory->verificaContaBaixada( $this->nrorg, $cdfilialnf, $nrlanctonf );
            
            //Verifica se tem boleto associado
            $verificaBoletoAssociado = $this->factory->verificaBoletoAssociado( $this->nrorg, $cdfilialnf, $nrlanctonf );
            
            // Se tiver conta baixada e/ou boleto associado, nao podera excluir, caso nao tenha nada, a rotina segue para excluir o registro.
            
            if(!empty($verificaContaBaixada)){
                throw new \Exception('Operação Bloqueada. Esta renegociação possui conta(s) baixada(s).');
            }else if(!empty($verificaBoletoAssociado)){
                throw new \Exception('Operação Bloqueada. Esta renegociação possui conta(s) relacionada(s) a um boleto real.');
            }else{
                $this->excluirRenegociacao($row, $cdfilialnf, $nrlanctonf);
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function excluirRenegociacao($row, $cdfilialnf, $nrlanctonf) {
    try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
    
            // Busca os documentos originados na renegociação e deleta os lançamentos.
            $getDocsOrigRenegoc = $this->factory->buscaDocOrigReneg($cdfilialnf, $nrlanctonf);
            foreach ($getDocsOrigRenegoc as $docs) {
                $params = array(
                    "CDEMPRESA" => $docs["CDEMPRESA"],
                    "NRLANCCONT" => $docs["NRLANCTBDOC"]
                );
                $excluirRenegociacaoLanccont = $this->factory->excluirRenegociacaoLanccont($params);
                $this->entityManager->flush();
                
                /* BUSCA OS AJUSTES DA CONTA */
            
                $dadosDocs = array(
                    'P_CDFILIAL' => $docs['CDFILIAL'],
                    'P_NRLANCTONF' => $docs['NRLANCTONF'],
                    'P_NRSEQVENREC' => $docs['NRSEQVENREC'],
                    'P_NRPARCELAREC' => $docs['NRPARCELAREC']
                );
                
                $buscaAjuste = $this->factory->buscaAjusteConta($dadosDocs);
                
                if (!empty($buscaAjuste) && isset($buscaAjuste['CDEMMFAJCNTR'], $buscaAjuste['NRLCTBAJREC'])) {
                /* DELETA O LANÇAMENTOS CONTÁBIL DO AJUSTE DO DOCUMENTO ORIGINADO */
                    $params = array(
                        'CDEMPRESA' => $buscaAjuste['CDEMMFAJCNTR'],
                        'NRLANCCONT' => $buscaAjuste['NRLCTBAJREC']
                    );
                    $excluirRenegociacaoLanccont = $this->factory->excluirRenegociacaoLanccont($params);
                    $this->entityManager->flush();
                }
                /* DELETE AJUSTE RECEBER E CONTA RECEBER*/
                $this->factory->excluirAjusteRec($dadosDocs);
                
                $this->entityManager->flush();
                $this->factory->excluirContaRec($dadosDocs);
                $this->entityManager->flush();
                
                /* DELETE BOLETORECEBER*/
                $params = array(
                    'P_CDFILIAL' => $docs['CDFILIBOLREC'],
                    'P_NRLANBOLREC' => $docs['NRLANBOLREC']
                );
                $excluirBoletoRec = $this->factory->excluirBoletoRec($params);
                $this->entityManager->flush();
                
            }
            /* DELETE NOTAFISCAL e ORÇAMENTO DE EVENTOS*/
            $params = array(
                'P_CDFILIAL' => $docs['CDFILIAL'],
                'P_NRLANCTONF' => $docs['NRLANCTONF']
            );
            $excluirNotaFiscal = $this->factory->excluirNotaFiscal($params);
            $this->entityManager->flush();
            $excluirOrcamentosEventos = $this->factory->excluirOrcamentosEventos($params);
            $this->entityManager->flush();
                    
            /*BUSCA A BAIXA DA NEGOCIAÇÃO PARA ATUALIZAR O STATUS DO BOLETO E DA CONTA*/
            $params = array(
                'P_NRSEQHISTRENEGO' => $row['NRSEQHISTRENEGO'],
                'P_CDCLIENTERENEGO' => $row['CDCLIENTE']
            );
            
            $buscaBaixaNegociacao = $this->factory->buscaBaixaNegociacao($params);

            /* BUSCA TODOS OS BOLETOS DA BAIXA DE RENEGOCIAÇÃO */
            $params = array(
                'P_CDFILIBAIREC' => $buscaBaixaNegociacao[0]['CDFILIAL'],
                'P_NRBAIXAREC' => $buscaBaixaNegociacao[0]['NRBAIXAREC']
            );
            $buscaBoletoBaixaRenegociacao = $this->factory->buscaBoletoBaixaRenegociacao($params);
 
            foreach ($buscaBoletoBaixaRenegociacao as $baixa) {
                /* BUSCA O AJUSTE DA RENEGOCIAÇÃO */
                $params = array(
                    'P_CDFILIAL' => $baixa['CDFILIAL'],
                    'P_NRLANBOLREC' => $baixa['NRLANBOLREC']
                );
                $buscaAjusteRenegociacao = $this->factory->buscaAjusteRenegociacao($params);
                  
                /* DELETA OS AJUSTES DE RENEGOCIAÇÃO*/
                if ($buscaAjusteRenegociacao) {
                    $params = array(
                        'P_CDFILIAL' => $baixa['CDFILIAL'],
                        'P_NRLANBOLREC' => $baixa['NRLANBOLREC']
                    );
                    $excluirAjusteRenegociacao = $this->factory->excluirAjusteRenegociacao($params);
                    $this->entityManager->flush();
                }
                
                /* ATUALIZA OS BOLETOS DA BAIXA DE RENEGOCIAÇÃO */
                $idStatBrec = empty($baixa['CDBARRAREC']) ? 'A' : 'E';
                $params = array(
                    "P_CDFILIAL" => $baixa["CDFILIAL"],
                    "P_NRLANBOLREC" => $baixa["NRLANBOLREC"],
                    "P_VRAJUSTE" => $buscaAjusteRenegociacao["VRAJBOLREC"] ? $buscaAjusteRenegociacao["VRAJBOLREC"] : 0,
                    "P_IDSTATBREC" => $idStatBrec
                );
                
                $this->factory->updateBoletoBaixaNegociacao($params);
                $this->entityManager->flush();
                
                /* ATUALIZA AS CONTAS DA BAIXA DE RENEGOCIAÇÃO*/
                $params = array(
                    "P_CDFILIBOLREC" => $baixa["CDFILIAL"],
                    "P_NRLANBOLREC" => $baixa["NRLANBOLREC"]
                );
                $this->factory->updateContaBaixaNegociacao($params);
                $this->entityManager->flush();
            }
                 
            /* DELETA LANÇAMENTO CONTÁBIL DA BAIXA */
            $params = array(
                'CDEMPRESA' => $buscaBaixaNegociacao[0]['CDEMPRESA'],
                'NRLANCCONT' => $buscaBaixaNegociacao[0]['NRLCTBBREC']
            );
            $excluiLancContabilBaixa = $this->factory->excluiLancContabilBaixa($params);
            $this->entityManager->flush();
            /* DELETA A BAIXA DA NEGOCIAÇÃO */
            $params = array(
                'P_CDFILIAL' => $buscaBaixaNegociacao[0]['CDFILIAL'],
                'P_NRBAIXAREC' => $buscaBaixaNegociacao[0]['NRBAIXAREC']
            );
            $excluiBaixaNegociacao = $this->factory->excluiBaixaNegociacao($params);
            $this->entityManager->flush();
            
            /*Deleta historico*/
            $params = array(
                'CDCLIENTE' => $row['CDCLIENTE'],
                'NRSEQHISTRENEGO' => $row['NRSEQHISTRENEGO']
            );
            $exluiHistorico = $this->factory->excluiHistorico($params);
            $this->entityManager->flush();
                
            $connection->commit();
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
}
