<?php
namespace Teknisa\FIN\Modules\ContaReceber\Services\Remessa\Banco;

use Financeiro\Entities\Boletreceber;
use Financeiro\Entities\Emprbanc;
use Financeiro\Logic\ContasReceber\Cnab\Remessa\GeraCnab;
use Geral\Entities\Banco;
use Geral\Entities\Endeclie;
use Geral\Entities\Licencauso;
use Geral\Entities\Contcorr;
use Geral\Entities\Empresa;
use Geral\Entities\Municipio;
use Teknisa\Libs\Util\DateUtil;
use Financeiro\Util\Util;
use Geral\Entities\Cliente;
use Geral\Entities\Filial;
use Teknisa\FIN\Modules\ContaReceber\Services\Remessa\GeraCnabService;

class GeraCnabDaycovalService extends GeraCnabService
{
  const IDENTIFICACAO_REGISTRO = '0';
  const IDENTIFICACAO_ARQUIVO_REMESSA = '1';
  const LITERAL_REMESSA = 'REMESSA';
  const CODIGO_SERVICO = '01';
  const LITERAL_SERVICO = 'COBRANCA';
  const CODIGO_COMPENSACAO_DAYCOVAL = '707';
  const NOME_BANCO_EXTENSO = 'BANCO DAYCOVAL';
  const TIPO_INSCRICAO_EMPRESA_CPF = '01';
  const TIPO_INSCRICAO_EMPRESA_CGC = '02';
  const CODIGO_REMESSA = '1';
  const IDENTIFICACAO_REGISTRO_TRANSACAO = '1';
  const IDENTIFICACAO_REGISTRO_TRAILLER = '9';
  const IDENTIFICACAO_REGISTRO_MENSAGEM = '2';
  
    public function geraHeaderArquivo($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec, $nrcnab, $cdreducont, $carteira = NULL, $contreg = NULL, $idtipcnabrec = NULL, $cdfilial, $contador = NULL)
    {
        try
        {
            if (!is_dir($caminho)) mkdir($caminho, "0777", true);
            $nmarquivo = $caminho . $nmarquivo;
            $file = fopen($nmarquivo, "w+");

            // CAMPO: Identificação do registro Header
            // POSIÇÃO: 001 a 001
            // PICTURE: 9(1)
            $arquivo = self::IDENTIFICACAO_REGISTRO;
            
            // CAMPO: Código da Remessa
            // POSIÇÃO: 002 a 002
            // PICTURE: 9 (1)
            $arquivo .= self::IDENTIFICACAO_ARQUIVO_REMESSA;
            
            // CAMPO: Identificação por Extenso do tipo de movimento
            // POSIÇÃO: 003 a 009
            // PICTURE: X (7)
            //CONTEUDO: REMESSA
            $arquivo .= self::LITERAL_REMESSA;
            
            // CAMPO: Identificação do tipo de Serviço
            // POSIÇÃO: 010 a 011
            // PICTURE: 9 (2)
            $arquivo .= self::CODIGO_SERVICO;
            
            // CAMPO: Identificação por Extenso do tipo de serviço
            // POSIÇÃO: 012 a 026
            // PICTURE: X (15)
            $arquivo .= self::LITERAL_SERVICO;
            $arquivo .= self::preenche_brancos('', 7);
            
            // CAMPO: a Identificação da Empresa no Banco
            // POSIÇÃO: 027 a 046
            // PICTURE: X (20)
            $codigoEmpresa = self::getCodigoEmpresa($cdbanco, $cdempresa, $cdidrec);
            if ($codigoEmpresa)
                $arquivo .= str_pad($codigoEmpresa, 20);
            else
                $arquivo = str_pad($arquivo, 20, "0");
            
            // CAMPO:  Nome por extenso da Empresa.
            // POSIÇÃO: 047 a 076
            // PICTURE: X (30)
            $empresa = Empresa::findOneByCdempresa($cdempresa);
            $filial = Filial::findOneByCdfilial($cdfilial);
            
            // Caso seja informado filial na geração do CNAB, sera inserido nome da razão social da filial e não da empresa no arquivo
            if($filial) {
                $nmrazsocemp = Util::campoCnabValido($filial->getNmrazsocfili());
                $nmrazsocemp = substr($nmrazsocemp, 0, 30);
                $arquivo .= str_pad($nmrazsocemp, 30);
            } else {
                $nmrazsocemp = Util::campoCnabValido($empresa->getNmrazsocempr());
                $nmrazsocemp = substr($nmrazsocemp, 0, 30);
                $arquivo .= str_pad($nmrazsocemp, 30);
            }
            
            // CAMPO: No. do Banco na câmara de compensação bancária
            // POSIÇÃO: 077 a 079
            // PICTURE: 9 (3)
            $arquivo .= self::CODIGO_COMPENSACAO_DAYCOVAL;
             
            // CAMPO: Nome por extenso do Banco
            // POSIÇÃO: 080 a 094
            // PICTURE: X (15)
            $arquivo .= str_pad(self::NOME_BANCO_EXTENSO, 15);
            
            // CAMPO: Data da Gravação da fita
            // POSIÇÃO: 095 a 100
            // PICTURE: 9 (6)
            $now = DateUtil::getDataAtual();
            $data = DateUtil::getDia($now) . DateUtil::getMes($now) . substr(DateUtil::getAno($now), -2);
            $arquivo .= Util::campoCnabValido($data);
            
            // CAMPO: Complementação do Registro
            // POSIÇÃO: 101 a 394
            // PICTURE: X (294)
            $arquivo = str_pad($arquivo, 394);
            
            // CAMPO: No. Sequencial do Registro no Arquivo
            // POSIÇÃO: 395 400
            // PICTURE: 9 (6)
            $arquivo .= str_pad($contreg + 1, 6, "0", STR_PAD_LEFT) . "\r\n";
            
            fprintf($file, "%s", $arquivo);
            fclose($file);
        }
        catch(\Exception $e)
        {
            throw $e;
        }
    }

    public function geraTransacao($caminho, $nmarquivo,$nrorg, $cdempresa, $cdcartcobbol, $cdagencia, $cdcontcorr, $cdfilial, $nrlanbolrec, $vrpemobrec, $idimprboleto, $cdnosnumrec, $dtvenbolrec, $cdins1cobbol, $qtdiaprotbol, $vrmordiabrec, $vrfinalbrec, $idopecnabrec, $vrpemulbrec, $idtpijurclie, $nrinsjurclie, $cdcliente, $cdins2cobbol, $contreg = NULL, $idtipcnabrec = null, $idfilialcnabrec, $cddiaprotbol = NULL, $contador = NULL, $cdreducont, $cdbanco)
    {
        if (!is_dir($caminho)) mkdir($caminho, "0777", true);
        $nmarquivo = $caminho . $nmarquivo;
        $file = fopen($nmarquivo, "a+");

        // CAMPO: Identificação do registro de transação
        // POSIÇÃO: 001 001
        // PICTURE: 9 (1)
        $arquivo = self::IDENTIFICACAO_REGISTRO_TRANSACAO;
        
        /**
         * CAMPO: Identificação do tipo de inscrição da empresa
         * POSIÇÃO: 002 a 003
         * PICTURE: 9 (2)
         * 01-CPF do cedente
         * 02-CNPJ do cedente
         * 03-CPF do Sacador
         * 04-CNPJ Sacador
        */
        $empresa = Empresa::findOneByCdempresa($cdempresa);
        $filial = Filial::findOneByCdfilial($idfilialcnabrec);
        
        if($filial){
            if($filial && $filial->getIdtpijurfili() == 'F'){
                $arquivo .= self::TIPO_INSCRICAO_EMPRESA_CPF;
            } else {
                $arquivo .= self::TIPO_INSCRICAO_EMPRESA_CGC;
            }
        } else {
            if($empresa && $empresa->getIdtpijurempr() == 'F'){
                $arquivo .= self::TIPO_INSCRICAO_EMPRESA_CPF;
            }else{
                $arquivo .= self::TIPO_INSCRICAO_EMPRESA_CGC;
            }
        }

        // CAMPO: a Número de inscrição da empresa (CNPJ/CPF) ou do Sacador
        // POSIÇÃO: 004 a 017
        // PICTURE: 9 (14)
        if($idfilialcnabrec){
            $filial = Filial::findOneByCdfilial($idfilialcnabrec);
            $arquivo .= str_pad($filial->getNrinsjurfili(), 14, '0', STR_PAD_LEFT);
        } else {
           $arquivo .= str_pad($empresa->getNrinsjurempr(), 14, '0', STR_PAD_LEFT); 
        }

        // CAMPO: a Identificação da empresa no Banco
        // POSIÇÃO: 018 a 037
        // PICTURE: X (20)
        $codigoEmpresa = self::getCodigoEmpresa($cdbanco, $cdempresa, $idtipcnabrec);
        if ($codigoEmpresa)
            $arquivo .= str_pad($codigoEmpresa, 20);
        else
            $arquivo = str_pad($arquivo, 20, "0");
         
        // CAMPO:  Identificação do título na Empresa
        // POSIÇÃO: 038 a 062
        // PICTURE: X (25)
        $arquivo .= str_pad($cdfilial.$nrlanbolrec,25);
        
        // CAMPO: Identificação Do título
        // POSIÇÃO: 063 070
        // PICTURE: 9 (8)
        $boleto = Boletreceber::findOneByNrlanbolrecAndCdfilial($nrlanbolrec, $cdfilial);
        if($boleto->getIdimprbolbol() == 'B'){
           $arquivo .= str_repeat('0',8);
        } else {
            $nossoNumero = str_replace(' ', '', Util::campoCnabValido($cdnosnumrec));
            $arquivo .= substr($nossoNumero, -8);
        }
        
        // CAMPO:  BRANCOS
        // POSIÇÃO: 071 a 075
        // PICTURE: 9(5)
        $arquivo .= self::preenche_brancos('', 5);
        
        // CAMPO: BRANCOS
        // POSIÇÃO: 076 a 083
        // PICTURE: X (8)
        $arquivo .= self::preenche_brancos('', 8);
        
        // CAMPO: Identificação da operação no banco - BRANCOS
        // POSIÇÃO: 084 a 107
        // PICTURE: X (24)
        $arquivo .= self::preenche_brancos('', 24);
        
        // CAMPO: Código de remessa
        // POSIÇÃO: 108 a 108
        // PICTURE: X (1)
        $arquivo .= "6";
        
        /**
         * CAMPO: Identificação da Ocorrência
         * POSIÇÃO: 109 a 110
         * PICTURE: 9 (2)
         * 01 . Remessa
         * 02 . Pedido de Baixa
         * 04 . Concessão de Abatimento
         * 06 . Alteração de Vencimento
         * 09 . Protestar
         * 10 . Pedido de não protestar
         * 18 . Sustar protesto
         */
        if($idopecnabrec && $idopecnabrec == 'E')
            $arquivo .= '02';    //Exclusão
        elseif($idopecnabrec == 'I')
            $arquivo .= '01';    //Inclusão
        else
            $arquivo .= '06';    //Alteração

        // CAMPO: Nº. Documento de cobrança (duplicata,promissória etc.)
        // POSIÇÃO: 111 a 120
        // PICTURE: X (10)
        $entitieDup = Boletreceber::getRepository();
        $duplicata = $entitieDup->getDuplicata($cdfilial, $nrlanbolrec);
        if($duplicata)
            $arquivo .= str_pad(substr($duplicata[0]['DUPLICATA'], -7), 8, '0', STR_PAD_LEFT) . str_pad($duplicata[0]['NRPARCELA'], 2, '0', STR_PAD_LEFT);
        else
            $arquivo .= $nrlanbolrec;

        // CAMPO:  Nº. Documento de cobrança (duplicata,promissória etc.)
        // POSIÇÃO: 121 a 126
        // PICTURE: 9 (6)
        $vencimento = DateUtil::getDataDeString(DateUtil::truncateString($dtvenbolrec));
        $vencimento = DateUtil::getDia($vencimento) . DateUtil::getMes($vencimento) . substr(DateUtil::getAno($vencimento), -2);
        $arquivo .= Util::campoCnabValido($vencimento);
        
        // CAMPO: Valor nominal do Título
        // POSIÇÃO: 127 a 139
        // PICTURE: 9(11) V9(2)
        $vr = $vrfinalbrec * 100;
        $vr = str_pad($vr, 13, "0", STR_PAD_LEFT);
        $arquivo .= Util::campoCnabValido($vr);
        
        // CAMPO: Nº Banco na câmara de compensação Bancária
        // POSIÇÃO: 140 a 142
        // PICTURE: 9 (3)
        $arquivo .= self::CODIGO_COMPENSACAO_DAYCOVAL;
        
        // CAMPO:  Agência cobradora
        // POSIÇÃO: 143 a 146
        // PICTURE: 9 (4)
        //CONTEUDO: Zeros
        $arquivo .= str_repeat('0',4);
        
        // CAMPO: Dígito de autoConferência da ag. Cobradora
        // POSIÇÃO: 147 a 147
        // PICTURE: 9 (1)
        //CONTEUDO: Zeros
        $arquivo .= str_repeat('0',1);
        
        // CAMPO: Espécie do título
        // POSIÇÃO: 148 a 149
        // PICTURE: X (2)
        $arquivo .= '12';
        
        // CAMPO: Identificação do título aceito ou não aceito
        // POSIÇÃO: 150 a 150
        // PICTURE: X (1)
        $arquivo .= 'N';
        
        // CAMPO: Data da emissão do título
        // POSIÇÃO: 151 a 156
        // PICTURE: 9 (6)
        $now = DateUtil::getDataAtual();
        $data = DateUtil::getDia($now) . DateUtil::getMes($now) . substr(DateUtil::getAno($now), -2);
        $arquivo .= Util::campoCnabValido($data);
        
        // CAMPO: ZEROS
        // POSIÇÃO: 157 a 158
        // PICTURE: 9 (2)
        $arquivo .= '00';
        
        // CAMPO: ZEROS
        // POSIÇÃO: 159 a 160
        // PICTURE: 9 (2)
        $arquivo .= '00';
        
        // CAMPO: Valor de mora por dia de atraso
        // POSIÇÃO: 161 a 173
        // PICTURE: 9 (11) V9 (2)
        //ZEROS
        $arquivo = str_pad($arquivo, 173, '0');
        
        // CAMPO: Data limite para concessão de desconto
        // POSIÇÃO: 174 a 179
        // PICTURE: 9 (6)
        $arquivo = str_pad($arquivo, 179, '0');
        
        // CAMPO: Valor do desconto a ser concedido
        // POSIÇÃO: 180 a 192
        // PICTURE: 9 (11) V9 (2)
        $arquivo = str_pad($arquivo, 192, '0');
        
        // CAMPO: Uso do Banco - ZEROS
        // POSIÇÃO: 193 a 205
        // PICTURE: 9 (13)
        //Zeros
        $arquivo = str_pad($arquivo, 205, '0');

        // CAMPO: Valor de Abatimento
        // POSIÇÃO: 206 a 218
        // PICTURE: 9 (11) V9(2)
        $arquivo = str_pad($arquivo, 218, '0');
        
        // CAMPO: Identificação do tipo de inscrição do sacado
        // 01 – CPF - 02 – CGC
        // POSIÇÃO: 219 a 220
        // PICTURE: 9 (2)
        if ($idtpijurclie) {
            switch ($idtpijurclie){
                case 'F':
                    $arquivo .= '01';
                    break;
                case 'J':
                    $arquivo .= '02';
                    break;
                case 'I':
                case 'O':
                    $arquivo .= '99';
            }
        }
        else
            $arquivo .= '98';
        
        // CAMPO: CNPJ ou CPF Número de inscrição do sacado
        // POSIÇÃO: 221 a 234
        // PICTURE: 9 (14)
        $nrinsjurempr = Cliente::findOneByNrorgAndCdcliente($nrorg, $cdcliente);
        
        $arquivo .= Util::campoCnabValido(str_pad($nrinsjurempr->getNrinsjurclie(), 14, "0", STR_PAD_LEFT));

        // CAMPO: Nome do sacado
        // POSIÇÃO: 235 a 264
        // PICTURE: X (30)
        
        $razaoSocial = Util::campoCnabValido($nrinsjurempr->getNmrazsocclie());
        $razaoSocial = substr($razaoSocial, 0, 30);
        $arquivo .= str_pad($razaoSocial, 30);
        
        // CAMPO: Complementação do Registro
        // POSIÇÃO: 265 a 274
        // COTEUDO: BRANCOS
        // PICTURE: X (10)
        $arquivo .= self::preenche_brancos('', 10);
        
        // CAMPO: Logradouro (Rua, Número e complemento do sacado )
        // POSIÇÃO: 275 a 314
        // PICTURE: X (40)
        $endeclie = Endeclie::findOneByCdclienteAndIdtpendeclie($cdcliente, 'C');
        $enderecoCliente = Util::campoCnabValido($endeclie->getDsendeclie());
        $enderecoCliente = substr($enderecoCliente, 0, 40);
        $arquivo .= str_pad($enderecoCliente, 40);
        
        // CAMPO: Bairro
        // POSIÇÃO: 315 a 326
        // PICTURE: X (12)
        $nmbairclie = str_pad(Util::campoCnabValido(trim($endeclie->getNmbairclie())), 12);
        if((bool) $nmbairclie !== false){
             $arquivo .= substr($nmbairclie, 0, 12);
        } else{
            $arquivo .= self::preenche_brancos('', 12);
        }
        
        // CAMPO: CEP
        // POSIÇÃO: 327 a 334
        // PICTURE: 9 (8)
        $nrcepclie = str_pad(str_replace(" ", null, $endeclie->getNrcepclie()), 8, "0", STR_PAD_LEFT);
        if((bool) $nrcepclie !== false){
          $arquivo .= Util::campoCepTel($nrcepclie);
        } else{
            $arquivo .= self::preenche_brancos('', 3);
        }
        
        // CAMPO: Cidade do sacado
        // POSIÇÃO:  335 a 349
        // PICTURE: X (15)
        $municipio = Municipio::findOneByNrorgAndCdmunicipio($nrorg, $endeclie->getCdmunicipio());
        $nmmunicipio = substr($municipio->getNmmunicipio(), 0, 15);
        if((bool) $nmmunicipio !== false){
          $arquivo .= Util::campoCnabValido(str_pad($nmmunicipio, 15));
        }else{
            $arquivo .= self::preenche_brancos('', 15);
        }
        
        // CAMPO:  Estado (UF Unidade da Federação) do sacado
        // POSIÇÃO:  350 a 351
        // PICTURE: X (2)
        $sgestado = str_pad($endeclie->getSgestado(), 2);
        if((bool) $sgestado !== false){
          $arquivo .= Util::campoCnabValido($sgestado);
        } else{
          $arquivo .= self::preenche_brancos('', 2);
        }
        
        // CAMPO: Nome do sacador ou avalista
        // POSIÇÃO: 352 a 381
        // PICTURE: X (30)
        $empresa = Empresa::findOneByCdempresa($cdempresa);
        $nmrazsocemp = Util::campoCnabValido($empresa->getNmrazsocempr());
        $nmrazsocemp = substr($nmrazsocemp, 0, 30);
        $arquivo .= str_pad($nmrazsocemp, 30);
        
        // CAMPO: Complementação do registro
        // POSIÇÃO: 382 a 385
        // CONTEUDO: BRANCOS
        // PICTURE: X (4)
        $arquivo .= self::preenche_brancos('', 4);
        
        // CAMPO: BRANCOS
        // POSIÇÃO: 386 a 391
        // CONTEUDO: BRANCOS
        // PICTURE: X (6)
        $arquivo .= self::preenche_brancos('', 6);
        
        // CAMPO: Prazo
        // POSIÇÃO: 392 a 393
        // CONTEUDO: ZEROS
        // PICTURE: 9 (2)
        $arquivo .= str_repeat('0',2);
        
        // CAMPO: Moeda
        // POSIÇÃO: 394 394
        // PICTURE: X (1)
        $arquivo .= str_repeat('0',1);
        
        // CAMPO: Número sequencial do registro no arquivo
        // POSIÇÃO: 395 a 400
        // PICTURE: 9 (6)
        $arquivo .= str_pad($contreg, 6, "0", STR_PAD_LEFT) . "\r\n";
        
        //Grava a variavel arquivo no arquivo texto
        fprintf($file, "%s", $arquivo);
        fclose($file);
    }

    public function geraTrailler($caminho, $nmarquivo, $nrorg, $cdbanco, $contlote = NULL, $contreg = NULL, $cdreducont = NULL)
    {
        if (!is_dir($caminho)) mkdir($caminho, "0777", true);
        $nmarquivo = $caminho . $nmarquivo;
        $file = fopen($nmarquivo, "a+");

        // CAMPO: Código do Registro I
        // POSIÇÃO: 001 a 001
        // PICTURE: 9 (1)
        $arquivo = self::IDENTIFICACAO_REGISTRO_TRAILLER;
        
        // CAMPO: Complementação do registro
        // POSIÇÃO: 002 a 394
        // CONTEUDO: BRANCOS
        // PICTURE: X (393)
        $arquivo = str_pad($arquivo, '394');
        
        // CAMPO: Número sequencial do registro no arquivo
        // POSIÇÃO: 395 a 400
        // PICTURE: 9 (6)
        $arquivo .= str_pad($contreg, 6, "0", STR_PAD_LEFT) . "\r\n";
        
        fprintf($file, "%s", $arquivo);
        fclose($file);
    }
    
    public function geraRegistroMensagem($caminho, $nmarquivo, $cdfilial, $nrlanbolrec,$contreg = NULL)
    {
        if (!is_dir($caminho)) mkdir($caminho, "0777", true);
        $nmarquivo = $caminho . $nmarquivo;
        $file = fopen($nmarquivo, "a+");

        // CAMPO: Identificação do Registro Header
        // POSIÇÃO: 001 a 001
        // PICTURE: X (1)
        $arquivo = self::IDENTIFICACAO_REGISTRO_MENSAGEM;
        
        // CAMPO: Zero
        // POSIÇÃO: 002 a 002
        // PICTURE: 9 (1)
        $arquivo .= str_repeat('0',1);
        
        // CAMPO: Mensagem Livre 69 posições
        // POSIÇÃO: 003  a 071
        // PICTURE: X (69)
        
        // CAMPO: Mensagem Livre 69 posições
        // POSIÇÃO: 072 a 140
        // PICTURE: X (69)
        
        // CAMPO: Mensagem Livre 69 posições
        // POSIÇÃO: 141 a 209
        // PICTURE: X (69)
        
        // CAMPO: Mensagem Livre 69 posições
        // POSIÇÃO: 210 a 278
        // PICTURE: X (69)
        $boleto = Boletreceber::getRepository();
        $txboletos = $boleto->getTxBoleto($cdfilial, $nrlanbolrec);
        $mensagem = strtoupper($txboletos[0]['TXMENSBOLBOL']);
        $arquivo .= str_pad($mensagem, 276, ' ');
        
        // CAMPO: Brancos
        // POSIÇÃO: 279 a 394
        // PICTURE: X (116)
        $arquivo .= self::preenche_brancos('', 116);
        
        // CAMPO: Número sequencial do registro no arquivo
        // POSIÇÃO: 395 a 400
        // PICTURE: 9 (6)
        $arquivo .= str_pad($contreg, 6, "0", STR_PAD_LEFT) . "\r\n";
        fprintf($file, "%s", $arquivo);
        fclose($file);
    }

    private static function getCodigoEmpresa($cdbanco, $cdempresa, $cdidrec)
    {
        $resultado = '';
        /** @var $banco Banco */
        $banco = Banco::findOneByCdbanco($cdbanco);
        $banccnab = $banco->getCdbanccnab();

        if ($banccnab == '001')
        {
            /** @var $emprbanc Emprbanc */
            $emprbanc = Emprbanc::findOneByCdbancoAndCdempresa($cdbanco, $cdempresa);
        }

        if ($cdidrec) $resultado = $cdidrec;

        else
        {
            if ($banccnab != '001')
            /** @var $emprbanc Emprbanc */
            $emprbanc = Emprbanc::findOneByCdbancoAndCdempresa($cdbanco, $cdempresa);

            if ($emprbanc->getCdempbancrec())
            {
                $resultado = $emprbanc->getCdempbancrec();
            }
        }
        return $resultado;
    }

    /**
     * @param $valor - valor a ser alterado
     * @param $tamanho - tamanho total da string desejado
     * @return string
    */
     
    private function preenche_brancos($valor, $tamanho){
        return str_pad($valor, $tamanho);
    }
    
    public function geraHeaderLote($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec, $nrcnabrec, $cdreducont, $carteira = NULL, $cdfilial)
    {
    }
    public function geraDetalheSegmentoP($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec, $nrcnabrec, $idoperacao, $cdagenciacnab, $cdcontacorrente, $cdcontcorrcobranca, $nossonumero, $idtipocobranca, $nrlanbolrec, $cdfilial, $dtvenbolrec, $valorfinalboleto, $valorpemobrec, $valormordiabrec, $valorpemulbrec, $valormultabrec, $valorfinalbrec, $cdcartcobbol, $cddiaprotbol, $cdreducont, $imprbol, $qtdiaprotbol, $contador = NULL)
    {
    }
    public function geraDetalheSegmentoQ($caminho, $nmarquivo, $nrorg, $cdbanco, $cdcliente, $idoperacao, $cdempresa, $cdfilial, $nrlanbolrec, $contador = NULL)
    {
    }
    public function geraDetalheSegmentoR($caminho, $nmarquivo, $nrorg, $cdbanco, $idoperacao, $dtvenbolrec, $valorpemulbrec, $nrlanbolrec, $valormultabrec, $contador = NULL)
    {
    }
    public function GeraTraillerLote($caminho, $nmarquivo, $nrorg, $cdbanco, $contreg = NULL)
    {
    }
}

