<?php
namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Controllers;

use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Teknisa\FIN\Controllers\TeknisaCrud;
use Zeedhi\Framework\DataSource\ParameterBag;
use Teknisa\FIN\Modules\MovimentacaoBancaria\Services\Bco11100TransferenciaContaCorrenteService;
use Teknisa\Libs\Util\DateUtil;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Message;

class Bco11100TransferenciaContaCorrenteController extends TeknisaCrud
{
    protected $environment;
    protected $entityManager;
    protected $dataSourceName = 'ittransfbanc_transfbanc';

    public function __construct($crudService, ParameterBag $parameterBag, $environment, Bco11100TransferenciaContaCorrenteService $service, EntityManager $entityManager) {
        $this->environment = $environment;
        $this->service = $service;
        $this->entityManager = $entityManager;
        parent::__construct($crudService, $parameterBag);
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $dataSource = substr($request->getRoutePath(), 1, strlen($request->getRoutePath()));
        $this->dataSourceName = $dataSource;
        parent::find($request, $response);
    }

    public function validaAcessoOperadorPorContaFilial(DTO\Request\Row $request, DTO\Response $response){
        try {
            $row = $request->getRow();
            $this->service->validaAcessoOperadorPorContaFilial($this->environment->getCdOperador(), $row['CDFILIAL'], $row['CDEMPRESA'], $this->environment->getNrOrgTrab());
        }catch (\Exception $e){
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function saveTransferenciaSaldo(DTO\Request $request, DTO\Response $response){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $rows = $request->getDataSet()->getRows();
            $tamanho = count($rows) - 1;
            $contabiliza = isset($rows[$tamanho]['contabiliza']) ? $rows[$tamanho]['contabiliza'] : true;
            $rows = self::preparaDataSet($rows);
            $nrorg = $this->environment->getNrOrgTrab();
            $transferencia = array_shift($rows);
            $itensTransferencia = $rows;
            
            if(!$contabiliza){    
                array_pop($itensTransferencia);
            } 
            
            $nrtranbanc = null;
            
            if ($rows[0]['__is_new']) {
                $nrtranbanc = $this->service->saveTransferenciaSaldo($transferencia, $itensTransferencia, $nrorg, $response, $contabiliza);
            }else{
                $this->service->editaTransferenciaSaldo($transferencia, $itensTransferencia, $nrorg, $response, $contabiliza);
            }
            $retorno = array(
                "NRTRANBANC" => $nrtranbanc,
                "NRORG" => $nrorg,
                "NRMOVBANC" => $this->service->nrmovbanc,
                "VRTRANBANC" => $rows[0]['VRMOVITBANC'],
                "CDFILIAL" => $rows[0]['CDFILIAL'],
                "CDCLASSFINA" => $rows[0]['CDCLASSFINA'],
                "CDREDUCONT" => $rows[0]['CDREDUCONT'],
                "NRLCTBTRAN" => $rows[0]['NRLCTBTRAN']
            );
            $response->addDataSet(new DataSet("transferencia", $retorno));
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteTransferenciaContaCorrente(DTO\Request $request, DTO\Response $response){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $rows = $request->getDataSet()->getRows();
            $transferencia = array_shift($rows);
            $this->service->deleteTransferenciaContaCorrente($transferencia, $rows);
            $this->entityManager->flush();
            $connection->commit();
            $response->addMessage(new Message("Deletado com sucesso."));
        } catch (\Exception $e) {
            $connection->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

    public function preparaDataSet($dataSet){
        $nrorg = $this->environment->getNrorg();
        foreach($dataSet as $data) {
            if(!(isset($data['contabiliza']))){    
                $data['NRORG'] = $nrorg;
                $data['DTTRANBANC'] = self::formataData($data['DTTRANBANC']);
                $data['CDCLASSFINA'] = null;
                $data['CDFILIAL'] = $this->environment->getCdFilial();
                $data['DTMOVFLUXO'] = null;
                $data['IDMOVFLUXO'] = null;
                $data['NRLCTBTRAN'] = null;
            }
        }
        return $dataSet;
    }

    public static function formataData($data){
        if(strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDatadeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }
}