<?php

namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Controllers;

use Teknisa\FIN\Controllers\TeknisaCrud;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource\Manager;
use Teknisa\FIN\Modules\MovimentacaoBancaria\Services\Bco54500ConciliacaoManualService;
use Teknisa\FIN\Modules\MovimentacaoBancaria\Services\Bco54500ConciliacaoAutomaticaService;

class Bco54500ConciliacaoController extends TeknisaCrud {

    const DESCONCILIAR = 0;
    const CONCILIAR = 1;
    const MANUAL = 'M';
    const NOVOCODIGO_CONCILIACAO = 'CONCIMOVBANC';
    protected $dataSourceName;
    protected $entityManager;
    protected $novoCodigo;
    protected $service;
    protected $environment;
    protected $serviceAuto;
    protected $parameterBagColumns = array("NRORG", "CDREDUCONT", "IDMOVREAL", "IDSTATUSEXTRATO", "TVALOR", "VALOR", "NRDOC", "DTINICIAL", "DTFINAL",
        "TDATAS", "IDSTATUSBANC", "VALORBANC", "NRDOCMBANC", "IDMOVBANC", "TVALORBANC", "DTMOVBANC", "DTMOVREAL");
    public function __construct($crudService, $parameterBag, $entityManager, $environment, $novoCodigo, Bco54500ConciliacaoManualService $service, Bco54500ConciliacaoAutomaticaService $serviceAuto){
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->novoCodigo = $novoCodigo;
        $this->service = $service;
        $this->serviceAuto = $serviceAuto;
        parent::__construct($crudService, $parameterBag);
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $dataSource = substr($request->getRoutePath(), 1, strlen($request->getRoutePath()));
        $this->dataSourceName = $dataSource;
        parent::find($request, $response);
    }

    public function conciliarExtratosMovimentos(DTO\Request $request, DTO\Response $response) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            $nrorg = $this->environment->getNrOrgTrab();
            $rows = $request->getDataSet()->getRows();
            $cdreducont = $rows[0]['CDREDUCONT'];
            $qtdeExtrato = $rows[0]['QTDEEXTRATO'];
            if (count($rows) > 2){
                $nrconcimovb = $this->novoCodigo->getCodeByTableName(self::NOVOCODIGO_CONCILIACAO.$cdreducont,$this->environment->getNrOrgTrab());
                $this->service->insereConcimovbanc($cdreducont, $nrorg, $nrconcimovb);
                $this->entityManager->flush();
                for ($i = 0; $i < count($rows); $i++){
                    if ($i < $qtdeExtrato){
                        $this->service->updateMovreal(self::CONCILIAR, self::MANUAL, $nrconcimovb, $rows[$i]['CDSEQUEXTR'], $rows[$i]['NRSEQMOVREAL'],
                            $cdreducont, $nrorg, null);
                    }else{
                        $this->service->updateMovbancario(self::CONCILIAR, self::MANUAL, $nrconcimovb, $rows[$i]['NRMOVBANC'],
                            $cdreducont, $nrorg, null, null);
                    }
                }
            }else{
                $nrconcimovb = null;
                $this->service->updateMovreal(self::CONCILIAR, self::MANUAL, null, $rows[0]['CDSEQUEXTR'], $rows[0]['NRSEQMOVREAL'],
                    $rows[0]['CDREDUCONT'], $nrorg, $rows[1]['NRMOVBANC']);
                $this->service->updateMovbancario(self::CONCILIAR, self::MANUAL, null, $rows[1]['NRMOVBANC'],
                    $rows[0]['CDREDUCONT'], $nrorg, $rows[0]['CDSEQUEXTR'], $rows[0]['NRSEQMOVREAL']);
            }
            $this->entityManager->flush();
            $connection->commit();
        }catch (\Exception $e){
            $connection->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

    public function desconciliarExtratosMovimentos(DTO\Request $request, DTO\Response $response) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            $rows  = $request->getDataSet()->getRows();
            $nrorg = $this->environment->getNrOrgTrab();
            
            foreach($rows as $key => $row) {
                if(isset($row['NRDOCMOVREAL']))
                    $this->service->updateMovreal(self::DESCONCILIAR, null, null, $rows[$key]['CDSEQUEXTR'], $rows[$key]['NRSEQMOVREAL'], $rows[$key]['CDREDUCONT'], $nrorg, null);
                elseif (isset($row['NRMOVBANC']))
                    $this->service->updateMovbancario(self::DESCONCILIAR, null, null, $rows[$key]['NRMOVBANC'], $rows[$key]['CDREDUCONT'], $nrorg, null, null);
            }
            
            $cdreducont  = $rows[0]['CDREDUCONT'];
            $nrconcimovb = $rows[0]['NRCONCIMOVB'];
            
            if($nrconcimovb)
                $this->service->deleteConcimovbanc($cdreducont, $nrorg, $nrconcimovb);
            
            $this->entityManager->flush();
            $connection->commit();
        }catch (\Exception $e){
            $connection->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

    /**
     * Essa função tem como finalidade executar os metodos do serviço de conciliação automática para que a conciliação
     * seja realizada.
     *
     * @param  DTO\Request\Row $request  Dados passados do frontend
     * @param  DTO\Response    $response O que sera retornado ao backend
     */
    public function conciliacaoAutomatica(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $connection = $this->entityManager->getConnection();

        $connection->beginTransaction();

        try {
            $registros = $this->serviceAuto->getRegisForConcilia($row[0]);
            $this->serviceAuto->updateTables($registros);
            if($this->serviceAuto->existsMoreLancs($row[0])) {
                $response->addMessage(new Message("Operação bloqueada. Ainda existem lançamentos a serem conciliados, favor verificar na Conciliação Manual."));
            } else {
                $response->addMessage(new Message("Conciliação realizada com sucesso."));
            }
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e){
            $connection->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

    /**
     * Busca Saldo da Conta Corrente informada de acordo com a data referente passada
     * por parâmetro
     *
     * @param  DTO\Request\Row $request  Dados passados do frontend
     * @param  DTO\Response    $response O que sera retornado ao backend
     */
    public function getSaldoContaCorr(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            
            $cdreducont = $row['cdreducont'];
            $datarefe   = $row['datarefe'];
            
            $saldoContaCorr = $this->service->getSaldoContaCorr($cdreducont, $datarefe);
            
            $response->addDataSet(new DataSet('getSaldoContaCorr', $saldoContaCorr));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}
