<?php

namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Controllers;


use Teknisa\Libs\Service\CRUD as CrudService;
use Teknisa\Libs\Util\Environment;
use Teknisa\FIN\Controllers\TeknisaCrud;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO\Request\DataSet as Data;
use Teknisa\FIN\Modules\MovimentacaoBancaria\Services\Bco56200LancamentoDespesaService;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;

/**
 * Class Bco56200LancamentoDespesaController
 * @package Teknisa\FIN\Modules\MovimentacaoBancaria\Controllers
 */
class Bco56200LancamentoDespesaController extends TeknisaCrud {

    protected $crudService, $parameterBag, $environment, $bco56200LancamentoDespesaService;

    /**
     * @param CrudService  $crudService
     * @param ParameterBag $parameterBag
     * @param Environment  $environment
     * @param Bco56200LancamentoDespesaService $bco56200LancamentoDespesaService
     */
    public function __construct(CrudService $crudService, ParameterBag $parameterBag, Environment $environment, Bco56200LancamentoDespesaService $bco56200LancamentoDespesaService, ReportService $reportService)
    {
        $this->crudService  = $crudService;
        $this->parameterBag = $parameterBag;
        $this->environment  = $environment;
        $this->bco56200LancamentoDespesaService = $bco56200LancamentoDespesaService;
        $this->reportService       = $reportService;
    }
    
    /**
     * Salva Lançamento de Despesa Caixa
     * @param Row $row
     * @param Response $response
     */
    public function saveLancDespCaixa(Row $row, Response $response)
    {
        try {
            $row = $row->getRow();

            $this->bco56200LancamentoDespesaService->saveLancDespCaixa($row, $response);

            $response->addDataSet(new DataSet('saveLancDespCaixa', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Deleta Lançamento de Despesa Caixa
     * @param Row $row
     * @param Response $response
     */
    public function deleteLancDespCaixa(Row $row, Response $response)
    {
        try {
            $row = $row->getRow();

            $this->bco56200LancamentoDespesaService->deleteLancDespCaixa($row, $response);

            $response->addDataSet(new DataSet('deleteLancDespCaixa', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Gera Relatório Livro Caixa
     * @param Row $row
     * DATA,CDFILIAL,CDCENTCUST, __format
     * @param Response $response
     */
     
    public function geraRelLivroCaixa(Row $row, Response $response)
    {
        try {
            $row  = $row->getRow();
            $reportName = $row["__report_name"];
            $parametersRow = array (
                "DTINICIAL"  => $row["DTINICIAL"],
                "DTFINAL" => $row['DTFINAL'],
                "CDCENTCUST" => $row['CDCENTCUST'],
                "CDFILIAL" => $row['CDFILIAL'],
                "NRORG" =>  $this->environment->getNrOrgTrab(),
                "__format" => $row["__FORMAT"]
            );
            
            $birtURL = $this->reportService->createRemoteReport('BIRT', $reportName, $parametersRow);
            $response->addMessage(new DTO\Response\Message($birtURL));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
}
