<?php

namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Controllers;

use Teknisa\FIN\Modules\MovimentacaoBancaria\Services\Bco57000RecalculoOrcamentoCentroCustoService;
use Teknisa\FIN\Controllers\TeknisaCrud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Exception;
use Teknisa\Libs\Util\XLS;

class Bco57000RecalculoOrcamentoCentroCustoController extends TeknisaCrud {

    protected $dataSourceName, $masks, $service;

    public function __construct($crudService, $parameterBag, Bco57000RecalculoOrcamentoCentroCustoService $bco57000RecalculoOrcamentoCentroCustoService) {
        $this->bco57000RecalculoOrcamentoCentroCustoService = $bco57000RecalculoOrcamentoCentroCustoService;
        //parent::__construct($crudService, $parameterBag);
    }

    public function recalcularOrcamentoCentroCusto(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $data = $request->getDataSet()->getRows();
            $row = $data[0];
            $centroCusto = $data[1];
            
            $this->bco57000RecalculoOrcamentoCentroCustoService->recalcularOrcamentoCentroCusto($centroCusto, $row);
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

    public function saveOrcamentoCentroCusto(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $data = $request->getDataSet()->getRows();
            $row = $data[0];
            $orcamentos = $data[1];
            $this->bco57000RecalculoOrcamentoCentroCustoService->saveOrcamentoCentroCusto($orcamentos, $row);
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

    public function cadastramentoAutomaticoCentCust(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $data = $request->getDataSet()->getRows();
            $opcao = $data[0];
            $origem = $data[1];
            $destino = $data[2];
            $this->bco57000RecalculoOrcamentoCentroCustoService->CadastramentoAutomaticoDoOrcamento($opcao, $origem, $destino);
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }
    
    // public function historicoTabbed(DTO\Request\DataSet $request, DTO\Response $response) {
    //     try {
    //         $data = $request->getDataSet()->getRows();
    //         $row = $data[0];
    //         //  var_dump($data);die;
    //         // $centroCusto = $data[1];
    //         // $this->bco57000RecalculoOrcamentoCentroCustoService->historicoTabbed(/*$centroCusto,*/ $row);
    //     } catch(\Exception $e) {
    //         $response->setError(new DTO\Response\Error($e->getMessage(), 500));
    //     }
    // }
    
    
    /**
     * Função Responsável por receber os dados da Tela Orçamento Financeiro por Centro de Custo 
     * Validar e realizar a trasnferencia de um Centro de Custo/ Classificação Origem para o Destino
     * @param DataSet 
     */
    public function transfereOrcamento(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            
            $row = $request->getDataSet()->getRows();
            $this->bco57000RecalculoOrcamentoCentroCustoService->transfereOrcamento($row[0]);
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }
    
    
    
    
    
    
    public function importaOrcamentoFinanceiro(DTO\Request $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            
            // Leitura da Planilha
            $table = $this->bco57000RecalculoOrcamentoCentroCustoService->leituraOrcamentoFinanceiro($row['FILE']);
            
            // Formata Dados Planilha - Orçamento Financeiro por Centro de Custo
            $tableRows = $this->bco57000RecalculoOrcamentoCentroCustoService->formataOrcamentoFinanceiro($table);
            
            // Valida Dados Orçamento Financeiro por Centro de Custo
            $this->bco57000RecalculoOrcamentoCentroCustoService->validaOrcamentoFinanceiro($table);
            
            // Controi Orçamento Financeiro por Centro de Custo
            $orcamentos = $this->bco57000RecalculoOrcamentoCentroCustoService->constroiOrcamentoFinanceiro($tableRows);
            
            // Gera Orçamentos Financeiros por Centro de Custo
            $this->bco57000RecalculoOrcamentoCentroCustoService->geraOrcamentosFinanceirosCentroCusto($orcamentos);
            
            $response->addDataSet(new DataSet('importaOrcamentoFinanceiro', array('success' => true)));
        }
        catch(XLS\Exception $e){
            $errors = $e->getErrors();
            $response->addDataSet(new DataSet('errors', $errors));
            $response->setError(new Error($e->getMessage(), 500));
        }
        catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    
}
