<?php

namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Factories;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Geral\Entities\Paramgeral;
use Teknisa\Libs\Util\DateUtil;

class Bco57000RecalculoOrcamentoCentroCustoFactory{

    public function __construct($entityManager) {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
    }

    public function getDatasPeriodos($dataInicial, $dataFinal, $nrorg) {
        $params = array(
            'DTINICIAL' => $dataInicial,
            'DTFINAL'   => $dataFinal,
            'NRORG'     => $nrorg
          );
        $sql = "SELECT DISTINCT TO_CHAR(DTMES, 'DD/MM/YYYY') DTMES
                  FROM DATA
                 WHERE (DTDATA between :DTINICIAL and :DTFINAL)
                   AND (NRORG = :NRORG)
                 ORDER BY DTMES";
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function getValorTotalContas($params) {
        $sql = "SELECT SUM(VRATUAPAG) VRATUAPAG
                  FROM CONTAPAGAR CT,NOTAFISCAL NF
                 WHERE (CT.CDFILIAL    = NF.CDFILIAL)
                   AND (CT.NRLANCTONF  = NF.NRLANCTONF)
                   AND (CT.NRORG       = NF.NRORG)
                   AND (NF.IDSTATUS    IN ('A','V'))
                   AND (CT.CDEMPRESA   = :CDEMPRESA)
                   AND (CDFIMOVFLPAG   = :CDFILIAL)
                   AND (CT.NRORG       = :NRORG)
                   AND (CT.DTMOVFLUXO  BETWEEN :DTMOVFLUXO_INI  AND :DTMOVFLUXO_FIN)";
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }

    public function getTotalAjusteBoleto($params) {
        $sql = "SELECT VRAJUSTE,
                       TIPOAJUS
                  FROM
                     (SELECT SUM(MF.VRFXAJBOLPAG) VRAJUSTE,
                             'DESCONTO'           TIPOAJUS
                        FROM MOVFXAJBOLP MF,
                             TIPOAJPAGAR TP
                       WHERE (MF.CDEMPRESA   = :CDEMPRESA) AND
                             (MF.CDFILIAL    = :CDFILIAL) AND
                             (MF.NRORG       = :NRORG) AND
                             (MF.NRORG       = TP.NRORG) AND
                             (MF.DTMOVFLUXO BETWEEN :DTMOVFLUXO_INI AND :DTMOVFLUXO_FIN) AND
                             (TP.IDAJUCTPAG = 'D') AND
                             (TP.CDAJUCTPAG = MF.CDAJUCTPAG)
                       UNION ALL
                      SELECT SUM(MF.VRFXAJBOLPAG) VRAJUSTE,
                             'ACRESCIMO'        TIPOAJUS
                        FROM MOVFXAJBOLP MF,
                             TIPOAJPAGAR TP
                       WHERE (MF.CDEMPRESA   = :CDEMPRESA) AND
                             (MF.CDFILIAL    = :CDFILIAL) AND
                             (MF.NRORG       = :NRORG) AND
                             (MF.NRORG       = TP.NRORG) AND
                             (MF.DTMOVFLUXO BETWEEN :DTMOVFLUXO_INI AND :DTMOVFLUXO_FIN) AND
                             (TP.IDAJUCTPAG = 'A') AND
                             (TP.CDAJUCTPAG  = MF.CDAJUCTPAG))";
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function getParamGeral($nrorg){
        $params = array();
        $params['nrorg'] = $nrorg;
        $paramGeral = $this->entityManager->getRepository(Paramgeral::class)->findOneBy($params);
        return $paramGeral;
    }

    public function getCentcust($cdempresa, $cdfilial, $cdcentcust, $cdclassfina, $dtorcafina, $nrorg){
        $params = array();
        $params['CDEMPRESA'] = $cdempresa;
        $params['CDFILIAL'] = $cdfilial;
        $params['CDCENTCUST'] = $cdcentcust;
        $params['CDCLASSFINA'] = $cdclassfina;
        $params['DTORCAFINA'] = $dtorcafina;
        $params['NRORG'] = $nrorg;
        
        $sql="SELECT CDEMPRESA,
                CDFILIAL,
                CDCENTCUST,
                CDCLASSFINA,
                DTORCAFINA,
                NRORG
				FROM ORCAFINACENTCUST
				WHERE CDEMPRESA  = :CDEMPRESA
				  AND CDFILIAL   = :CDFILIAL
				  AND CDCENTCUST = :CDCENTCUST
				  AND CDCLASSFINA = :CDCLASSFINA
				  AND DTORCAFINA = TO_DATE(:DTORCAFINA, 'DD/MM/YYYY')
				  AND NRORG      = :NRORG";

		$params = array("CDEMPRESA"=>$cdempresa,
						"CDFILIAL"=>$cdfilial,
		                "CDCENTCUST"=>$cdcentcust,
		                "CDCLASSFINA"=>$cdclassfina,
		                "DTORCAFINA"=>$dtorcafina,
						"NRORG"=>$nrorg);
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function insereCentcust($vrrealorca, $vrextoleorca, $vrorcafina, $cdempresa, $filial, $cdcentcust, $cdclassfina, $dataMes, $nrorg){
        $sql = "INSERT INTO ORCAFINACENTCUST
				  (VRREALORCA, VREXTOLEORCA, VRORCAFINA, CDEMPRESA, CDFILIAL, CDCENTCUST, CDCLASSFINA, DTORCAFINA, NRORG)
				VALUES
				  (:VRREALORCA, :VREXTOLEORCA, :VRORCAFINA, :CDEMPRESA, :CDFILIAL, :CDCENTCUST, :CDCLASSFINA, :DTORCAFINA, :NRORG)";

		$params = array("VRREALORCA"=>$vrrealorca,
						"VREXTOLEORCA"=>$vrextoleorca,
						"VRORCAFINA"=>$vrorcafina,
						"CDEMPRESA"=>$cdempresa,
						"CDFILIAL"=>$filial,
		                "CDCENTCUST"=>$cdcentcust,
		                "CDCLASSFINA"=>$cdclassfina,
						"DTORCAFINA"=>$dataMes,
						"NRORG"=>$nrorg);
		$query = $this->connection->prepare($sql);
		$query->execute($params);
    }
    
    public function updateCentcust($vrextoleorca, $vrorcafina, $dataMes, $cdcentcust, $cdclassfina) {
		$sql="UPDATE ORCAFINACENTCUST SET   
				VREXTOLEORCA = :VREXTOLEORCA,
				VRORCAFINA   = :VRORCAFINA
			WHERE DTORCAFINA = :DTORCAFINA
			  AND CDCENTCUST = :CDCENTCUST
			  AND CDCLASSFINA = :CDCLASSFINA";

		$params = array("VREXTOLEORCA"=>$vrextoleorca,
						"VRORCAFINA"=>$vrorcafina,
						"DTORCAFINA"=>$dataMes,
						"CDCENTCUST"=>$cdcentcust,
						"CDCLASSFINA"=>$cdclassfina);
		$query = $this->connection->prepare($sql);
		$query->execute($params);
	}

    public function getPeriodoCentcust($empresa, $filial, $anoini, $anofim) {
        $params = array(
            'EMPRESA' => $empresa,
            'FILIAL'  => $filial,
            'ANOFIM'  => $anofim,
            'ANOINI'  => $anoini
        );
        $sql = "SELECT CDCENTCUST,
                       CDCLASSFINA,
                       CDEMPRESA,
                       DTORCAFINA,
                       VRORCAFINA,
                       VREXTOLEORCA,
                       CDFILIAL
                  FROM ORCAFINACENTCUST
                 WHERE CDEMPRESA  = :EMPRESA AND
                       CDFILIAL   = :FILIAL  AND
                       DTORCAFINA BETWEEN :ANOINI AND :ANOFIM
                 ORDER BY
                       CDEMPRESA,
                       CDCLASSFINA,
                       CDFILIAL,
                       CDCENTCUST,
                       DTORCAFINA";
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function getFilial($cdempresa, $nrorg){
        $params = array(
            'DTINICIAL' => $dataInicial,
            'DTFINAL'   => $dataFinal,
            'NRORG'     => $nrorg
        );
        $sql = "SELECT CDEMPRESA,
                       CDFILIAL,
                       NMFILIAL
                  FROM FILIAL
                 WHERE (CDEMPRESA = :P_EMPRESA)  AND
                       (INSTR(:P_FILIAL, CDFILIAL) <> 0 OR :P_TFILIAL = 'T' )
                 ORDER BY CDFILIAL";
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    
    /**
     * Função responsável por inserir o histórico de transferencia de orçamento
    */
    public function insereJustificativaOrcaCentCust($params){
        try {
            $sql = "INSERT INTO HISTORCAFINCC (NRHISTORCAFINCC, 
                                               CDCENTCUST, 
                                               CDEMPRESA, 
                                               CDFILIAL, 
                                               CDCLASSFINA,
                                               DTORCAFINA, 
                                               DSJUSTIFICATIVA, 
                                               DTALTERACAO, 
                                               CDOPERADOR, 
                                               VRORCAFINAANT, 
                                               VREXTOLEORCAANT,
                                               TPTRANSF,
                                               NRORG)
                                       VALUES (:NRHISTORCAFINCC, 
                                               :CDCENTCUST, 
                                               :CDEMPRESA, 
                                               :CDFILIAL, 
                                               :CDCLASSFINA,
                                               :DTORCAFINA, 
                                               :DSJUSTIFICATIVA, 
                                               :DTALTERACAO, 
                                               :CDOPERADOR, 
                                               :VRORCAFINAANT, 
                                               :VREXTOLEORCAANT,
                                               :TPTRANSF,
                                               :NRORG)";
                                               
            $query = $this -> connection -> prepare($sql);
            $query -> execute($params);
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    
    /**
     * Função responsável por retornar os dados cadastrados do orçamento de origem
     * @param CDCENTCUST 
     * @param CDFILIAL
     * @param CDEMPRESA
     * @param DTORCAFINA
     * @param NRORG
    */
    public function validaValorOrcamento($param){
        
        $sql = "SELECT
                    CDCENTCUST,
                    CDEMPRESA,
                    CDFILIAL,
                    DTORCAFINA,
                    VRORCAFINA,
                    VRREALORCA,
                    VREXTOLEORCA,
                    NRORG,
                    CDCLASSFINA
                FROM
                    ORCAFINACENTCUST
                WHERE
                        CDCENTCUST =:CDCENTCUST
                    AND CDCLASSFINA =:CDCLASSFINA
                    AND CDFILIAL =:CDFILIAL
                    AND CDEMPRESA =:CDEMPRESA
                    AND DTORCAFINA =:DTORCAFINA
                    AND NRORG =:NRORG";
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($param);
        return $std->fetchAll();
    }
    
    public function insereOrcafinacentcust(
        $nrorg, $cdcentcust, $cdclassfina, $cdempresa, $cdfilial, $dtorcafina, $vrorcafina, $vrextoleorca
    ){
        $sql = "INSERT INTO ORCAFINACENTCUST
				  (NRORG, CDCENTCUST, CDCLASSFINA, CDEMPRESA, CDFILIAL, DTORCAFINA, VRORCAFINA, VREXTOLEORCA)
				VALUES
				  (:NRORG, :CDCENTCUST, :CDCLASSFINA, :CDEMPRESA, :CDFILIAL, :DTORCAFINA, :VRORCAFINA, :VREXTOLEORCA)";

		$params = array(
			"NRORG"         => $nrorg,
            "CDCENTCUST"    => $cdcentcust,
            "CDCLASSFINA"   => $cdclassfina,
			"CDEMPRESA"     => $cdempresa,
			"CDFILIAL"      => $cdfilial,
			"DTORCAFINA"    => $dtorcafina,
			"VRORCAFINA"    => $vrorcafina,
			"VREXTOLEORCA"  => $vrextoleorca
		);
		
		$query = $this->connection->prepare($sql);
		$query->execute($params);
    }
    
    public function insereHistorcafincc(
        $nrorg, $nrhistorcafincc, $cdcentcust, $cdclassfina, $cdempresa, $cdfilial, $dtorcafina, $dsjustificativa,
        $cdoperador, $vrorcafinaant, $vrextoleorcaant
    ){
        try {
            $sql = "INSERT INTO HISTORCAFINCC (NRHISTORCAFINCC, 
                                               CDCENTCUST, 
                                               CDEMPRESA, 
                                               CDFILIAL, 
                                               CDCLASSFINA,
                                               DTORCAFINA, 
                                               DSJUSTIFICATIVA, 
                                               DTALTERACAO, 
                                               CDOPERADOR, 
                                               VRORCAFINAANT, 
                                               VREXTOLEORCAANT, 
                                               NRORG)
                                       VALUES (:NRHISTORCAFINCC, 
                                               :CDCENTCUST, 
                                               :CDEMPRESA, 
                                               :CDFILIAL, 
                                               :CDCLASSFINA,
                                               :DTORCAFINA, 
                                               :DSJUSTIFICATIVA, 
                                               SYSDATE, 
                                               :CDOPERADOR, 
                                               :VRORCAFINAANT, 
                                               :VREXTOLEORCAANT, 
                                               :NRORG)";
                                               
       		$params = array(
                "NRORG"           => $nrorg,
       		    "NRHISTORCAFINCC" => $nrhistorcafincc,
                "CDCENTCUST"      => $cdcentcust,
                "CDCLASSFINA"     => $cdclassfina,
                "CDEMPRESA"       => $cdempresa,
                "CDFILIAL"        => $cdfilial,
                "DTORCAFINA"      => $dtorcafina,
                "DSJUSTIFICATIVA" => $dsjustificativa,
                "CDOPERADOR"      => $cdoperador,
                "VRORCAFINAANT"   => $vrorcafinaant,
                "VREXTOLEORCAANT" => $vrextoleorcaant
            );
                                               
            $query = $this -> connection -> prepare($sql);
            $query -> execute($params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function deletaOrcafinacentcust(
        $nrorg, $cdcentcust, $cdclassfina, $cdempresa, $cdfilial, $dtorcafina
    ){
        $sql = "DELETE 
                  FROM ORCAFINACENTCUST 
                 WHERE NRORG       = :NRORG 
                   AND CDCENTCUST  = :CDCENTCUST 
                   AND CDCLASSFINA = :CDCLASSFINA 
                   AND CDEMPRESA   = :CDEMPRESA
                   AND CDFILIAL    = :CDFILIAL
                   AND DTORCAFINA  = :DTORCAFINA";

		$params = array(
			"NRORG"         => $nrorg,
            "CDCENTCUST"    => $cdcentcust,
            "CDCLASSFINA"   => $cdclassfina,
			"CDEMPRESA"     => $cdempresa,
			"CDFILIAL"      => $cdfilial,
			"DTORCAFINA"    => $dtorcafina
		);
		
		$query = $this->connection->prepare($sql);
		$query->execute($params);
    }
    
    public function getCentcustImp($cdcentcust, $nrorg){
        $params = array();
        $params['P_CDCENTCUST'] = $cdcentcust;
        $params['P_NRORG']      = $nrorg;
        $sql = "SELECT CDCENTCUST, NRORG FROM CENTCUST WHERE CDCENTCUST = :P_CDCENTCUST AND NRORG = :P_NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getClassfinancImp($cdclassfina, $nrorg){
        $params = array();
        $params['P_CDCLASSFINA'] = $cdclassfina;
        $params['P_NRORG']       = $nrorg;
        $sql = "SELECT CDCLASSFINA, NRORG FROM CLASSFINANC WHERE CDCLASSFINA = :P_CDCLASSFINA AND NRORG = :P_NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getClassfina($cdcentcust, $cdfilial){
        $params = array();
        $params['CDCENTCUST'] = $cdcentcust;
        $params['CDFILIAL']       = $cdfilial;
        $sql = "SELECT CDCLASSFINA FROM ORCAFINACENTCUST WHERE CDCENTCUST = :CDCENTCUST AND CDFILIAL = :CDFILIAL";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }  
    
    public function getEmpresaImp($cdempresa, $nrorg){
        $params = array();
        $params['P_CEMPRESA'] = $cdempresa;
        $params['P_NRORG']    = $nrorg;
        $sql = "SELECT CDEMPRESA, NRORG FROM EMPRESA WHERE CDEMPRESA = :P_CEMPRESA AND NRORG = :P_NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getFilialImp($cdfilial, $nrorg){
        $params = array();
        $params['P_CDFILIAL'] = $cdfilial;
        $params['P_NRORG']    = $nrorg;
        $sql = "SELECT CDFILIAL, NRORG FROM FILIAL WHERE CDFILIAL = :P_CDFILIAL AND NRORG = :P_NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    } 
    
    public function buscaOrcafinacentcust($nrorg, $cdcentcust, $cdclassfina, $cdempresa, $cdfilial, $dtorcafina){
        $params = array();
        $params['P_NRORG']       = $nrorg;
        $params['P_CDCENTCUST']  = $cdcentcust;
        $params['P_CDCLASSFINA'] = $cdclassfina;
        $params['P_CDEMPRESA']   = $cdempresa;
        $params['P_CDFILIAL']    = $cdfilial;
        $params['P_DTORCAFINA']  = $dtorcafina;
        $sql = "SELECT VRORCAFINA, VREXTOLEORCA, NVL(VRREALORCA, 0) VRREALORCA, NRORG
                  FROM ORCAFINACENTCUST 
                 WHERE NRORG       = :P_NRORG
                   AND CDCENTCUST  = :P_CDCENTCUST
                   AND CDCLASSFINA = :P_CDCLASSFINA
                   AND CDEMPRESA   = :P_CDEMPRESA
                   AND CDFILIAL    = :P_CDFILIAL
                   AND DTORCAFINA  = TO_DATE(:P_DTORCAFINA, 'DD/MM/YYYY')";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    } 
    
    public function updataOrcafinacentcust($nrorg, $cdcentcust, $cdclassfina, $cdempresa, $cdfilial, $dtorcafina, $vrorcafina, $vrextoleorca){
        $params = array();
        $params['P_NRORG']         = $nrorg;
        $params['P_CDCENTCUST']    = $cdcentcust;
        $params['P_CDCLASSFINA']   = $cdclassfina;
        $params['P_CDEMPRESA']     = $cdempresa;
        $params['P_CDFILIAL']      = $cdfilial;
        $params['P_DTORCAFINA']    = $dtorcafina;
        $params['P_VRORCAFINA']    = $vrorcafina;
        $params['P_VREXTOLEORCA']  = $vrextoleorca;
        $sql = "UPDATE ORCAFINACENTCUST 
                   SET VRORCAFINA = :P_VRORCAFINA, VREXTOLEORCA = :P_VREXTOLEORCA
                 WHERE NRORG       = :P_NRORG
                   AND CDCENTCUST  = :P_CDCENTCUST
                   AND CDCLASSFINA = :P_CDCLASSFINA
                   AND CDEMPRESA   = :P_CDEMPRESA
                   AND CDFILIAL    = :P_CDFILIAL
                   AND DTORCAFINA  = :P_DTORCAFINA";
		$query = $this->connection->prepare($sql);
		$query->execute($params);
    } 
}
