<?php
namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Factories;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Model\Financeiro\Entities\Lancsaldcaixa;
use Teknisa\Model\Financeiro\Entities\Movbancario;
use Teknisa\Model\Contabilidade\Entities\Lanccont;

class LancamentoCreditoFactory
{
    private $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager = $entityManager;
    }

    public function insereLancamentoCredito(
        $cdcentcust,
        $cdempresa,
        $cdfilial,
        $cdreducont,
        $dtlancsald,
        $nrorg,
        $vrlancsald,
        $nrseqlancsald,
        $vrlancsalddisp,
        $nrmovbanc,
        $nrlanccont,
        $vrlancsaldutil
    ){
        $lancdecredito = new Lancsaldcaixa(); 
        $lancdecredito->setCdcentcust($cdcentcust);
        $lancdecredito->setCdempresa($cdempresa);
        $lancdecredito->setCdfilial($cdfilial);
        $lancdecredito->setCdreducont($cdreducont);
        $lancdecredito->setDtlancsald($dtlancsald);
        $lancdecredito->setNrorg($nrorg);
        $lancdecredito->setVrlancsald($vrlancsald); 
        $lancdecredito->setNrseqlancsald($nrseqlancsald);
        $lancdecredito->setVrlancsalddisp($vrlancsalddisp);
        $lancdecredito->setNrmovbanc($nrmovbanc);
        $lancdecredito->setNrlanccont($nrlanccont);
        $lancdecredito->setVrlancsaldutil($vrlancsaldutil);
    
        return $lancdecredito;
    }
    
    public function findLancsaldcaixa(
        $nrorg, $nrseqlancsald
    ){
        /** @var $params */
        $params = array();
        $params['nrorg']         = $nrorg;
        $params['nrseqlancsald'] = $nrseqlancsald;

        /** @var Lancsaldcaixa $lancdecredito */
        $lancdecredito = $this->entityManager->getRepository(Lancsaldcaixa::class)->findOneBy($params);
    
        return $lancdecredito;
    }
    
    public function findMovbancario(
        $nrorg, $cdreducont, $nrmovbanc
    ){
        /** @var $params */
        $params = array();
        $params['nrorg']      = $nrorg;
        $params['cdreducont'] = $cdreducont;
        $params['nrmovbanc']  = $nrmovbanc;

        /** @var Movbancario $movbancario */
        $movbancario = $this->entityManager->getRepository(Movbancario::class)->findOneBy($params);
    
        return $movbancario;
    }
    
    /**
     * Insere Movimentação Bancária
     * @param $lancdecredito
     * @param $cdcentcust
     * @return Lancsaldcaixa
     */
    public static function updateLancsaldcaixa(
        Lancsaldcaixa $lancdecredito, $cdcentcust, $cdempresa,
        $cdfilial, $cdreducont, $dtlancsald, $vrlancsald, $vrlancsalddisp, $nrlanccont, $vrlancsaldutil
    ){
        
        $lancdecredito->setCdcentcust($cdcentcust);
        $lancdecredito->setCdempresa($cdempresa);
        $lancdecredito->setCdfilial($cdfilial);
        $lancdecredito->setCdreducont($cdreducont);
        $lancdecredito->setDtlancsald($dtlancsald);
        $lancdecredito->setVrlancsald($vrlancsald);
        $lancdecredito->setVrlancsalddisp($vrlancsalddisp);
        $lancdecredito->setNrlanccont($nrlanccont);
        $lancdecredito->setVrlancsaldutil($vrlancsaldutil);

        return $lancdecredito;
    }
    
    /**
     * Insere Movimentação Bancária
     * @param $nrorg
     * @param $nrmovbanc
     * @param $cdreducont
     * @param $dtmovbanc
     * @param $vrmovbanc
     * @return Movbancario
     */
    public static function insereMovBancario(
        $nrorg, $nrmovbanc, $cdreducont, $dtmovbanc, $vrmovbanc, $dshismbanc = 'bxRec'
    ){
        /** @var Movbancario $movbancario */
        $movbancario = new Movbancario;
        $movbancario->setNrorg($nrorg);
        $movbancario->setCdreducont($cdreducont);
        $movbancario->setNrmovbanc($nrmovbanc);
        $movbancario->setDtmovbanc($dtmovbanc);
        $movbancario->setVrmovbanc($vrmovbanc);
        $movbancario->setIdativo('S');
        $movbancario->setIdmovbanc('C');
        $movbancario->setIdorigmbanc('R');
        $movbancario->setIdstatusctbbanc('P');
        $movbancario->setDshismbanc($dshismbanc);
        $movbancario->setDtinclusao(DateUtil::getDataAtual());

        return $movbancario;
    }
    
    /**
     * Busca Lançamento Contabil
     * @param $nrorg
     * @param $cdempresa
     * @param $nrlanccont
     * @return Lanccont
     */
    public function buscaLanccont($nrorg, $cdempresa, $nrlanccont){
        /** @var $params */
        $params = array();
        $params['nrorg']      = $nrorg;
        $params['cdempresa']  = $cdempresa;
        $params['nrlanccont'] = $nrlanccont;

        /** @var Lanccont $lanccont */
        $lanccont = $this->entityManager->getRepository(Lanccont::class)->findBy($params);

        return $lanccont;
    }
        
}