<?php

namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\FIN\Modules\MovimentacaoBancaria\Factories\Bco57000RecalculoOrcamentoCentroCustoFactory;
use Teknisa\Libs\Util\XLS\FileLoader;
use Teknisa\Libs\Util\XLS\FileReader;
use PhpOffice\PhpSpreadsheet;
use PHPExcel_Cell;
use Teknisa\Libs\Util\XLS\Table;
use Teknisa\Libs\Util\XLS\Exception;
use Teknisa\Libs\Util\XLS;
use Teknisa\FIN\Modules\MovimentacaoBancaria\Services\ImportacaoOrcamentoFinanceiroError;

class Bco57000RecalculoOrcamentoCentroCustoService{
    
    const IMPORT_FILE_FORMAT_ERROR = 'IMPORT_FILE_FORMAT_ERROR';
    const IMPORT_NOT_FOUND_CELL_ERROR = 'IMPORT_NOT_FOUND_CELL_ERROR';
    const IMPORT_EMPTY_CELL_VALUE_ERROR = 'IMPORT_EMPTY_CELL_VALUE_ERROR';
    const IMPORT_CALC_VRORCAFINA_ERROR  = 'IMPORT_CALC_VRORCAFINA_ERROR';

    protected $dataSourceName, $masks, $service, $operador;

    public function __construct($entityManager, $environment, Bco57000RecalculoOrcamentoCentroCustoFactory $bco57000RecalculoOrcamentoCentroCustoFactory, $newCode) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->bco57000RecalculoOrcamentoCentroCustoFactory = $bco57000RecalculoOrcamentoCentroCustoFactory;
        $this->nrorg = $this->environment->getNrOrgTrab();
        $this->filial = $this->environment->getCdFilial();
        $this->operador = $this->environment->getCdOperador();
        $this->newCode = $newCode;
    }

    /**
     * [saveOrcamentoCentroCusto Realiza o salvamento das cadastramento automático
     * para todas as classificações financeiras por empresa informada.]
     */
    public function saveOrcamentoCentroCusto($orcamentos, $row) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            //var_dump($row);die;
            // Busca Parâmetros Gerais
            $paramGeral = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getParamGeral($this->nrorg);
            
            // Verifica Tipo de Orçamento Centro de Custo
            if ($paramGeral->getIdtporcafina() == 'F')
                 $filial = $row['CDFILIAL'];
            else $filial = $this->environment->getCdfilial();
            
            // Percorre Orçamentos Financeiros
            foreach ($orcamentos as $orcamento) {
                
                // Formata Data de Orçamento Centro de Custo
                $dataMes = '01/'.$orcamento['DTMES'].'/'.$row['DTMES'];
              //  $dataMes = $this->formataData($dataMes);
               
                // Busca Orçamento Centro de Custo
                $centcust = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getCentcust(
                    $row['CDEMPRESA'], $filial, $row['CDCENTCUST'], $row['CDCLASSFINA'], $dataMes, $this->nrorg
                );
                // Verifica existência Orçamento Centro de Custo
                if (!$centcust){
                    // Insere Orçamento Centro de Custo
                    $centcust = $this->bco57000RecalculoOrcamentoCentroCustoFactory->insereCentcust(
                        0, $orcamento['VREXTOLEORCA'], $orcamento['VRORCAFINA'], $row['CDEMPRESA'],
                        $filial, $row['CDCENTCUST'], $row['CDCLASSFINA'], $dataMes, $this->nrorg
                    );
                }else{
                    // Atualiza Orçamento Financeiro
                    $centcust = $this->bco57000RecalculoOrcamentoCentroCustoFactory->updateCentcust(
                        $orcamento['VREXTOLEORCA'], $orcamento['VRORCAFINA'], $dataMes, $row['CDCENTCUST'], $row['CDCLASSFINA']
                    );
                   // $centcust = $orcamento['VRORCAFINA'];
                }
                
                // Gera novo sequencial para inserção dos dados na tabela HISTORCAFINCC
                $nrHistorcafiancc = $this->newCode->getCodeByTableName('NRHISTORCAFINCC', $this->nrorg, 1, 10);
                $dia =  DateUtil::getDia(DateUtil::getDataAtual());
                $mes =  DateUtil::getMes(DateUtil::getDataAtual());
                $ano =  DateUtil::getAno(DateUtil::getDataAtual());
                $dataAtual =  $dia.'/'.$mes.'/'.$ano;
                
                // Calcula a diferença ao mudar o orçamento
                if(isset($orcamento["changed_VRORCAFINA"]) && $orcamento["changed_VRORCAFINA"] < $orcamento["VRORCAFINA"]){
    
                    $diferenca = $orcamento["VRORCAFINA"] - $orcamento["changed_VRORCAFINA"];
    
                }else if (isset($orcamento["changed_VRORCAFINA"]) && $orcamento["changed_VRORCAFINA"] > $orcamento["VRORCAFINA"]){
    
                    $diferenca = $orcamento["changed_VRORCAFINA"] - $orcamento["VRORCAFINA"];
    
                }else{
                    
                    $diferenca = $orcamento["VRORCAFINA"];
                }
                
                // Parametros para inserção de Histórico de alteração
                
                $paramInsert = array(
                    'NRHISTORCAFINCC' => $nrHistorcafiancc,
                    'CDCENTCUST'      => $row['CDCENTCUST'],
                    'CDEMPRESA'       => $row['CDEMPRESA'],
                    'CDCLASSFINA'     => $row['CDCLASSFINA'],
                    'CDFILIAL'        => $filial,
                    'DTORCAFINA'      => $dataMes,
                    'DSJUSTIFICATIVA' => $row["DSJUSTIFICATIVA"],
                    'DTALTERACAO'     => $dataAtual,
                    'CDOPERADOR'      => $this->operador,
                    'VRORCAFINAANT'   => $diferenca,
                    'VREXTOLEORCAANT' => isset($orcamento["changed_VRORCAFINA"])?$orcamento["VRORCAFINA_inline"]:$orcamento["VRORCAFINA"],
                    'TPTRANSF'        => $orcamento["changed_VRORCAFINA"] < $orcamento["VRORCAFINA"] ? "E" : "S",
                    'NRORG'           => $this->nrorg
                  );
                // Salva Justificativa de alteração do Orçamento de Centro de Custo
                $this->bco57000RecalculoOrcamentoCentroCustoFactory->insereJustificativaOrcaCentCust($paramInsert);
                
               // $this->entityManager->persist($centcust);
                $this->entityManager->flush();
            }

            $connection->commit();
        }catch (\Exception $e){
            $connection->rollback();
            throw $e;
        }
    }

    /**
     * [CadastramentoAutomaticoDoOrcamento Realiza o cadastramento automático
     * para todas os centros du custos por empresa informada.]
     */
    public function CadastramentoAutomaticoDoOrcamento($row, $origem, $destino) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $paramGeral = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getParamGeral($this->nrorg);
            $anoInicial = '01/01/'.$row['DTMES'];
            $anoFinal = '31/12/'.$row['DTMES'];

            $tipoCadastramento = null;
            
            if ($paramGeral->getIdtporcafina() == 'F'){
                $tipoCadastramento = 'F';
                $orcamentoOrigem = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getPeriodoCentcust(
                    $origem['CDEMPRESA'],
                    $origem['CDFILIAL'],
                    $anoInicial,
                    $anoFinal
                );

            }else{
                $filialOrigem = $this->environment->getCdfilial();
                $tipoCadastramento = 'E';
                $orcamentoOrigem = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getPeriodoCentcust(
                    $origem['CDEMPRESA'],
                    $filialOrigem,
                    $anoInicial,
                    $anoFinal
                );
            }
            
            foreach ($orcamentoOrigem as $origem) {
                if ($tipoCadastramento == 'E'){
                    $this->efetuaOrcamentoPorEmpresa(
                        $filialOrigem,
                        $origem,
                        $destino,
                        $paramGeral,
                        $row['OPCAOOPERACAO']
                    );
                }else{
                    $this->efetuaOrcamentoPorFilial(
                        $origem,
                        $destino,
                        $row['OPCAOOPERACAO']
                    );
                }
            }
            $this->entityManager->flush();
            $connection->commit();
        }catch (\Exception $e){
            $connection->rollback();
            throw $e;
        }
    }

    /**
     * [efetuaOrcamentoPorFilial Realiza o cadastramento Automático
     * da filial de origem para N filiais de destino]
     *
     * @param  [type] $origem  [Filial do orçamento a ser replicado.
     * Caso a filial não esteja parametrizada na tela orçamento para
     * financeiro, a filial considerada será a filial logada]
     * @param  [type] $destino [Filiais que receberão a replicação]
     * @param  [type] $opcao   [description]
     */
    public function efetuaOrcamentoPorFilial($origem, $destino, $opcao){
        // Unidades para Inclusão/Sobreposição
        $filiais = $destino['CDFILIAL'];
        
        // Percorre Unidades p/ Inclusão/Sobreposição
        foreach ($filiais as $filial) {
            
            // Busca Orçamento Centro de Custo de Destino
            $centcustDestino = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getCentcust(
                $destino['CDEMPRESA'], $filial, $origem['CDCENTCUST'],
                $this->formataData($origem['DTORCAFINA']), $this->nrorg
            );
            
            // Verifica Inclusão/Sobreposição
            if ($opcao == 'I'){
                if (!$centcustDestino)
                    $this->insereCentcust($origem, $filial, $destino);
            }else{
                if ($centcustDestino){
                    if (($centcustDestino->getCdempresa() != $origem['CDEMPRESA']) || ($centcustDestino->getCdfilial() != $origem['CDFILIAL'])){
                        if (($origem['VRORCAFINA'] + $origem['VREXTOLEORCA']) > $centcustDestino->getVrrealorca()){
                            if ($centcustDestino)
                                 $this->updateCentcust($centcustDestino, $origem);
                            else $this->insereCentcust($origem, $filial, $destino);
                        }
                    }
                }else $this->insereCentcust($origem, $filial, $destino);
            }
        }
    }

    /**
     * [efetuaOrcamentoPorEmpresa Realiza o cadastramento Automático
     * da empresa de origem para a empresa de destino]
     */
    public function efetuaOrcamentoPorEmpresa($filial, $origem, $destino, $paramGeral, $opcao){
        $centcustDestino = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getCentcust(
            $destino['CDEMPRESA'],
            $filial,
            $origem['CDCENTCUST'],
            $origem['CDCLASSFINA'],
            $origem['DTORCAFINA'],
            $this->nrorg
        );
        if ($opcao == 'I'){
            if ($centcustDestino){
                if (($origem['VREXTOLEORCA'] == 0) && ($origem['VRORCAFINA'] == 0)){
                    if ($paramGeral->getIdconsorcfin() == 'S'){
                        if (($origem['VRORCAFINA'] + $origem['VREXTOLEORCA']) >= $centcustDestino->getVrrealorca()){
                            $this->updateCentcust($centcustDestino, $origem);
                        }
                    }else{
                        $this->updateCentcust($centcustDestino, $origem);
                    }
                }
            }else{
                $this->insereCentcust($origem, $filial, $destino);
            }
        }else{
            if ($centcustDestino){
                if (($destino['CDEMPRESA'] != $origem['CDEMPRESA']) ||
                    ($filial != $origem['CDFILIAL'])){
                    if ($paramGeral->getIdconsorcfin() == 'S'){
                        if (($origem['VRORCAFINA'] + $origem['VREXTOLEORCA']) >= $centcustDestino->getVrrealorca()){
                            $this->updateCentcust($centcustDestino, $origem);
                        }
                    }else{
                        $this->updateCentcust($centcustDestino, $origem);
                    }
                }
            }else{
                $this->insereCentcust($origem, $filial, $destino);
            }
        }
    }

    public function insereCentcust($origem, $cdfilial, $destino){
        $centcustDestino = $this->bco57000RecalculoOrcamentoCentroCustoFactory->insereCentcust(
            0,
            $origem['VREXTOLEORCA'],
            $origem['VRORCAFINA'],
            $destino['CDEMPRESA'],
            $cdfilial,
            $origem['CDCENTCUST'],
            $origem['CDCLASSFINA'],
            $origem['DTORCAFINA'],
            $this->nrorg
        );
        //$this->entityManager->persist($centcustDestino);
        $this->entityManager->flush();
    }

    public function updateCentcust($centcustDestino, $origem){
    /*  $centcustDestino->$origem['VREXTOLEORCA'];
        $centcustDestino->$origem['VRORCAFINA'];
        $this->entityManager->persist($centcustDestino);
        $this->entityManager->flush();
    */
        // update orcamento
        $centcustDestino = $this->bco57000RecalculoOrcamentoCentroCustoFactory->updateCentcust(
            $origem['VREXTOLEORCA'], $origem['VRORCAFINA'], $origem['DTORCAFINA'], $origem['CDCENTCUST'], $origem['CDCLASSFINA']
        );
    }

    public function recalcularOrcamentoCentroCusto($centroCusto, $row) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $paramGeral = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getParamGeral($this->nrorg);
            foreach ($centroCusto as $centCusto) {
                $periodos = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getDatasPeriodos(
                    $row['DTORCAFINA_INI'],
                    $row['DTORCAFINA_FIN'],
                    $this->nrorg
                );

                foreach($periodos as $periodo){
                    $valorTotal = 0;
                    list($dia, $mes, $ano) = explode("/", $periodo['DTMES']);
                    $ultimoDia = cal_days_in_month(CAL_GREGORIAN, $mes , $ano);
                    $ultimoDiaMes = $ultimoDia.'/'.$mes.'/'.$ano;
                    $tipoRecalculo = null;
                    $filial = null;
                    
                    if ($paramGeral->getIdtporcafina() == 'F'){
                        $filial = $row['CDFILIAL'];
                        $tipoRecalculo = 'F';
                    }else{
                        $filial = $this->environment->getCdfilial();
                        $tipoRecalculo = 'E';
                    }
                    $valorTotal = $this->getValorTotalTitulos(
                        $row['CDEMPRESA'], $filial, $periodo['DTMES'], $ultimoDiaMes
                    );
                    
                    $centcust = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getCentcust(
                        $row['CDEMPRESA'], $filial, $centCusto['CDCENTCUST'],
                        $periodo['DTMES'], $this->nrorg
                    );
                    if ($centcust){
                        $centcust->setVrrealorca($valorTotal);
                        $this->entityManager->persist($centcust);
                        $this->entityManager->flush();
                    }else{
                        if ($valorTotal != 0){
                            $centcust = $this->bco57000RecalculoOrcamentoCentroCustoFactory->insereCentcust(
                                $valorTotal, 0, 0, $row['CDEMPRESA'], $filial,
                                $centCusto['CDCENTCUST'], $periodo['DTMES'], $this->nrorg
                            );
                            
                            $this->entityManager->persist($centcust);
                            $this->entityManager->flush();
                        }
                    }
                }
            }
            $this->entityManager->flush();
            $connection->commit();
        }catch (\Exception $e){
            $connection->rollback();
            throw $e;
        }
    }

    public function getValorTotalTitulos($cdempresa, $cdfilial, $dtmes, $ultimoDiaMes){
        $paramsFilial = array(
            'CDEMPRESA'     => $cdempresa,
            'NRORG'         => $this->nrorg,
            'CDFILIAL'      => $cdfilial,
            'DTMOVFLUXO_INI' => $dtmes,
            'DTMOVFLUXO_FIN' => $ultimoDiaMes
        );
        $vrrealorca = 0;
        $totalValorAtual = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getValorTotalContas($paramsFilial);
        $ajustesBoleto = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getTotalAjusteBoleto($paramsFilial);
        $vrrealorca = $totalValorAtual['VRATUAPAG'];
        foreach($ajustesBoleto as $ajuste){
            if ($ajuste['VRAJUSTE']){
                if ($ajuste['TIPOAJUS'] == 'DESCONTO'){
                    $vrrealorca = $vrrealorca - $ajuste['VRAJUSTE'];
                }
                if ($ajuste['TIPOAJUS'] == 'ACRESCIMO'){
                    $vrrealorca = $vrrealorca + $ajuste['VRAJUSTE'];
                }
            }
        }
        return $vrrealorca;
    }

    public function formataData($data)
    {
        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDatadeString($data);
        $data = DateUtil::truncateData($data);
    
        return $data;
    }


    /**
     * Função Responsável por receber os dados do controller validar a 
     * a transferencia de um Centro de Custo/ Classificação Origem para o Destino
     * @param DataSet 
     */
    public function transfereOrcamento($dados) {
        try {
            
            //Valida se a parametrização do cliente Parametros para Financeiro é por FILIAl
            // Caso for ele vai habilitar o campo de filial na tela e sera passado na current Row o indice com valor da Filial
            if(!$dados['CDFILIAL']) {
                $filial =  $this->environment->getCdfilial();
            } else {
                $filial = $dados['CDFILIAL'];
            }
            // Valida se o centro de custo e classificação financeira são os mesmos origem e destino
            if(($dados["CDCENTCUSTORIGEM"] == $dados["CDCENTCUST"]) && ($dados["CDCLASSFINAORIGEM"] == $dados["CDCLASSFINA"])){
                throw new \Exception('Operação Bloqueada!. </br> Não é permitido fazer a transferência de orçamento de para a mesma origem e destino');
            }
            
            $paramBusca = array(
                'CDCENTCUST'    => $dados["CDCENTCUSTORIGEM"],
                'CDCLASSFINA'   => $dados["CDCLASSFINAORIGEM"],
                'CDFILIAL'      => $filial,
                'CDEMPRESA'     => $dados["CDEMPRESA"],
                'DTORCAFINA'    => '01'.'/'.$dados["DTMES"].'/'.$dados["ANO"],
                'NRORG'         => $this->nrorg
            );
            
            //Busca os dados de Centro de Custo/Classificação Financeira de origem para validações dos dados
            $retorno = $this->bco57000RecalculoOrcamentoCentroCustoFactory->validaValorOrcamento($paramBusca);
           
            //Valida se existe cadastro para os parmetros informados
            if(!$retorno){
                throw new \Exception('Atenção! A transferência não pode ser concluída devido aos valores do orçamento de origem não possuir dados cadastrados para a competência '.$dados["DTMES"].'/'.$dados["ANO"].' de referência');
            } else {
                //Itera sobre o retorno para validar se o valor da transferência é menor que o de origem
                foreach($retorno as $dadosOrigem) {
                    //Retorna mensagem para o usuário impedindo que seja tranferido valores negativos e nulos
                    if($dadosOrigem['VRORCAFINA'] < floatval($dados["VRORCAFINA"])){
                       throw new \Exception('Operação Bloqueada!</br> O valor a ser transferido deve ser menor ou igual ao valor do centro de custo / classificação financeira de origem cadastrado!
                            </br> </br>Valor do orçamento de origem:
                           ' . number_format($dadosOrigem['VRORCAFINA'], 2, ',', ' ').' </br> 
                           Valor a ser transferido: '.number_format($dados["VRORCAFINA"], 2, ',', ' '));
                    } else {
                        
                        // Verifica se existe algum orçamento de destino para o mês de referência, se tiver atualiza, se não, insere.
                         $param = array(
                            'CDCENTCUST'    => $dados["CDCENTCUST"],
                            'CDCLASSFINA'   => $dados["CDCLASSFINA"],
                            'CDFILIAL'      => $filial,
                            'CDEMPRESA'     => $dados["CDEMPRESA"],
                            'DTORCAFINA'    => '01'.'/'.$dados["DTMES"].'/'.$dados["ANO"],
                            'NRORG'         => $this->nrorg
                        );
                        
                        // Busca os dados de Orçamento Destino
                        $orcamentoDestino = $this->bco57000RecalculoOrcamentoCentroCustoFactory->validaValorOrcamento($param);
                        
                        if(!$orcamentoDestino){ 
                            
                            // Caso não tenha; insere um novo no destivo 
                            $dataOrcamento = '01'.'/'.$dados["DTMES"].'/'.$dados["ANO"];
                            
                            //Insere Orçamento Centro de Custo no Destino
                            $this->bco57000RecalculoOrcamentoCentroCustoFactory->insereCentcust(
                                0, $dadosOrigem['VREXTOLEORCA'], floatval($dados["VRORCAFINA"]), $dados["CDEMPRESA"],
                                $filial, $dados["CDCENTCUST"], $dados["CDCLASSFINA"], $dataOrcamento, $this->nrorg
                            );
                            
                            $this->entityManager->flush();
                                
                            $vrDiferenca =  floatval($dadosOrigem["VRORCAFINA"]) - floatval($dados["VRORCAFINA"]); // Atenção. diminuí com o valor que é preenchido na tela
                            
                            //Atualiza o Orçamento Financeiro Origem
                            $this->bco57000RecalculoOrcamentoCentroCustoFactory->updateCentcust(
                                $dadosOrigem['VREXTOLEORCA'], $vrDiferenca, $dadosOrigem["DTORCAFINA"], $dadosOrigem['CDCENTCUST'], $dadosOrigem['CDCLASSFINA']
                            );
                            
                            $this->entityManager->flush();
                           
                        } else { // Se tiver. altera o valor e atualiza.
                            
                            //Itera sobre os dados do orcamento já cadastrado para atualizar os valores.  
                            foreach($orcamentoDestino as $orcDestino){
                                
                                //Verifica se o valor não esta zerado para acumular com valor de destino.  
                                if($orcDestino["VRORCAFINA"] > 0){
                                    $vr =  floatval($orcDestino["VRORCAFINA"]) + $dados["VRORCAFINA"];
                                } else {
                                    $vr =  floatval($dados["VRORCAFINA"]);
                                }
                                
                                // Atualiza Orçamento Financeiro Destino
                                $this->bco57000RecalculoOrcamentoCentroCustoFactory->updateCentcust(
                                    $dadosOrigem['VREXTOLEORCA'], $vr, $orcDestino["DTORCAFINA"], $orcDestino['CDCENTCUST'], $orcDestino['CDCLASSFINA']
                                );
                                
                                $this->entityManager->flush();
                                
                                
                                $vrDiferenca =  floatval($dadosOrigem["VRORCAFINA"]) - floatval($dados["VRORCAFINA"]); 
                                
                                
                                //Atualiza o Orçamento Financeiro Origem
                                $this->bco57000RecalculoOrcamentoCentroCustoFactory->updateCentcust(
                                    $dadosOrigem['VREXTOLEORCA'], $vrDiferenca, $dadosOrigem["DTORCAFINA"], $dadosOrigem['CDCENTCUST'], $dadosOrigem['CDCLASSFINA']
                                );
                                
                                $this->entityManager->flush();
                               
                            }
                        }
                    }
                    
                    //Grava Histórico de Alteração de Orçamento nos dados de Origem
                    //Gera novo sequencial para inserção dos dados na tabela HISTORCAFINCC
                    $nrHistorcafiancc = $this->newCode->getCodeByTableName('NRHISTORCAFINCC', $this->nrorg, 1, 10);
                    $nrHistorcafianccDest = $this->newCode->getCodeByTableName('NRHISTORCAFINCC', $this->nrorg, 1, 10);
                    $dia =  DateUtil::getDia(DateUtil::getDataAtual());
                    $mes =  DateUtil::getMes(DateUtil::getDataAtual());
                    $ano =  DateUtil::getAno(DateUtil::getDataAtual());
                    $dataAtual =  $dia.'/'.$mes.'/'.$ano;
                    
                    // Parametros para inserção de Histórico de alteração Centro de custo de origem
                    $paramInsert = array(
                        'NRHISTORCAFINCC' => $nrHistorcafiancc,
                        'CDCENTCUST'      => $dadosOrigem['CDCENTCUST'],
                        'CDEMPRESA'       => $dadosOrigem['CDEMPRESA'],
                        'CDFILIAL'        => $dadosOrigem['CDFILIAL'],
                        'CDCLASSFINA'     => $dadosOrigem['CDCLASSFINA'],
                        'DTORCAFINA'      => $dadosOrigem['DTORCAFINA'],
                        'DSJUSTIFICATIVA' => $dados["DSJUSTIFICATIVA"],
                        'DTALTERACAO'     => $dataAtual,
                        'CDOPERADOR'      => $this->operador,
                        'VRORCAFINAANT'   => $dados["VRORCAFINA"], // Valor da transferência
                        'VREXTOLEORCAANT' => $vrDiferenca,
                        'TPTRANSF'        => 'S',
                        'NRORG'           => $this->nrorg
                    );
                      
                    // Salva Justificativa de alteração do Orçamento de Centro de Custo
                    $this->bco57000RecalculoOrcamentoCentroCustoFactory->insereJustificativaOrcaCentCust($paramInsert);
                    
                    $paramInsert = array(   // Dados a serem salvos no histórico do centro de cuso de destino
                        'NRHISTORCAFINCC' => $nrHistorcafianccDest,
                        'CDCENTCUST'      => $dados['CDCENTCUST'],
                        'CDEMPRESA'       => $dados['CDEMPRESA'],
                        'CDFILIAL'        => $dados['CDFILIAL'],
                        'CDCLASSFINA'     => $dados['CDCLASSFINA'],
                        'DTORCAFINA'      => $orcDestino['DTORCAFINA'],
                        'DSJUSTIFICATIVA' => $dados["DSJUSTIFICATIVA"],
                        'DTALTERACAO'     => $dataAtual,
                        'CDOPERADOR'      => $this->operador,
                        'VRORCAFINAANT'   => $dados["VRORCAFINA"], // Valor da transferência
                        'VREXTOLEORCAANT' => $vrDiferenca,
                        'TPTRANSF'        => 'E',
                        'NRORG'           => $this->nrorg
                    );
                    // Salva Justificativa de alteração do Orçamento de Centro de Custo no destino
                    $this->bco57000RecalculoOrcamentoCentroCustoFactory->insereJustificativaOrcaCentCust($paramInsert);
                    $this->entityManager->flush();
                }
                
            }

        } catch(\Exception $e) {
             throw $e;
        }
    }
    
    /***************************************************************************************************************
      ************* IMPORTAÇÃO DE ORÇAMENTO FINANCEIRO POR CENTRO DE CUSTO - PLANILHA XLS/XLSX (EXCEL) ************* 
      **************************************************************************************************************/
    
    public function leituraOrcamentoFinanceiro($file){
        try {
            $xlsFile = FileLoader::loadFromBase64($file['content'], $file['name']);
            
            if(!$xlsFile)
                throw new XLS\Exception(self::IMPORT_LOAD_FILE_ERROR);
                
            $worksheet   = $xlsFile->getActiveSheet();
            
            $columnNames = array(
                'CDCENTCUST',	    'CDCLASSFINA',	    'CDEMPRESA',	    'CDFILIAL',	        'DTANO',	
                'VRORCAFINAJAN',	'VREXTOLEORCAJAN',	'VRORCAFINAFEV',	'VREXTOLEORCAFEV',	'VRORCAFINAMAR',	
                'VREXTOLEORCAMAR',	'VRORCAFINAABR',	'VREXTOLEORCAABR',	'VRORCAFINAMAI',	'VREXTOLEORCAMAI',	
                'VRORCAFINAJUN',	'VREXTOLEORCAJUN',	'VRORCAFINAJUL',	'VREXTOLEORCAJUL',	'VRORCAFINAAGO',	
                'VREXTOLEORCAAGO',	'VRORCAFINASET',	'VREXTOLEORCASET',	'VRORCAFINAOUT',	'VREXTOLEORCAOUT',	
                'VRORCAFINANOV',	'VREXTOLEORCANOV',	'VRORCAFINADEZ',	'VREXTOLEORCADEZ',	'DSJUSTIFICALT'
            );
            
            $table = FileReader::readWorksheetTable($worksheet, $columnNames);
            
            return $table;
        } catch(\Exception $e) {
             throw $e;
        }
    }
    
    public function formataOrcamentoFinanceiro(Table $table){
        $columnsReference = $table->getColumnsReference();
        $tableRows        = $table->getRows();
        $columnsNumber    = $table->getNumberOfColumns();
        
        $paramgeral = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getParamGeral($this->nrorg);
        
        foreach ($tableRows as &$row) {
            for($column = 0; $column < $columnsNumber; $column++){
                if(PHP_VERSION_ID >= 70400)
                     $columnLetterIndex = PhpSpreadsheet\Cell\Coordinate::stringFromColumnIndex($column + 1);
                else $columnLetterIndex = PhpSpreadsheet\Cell\Coordinate::stringFromColumnIndex($column + 1);
                
                $columnName = $columnsReference[$columnLetterIndex];
                
                $row['content'][$columnName] = self::formataValor($row['content'][$columnName], $columnName, $paramgeral); 
            }
        }
        
        return $tableRows;
    }
    
    public function validaOrcamentoFinanceiro(Table $table){
        $errors = array();
        
        $tableRows        = $table->getRows();
        $columnsReference = $table->getColumnsReference();
        $columnsNumber    = $table->getNumberOfColumns();
        
        $paramgeral = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getParamGeral($this->nrorg);
        
        foreach ($tableRows as $tableRow) {
            for($column = 0; $column < $columnsNumber; $column++){
                if(PHP_VERSION_ID >= 70400)
                     $columnLetterIndex = PhpSpreadsheet\Cell\Coordinate::stringFromColumnIndex($column + 1);
                else $columnLetterIndex = PhpSpreadsheet\Cell\Coordinate::stringFromColumnIndex($column + 1);
                
                $columnName = $columnsReference[$columnLetterIndex];
                $value      = $tableRow['content'][$columnName];
                $valores    = $tableRow['content'];
                
                $error = self::validaCampoOrcamentoFinanceiro($valores, $value, $tableRow['index'], $columnLetterIndex, $columnName, $paramgeral);
                
                if($error !== NULL)
                    array_push($errors, $error->toArray());
            }
        }
        
        if(count($errors) > 0)
            throw new XLS\Exception($errors, self::IMPORT_FILE_FORMAT_ERROR);
    }
    
    private function validaCampoOrcamentoFinanceiro($valores, $value, $tableRow, $column, $columnName, $paramgeral){
        $error = NULL;
        
        $optional = self::isFieldValueOptional($columnName, $paramgeral);
        
        if($value == NULL && $value !== 0 && !$optional){
            $error = new ImportacaoOrcamentoFinanceiroError(self::IMPORT_EMPTY_CELL_VALUE_ERROR, $tableRow, $column, $columnName);
        }else{
            if(self::isFieldToSearch($columnName)){
                $data = self::existsFieldOnDB($value, $columnName, $paramgeral);
                
                if(!$data && ($value != NULL || $value === 0))
                    $error = new ImportacaoOrcamentoFinanceiroError(self::IMPORT_NOT_FOUND_CELL_ERROR, $tableRow, $column, $columnName);
            }else{
                $columnName_vrorcafina = substr($columnName, 0, 10);
                $columnName_mes        = substr($columnName, -3, 3);
                
                if($columnName_vrorcafina == 'VRORCAFINA'){
                    $columnName_vrextoleorca_name = 'VREXTOLEORCA'.$columnName_mes;
                    
                    $value = !empty($value)?$value:0;
                    $columnName_vrextoleorca_value = !empty($valores[$columnName_vrextoleorca_name])?$valores[$columnName_vrextoleorca_name]:0;
                    
                    $vrorcafina_total = $value * (1 +($columnName_vrextoleorca_value / 100));
                    
                    $meses = [
                        'JAN' => '01',
                        'FEV' => '02',
                        'MAR' => '03',
                        'ABR' => '04',
                        'MAI' => '05',
                        'JUN' => '06',
                        'JUL' => '07',
                        'AGO' => '08',
                        'SET' => '09',
                        'OUT' => '10',
                        'NOV' => '11',
                        'DEZ' => '12'
                    ];
                    $dtorcafina = '01/'.$meses[$columnName_mes].'/'.$valores['DTANO'];
                    // Busca Orçamento Financeiro por Centro de Custo
                    $orcamento_old = $this->bco57000RecalculoOrcamentoCentroCustoFactory->buscaOrcafinacentcust(
                        $this->nrorg, $valores['CDCENTCUST'], $valores['CDCLASSFINA'], $valores['CDEMPRESA'], $valores['CDFILIAL'],
                        $dtorcafina
                    );
 
                    if($orcamento_old)
                        if($orcamento_old['VRREALORCA'] > $vrorcafina_total)
                            $error = new ImportacaoOrcamentoFinanceiroError(self::IMPORT_CALC_VRORCAFINA_ERROR, $tableRow, $column, [$columnName, $columnName_vrextoleorca_name], [$orcamento_old['VRREALORCA'], $vrorcafina_total]);
                
                }
            }
        }
        
        return $error;
    }
    
    private function isFieldValueOptional($columnName, $paramgeral){
        $optionalColumnNames = array(
            'VRORCAFINAJAN',	'VREXTOLEORCAJAN',	'VRORCAFINAFEV',	'VREXTOLEORCAFEV',	'VRORCAFINAMAR',	
            'VREXTOLEORCAMAR',	'VRORCAFINAABR',	'VREXTOLEORCAABR',	'VRORCAFINAMAI',	'VREXTOLEORCAMAI',	
            'VRORCAFINAJUN',	'VREXTOLEORCAJUN',	'VRORCAFINAJUL',	'VREXTOLEORCAJUL',	'VRORCAFINAAGO',	
            'VREXTOLEORCAAGO',	'VRORCAFINASET',	'VREXTOLEORCASET',	'VRORCAFINAOUT',	'VREXTOLEORCAOUT',	
            'VRORCAFINANOV',	'VREXTOLEORCANOV',	'VRORCAFINADEZ',	'VREXTOLEORCADEZ',	'DSJUSTIFICALT'
        );
        
        // Caso o Orçamento Financeiro seja de Tipo Empresa, a Filial passa a ser Opcional
        if($paramgeral->getIdtporcafina() == 'E')
            array_push($optionalColumnNames, 'CDFILIAL');
        
        return in_array($columnName, $optionalColumnNames);
    }
    
    private function isFieldToSearch($columnName){
        $seachColumnNames = array(
            'CDCENTCUST', 'CDCLASSFINA', 'CDEMPRESA', 'CDFILIAL'
        );
        return in_array($columnName, $seachColumnNames);
    }
    
    private function existsFieldOnDB($value, $columnName, $paramgeral){
        switch($columnName){
            case 'CDCENTCUST':
                $value = (strval($value) != NULL)?self::leftZeroFill(strval($value), 4):$value;
                return ($this->bco57000RecalculoOrcamentoCentroCustoFactory->getCentcustImp($value, $this->nrorg));
                break;
            case 'CDCLASSFINA':
                $value = (strval($value) != NULL)?self::leftZeroFill(strval($value), 6):$value;
                return ($this->bco57000RecalculoOrcamentoCentroCustoFactory->getClassfinancImp($value, $this->nrorg));
                break;
            case 'CDEMPRESA':
                $value = (strval($value) != NULL)?self::leftZeroFill(strval($value), 2):$value;
                return ($this->bco57000RecalculoOrcamentoCentroCustoFactory->getEmpresaImp($value, $this->nrorg));
                break;
            case 'CDFILIAL':
                if($paramgeral->getIdtporcafina() == 'E')
                    $value = $value??$this->filial;
                $value = (strval($value) != NULL)?self::leftZeroFill(strval($value), 4):$value;
                return ($this->bco57000RecalculoOrcamentoCentroCustoFactory->getFilialImp($value, $this->nrorg));
                break;
            default:
                return false;
        }
    }
    
    private function formataValor($value, $columnName, $paramgeral){
        switch($columnName){
            case 'CDCENTCUST':
                $value = (strval($value) != NULL)?self::leftZeroFill(strval($value), 4):$value;
                break;
            case 'CDCLASSFINA':
                $value = (strval($value) != NULL)?self::leftZeroFill(strval($value), 6):$value;
                break;
            case 'CDEMPRESA':
                $value = (strval($value) != NULL)?self::leftZeroFill(strval($value), 2):$value;
                break;
            case 'CDFILIAL':
                if($paramgeral->getIdtporcafina() == 'E')
                    $value = $value??$this->filial;
                $value = (strval($value) != NULL)?self::leftZeroFill(strval($value), 4):$value;
                break;
            case 'DTANO':
                $value = strval($value);
                break;
            case 'VRORCAFINAJAN':
            case 'VREXTOLEORCAJAN':
            case 'VRORCAFINAFEV':
            case 'VREXTOLEORCAFEV':
            case 'VRORCAFINAMAR':
            case 'VREXTOLEORCAMAR':
            case 'VRORCAFINAABR':
            case 'VREXTOLEORCAABR':
            case 'VRORCAFINAMAI':
            case 'VREXTOLEORCAMAI':
            case 'VRORCAFINAJUN':
            case 'VREXTOLEORCAJUN':
            case 'VRORCAFINAJUL':
            case 'VREXTOLEORCAJUL':
            case 'VRORCAFINAAGO':
            case 'VREXTOLEORCAAGO':
            case 'VRORCAFINASET':
            case 'VREXTOLEORCASET':
            case 'VRORCAFINAOUT':
            case 'VREXTOLEORCAOUT':
            case 'VRORCAFINANOV':
            case 'VREXTOLEORCANOV':
            case 'VRORCAFINADEZ':
            case 'VREXTOLEORCADEZ':
                $value = $value??0;
                $value = str_replace(',', '.', $value);
                $value = floatval($value);
                break;
            case 'DSJUSTIFICALT':
                $value = $value?substr('[VIA IMPORTAÇÃO] '.strval($value), 0, 250):'[VIA IMPORTAÇÃO]';
                break;
        }
        
        return $value;
    }
    
    private function leftZeroFill($string, $length){
        $string = str_pad(strval($string), $length, '0', STR_PAD_LEFT); 
        return $string;
    }

    public function constroiOrcamentoFinanceiro($tableRows) {
        
        // Orçamentos Financeiros
        $orcamentos_aux = array();
        
        // Monta Orçamentos Financeiros
        foreach($tableRows as $tableRow){
            // Dados do Orçamento Financeiro
            array_push($orcamentos_aux, $tableRow['content']);
        }
        
        $orcamentos = array();
        
        foreach($orcamentos_aux as $orcamento_aux){
            
            $meses = [
                '01' => 'JAN',
                '02' => 'FEV',
                '03' => 'MAR',
                '04' => 'ABR',
                '05' => 'MAI',
                '06' => 'JUN',
                '07' => 'JUL',
                '08' => 'AGO',
                '09' => 'SET',
                '10' => 'OUT',
                '11' => 'NOV',
                '12' => 'DEZ'
            ];
            
            foreach($meses as $key => $mes){
                $aux = [
                    'CDCENTCUST'    => $orcamento_aux['CDCENTCUST'],
                    'CDCLASSFINA'   => $orcamento_aux['CDCLASSFINA'],
                    'CDEMPRESA'     => $orcamento_aux['CDEMPRESA'],
                    'CDFILIAL'      => $orcamento_aux['CDFILIAL'],
                    'DTORCAFINA'    => '01/'.$key.'/'.$orcamento_aux['DTANO'],
                    'VRORCAFINA'    => $orcamento_aux['VRORCAFINA'.$mes],
                    'VREXTOLEORCA'  => $orcamento_aux['VREXTOLEORCA'.$mes],
                    'DSJUSTIFICALT' => $orcamento_aux['DSJUSTIFICALT']
                ];
                
                array_push($orcamentos, $aux);
            }
        }
        
        return $orcamentos;
    }

    public function geraOrcamentosFinanceirosCentroCusto($orcamentos) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            
            // Percorre Orçamentos Financeiros
            foreach($orcamentos as $orcamento){
                
                // Converte para duas casas decimais
                $orcamento['VRORCAFINA'] = floatval(number_format($orcamento['VRORCAFINA'], 2));
                $orcamento['VREXTOLEORCA'] = floatval(number_format($orcamento['VREXTOLEORCA'], 2));
                
                // Busca Orçamento Financeiro por Centro de Custo
                $orcamento_old = $this->bco57000RecalculoOrcamentoCentroCustoFactory->buscaOrcafinacentcust(
                    $this->nrorg, $orcamento['CDCENTCUST'], $orcamento['CDCLASSFINA'], $orcamento['CDEMPRESA'], $orcamento['CDFILIAL'],
                    $orcamento['DTORCAFINA']
                );
                
                // Verifica se o Orçamento Financeiro existe para o Periodo
                if($orcamento_old){
                    // Updata Orçamento Financeiro por Centro de Custo
                    $this->bco57000RecalculoOrcamentoCentroCustoFactory->updataOrcafinacentcust(
                        $this->nrorg, $orcamento['CDCENTCUST'], $orcamento['CDCLASSFINA'], $orcamento['CDEMPRESA'], $orcamento['CDFILIAL'],
                        $orcamento['DTORCAFINA'], $orcamento['VRORCAFINA'], $orcamento['VREXTOLEORCA']
                    );
                }else{
                    // Insere Orçamento Financeiro por Centro de Custo
                    $this->bco57000RecalculoOrcamentoCentroCustoFactory->insereOrcafinacentcust(
                        $this->nrorg, $orcamento['CDCENTCUST'], $orcamento['CDCLASSFINA'], $orcamento['CDEMPRESA'], $orcamento['CDFILIAL'],
                        $orcamento['DTORCAFINA'], $orcamento['VRORCAFINA'], $orcamento['VREXTOLEORCA']
                    );
                }
                
                $nrhistorcafincc = $this->newCode->getCodeByTableName('NRHISTORCAFINCC', $this->nrorg, 1, 10);
                $vrorcafinaant   = isset($orcamento_old["VRORCAFINA"])   ? $orcamento_old["VRORCAFINA"]   : $orcamento['VRORCAFINA'];
                $vrextoleorcaant = isset($orcamento_old["VREXTOLEORCA"]) ? $orcamento_old["VREXTOLEORCA"] : $orcamento['VREXTOLEORCA'];
                
                $this->bco57000RecalculoOrcamentoCentroCustoFactory->insereHistorcafincc(
                    $this->nrorg, $nrhistorcafincc, $orcamento['CDCENTCUST'], $orcamento['CDCLASSFINA'], $orcamento['CDEMPRESA'], 
                    $orcamento['CDFILIAL'], $orcamento['DTORCAFINA'], $orcamento['DSJUSTIFICALT'],
                    $this->operador, $vrorcafinaant, $vrextoleorcaant
                );
            }
            
            $this->entityManager->flush();
            $connection->commit();
        }catch (\Exception $e){
            $connection->rollback();
            throw $e;
        }
    }
    
}
