<?php
namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Services;
use Doctrine\ORM\EntityManager;
use Teknisa\GER\Modules\Fiscal\Factory;
use Geral\Entities\Prazopgtclie;
use Financeiro\Entities\Sdoccorr;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\FIN\Modules\MovimentacaoBancaria\Factories\LancamentoCreditoFactory;
use Teknisa\FIN\Modules\FluxoCaixa\Services\FluxoDeCaixaService;
use Teknisa\FIN\Services\ContabilizacaoService;

class LancamentoCreditoService
{
    protected $environment;
    protected $newCode;
    protected $entityManager;
    protected $factory;
    protected $serviceFluxo;
    protected $contabilizacaoService;
    
    public function __construct($environment, $newCode, EntityManager $entityManager,
                                LancamentoCreditoFactory $factory,
                                FluxoDeCaixaService $serviceFluxo,
                                ContabilizacaoService $contabilizacaoService) {
        $this->environment = $environment;
        $this->newCode = $newCode;
        $this->entityManager = $entityManager;
        $this->factory = $factory;
        $this->serviceFluxo = $serviceFluxo;
        $this->contabilizacaoService = $contabilizacaoService;
    }


    public function insereLancamento($row, $response){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $nrorg          = $this->environment->getNrOrgTrab();
            $dtlancsald     = $this->formataData($row["DTLANCSALD"]);
            $vrlancsald     = $row["VRLANCSALD"];
            $cdreducont     = $row["CDREDUCONT"];
            $cdfilial       = $row["CDFILIAL"];
            $nmfilial       = $row["NMFILIAL"];
            $cdcentcust     = $row["CDCENTCUST"];
            $nmcentcust     = $row["NMCENTCUST"];
            $cdempresa      = $row["CDEMPRESA"];
            $nmempresa      = $row["NMEMPRESA"];
            $vrlancsalddisp = $row["VRLANCSALD"];
            $vrlancsaldutil = 0;
            
            if($row["__is_new"]){
                // Gera Novo Código Lançamento de Crédito
                $nrseqlancsald  = $this->newCode->getCodeByTableName('LANCSALDCAIXA', $nrorg, 1, 10);
                
                // Novo Código Movimentação Bancária
                $nrmovbanc = self::geraNovoCodigo($nrorg, 'MOVBANCARIO'.$cdreducont);
                
                // Histório da Movimentação Bancária
                $dshismbanc = 'Lanc. Saldo - Unid.: '.$cdfilial.' C. Custo: '.$cdcentcust;
    
                // Insere Movimentação Bancária
                /** @var Movbancario $movbancario */
                $movbancario = $this->factory->insereMovBancario(
                    $nrorg, $nrmovbanc, $cdreducont, $dtlancsald, $vrlancsald, $dshismbanc
                );
                
                $this->entityManager->persist($movbancario);
                $this->entityManager->flush();
                
                // Contabiliza Lançamento de Crédito
                $nrlanccont = $this->contabilizacaoService->contabilizaLancamentoControleBancario($cdempresa, $cdreducont, $nrmovbanc, "S", "S", $cdcentcust, $nrseqlancsald);
                
                // Insere Lançamento de Crédito
                /** @var Lancsaldcaixa $lancdecredito */
                $lancdecredito = $this->factory->insereLancamentoCredito(
                    $cdcentcust, $cdempresa, $cdfilial, $cdreducont, $dtlancsald, $nrorg, $vrlancsald, $nrseqlancsald, $vrlancsalddisp, $nrmovbanc, $nrlanccont, $vrlancsaldutil
                );
                
                $this->entityManager->persist($lancdecredito);
            }
            
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            $this->contabilizacaoService->saveContabilizacaoData($response);
            throw $e;
        }
    }
    
    /**
     * Formata Data
     * @param $data
     * @return \DateTime|null|\Setforms2\Shared\ORM\DateTime|string
     */
    public static function formataData(
        $data
    ){
        if (!$data)
            return null;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }
    
    /**
     * Gera Novo Codigo
     * @param $nrorg
     * @param $cdcontador
     * @param int $intervalo
     * @param int $tamanho
     * @return int|null|string
     * @throws \Exception
     */
    public function geraNovoCodigo(
        $nrorg, $cdcontador, $intervalo = 1, $tamanho = 10
    ){
        try {
            // Gera Novo Código Boleto
            $novocodigo = $this->newCode->getCodeByTableName($cdcontador, $nrorg, $intervalo, $tamanho);
            return $novocodigo;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function removeLancamento($row){
        try {
            // Variáveis de Inicialização
            $nrseqlancsald  = $row["NRSEQLANCSALD"];
            $nrorg          = $row["NRORG"];
            $nrlanccont     = $row["NRLANCCONT"];
            $cdempresa      = $row["CDEMPRESA"];
            $cdreducont     = $row["CDREDUCONT"];
            $nrmovbanc      = $row["NRMOVBANC"];
            
            // Busca Lançamento Contábil
            /** @var Lanccont $nrlanccont */
            $nrlancconts = $this->factory->buscaLanccont(
                $nrorg, $cdempresa, $nrlanccont
            );
            
            // Remove Lançamento Contabil
            if($nrlancconts){
                foreach($nrlancconts as $nrlanccont)
                    $this->entityManager->remove($nrlanccont);
            }
            
            // Busca Lançamento Saldo Caixa
            /** @var Lancsaldcaixa $lancdecredito */
            $lancdecredito = $this->factory->findLancsaldcaixa($nrorg, $nrseqlancsald);
            
            // Remove Lançamento Saldo Caixa
            if($lancdecredito){
                $this->entityManager->remove($lancdecredito);
                
                // Busca Movimento Bancário
                $movbancario = $this->factory->findMovbancario($nrorg, $cdreducont, $nrmovbanc);
                if($movbancario)
                    $this->entityManager->remove($movbancario);
            }
            
            $this->entityManager->flush();
        }catch (\Exception $e) {
            throw $e;
        }
    }
}