<?php
namespace Teknisa\FIN\Services;

use Doctrine\ORM\EntityManager;

/**
 * Class ArquivosService
 * @package Teknisa\FIN\Services
 */
class ArquivosService{
    
    protected $fileUploader;
    protected $fileServerUrl;
    
    /**
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager, $fileUploader, $fileServerUrl){
        $this->entityManager  = $entityManager;
        $this->fileUploader   = $fileUploader;
        $this->_fileServerUrl = $fileServerUrl;
    }

    public function getFileServerUrl(){
        return $this->_fileServerUrl;
    }
    /**
     * @desc   Download Arquivo
     * @param  $arquivo
     * @throws \Exception
     */
    public function downloadArquivo($arquivo){
        try{
            if(isset($arquivo) && file_exists($arquivo)){
                $tipo = null;

                switch(strtolower(substr(strrchr(basename($arquivo),"."),1))){
                    case "gif" : $tipo="image/gif"                    ; break;
                    case "png" : $tipo="image/png"                    ; break;
                    case "jpg" : $tipo="image/jpg"                    ; break;
                    case "wma" : $tipo="audio/mime"                   ; break;
                    case "txt" : $tipo="text/plain"                   ; break;
                    case "rem" : $tipo="text/plain"                   ; break;
                    case "mp3" : $tipo="audio/mpeg"                   ; break;
                    case "mp4" : $tipo="video/mpeg-4"                 ; break;
                    case "pdf" : $tipo="application/pdf"              ; break;
                    case "zip" : $tipo="application/zip"              ; break;
                    case "doc" : $tipo="application/msword"           ; break;
                    case "xls" : $tipo="application/vnd.ms-excel"     ; break;
                    case "exe" : $tipo="application/octet-stream"     ; break;
                    case "ppt" : $tipo="application/vnd.ms-powerpoint"; break;
                    case "php" :                                      ; break;
                    case "htm" :                                      ; break;
                    case "html":                                      ; break;
                }

                header("Content-Type: ".$tipo);
                header("Content-Length: ".filesize($arquivo));
                header("Content-Disposition: attachment; filename=".basename($arquivo));
                readfile($arquivo);
                unlink($arquivo);

                die();
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function downloadArquivoAnexo($arquivo){
        try{
            if(isset($arquivo) && file_exists($arquivo)){
                $tipo = null;

                switch(strtolower(substr(strrchr(basename($arquivo),"."),1))){
                    case "gif" : $tipo="image/gif"                    ; break;
                    case "png" : $tipo="image/png"                    ; break;
                    case "jpg" : $tipo="image/jpg"                    ; break;
                    case "wma" : $tipo="audio/mime"                   ; break;
                    case "txt" : $tipo="text/plain"                   ; break;
                    case "rem" : $tipo="text/plain"                   ; break;
                    case "mp3" : $tipo="audio/mpeg"                   ; break;
                    case "mp4" : $tipo="video/mpeg-4"                 ; break;
                    case "pdf" : $tipo="application/pdf"              ; break;
                    case "zip" : $tipo="application/zip"              ; break;
                    case "doc" : $tipo="application/msword"           ; break;
                    case "xls" : $tipo="application/vnd.ms-excel"     ; break;
                    case "exe" : $tipo="application/octet-stream"     ; break;
                    case "ppt" : $tipo="application/vnd.ms-powerpoint"; break;
                    case "php" :                                      ; break;
                    case "htm" :                                      ; break;
                    case "html":                                      ; break;
                }

                header("Content-Type: ".$tipo);
                header("Content-Length: ".filesize($arquivo));
                header("Content-Disposition: attachment; filename=".basename($arquivo));
                readfile($arquivo);
                
                die();
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function downloadArquivoAnexoFileServer($arquivo, $fileName){
        try{
            
            $contentLenghtArray = array();
            foreach (get_headers($arquivo) as $value){
            		
                if (strpos($value, 'Content-Length: ') !== false) {
                	array_push($contentLenghtArray, str_replace('Content-Length: ', '',$value));
                }
            
            };
            $contentLenght = max($contentLenghtArray);
            
            if(isset($arquivo) && (!array_search('HTTP/1.1 404 Not Found', get_headers($arquivo)))){
                $tipo = null;

                switch(strtolower(substr(strrchr(basename($arquivo),"."),1))){
                    case "gif" : $tipo="image/gif"                    ; break;
                    case "png" : $tipo="image/png"                    ; break;
                    case "jpg" : $tipo="image/jpg"                    ; break;
                    case "wma" : $tipo="audio/mime"                   ; break;
                    case "txt" : $tipo="text/plain"                   ; break;
                    case "rem" : $tipo="text/plain"                   ; break;
                    case "mp3" : $tipo="audio/mpeg"                   ; break;
                    case "mp4" : $tipo="video/mpeg-4"                 ; break;
                    case "pdf" : $tipo="application/pdf"              ; break;
                    case "zip" : $tipo="application/zip"              ; break;
                    case "doc" : $tipo="application/msword"           ; break;
                    case "xls" : $tipo="application/vnd.ms-excel"     ; break;
                    case "exe" : $tipo="application/octet-stream"     ; break;
                    case "ppt" : $tipo="application/vnd.ms-powerpoint"; break;
                    case "php" :                                      ; break;
                    case "htm" :                                      ; break;
                    case "html":                                      ; break;
                }
                
                header("Content-Type: ".$tipo);
                header("Content-Length: ".$contentLenght);
                header("Content-Disposition: attachment; filename=".basename($fileName));
                readfile($arquivo);
                
                die();
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
}