var bco56200_LancamentoDespesa_Controller = function(ScreenService, General, templateManager, contabilizacaoController){

    
    this.newLancDespCaixa = function (args) {
        var widget = templateManager.container.getWidget('LancamentoDespesasGrid');
        widget.newRow();
        
        widget.getField('CDCENTCUST').setValue('');
        widget.getField('NMCENTCUST').setValue('');
        widget.getField('NMCENTCUST').dataSourceFilter[0].value = '';
        widget.getField('NMCENTCUST').readOnly = true;
        widget.getField('NMCENTCUST').reload();
        
        templateManager.updateTemplate();
        
    };
    
    this.setDescLancDesp = function (args) {
        args.row.DSLANCDESP = args.row.NMDESPESA;
    };
    
    this.saveLancDespCaixa = function (args) {
        var widget = templateManager.container.getWidget('LancamentoDespesasGrid');
        
        if(widget.isValid()){
            
            if(widget.currentRow.VRLANCDESP <= 0){
                ScreenService.showMessage('Operação bloqueada. Valor deve ser maior que zero.');
                return;
            }
    
            General.callRestEngine('fin_movimentacaoBancaria#/saveLancDespCaixa', 'Row', widget.currentRow).then(function() {
                ScreenService.showMessage('Registros salvo com sucesso.');
                ScreenService.closeSwipe();
                
                widget.activate();
                widget.reload();
            }, function (error) {
                if (contabilizacaoController.checkErrosContabilizacao(error)) {
                    ScreenService.showMessage(error.data.error);
                }
            });
            
        }
    };
    
    this.deleteLancDespCaixa = function (args) {
        var widget = templateManager.container.getWidget('LancamentoDespesasGrid');
        
        ScreenService.confirmMessage('Deseja realmente excluir o registro?', 'question', function() {

            General.callRestEngine('fin_movimentacaoBancaria#/deleteLancDespCaixa', 'Row', widget.currentRow).then(function() {
                ScreenService.showMessage('Registros excluído com sucesso.');
                ScreenService.closeSwipe();
                
                widget.activate();
                widget.reload();
            });

        });
        
    };
    
    this.setCentCustFil = function (args) {
        var widget = templateManager.container.getWidget('LancamentoDespesasGrid');
        var value = '';
        var readOnly = true;
        
        var unidade = widget.getField('CDFILIAL').value();
        
        if(unidade){
            value = unidade;
            readOnly = false;
        }
        
        widget.getField('CDCENTCUST').setValue('');
        widget.getField('NMCENTCUST').setValue('');
        widget.getField('NMCENTCUST').dataSourceFilter[0].value = value;
        widget.getField('NMCENTCUST').readOnly = readOnly;
        widget.getField('NMCENTCUST').reload();
        
        templateManager.updateTemplate();
    };
    
    /**
     * função responsável por Abrir a tela de Filtro do Relatório Birt e 
     * setar os campos de Relatório, filial e Centro de custo com o padrão
    */
    this.openFilterReport = function(){
        var widget = templateManager.container.getWidget('report');
        ScreenService.openPopup(widget).then(function () {
            widget.getField('TCDFILI').setValue('T');
            widget.getField('TCDCENTCUST').setValue('T');
            widget.getField('NAMEREPORT').setValue('Livro Caixa');
        });
    };
    
    /**
     * Função responsável por controlar os campos de Filtro abaixo:
     * Filial(Lista ou Todos)
     * Centro de Custo(Lista ou Todos)
     * Controla a Visibilidade, Valor e Campo obrigatório de acordo com o filtro de Filial 
    */
    this.setCentCustFilFilterReport = function (args) {
        var widget = templateManager.container.getWidget('report');
        var unidade = widget.getField('CDFILIAL').value();
        
        widget.getField('NMCENTCUST').setValue('');
        widget.getField('NMCENTCUST').validations = {};
        if(unidade.length){
            widget.getField("TCDCENTCUST").readOnly = false;
        } else {
            widget.getField('TCDCENTCUST').setValue('T');
            widget.getField("TCDCENTCUST").readOnly = true;
            widget.getField('NMCENTCUST').dataSource.data = [];
            widget.getField('NMCENTCUST').setValue('');
            widget.getField("NMCENTCUST").readOnly = true;
            widget.getField('NMCENTCUST').reload();
        }
        
        
        if(Array.isArray(unidade) && unidade){
            widget.getField('NMCENTCUST').dataSourceFilter[0].value = unidade;
            widget.getField('NMCENTCUST').reload();
        } else {
            widget.getField('CDCENTCUST').setValue('');
            widget.getField('NMCENTCUST').setValue('');
            widget.getField('NMCENTCUST').dataSourceFilter[0].value = null;
            widget.getField('NMCENTCUST').reload();
        }
        
        templateManager.updateTemplate();
    };
    
    /**
     * Função responsável por controlar os campos de Filtro abaixo:
     * Lista de Filial e Centro de Custo
     * Controla a Visibilidade e Campo obrigatório de acordo com o filtro de Filial 
    */
    this.setReadSelect = function(fieldNative, widgetFiltro, nameField){
		var field = widgetFiltro.getField(nameField);
		if(fieldNative.value() === 'L'){
			field.readOnly = false;
			field.validations = {"required": true};
		} else {
			field.setValue(null);
			delete field.validations;
			field.readOnly = true;
			if (field.widget.currentRow.$error) {
        		field.widget.currentRow.$error[nameField] = [];
    		}
		}
		 templateManager.updateTemplate();
	};
	
	/**
     * Função responsável por gerar o Relatório Livro Caixa:
     * @param DTINICIAL String
     * @param DTFINAL String
     * @param CDCENTCUST String
     * @param CDFILIAL String
     * @param FORMAT String
    */
	this.generateReport = function() {
	    var widget = templateManager.container.getWidget('report');
	    if (widget.isValid()){
    		var paramRel = {
                'DTINICIAL': widget.currentRow.DTPERIODO.start,
                'DTFINAL': widget.currentRow.DTPERIODO.end,
                'CDCENTCUST': (widget.currentRow.TCDCENTCUST == 'T')? 'T': widget.currentRow.CDCENTCUST.join(","),
                'CDFILIAL': (widget.currentRow.TCDFILI == 'T')? 'T': widget.currentRow.CDFILIAL.join(","),
                '__FORMAT': widget.currentRow.FORMAT
            };
    		
    		General.openReport("fin#/geraRelLivroCaixa", 'BCO56200' ,paramRel);
    	}
	};
    
};
/*global Configuration*/
Configuration(function (ContextRegister){
    ContextRegister.register('Bco56200_LancamentoDespesa_Controller', bco56200_LancamentoDespesa_Controller);
});