const ZeedhiAPIConstructor = require('zeedhi-functional-test-api');
const z = new ZeedhiAPIConstructor(browser, protractor);
const Helper = require('teknisa-helper');
const geral = new Helper.geral();
const loginPage = new Helper.login();
const protractorHelper = require('protractor-helper');
const j = require('../../../../../json/leitorJson.po');
const titulosAPagarPagosPage = require('../../../../../page-objects/financeiro/titulosAPagar/operacoes/titulosAPagarPagos/titulosAPagarPagos.po');


describe('Títulos a Pagar/Pagos', () => {
    beforeAll(() => {
        loginPage.login(j.getValor('login'), j.getValor('senha'));
        titulosAPagarPagosPage.abrir();
    });


    afterAll(() => {
        geral.sair();
    });


    it('Tela de Títulos a Pagar/Pagos', () => {
        expect(z.component.header.getTitle(element(by.id('Títulos a Pagar/Pagos'))));
    });


    it('LANÇAMENTO DA CONTA A PAGAR', () => {
        protractorHelper.waitForElementVisibility(element(by.id('grid-10511995312368698244628')));
        geral.clickAcoes();
        geral.clickActionPorIndice(1);
        geral.esperarLoading();

        titulosAPagarPagosPage.lancamento(
            j.getValor('unidadeDeOrigem'),
            j.getValor('fornecedor'),
            j.getValor('operacao'),
            j.getValor('serie'),
            j.getValor('valorDocumento'),
            j.getValor('tipoDeConta'));
        geral.clickFinalizar();
        geral.clickOk();
    });


    it('INCLUSÃO DO AJUSTE NA CONTA A PAGAR', () => {
        protractorHelper.waitForElementVisibility(element(by.id('grid-10511995312368698244628')));
        geral.filterIt(j.getValor('fornecedor'));
        geral.clickGrid('NMRAZSOCFORN', j.getValor('fornecedor'), '10511995312368698244628');
        geral.trocarAba('Ajuste');
        geral.clickAdicionar();
        titulosAPagarPagosPage.editar(
            j.getValor('ajuste'),
            j.getValor('valor'));
        geral.clickSalvar();
        geral.clickOk();
        geral.clickVoltar();
    });


    it('INCLUSÃO DA PRORROGAÇÃO NA CONTA A PAGAR', () => {
        z.widget.grid.checkRow('10511995312368698244628', '0', 'true');
        geral.clickAcoes();
        geral.clickActionPorIndice(1);
        titulosAPagarPagosPage.incluir(
            j.getValor('vencimento'),
            j.getValor('motivo'));
        geral.clickConfirmar();
        geral.clickOk();

        // FILTRAR A CONTA A PAGAR QUE FOI PRORROGADA!!!

        browser.executeScript('$(\'div.zh-floating-control li.float-action .zh-icon-filter\').click()');
        z.field.calendar.selectDateCalendarField('DTVENBOLPAG', '31/12/2021', 'pt_br');
        z.component.footer.clickRightActionByLabel('OK', true);
        z.component.footer.clickRightActionByLabel('Aplicar filtro', true);

        geral.filterIt(j.getValor('fornecedor'));
        geral.clickGrid('NMRAZSOCFORN', j.getValor('fornecedor'), '10511995312368698244628');
        geral.clickVoltar();


        // VALIDANDO (PARCELA, FORNECEDOR E TIPO DE CONTA A PAGAR)***!


        expect(geral.gridHasText('10511995312368698244628', j.getValor('fornecedor'), 3)).
            toEqual('HELA GEWÜRZWERK HERMANN LAUE GMBH');

        geral.clickGrid('NMRAZSOCFORN', j.getValor('fornecedor'), '10511995312368698244628');
        const el = element.all(by.css('#span-field-DSBOLETPAG > p'));

        el.getText().then(function (tipoDeConta) {
            expect(tipoDeConta).toEqual(['CORREIOS']);
        });
    });


    it('EXCLUSÃO DO AJUSTE', () => {
        geral.trocarAba('Ajuste');
        geral.clickGrid('DSAJUCTPAG', j.getValor('ajuste'), '10511995311283874450629');
        geral.clickExcluir();
        geral.clickSim();
        geral.clickOk();
        geral.clickVoltar();

        // Validação (Exclusão do Ajuste)...!!!


        expect(geral.gridHasText('10511995311283874450629', j.getValor('ajuste'), 0, false)).
            not.toEqual('JUROS');
    });


    it('PARCELAMENTO DA CONTA', () => {
        geral.clickGrid('NMRAZSOCFORN', j.getValor('fornecedor'), '10511995312368698244628');

        geral.trocarAba('Parcelamento');
        geral.clickAdicionar();
        titulosAPagarPagosPage.parcela(
            j.getValor('valorLiquido'),
            j.getValor('motivoParcelamento'));
        geral.clickSalvar();

        geral.clickAdicionar();
        titulosAPagarPagosPage.parcela1(
            j.getValor('valorLiquido1'),
            j.getValor('motivoParcelamento'));
        geral.clickSalvar();

        geral.clickFinalizar();
        geral.clickOk();
        geral.clickVoltar();
        geral.filterIt(j.getValor('fornecedor'));

        expect(geral.gridHasText('10511995312368698244628', j.getValor('parcela01'), 4)).
            toEqual('001');
    });


    it('EXCLUSÃO DA PARCELA', () => {
        //   geral.filterIt(j.getValor('fornecedor'));
        geral.clickGrid('NMRAZSOCFORN', j.getValor('fornecedor'), '10511995312368698244628');
        geral.clickExcluir();
        geral.clickVencimento();
        geral.clickSim();
        geral.clickOk();
        // geral.clickVoltar();

        expect(geral.gridHasText('10511995312368698244628', j.getValor('parcela02'), 4)).
            toBeUndefined();
    });


    it('REALIZAÇÃO DA BAIXA', () => {
        z.widget.grid.checkRow('10511995312368698244628', '0', 'true');
        geral.clickAcoes();
        geral.clickActionPorIndice(4);
        geral.clickSim();
        titulosAPagarPagosPage.incluirBaixa(
            j.getValor('contaPagamento'),
            j.getValor('tipoDeBaixa'));
        geral.clickConfirmar();
        geral.clickOk();
        geral.filterIt(j.getValor('fornecedor'));
    });


    it('CANCELAMENTO DA BAIXA', () => {
        z.widget.grid.checkRow('10511995312368698244628', '0', 'true');
        geral.clickAcoes();
        geral.clickActionPorIndice(1);
        geral.clickBaixa();
        geral.clickSim();
        geral.clickOk();
    });


    it('EXCLUSÃO DO TITULO', () => {
        geral.clickGrid('NMRAZSOCFORN', j.getValor('fornecedor'), '10511995312368698244628');
        geral.clickExcluir();
        geral.clickDocumento();
        geral.clickSim();
        geral.clickOk();
        geral.filterIt(j.getValor('fornecedor'));

        expect(geral.gridHasText('10511995312368698244628', j.getValor('fornecedor'), 2)).
            toBeUndefined();
    });
});